/*****************************************************************************
 *  Rohde&Schwarz CMW500 Wideband Radio Communication Tester instrument driver
 *  1xEVDO Measurements
 *  Original Release: June 2008
 *  By: Jiri Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Version: 3.7.100
 *
 *  Modification History:
 *     see ChangeLog
 *
 *****************************************************************************/
#include "rscmwevm.h"
#include <time.h>
#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
    #define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
    #include <sys/timeb.h>
    #include <sys/types.h>
    #include <unistd.h>
    #define Sleep sleep
#else                       /* ANSI compatible */
        #include <windows.h>
#endif
#include <math.h> 

#if defined (_CVI_)
	#include <utility.h>
#endif

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
static ViString switchArr[] = {"OFF","ON",NULL};
static ViString resultTypeArr[] = {"CURR","AVER","MAX","MIN","SDEV","PMAX:CURR","PMIN:CURR",NULL};  
static ViString resultMeasTypeArr[] = {"RMS","PEAK",NULL};   
static ViString resultCodeDomainArr[] = {"CDP","CDE",NULL}; 
static ViString resultCodeDomainMonitorTypeArr[] = {"ISIG","QSIG","INV",NULL}; 
static ViString mevFilterArr[] = {"chDRC","chDATA","chAPIL",NULL};
static ViString modCharTypeArr[] = {"EVM","MERR","PERR",NULL}; 
static ViString limitPowerArr[] = {"MINP","MAXP",NULL};
static ViString layerSubtypeArr[] = {"ST01","ST2",NULL}; 
static ViString halfSlotArr[] = {"BHSL","FHSL","SHSL",NULL};
static ViString retriggerArr[] = {"OFF","ON","IFP","IFPS",NULL};
static ViString arrayCodes[] = {"OK", "ULEU", "ULEL", NULL}; 
static ViString limitSetArr[] = {"A", "B", NULL};
static ViString resultACPDirectionArr[] = {"ACPP","ACPM",NULL};
static ViString resultACPBandArr[] = {"NPOW","WPOW",NULL};
static ViString resultOLTRTraceArr[] = {"OLTR","RPIN",NULL};
static ViString resultOLTRDirectionArr[] = {"DOWN","UP",NULL};
static ViString resultChannelTypeArr[] = {"RRI","PIL","ADSC","APIL","DRC","DATA",NULL};
static ViString resultOBWFreqArr[] = {"LOW","UPP",NULL};
static ViString resultModulationTypeArr[] = {"RMS","PEAK",NULL};
static ViString resultBandArr[] = {"PNB","PWB",NULL};
static ViString resultDWCTypeArr[] = {"WOTI","WOTQ","WTFI","WTFQ",NULL};
static ViString DirectionArr[] = {"DOWN","UP", NULL};
static ViString RBWArr[] = {"F1K0","F6K25","F10K","F12K5","F25K","F30K",
							"F50K","F100","F1M0","F1M23",NULL};
/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/
ViStatus rscmwevm_CheckStatusCallback (ViSession io);
ViStatus rscmwevm_WaitForOPCCallback (ViSession io);

ViStatus _VI_FUNC rscmwevm_RsClose (ViSession vi);

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rscmwevm_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rscmwevm_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == NULL)
        {
        (void) Rs_SetErrorInfo (0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    checkErr( rscmwevm_InitWithOptions (resourceName, IDQuery, resetDevice,
                                      "QueryInstrStatus=1", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rscmwevm_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession
)
{
    ViStatus                error = VI_SUCCESS;
    ViChar                  newResourceName[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar                  newOptionString[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViBoolean               isLogicalName = 0;
    ViSession               instrSession = 0, rmSession = 0;
    RsAttrPropertiesPtr     *attrList=NULL;
    RsRepCapPtr             repCapTable = NULL;

    if (newInstrSession == NULL)
        {
        (void) Rs_SetErrorInfo (0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                         VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    /* Prepare resource manager session handle */
    checkErr (viOpenDefaultRM (&rmSession));

    /* Establish communication session with the instrument */

    viCheckErr( viOpen (rmSession, resourceName, 0, 0, &instrSession));
	viCheckErr(viClear(instrSession));
    
    /* Configure VISA Formatted I/O */
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 5000 ));
    viCheckErr( viSetBuf (instrSession, (ViUInt16) (VI_READ_BUF | VI_WRITE_BUF), 4000));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));

    checkErr( Rs_GetInfoFromResourceName (resourceName,
                                          (ViString)optionString,
                                          newResourceName,
                                          newOptionString,
                                          &isLogicalName));

    ATTRLIST(attrList,rscmwevm); 
    checkErr( Rs_SpecificDriverNew (instrSession, "rscmwevm", newOptionString, attrList));
   
    /* Build repeated capability table */
    REPCAPTABLE(repCapTable,rscmwevm);
    checkErr( Rs_BuildRepCapTable (instrSession, repCapTable));

    /* With remote control via Ethernet, remote control is not automatically
       set by means of a command. The instrument must be explicitly set to
       the REMOTE state, e.g. by sending the intf command &GTR
    */
    checkErr (viWrite (instrSession, (ViBuf) "&GTR\n", 5, NULL));
    
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
    {
        checkErr( rscmwevm_reset (instrSession));
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( rscmwevm_DefaultInstrSetup (instrSession));

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery == VI_TRUE)
        {
        ViChar rdBuffer[RSCMWEVM_IO_BUFFER_SIZE] = "";

        checkErr( Rs_GetAttribute (instrSession, NULL, (ViAttr) RSCMWEVM_ATTR_ID_QUERY_RESPONSE,
                                   0, RSCMWEVM_IO_BUFFER_SIZE, rdBuffer));

        if (strncmp (rdBuffer, RSCMWEVM_VALID_ID_RESPONSE_STRING, strlen(RSCMWEVM_VALID_ID_RESPONSE_STRING)) != 0)
            {
            viCheckErr( VI_ERROR_FAIL_ID_QUERY);
            }
        }

    if (isLogicalName == VI_FALSE)
        {
        ViInt32 oldFlag = 0;

        checkErr (Rs_GetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, &oldFlag));
        checkErr (Rs_SetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, oldFlag & 0xfffb | 0x0010));
        checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_RESOURCE_DESCRIPTOR, 0, newResourceName));
        }
    

    if (isLogicalName == VI_TRUE)
        checkErr( Rs_ApplyDefaultSetup (instrSession));

    /* Instrument session accessible by the end user is native VISA session,
       or when simulated virtual instrument (vi) session. */
    checkErr( rscmwevm_CheckStatus (instrSession));
	checkErr(Rs_SetAttribute(instrSession, "", RS_ATTR_VISA_RM_SESSION, 0, &rmSession));

Error:

	if (error < VI_SUCCESS)
	    {
	        if (instrSession != 0)
	        {
	            (void) viClose (instrSession);
	            (void) Rs_Dispose (instrSession);
	        }
        
	        if (rmSession != 0)
	            (void) viClose (rmSession);
	    }
	    else
	    {
	        *newInstrSession = instrSession;
	    }
    
    return error;
}

/*****************************************************************************
 * Function: Configure Analyzer Combined Signal Path Scenario
 * Purpose:  This function activates the combined signal path scenario and
             selects a master. The master controls the signal routing and
             analyzer settings while the combined signal path scenario is
             active.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureAnalyzerCombinedSignalPathScenario (ViSession instrSession,
                                                                        ViInt32 instance,
                                                                        ViString controlledBy)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm(rscmwevm_SetAttributeViString(instrSession, rep_cap,
        RSCMWEVM_ATTR_INPUT_COMBINED_SIGNAL_PATH_SCENARIO, controlledBy), 3, 
        "Controlled By");
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;

}

/*****************************************************************************
 * Function: Query Analyzer Combined Signal Path Catalog
 * Purpose:  This function lists all applications that can be set as master
             for the combined signal path scenario using
             ROUTe:EVDO:MEASurement<instance>:SCENario:CSPath.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_QueryAnalyzerCombinedSignalPathCatalog (ViSession instrSession,
                                                                   ViInt32 instance,
                                                                   ViInt32 arraySize,
                                                                   ViChar catalog[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    if (catalog == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 4, "catalog");   
    };
    
    sprintf (command, "ROUT:EVDO:MEAS%ld:SCEN:CAT:CSP?\n", instance);
    checkErr (rscmwevm_GetCatalog(instrSession, command, 15000, arraySize, catalog));

    checkErr( rscmwevm_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, 0);

    return error;

}

/*****************************************************************************
 * Function: Configure Analyzer Stand Alone Scenario
 * Purpose:  This function activates the standalone scenario and selects the
             input connector for the measured RF signal.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureAnalyzerStandAloneScenario (ViSession instrSession,
                                                                ViInt32 instance,
                                                                ViInt32 RXConnector,
                                                                ViInt32 RFConverter)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    ViChar      command[130] = "";
    ViString    RXConnectorArr[]={"RF1C","RF2C","RF3C","RF4C","RFAC","RFBC","R11","R12","R13","R14","R15",
                                 "R16","R17","R18","R21","R22","R23","R24","R25","R26","R27","R28","R31",
                                 "R32","R33","R34","R35","R36","R37","R38","RA1","RA2","RA3","RA4","RA5",
                                 "RA6","RA7","RA8","RB1","RB2","RB3","RB4","R11C","R12C","R13C","R14C",
								 "R21C","R22C","R23C","R24C","R31C","R32C","R33C","R34C","R41C","R42C",
								 "R43C","R44C",NULL};
    ViString    RFConverterArr[]={"RX1","RX2","RX3","RX4",
									"RX11","RX12","RX13","RX14",
									"RX21","RX22","RX23","RX24",
									"RX31","RX32","RX33","RX34",
									"RX41","RX42","RX43","RX44",NULL};  
    
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));
    
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, RXConnector, RSCMWEVM_VAL_ANALYZER_CONN_RF1C, RSCMWEVM_VAL_ANALYZER_CONNECTOR_R44C), 3, "RXConnector");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, RFConverter, RSCMWEVM_VAL_ANALYZER_SAL_CONV_RX1, RSCMWEVM_VAL_ANALYZER_SAL_CONV_RX44), 4, "RFConverter");
    sprintf (command, "ROUT:EVDO:MEAS%ld:SCEN:SAL %s,%s", instance, RXConnectorArr[RXConnector],
                                                         RFConverterArr[RFConverter]);
    
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr( rscmwevm_CheckStatus (instrSession));
    
    
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/*****************************************************************************
 * Function: Analyzer Measure Protocol Test Scenario
 * Purpose:  This function activates the Measure@ProtocolTest scenario. The
             signal routing and analyzer settings are ignored by the
             measurement application. The corresponding settings have to be
             configured within the protocol test application used in
             parallel.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_AnalyzerMeasureProtocolTestScenario (ViSession instrSession,
                                                                ViInt32 instance)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm(rscmwevm_SetAttributeViString(instrSession, rep_cap,
        RSCMWEVM_ATTR_MEASURE_PROTOCOL_TEST, NULL), 2, "Protocol Test Scenario");

Error:
    Rs_UnlockSession(instrSession, 0);

    return error;

}


/*****************************************************************************
 * Function: Query Analyzer Routing Settings
 * Purpose:  This function queries the configured routing settings.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_QueryAnalyzerRoutingSettings (ViSession instrSession,
                                                         ViInt32 instance,
                                                         ViInt32 *scenario,
                                                         ViChar  controller[],
                                                         ViInt32 *RXConnector,
                                                         ViInt32 *RFConverter)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   response [1024] = "";
    ViChar*  p2buf;
    ViString ScenarioArr[] = {"SAL","CSP","MAPR",NULL}; 
    ViString RXConnectorArr[] = {"RF1C","RF2C","RF3C","RF4C","RFAC","RFBC","R11","R12","R13","R14","R15",
                                 "R16","R17","R18","R21","R22","R23","R24","R25","R26","R27","R28","R31",
                                 "R32","R33","R34","R35","R36","R37","R38","RA1","RA2","RA3","RA4","RA5",
                                 "RA6","RA7","RA8","RB1","RB2","RB3","RB4","R11C","R12C","R13C","R14C",
								 "R21C","R22C","R23C","R24C","R31C","R32C","R33C","R34C","R41C","R42C",
								 "R43C","R44C",NULL};
    ViString    RFConverterArr[] ={ "RX1","RX2","RX3","RX4",
									"RX11","RX12","RX13","RX14",
									"RX21","RX22","RX23","RX24",
									"RX31","RX32","RX33","RX34",
									"RX41","RX42","RX43","RX44",NULL};
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
   
    sprintf (response, "ROUT:EVDO:MEAS%ld?", instance);

    checkErr (viWrite (instrSession, (ViBuf) response, (ViUInt32) strlen (response), NULL));
    checkErr (viRead (instrSession, (ViBuf) response, 1024, NULL));
    
    p2buf = strtok (response, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    *scenario = rscmwevm_StringIndex (ScenarioArr,p2buf);
    if (*scenario < 0)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
     
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    strcpy (controller, p2buf);
    if (*controller < 0)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    *RXConnector = rscmwevm_StringIndex (RXConnectorArr,p2buf);
    if (*RXConnector < 0)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;

    p2buf = strtok (NULL, ",\n");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    *RFConverter = rscmwevm_StringIndex (RFConverterArr,p2buf);
    if (*RFConverter < 0)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;

    checkErr( rscmwevm_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, 0);

    return error;

}

/*****************************************************************************
 * Function: Configure Analyzer External Attenuation
 * Purpose:  This function configures external attenuation value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureAnalyzerExternalAttenuation (ViSession instrSession,
                                                                 ViInt32 instance,
                                                                 ViReal64 externalAttenuation)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));
    
    viCheckParm(rscmwevm_SetAttributeViReal64(instrSession, rep_cap,
        RSCMWEVM_ATTR_INPUT_EXTERNAL_ATTENUTATION, externalAttenuation), 3, "External Attenuation");    
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/*****************************************************************************
 * Function: Configure Analyzer
 * Purpose:  Configures the RF input path.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureAnalyzer(ViSession   instrSession,
                                            ViInt32 instance,
                                            ViReal64    expectedNominalPower,
                                            ViReal64    margin,
                                            ViReal64    centerFrequency)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm(rscmwevm_SetAttributeViReal64(instrSession, rep_cap,
        RSCMWEVM_ATTR_ANALYZER_EXPECTED_NOMINAL_POWER, expectedNominalPower), 3, 
        "Expected Nominal Power");
    
    viCheckParm(rscmwevm_SetAttributeViReal64(instrSession, rep_cap,
        RSCMWEVM_ATTR_ANALYZER_USER_MARGIN, margin), 4, "Margin");
    
    viCheckParm(rscmwevm_SetAttributeViReal64(instrSession, rep_cap,
        RSCMWEVM_ATTR_ANALYZER_CENTER_FREQUENCY, centerFrequency), 5, "Center Frequency");
    
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/*****************************************************************************
 * Function: Configure Analyzer Channel
 * Purpose:  Selects the frequency band and frequency channel of the signal.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureAnalyzerChannel (ViSession instrSession,
                                                     ViInt32 instance,
                                                     ViInt32 band,
                                                     ViInt32 channel)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVM_ATTR_ANALYZER_BAND_CLASS, band), 3, "Analyzer Band Class");
    
    viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVM_ATTR_ANALYZER_CHANNEL, channel), 4, "Analyzer Channel");
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/*****************************************************************************
 * Function: Configure Analyzer Frequency Offset
 * Purpose:  Selects a positive or negative offset frequency to be added to
 *           the center frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureAnalyzerFrequencyOffset (ViSession instrSession,
                                                             ViInt32 instance,
                                                             ViReal64 frequencyOffset)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));
    
    viCheckParm(rscmwevm_SetAttributeViReal64(instrSession, rep_cap,
        RSCMWEVM_ATTR_ANALYZER_FREQUENCY_OFFSET, frequencyOffset), 3, "Analyzer Frequency Offset");
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/// HIFN  Selects the view to be shown when the display is switched on during 
/// HIFN  remote control.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR tab/Selects the view to be shown when the display is switched on during 
/// HIPAR tab/remote control.
ViStatus _VI_FUNC rscmwevm_ConfigureAnalyzerRemoteDisplay(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32	tab
)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, 0));

    checkErr (rscmwevm_instanceRepCap (instrSession, instance, rep_cap));
    
    viCheckParm (rscmwevm_SetAttributeViInt32 (instrSession, rep_cap,
        RSCMWEVM_ATTR_ANALYZER_REMOTE_DISPLAY, tab), 3, "Tab");
Error:
    Rs_UnlockSession (instrSession, 0);
    
    return error;    
}

/*****************************************************************************
 * Function: Configure Analyzer Code Channel Filter
 * Purpose:  This function configures code channel filter.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasCodeChannelFilter (ViSession instrSession,
                                                                ViInt32 instance,
                                                                ViInt32 DRC,
                                                                ViInt32 ACKDSC,
                                                                ViInt32 data,
                                                                ViInt32 auxPilot,
                                                                ViInt32 halfSlot)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    
    sprintf (rep_cap, "i%d,%s", instance, mevFilterArr[0]);
    viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_CODE_CHANNEL_FILTER, DRC), 3, "Multi Eval Measurement Code Channel Filter");    
    sprintf (rep_cap, "i%d,%s", instance, mevFilterArr[1]);
    viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_CODE_CHANNEL_FILTER, data), 5, "Multi Eval Measurement Code Channel Filter");    
    sprintf (rep_cap, "i%d,%s", instance, mevFilterArr[2]);
    viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_CODE_CHANNEL_FILTER, auxPilot), 6, "Multi Eval Measurement Code Channel Filter");    
    sprintf (rep_cap, "i%d", instance);
    viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_CODE_CHANNEL_ACK_DSC_FILTER, ACKDSC), 4, "Multi Eval Measurement Code Channel ACK DSC Filter");
    viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_CODE_CHANNEL_HALF_SLOT, halfSlot), 7, "Multi Eval Measurement Code Channel Half Slot");
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/*****************************************************************************
 * Function: Configure Multi Eval Measurement ACK Channel Filter
 * Purpose:  TThis function configures code ACK channel filter.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasACKChannelFilter
             (ViSession instrSession, ViInt32 instance, ViInt32 ack)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[15] = "";

    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    sprintf (rep_cap, "i%d,chACK", instance);

    viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_CODE_CHANNEL_FILTER, ack), 3, "ACK");

Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}   
    
/*****************************************************************************
 * Function: Configure Analyzer Data Channel Modulation Type
 * Purpose:  This function specifies the data channel modulation type of the
 *           reverse link signal.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasDataChannelModulationType (ViSession instrSession,
                                                                         ViInt32 instance,
                                                                         ViInt32 modulationType)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_DATA_CHANNEL_MODULATION_TYPE, modulationType), 3, "Multi Eval Measurement Data Channel Modulation Type");

Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/*****************************************************************************
 * Function: Configure Multi Eval Measurement Statistics
 * Purpose:  This function configures the scope of the multi-evaluation
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasStatistics (ViSession instrSession,
                                                        ViInt32 instance,
                                                        ViInt32 repetition,
                                                        ViInt32 stopCondition)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[50] = "";
    
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_REPETITION_MODE, repetition), 3, 
        "Repetition Mode");
    
    viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_STOP_CONDITION, stopCondition), 4, "Stop Condition");
    
    checkErr( rscmwevm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/*****************************************************************************
 * Function: Configure Multi Eval Measurement Statistics Count
 * Purpose:  Configures the statistics count. The statistic count (also termed 
 *           statistic length) is the integer number of measurement intervals 
 *           per measurement cycle (statistics cycle, single-shot measurement). 
 *           The length of a measurement interval is measurement-specific.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasStatisticsCount (ViSession instrSession,
                                                             ViInt32 instance,
                                                             ViInt32 measurement,
                                                             ViInt32 statisticCount)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[50] = "";
    ViString    mevResArr[] = {"mevMeasMOD","mevMeasSPEC"};
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, measurement, 0, 1), 3, "Measurement");     
    
    sprintf (rep_cap, "i%d,%s", instance, mevResArr[measurement]);    
    
    viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_STATISTIC_COUNT, statisticCount), 4, 
        "Statistic Count");
    
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/*****************************************************************************
 * Function: Configure Multi Eval Measurement Timeout
 * Purpose:  This function configures a timeout for the measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasTimeout (ViSession instrSession,
                                                      ViInt32 instance,
                                                      ViReal64 timeout)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm(rscmwevm_SetAttributeViReal64(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_TIMEOUT, timeout), 3, "Timeout");

Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}   
    
    
/*****************************************************************************
 * Function: Configure Multi Eval Measurement Results
 * Purpose:  This function select the evaluated results and the displayed
 *           views.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasResults (ViSession instrSession,
                                                     ViInt32 instance,
                                                     ViInt32 measurement,
                                                     ViBoolean state)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[50] = "";
    ViString    mevResArr[] = {"mevEVM","mevMERR","mevPERR","mevACP","mevCDP",
                               "mevCDE","mevPOW","mevTXM","mevMODQ","mevCP","mevOBW"};

    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, measurement, 0, 10), 3, "Measurement");     
    
    sprintf (rep_cap, "i%d,%s", instance, mevResArr[measurement]);    
    
    viCheckParm(rscmwevm_SetAttributeViBoolean(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_RESULT, state), 4, 
        "State");
    
    checkErr( rscmwevm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/*****************************************************************************
 * Function: Configure Multi Eval Measurement Statistics
 * Purpose:  Configures the scope of the multi-evaluation measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasParameters (ViSession instrSession,
                                                        ViInt32 instance,
                                                        ViBoolean measure_onException)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm(rscmwevm_SetAttributeViBoolean(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_RESULTS_REJECTION, measure_onException), 3, 
        "Measure on Exception");
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/*****************************************************************************
 * Function: Configure Multi Eval Measurement Code Domain
 * Purpose:  Defines code domain parameters.                 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasCodeDomain (ViSession instrSession,
                                                         ViInt32 instance,
                                                         ViString longCodeMaskI,
                                                         ViString longCodeMaskQ,
                                                         ViInt32 physicalLayerSubtype,
                                                         ViInt32 referencePowerMode)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";

    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    sprintf (buffer, "CONF:EVDO:MEAS%ld:MEV:ILCM %s", instance, longCodeMaskI);
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (rscmwevm_CheckStatus(instrSession));
    sprintf (buffer, "CONF:EVDO:MEAS%ld:MEV:QLCM %s", instance, longCodeMaskQ);
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (rscmwevm_CheckStatus(instrSession));
    
    viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_PHYSICAL_LAYER_SUBTYPE, physicalLayerSubtype), 5, "Multi Eval Measurement Physical Layer Subtype");
    
    viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_REF_POWER_MODE, referencePowerMode), 6, "Reference Power Mode");
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/*****************************************************************************
 * Function: Query Multi Eval Measurement Spreading Factor
 * Purpose:  This function returns the spreading factor.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasSpreadingFactor (ViSession instrSession,
                                                          ViInt32 instance,
                                                          ViInt32 *spreadingFactor)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm(rscmwevm_GetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVM_ATTR_QUERY_MULTI_EVAL_MEASUREMENT_SPREADING_FACTOR, spreadingFactor), 3, 
        "Spreading Factor");
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;

}

/*****************************************************************************
 * Function: Configure Multi Eval Measurement Results All
 * Purpose:  Enables or disables the evaluation of results and shows or hides
 *           the views in the multi evaluation measurement
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasResultsAll (ViSession instrSession,
                                                           ViInt32 instance,
                                                           ViInt32 arraySize,
                                                           ViBoolean states[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar      *pCmd = buffer;
    ViInt32     i;

    checkErr( Rs_LockSession (instrSession, 0));
    
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, arraySize, 10, 10), 3, "Array Size");
    if ( states == NULL )
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 4, "States");
    };      
    

    pCmd += sprintf (pCmd, "CONF:EVDO:MEAS%ld:MEV:RES ", instance);
    
    for ( i = 0; i < 10; i++ )
    {
        pCmd += sprintf(pCmd, "%s,", switchArr[states[i]]);
    }
    *pCmd = '\0';
    *--pCmd = '\n';
    
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (rscmwevm_CheckStatus(instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/*****************************************************************************
 * Function: Configure Multi Eval Measurement Trigger
 * Purpose:  Configures the trigger parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasTrigger (ViSession instrSession,
                                                      ViInt32 instance,
                                                      ViString source,
                                                      ViInt32 slope,
                                                      ViReal64 threshold,
                                                      ViReal64 timeout)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
   
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));
    
    viCheckParm(rscmwevm_SetAttributeViString(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_TRIGGER_SOURCE, source), 3, 
        "Source");
    
    viCheckParm(rscmwevm_SetAttributeViReal64(instrSession, rep_cap,
            RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_TRIGGER_TIMEOUT, timeout), 6, "Timeout");   
    
    viCheckParm(rscmwevm_SetAttributeViReal64(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_TRIGGER_THRESHOLD, threshold), 5, "Threshold"); 
    viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_TRIGGER_SLOPE, slope), 4, "Slope");
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/*****************************************************************************
 * Function: Configure Multi Eval Measurement Trigger Gap
 * Purpose:  Configures a minimum time during which the IF signal must be
 *           below the trigger threshold before the trigger is armed so that
 *           an IF power trigger event can be generated.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasurementTriggerGap
             (ViSession instrSession, ViInt32 instance, ViReal64 minTriggerGap)

{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm(rscmwevm_SetAttributeViReal64(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_TRIGGER_GAP, minTriggerGap), 3, "Min Trigger Gap");

Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}   

/// HIFN Defines a delay time for the measurement relative to the "IF Power" or external trigger events.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init 
/// HIPAR instrSession/or rscmwevm_InitWithOptions function. 
/// HIPAR instance/No. of generator firmware application.
/// HIPAR offset/Sets a delay time for the measurement relative to the "IF Power" or external trigger events.
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasMinimumTriggerOffset (ViSession instrSession,
                                                          ViInt32 instance,
                                                          ViInt32 offset)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm(rscmwevm_SetAttributeViInt32 (instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_TRIGGER_OFFSET, offset), 3, "Offset");

Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/*****************************************************************************
 * Function: Query Multi Eval Measurement Trigger Source Catalog
 * Purpose:  This function lists all trigger source values that can be set
 *           using TRIGger:EVDO:MEAS<instance>:SOURce.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasTriggerSourceCatalog (ViSession instrSession,
                                                               ViInt32 instance,
                                                               ViInt32 arraySize,
                                                               ViChar catalog[])
{
    ViStatus    error   = VI_SUCCESS;

    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    
    checkErr (viPrintf (instrSession, "TRIG:EVDO:MEAS%ld:CAT:SOUR?", instance));
    checkErr (viRead (instrSession, (ViBuf) catalog, arraySize, NULL));
    checkErr (rscmwevm_CheckStatus (instrSession));

Error:    
    return error;
}

/*****************************************************************************
 * Function: Configure Multi Eval Select Carrier
 * Purpose:  Defines the selected carrier, also displayed at the GUI. The GUI
 *           displays the results for this carrier (if the carrier is
 *           enabled).
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalSelectCarrier (ViSession instrSession,
                                                  ViInt32 instance,
                                                  ViInt32 selectCarrier)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, 0));
    
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    
    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));  
     
    viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_SELECT_CARRIER, selectCarrier), 3, "Select Carrier");
   
 Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}
/*****************************************************************************
 * Function: Configure Multi Eval Carrier
 * Purpose:  Selects carrier's parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalCarrier (ViSession instrSession,
                                                  ViInt32 instance,
                                                  ViInt32 carrier,
                                                  ViBoolean enable,
                                                  ViReal64 offset,
                                                  ViReal64 frequency,
                                                  ViInt32 bandwidth)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, 0));
    
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, carrier, 0, 2), 3, "Carrier");
  
    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));
     
    viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_SETTING_CARRIER, carrier), 3, "Carrier");
 
    if (carrier == RSCMWEVM_VAL_CARRIER_0) {
            viCheckParm(rscmwevm_SetAttributeViReal64(instrSession, rep_cap,
            RSCMWEVM_ATTR_MULTI_EVAL_FREQUENCY_CARRIER, frequency), 6, "Frequency");
    
            viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
            RSCMWEVM_ATTR_MULTI_EVAL_BAND_WIDTH_FILTER_CARRIER, bandwidth), 7, "Band Width");
        }
        else {
            viCheckParm(rscmwevm_SetAttributeViBoolean(instrSession, rep_cap,
            RSCMWEVM_ATTR_MULTI_EVAL_SWITCH_CARRIER, enable), 4, "Enable");
            
            viCheckParm(rscmwevm_SetAttributeViReal64(instrSession, rep_cap,
            RSCMWEVM_ATTR_MULTI_EVAL_OFFSET_CARRIER, offset), 5, "Offset");
        }
        
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}
/*****************************************************************************
 * Function: Configure Multi Eval ACP Frequency Lower Offset
 * Purpose:  Defines the negative (lower) frequency offsets to be used for
 *           ACP measurements of the selected carrier. The offsets are
 *           defined relative to the analyzer frequency
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalACPFrequencyLowerOffset (ViSession instrSession,
                                                                  ViInt32 instance,
                                                                  ViInt32 arraySize,
                                                                  ViBoolean states[],
                                                                  ViReal64 offsets[]
                                                                  )
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar      *pCmd = buffer;
    ViInt32     i;

    checkErr( Rs_LockSession (instrSession, 0));
    
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, arraySize, 10, 10), 3, "Array Size");
    if ( states == NULL )
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 4, "States");
    };  
    if ( offsets == NULL )
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Offsets");
    };  
    

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    pCmd += sprintf (pCmd, "CONF:EVDO:MEAS%ld:MEV:ACP:FOFF:LOW ", instance);

    for ( i = 0; i < 10; i++ )
    {
        if ( states[i] )
            pCmd += sprintf(pCmd, "%lf,", offsets[i]);
        else
            pCmd += sprintf(pCmd, "%s,", switchArr[states[i]]);
    }
    
    *pCmd = '\0';
    *--pCmd = '\n';
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (rscmwevm_CheckStatus(instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}
/*****************************************************************************
 * Function: Configure Multi Eval ACP Frequency Upper Offset
 * Purpose:  Defines the positive (upper) frequency offsets to be used for
 *           ACP measurements of the selected carrier. The offsets are
 *           defined relative to the analyzer frequency
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalACPFrequencyUpperOffset (ViSession instrSession,
                                                                  ViInt32 instance,
                                                                  ViInt32 arraySize,
                                                                  ViBoolean states[],
                                                                  ViReal64 offsets[]
                                                                  )
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar      *pCmd = buffer;
    ViInt32     i;

    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, arraySize, 10, 10), 3, "Array Size");
    if ( states == NULL )
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 4, "States");
    };  
    if ( offsets == NULL )
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Offsets");
    };  

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    pCmd += sprintf (pCmd, "CONF:EVDO:MEAS%ld:MEV:ACP:FOFF:UPP ", instance);

    for ( i = 0; i < 10; i++ )
    {
        if ( states[i] )
            pCmd += sprintf(pCmd, "%lf,", offsets[i]);
        else
            pCmd += sprintf(pCmd, "%s,", switchArr[states[i]]);
    }
     *pCmd = '\0';
    *--pCmd = '\n';
    
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (rscmwevm_CheckStatus(instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;    
}

/// HIFN  Defines the resolution bandwidth to be used for lower frequency offset 
/// HIFN  9 to 0 of the selected carrier for ACP measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Count of items at States and Offsets arrays.
/// HIPAR RBW/Defines the resolution bandwidth to be used for the upper and lower 
/// HIPAR RBW/frequency offsets 0 to 9 of ACP measurements.
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalACPFrequencyLowerRBW (ViSession instrSession,
                                                               ViInt32 instance,
                                                               ViInt32 arraySize,
                                                               ViInt32 RBW[])
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar     *p2buffer=buffer;
    ViInt32     i = 0;
    
    checkErr (Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    
    p2buffer += sprintf (buffer, "CONF:EVDO:MEAS%ld:MEV:ACP:RBW:LOW ", instance);
    
    for (i = 0; i < arraySize; i++)
    {
        viCheckParm (rscmwevm_invalidViInt32Range (instrSession, RBW[i], RSCMWEVM_VAL_RBW_F1K0, RSCMWEVM_VAL_RBW_F1M23), 4, "RBW");
        p2buffer += sprintf (p2buffer, "%s,", RBWArr[RBW[i]]); 
    }
    
    *p2buffer = '\0';
    *--p2buffer = '\n';
        
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}


/// HIFN  Defines the resolution bandwidth to be used for upper frequency offset 
/// HIFN  0 to 9 of the selected carrier for ACP measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Size of arrays States and Offsets.
/// HIPAR RBW/Defines the resolution bandwidth to be used for the upper and lower 
/// HIPAR RBW/frequency offsets 0 to 9 of ACP measurements.
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalACPFrequencyUpperRBW (ViSession instrSession,
                                                               ViInt32 instance,
                                                               ViInt32 arraySize,
                                                               ViInt32 RBW[])
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar     *p2buffer=buffer;
    ViInt32     i = 0;
    
    checkErr (Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    
    p2buffer += sprintf (buffer, "CONF:EVDO:MEAS%ld:MEV:ACP:RBW:UPP ", instance);
    
    for (i = 0; i < arraySize; i++)
    {
        viCheckParm (rscmwevm_invalidViInt32Range (instrSession, RBW[i], RSCMWEVM_VAL_RBW_F1K0, RSCMWEVM_VAL_RBW_F1M23), 4, "RBW");
        p2buffer += sprintf (p2buffer, "%s,", RBWArr[RBW[i]]); 
    }
    
    *p2buffer = '\0';
    *--p2buffer = '\n';
        
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}


/// HIFN  Defines the negative (lower) frequency offsets to be used for ACP 
/// HIFN  measurements of the selected carrier. The offsets are defined relative 
/// HIFN  to the analyzer frequency
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     none
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:EVDO:MEAS<instance>:MEValuation:ACP:EXTended:FOFFsets:LOWer 
/// HIFN  <FreqOffset9>, ..., <FreqOffset0>
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Count of items at States and Offsets arrays.
/// HIPAR states/Array of 10 states which switch frequency offsets.
/// HIPAR offsets/Array of 10 frequency offsets
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalACPFrequencyLowerOffsetExtended (ViSession instrSession,
                                                                          ViInt32 instance,
                                                                          ViInt32 arraySize,
                                                                          ViBoolean states[],
                                                                          ViReal64 offsets[])
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar      *pCmd = buffer;
    ViInt32     i;

    checkErr( Rs_LockSession (instrSession, 0));
    
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, arraySize, 20, 20), 3, "Array Size");
    if ( states == NULL )
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 4, "States");
    };  
    if ( offsets == NULL )
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Offsets");
    };  
    

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    pCmd += sprintf (pCmd, "CONF:EVDO:MEAS%ld:MEV:ACP:EXT:FOFF:LOW ", instance);

    for ( i = 0; i < arraySize; i++ )
    {
        if ( states[i] )
            pCmd += sprintf(pCmd, "%lf,", offsets[i]);
        else
            pCmd += sprintf(pCmd, "%s,", switchArr[states[i]]);
    }
    
    *pCmd = '\0';
    *--pCmd = '\n';
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (rscmwevm_CheckStatus(instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}


/// HIFN  Defines the positive (upper) frequency offsets to be used for ACP 
/// HIFN  measurements of the selected carrier. The offsets are defined relative 
/// HIFN  to the analyzer frequency
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     none
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:EVDO:MEAS<instance>:MEValuation:ACP:EXTended:FOFFsets:UPPer 
/// HIFN  <FreqOffset0>, ..., <FreqOffset9>
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Size of arrays States and Offsets.
/// HIPAR states/Array of 10 states which switch frequency offsets.
/// HIPAR offsets/Array of 20 frequency offsets:
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalACPFrequencyUpperOffsetExtended (ViSession instrSession,
                                                                          ViInt32 instance,
                                                                          ViInt32 arraySize,
                                                                          ViBoolean states[],
                                                                          ViReal64 offsets[])
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar      *pCmd = buffer;
    ViInt32     i;

    checkErr( Rs_LockSession (instrSession, 0));
    
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, arraySize, 20, 20), 3, "Array Size");
    if ( states == NULL )
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 4, "States");
    };  
    if ( offsets == NULL )
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Offsets");
    };  
    

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    pCmd += sprintf (pCmd, "CONF:EVDO:MEAS%ld:MEV:ACP:EXT:FOFF:UPP ", instance);

    for ( i = 0; i < arraySize; i++ )
    {
        if ( states[i] )
            pCmd += sprintf(pCmd, "%lf,", offsets[i]);
        else
            pCmd += sprintf(pCmd, "%s,", switchArr[states[i]]);
    }
    
    *pCmd = '\0';
    *--pCmd = '\n';
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (rscmwevm_CheckStatus(instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}


/// HIFN  Defines the resolution bandwidth to be used for lower frequency offset 
/// HIFN  9 to 0 of the selected carrier for ACP measurements.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     none
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:EVDO:MEAS<instance>:MEValuation:ACP:EXTended:RBW:LOWer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Count of items at States and Offsets arrays.
/// HIPAR RBW/Defines the resolution bandwidth to be used for the upper and lower 
/// HIPAR RBW/frequency offsets 0 to 19 of ACP measurements.
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalACPFrequencyLowerRBWExtended (ViSession instrSession,
                                                                       ViInt32 instance,
                                                                       ViInt32 arraySize,
                                                                       ViInt32 RBW[])
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar     *p2buffer=buffer;
    ViInt32     i = 0;
    
    checkErr (Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    
    p2buffer += sprintf (buffer, "CONF:EVDO:MEAS%ld:MEV:ACP:EXT:RBW:LOW ", instance);
    
    for (i = 0; i < arraySize; i++)
    {
        viCheckParm (rscmwevm_invalidViInt32Range (instrSession, RBW[i], RSCMWEVM_VAL_RBW_F1K0, RSCMWEVM_VAL_RBW_F1M23), 4, "RBW");
        p2buffer += sprintf (p2buffer, "%s,", RBWArr[RBW[i]]); 
    }
    
    *p2buffer = '\0';
    *--p2buffer = '\n';
        
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}


/// HIFN  Defines the resolution bandwidth to be used for upper frequency offset 
/// HIFN  0 to 9 of the selected carrier for ACP measurements.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     none
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:EVDO:MEAS<instance>:MEValuation:ACP:EXTended:RBW:UPPer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Size of arrays States and Offsets.
/// HIPAR RBW/Defines the resolution bandwidth to be used for the upper and lower 
/// HIPAR RBW/frequency offsets 0 to 19 of ACP measurements.
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalACPFrequencyUpperRBWExtended (ViSession instrSession,
                                                                       ViInt32 instance,
                                                                       ViInt32 arraySize,
                                                                       ViInt32 RBW[])
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar     *p2buffer=buffer;
    ViInt32     i = 0;
    
    checkErr (Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    
    p2buffer += sprintf (buffer, "CONF:EVDO:MEAS%ld:MEV:ACP:EXT:RBW:UPP ", instance);
    
    for (i = 0; i < arraySize; i++)
    {
        viCheckParm (rscmwevm_invalidViInt32Range (instrSession, RBW[i], RSCMWEVM_VAL_RBW_F1K0, RSCMWEVM_VAL_RBW_F1M23), 4, "RBW");
        p2buffer += sprintf (p2buffer, "%s,", RBWArr[RBW[i]]); 
    }
    
    *p2buffer = '\0';
    *--p2buffer = '\n';
        
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/// HIFN  Enables or disables the list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwgm_init or 
/// HIPAR instrSession/rscmwgm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR state/Enables or disables the list mode.
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasurementListModeState(
                                                            ViSession   instrSession,
                                                            ViInt32     instance,
                                                            ViBoolean   state)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  rep_cap[50] = "";

    checkErr (Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));
    
    viCheckParm(rscmwevm_SetAttributeViBoolean(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_LIST_MODE_STATE, state), 3, 
        "State");
    
    checkErr( rscmwevm_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, 0);    
    return error;
}


/// HIFN  Configures list mode number of measured segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwgm_init or 
/// HIPAR instrSession/rscmwgm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR measuredSegments/Defines the number of segments in the entire measurement interval.
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasurementListModeSegmentCount(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 measuredSegments
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar  rep_cap[50] = "";

    checkErr (Rs_LockSession (instrSession, 0)); 

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));
    
    viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_LIST_MODE_COUNT, measuredSegments), 3, "Measured Segments");
    
    checkErr( rscmwevm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, 0);    
    return error;
} 


/// HIFN  Specifies whether a trigger event will initiate a measurement of the 
/// HIFN  entire measurement interval (comprising the number of segments defined 
/// HIFN  via: RSCMWEVM_ATTR_MULTI_EVAL_LIST_MODE_COUNT) or the retrigger 
/// HIFN  information from the segments is used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR retriggerMode/Selects the retrigger mode:
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasListModeRetriggerMode(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 retriggerMode
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[50] = "";

    checkErr (Rs_LockSession (instrSession, 0)); 

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));
    
    viCheckParm(rscmwevm_SetAttributeViInt32 (instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_RETRIGGER_MODE, retriggerMode), 3, "Retrigger Mode");
    
    checkErr( rscmwevm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, 0);    
    return error;
}

/*****************************************************************************
 * Function: Configure Multi Eval Measurement List Mode Segment CMWS Connector
 * Purpose:  Selects the RF input connector for segment <no> for CDMA2000
 *           list mode measurements with the R&S CMWS. This setting is only
 *           relevant for connector mode LIST,
 *           rscmwevm_ConfigureMEvalMeasListModeCMWSConnectorMode
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasurementListModeSegmentConnector (ViSession instrSession,
                                                                           ViInt32 instance,
                                                                           ViInt32 segment,
                                                                           ViInt32 connector)
{

	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[25] = "";
	ViChar      buffer[15]  = "";
   
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));
	
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 1000), 3, "Segment");
	
	sprintf(buffer,",seg%ld",segment);
	
	strcat(rep_cap,buffer);
    
    viCheckParm(rscmwevm_SetAttributeViInt32(instrSession, rep_cap,
            RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_LIST_MODE_SEGMENT_CMWS_CONNECTOR, connector), 4, "Connector"); 
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Defines the length, the analyzer settings, the expected PCL, retrigger 
/// HIFN  setting and evaluation offset of a selected segment. In general this 
/// HIFN  must be configured for all measured segments.    
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwgm_init or 
/// HIPAR instrSession/rscmwgm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR segmentLength/Number of measured Power Control Groups (slots) in the segment.
/// HIPAR nominalPower/Expected nominal power in the segment. The range of the expected 
/// HIPAR nominalPower/nominal power can be calculated as follows:
/// HIPAR frequency/Center frequency of the RF analyzer.
/// HIPAR retrigger/Specifies whether a trigger event is required for the segment or
/// HIPAR physicalLayerSubtype/Selects the physical layer protocol subtype.
/// HIPAR evaluatedHalfSlot/Specifies which half-slot(s) of the code channel is/are evaluated for 
/// HIPAR evaluatedHalfSlot/measurements using physical layer subtype 2.
/// HIPAR longCodeMaskI/Sets the long code mask for the I branch
/// HIPAR longCodeMaskQ/Sets the long code mask for the Q branch
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasurementListModeSegment(
    ViSession   instrSession,
    ViInt32     instance,
    ViInt32     segment,
    ViInt32     segmentLength,
    ViReal64    nominalPower,
    ViReal64    frequency,
    ViBoolean   retrigger,
    ViInt32     physicalLayerSubtype,
    ViInt32     evaluatedHalfSlot,
    ViString    longCodeMaskI,
    ViString    longCodeMaskQ
	
)
{
    ViStatus    error = VI_SUCCESS;

    checkErr (Rs_LockSession (instrSession, 0)); 

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segmentLength, 1, 1000), 4, "segmentLength");
    viCheckParm (rscmwevm_invalidViReal64Range (instrSession, nominalPower, -120.0, 50.0), 5, "Nominal Power");
    viCheckParm (rscmwevm_invalidViReal64Range (instrSession, frequency, 100.0E+6, 6000.0E+6), 6, "Frequency");
	viCheckParm (rscmwevm_invalidViUInt16Range (instrSession, retrigger, VI_FALSE, RSCMWEVM_VAL_RETRIGGER_IFPS), 7, "Retrigger");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, physicalLayerSubtype, RSCMWEVM_VAL_ST01, RSCMWEVM_VAL_ST2), 8, "Physical Layer Subtype");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, evaluatedHalfSlot, RSCMWEVM_VAL_BOTH, RSCMWEVM_VAL_SECOND), 9, "Evaluated Half Slot");
	
	
    
    checkErr (viPrintf (instrSession, "CONF:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:SET %ld,%.12lG,%.12lG,%s,%s,%s,%s,%s\n", 
                                            instance,segment,segmentLength,nominalPower,frequency, retriggerArr[retrigger],
                                            layerSubtypeArr[physicalLayerSubtype], halfSlotArr[evaluatedHalfSlot], 
                                            longCodeMaskI, longCodeMaskQ));
	
    
    checkErr( rscmwevm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, 0);    
    return error;
} 


/// HIFN  Configures the statistical length for Average, Maximum, Minimum and 
/// HIFN  Standard Deviation calculation and enables the calculation of the 
/// HIFN  different modulation results in selected segment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwgm_init or 
/// HIPAR instrSession/rscmwgm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR statisticsLength/Sets the statistical length.
/// HIPAR measurementsState/Sets state of each measurement.
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasurementListModeModulationResults(
    ViSession   instrSession,
    ViInt32     instance,
    ViInt32     segment,
    ViInt32     statisticsLength,
    ViBoolean   measurementsState[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar     *p2buffer=buffer;
    ViInt32     i;
    
    
    
    checkErr (Rs_LockSession (instrSession, 0)); 

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, statisticsLength, 1, 3000), 4, "Statistics Length");

    for ( i = 0; i < 10; i++ )
    {
        p2buffer += sprintf(p2buffer, "%s,", switchArr[measurementsState[i]]);
    }
    *--p2buffer = '\0';
   
    checkErr (viPrintf (instrSession, "CONF:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:MOD %ld,%s\n", 
                                            instance, segment, statisticsLength, buffer)); 
    
    checkErr( rscmwevm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, 0);    
    return error;
}  


/// HIFN  Configures the statistical length for Average, Maximum, Minimum and 
/// HIFN  Standard Deviation calculation and enables the calculation of the 
/// HIFN  different spectrum results in selected segment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwgm_init or 
/// HIPAR instrSession/rscmwgm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR statisticsLength/Sets the statistical length.
/// HIPAR acpState/Enables or disables measurement of Adjacent Channel Power (RMS).
/// HIPAR occupiedBandwidthState/Enables or disables measurement of occupied bandwidth.
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalMeasurementListModeSpectrumResults(
    ViSession   instrSession,
    ViInt32     instance,
    ViInt32     segment,
    ViInt32     statisticsLength,
    ViBoolean   acpState,
    ViBoolean   occupiedBandwidthState
)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr (Rs_LockSession (instrSession, 0)); 

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, statisticsLength, 1, 3000), 4, "Statistics Length");
    
    checkErr (viPrintf (instrSession, "CONF:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:SPEC %ld,%s,%s\n", 
                        instance,segment,statisticsLength, acpState?"ON":"OFF", occupiedBandwidthState?"ON":"OFF"));
    
    checkErr( rscmwevm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, 0);    
    return error;
}

/*****************************************************************************
 * Function: Configure Multi Eval Limits Modulation
 * Purpose:  This function defines upper limits for the RMS and peak values
 *           of the selected modulation characteristic.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsModulation (ViSession instrSession,
                                                           ViInt32 instance,
                                                           ViInt32 modulationCharacteristic,
                                                           ViInt32 type,
                                                           ViReal64 limitValue,
                                                           ViInt32 limitStatus)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar     *p2buffer=buffer;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, modulationCharacteristic, 0, 2), 3, "Modulation Characteristic");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, type, 0, 1), 4, "Type");
    viCheckParm (rscmwevm_invalidViReal64Range (instrSession, limitValue, 0.0, 100.0), 5, "Limit Value");       
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, limitStatus, 0, 2), 6, "Limit Status");       
    
    p2buffer += sprintf (p2buffer, "CONF:EVDO:MEAS%ld:MEV:LIM:%s:%s ", instance,
        modCharTypeArr[modulationCharacteristic], resultMeasTypeArr[type]);
    
    switch (limitStatus)
    {
        case RSCMWEVM_VAL_STATE_OFF:
            p2buffer += sprintf (p2buffer, "OFF");
            break;
        case RSCMWEVM_VAL_STATE_ON:
            p2buffer += sprintf (p2buffer, "ON");
            break;
        case RSCMWEVM_VAL_STATE_VAL:
            p2buffer += sprintf (p2buffer, "%f", limitValue);
            break;
    }

    checkErr (viPrintf (instrSession, "%s\n", buffer));
        
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error; 
}

/*****************************************************************************
 * Function: Configure Multi Eval Limits IQ Offset
 * Purpose:  This function defines an upper limit for the I/Q origin offset.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsIQOffset (ViSession instrSession,
                                                         ViInt32 instance,
                                                         ViReal64 limit,
                                                         ViInt32 limitStatus)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar     *p2buffer=buffer;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViReal64Range (instrSession, limit, -120.0, -20.0), 3, "Limit");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, limitStatus, 0, 2), 4, "Limit Status");
    
    p2buffer += sprintf (p2buffer, "CONF:EVDO:MEAS%ld:MEV:LIM:IQOF ", instance);

    switch (limitStatus)
    {
        case RSCMWEVM_VAL_STATE_OFF:
            p2buffer += sprintf (p2buffer, "OFF");
            break;
        case RSCMWEVM_VAL_STATE_ON:
            p2buffer += sprintf (p2buffer, "ON");
            break;
        case RSCMWEVM_VAL_STATE_VAL:
            p2buffer += sprintf (p2buffer, "%f", limit);
            break;
    }
    
    checkErr (viPrintf (instrSession, "%s\n", buffer));
        
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error; 
}

/*****************************************************************************
 * Function: Configure Multi Eval Limits IQ Imbalance
 * Purpose:  This function defines an upper limit for the I/Q origin offset.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsIQImbalance (ViSession instrSession,
                                                            ViInt32 instance,
                                                            ViReal64 limit,
                                                            ViInt32 limitStatus)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar     *p2buffer=buffer;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViReal64Range (instrSession, limit, -120.0, -20.0), 3, "Limit");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, limitStatus, 0, 2), 4, "Limit Status");
    
    p2buffer += sprintf (p2buffer, "CONF:EVDO:MEAS%ld:MEV:LIM:IQIM ", instance);

    switch (limitStatus)
    {
        case RSCMWEVM_VAL_STATE_OFF:
            p2buffer += sprintf (p2buffer, "OFF");
            break;
        case RSCMWEVM_VAL_STATE_ON:
            p2buffer += sprintf (p2buffer, "ON");
            break;
        case RSCMWEVM_VAL_STATE_VAL:
            p2buffer += sprintf (p2buffer, "%f", limit);
            break;
    }
    
    checkErr (viPrintf (instrSession, "%s\n", buffer));
        
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error; 
}

/*****************************************************************************
 * Function: Configure Multi Eval Limits Carrier Frequency Error
 * Purpose:  This function defines an upper limit for the carrier frequency 
 *           error.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsCarrierFrequencyError (ViSession instrSession,
                                                                      ViInt32 instance,
                                                                      ViReal64 limit,
                                                                      ViInt32 limitStatus)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar     *p2buffer=buffer;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViReal64Range (instrSession, limit, 0.0, 1000.0), 3, "Limit");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, limitStatus, 0, 2), 4, "Limit Status");
    
    p2buffer += sprintf (p2buffer, "CONF:EVDO:MEAS%ld:MEV:LIM:CFER ", instance);
    switch (limitStatus)
    {
        case RSCMWEVM_VAL_STATE_OFF:
            p2buffer += sprintf (p2buffer, "OFF");
            break;
        case RSCMWEVM_VAL_STATE_ON:
            p2buffer += sprintf (p2buffer, "ON");
            break;
        case RSCMWEVM_VAL_STATE_VAL:
            p2buffer += sprintf (p2buffer, "%f", limit);
            break;
    }
    
    checkErr (viPrintf (instrSession, "%s\n", buffer));
        
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error; 
}

/*****************************************************************************
 * Function: Configure Multi Eval Limits Transport Time Error
 * Purpose:  This function defines the absolute upper limit for the transport
 *           time error.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsTransportTimeError (ViSession instrSession,
                                                                   ViInt32 instance,
                                                                   ViReal64 limit,
                                                                   ViInt32 limitStatus)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar     *p2buffer=buffer;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViReal64Range (instrSession, limit, 0.0, 10.0), 3, "Limit");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, limitStatus, 0, 2), 4, "Limit Status");
    
    p2buffer += sprintf (p2buffer, "CONF:EVDO:MEAS%ld:MEV:LIM:TTER ", instance);

    switch (limitStatus)
    {
        case RSCMWEVM_VAL_STATE_OFF:
            p2buffer += sprintf (p2buffer, "OFF");
            break;
        case RSCMWEVM_VAL_STATE_ON:
            p2buffer += sprintf (p2buffer, "ON");
            break;
        case RSCMWEVM_VAL_STATE_VAL:
            p2buffer += sprintf (p2buffer, "%f", limit);
            break;
    }
    
    checkErr (viPrintf (instrSession, "%s\n", buffer));
        
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error; 
}

/*****************************************************************************
 * Function: Configure Multi Eval Limits Waveform Quality
 * Purpose:  This function defines the absolute upper limit for the transport
 *           time error.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsWaveformQuality (ViSession instrSession,
                                                                ViInt32 instance,
                                                                ViReal64 limit,
                                                                ViInt32 limitStatus)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar     *p2buffer=buffer;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViReal64Range (instrSession, limit, 0.0, 1.0), 3, "Limit");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, limitStatus, 0, 2), 4, "Limit Status");
    
    p2buffer += sprintf (p2buffer, "CONF:EVDO:MEAS%ld:MEV:LIM:WQU ", instance);
    switch (limitStatus)
    {
        case RSCMWEVM_VAL_STATE_OFF:
            p2buffer += sprintf (p2buffer, "OFF");
            break;
        case RSCMWEVM_VAL_STATE_ON:
            p2buffer += sprintf (p2buffer, "ON");
            break;
        case RSCMWEVM_VAL_STATE_VAL:
            p2buffer += sprintf (p2buffer, "%f", limit);
            break;
    }
    
    checkErr (viPrintf (instrSession, "%s\n", buffer));
        
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error; 
}

/*****************************************************************************
 * Function: Configure Multi Eval Limits Power
 * Purpose:  This function defines upper or lower limit for the access
 *           terminal (AT) power.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsPower (ViSession instrSession,
                                                      ViInt32 instance,
                                                      ViInt32 power,
                                                      ViReal64 limitValue,
                                                      ViInt32 limitStatus)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar      *p2buffer=buffer;
    ViReal64    min = 0.0, max = 0.0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    switch (power)
    {
        case RSCMWEVM_VAL_POWER_MIN:
            min = -128.0; max = -0.1; break;
        case RSCMWEVM_VAL_POWER_MAX:
            min = -127.9; max = 0.0; break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Power");
    }
    viCheckParm (rscmwevm_invalidViReal64Range (instrSession, limitValue, min, max), 4, "Limit");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, limitStatus, 0, 2), 5, "Limit Status");
    
    p2buffer += sprintf (p2buffer, "CONF:EVDO:MEAS%ld:MEV:LIM:%s ", instance, limitPowerArr[power]);
    switch (limitStatus)
    {
        case RSCMWEVM_VAL_STATE_OFF:
            p2buffer += sprintf (p2buffer, "OFF");
            break;
        case RSCMWEVM_VAL_STATE_ON:
            p2buffer += sprintf (p2buffer, "ON");
            break;
        case RSCMWEVM_VAL_STATE_VAL:
            p2buffer += sprintf (p2buffer, "%f", limitValue);
            break;
    }
    
    checkErr (viPrintf (instrSession, "%s\n", buffer));
        
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error; 
}


/*****************************************************************************
 * Function: Configure Multi Eval Limits Code Domain
 * Purpose:  This function defines limits for code domain results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsCodeDomain (ViSession instrSession,
                                                           ViInt32 instance,
                                                           ViInt32 codeDomain,
                                                           ViReal64 limitValue,
                                                           ViBoolean IQLeakageCheck)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[15] = "";
    ViString    code_domain_arr[] = {"mevLimitCDP","mevLimitCDE"};

    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, codeDomain, 0, 1), 3, "Code Domain");
    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));
    viCheckParm(rscmwevm_SetAttributeViBoolean(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_CDP_IQ_LEAKAGE_CHECK, IQLeakageCheck), 5, "Multi Eval Measurement CDP IQ Leakage Check");
    
    sprintf (rep_cap, "i%ld,%s", instance, code_domain_arr[codeDomain]);

    viCheckParm(rscmwevm_SetAttributeViReal64(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_CODE_DOMAIN_LIMIT, limitValue), 3, "Multi Eval Measurement Code Domain Limit");
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/*****************************************************************************
 * Function: Configure Multi Eval Limits Channel Power
 * Purpose:  This function defines limits for channel power results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsChannelPower
             (ViSession instrSession, ViInt32 instance, ViInt32 channelPower,
              ViReal64 limitValue)

{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr( Rs_LockSession (instrSession, 0));
    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));
    
    switch (channelPower)
    {
        case RSCMWEVM_VAL_DOMAIN_POW_CP:
             viCheckParm(rscmwevm_SetAttributeViReal64(instrSession, rep_cap,
             RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_UPPER_CP_LIMIT, limitValue), 4, "Limit Value");
         break;   
        
        case RSCMWEVM_VAL_DOMAIN_POW_DWCP:
             viCheckParm(rscmwevm_SetAttributeViReal64(instrSession, rep_cap,
             RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_CODE_DWCP_LIMIT, limitValue), 4, "Limit Value");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Channel Power");
    }
   
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}   
    
/*****************************************************************************
 * Function: Configure Multi Eval Limits OBW
 * Purpose:  This function defines the overall occupied bandwidth limit and
 *	         the state of the limit check for multicarrier configurations.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsOBW (ViSession instrSession,
                                                      ViInt32 instance,
                                                      ViReal64 limitValue,
                                                      ViInt32 limitStatus)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar      *p2buffer=buffer;
   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViReal64Range (instrSession, limitValue, 0, 16.0), 4, "Limit");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, limitStatus, 0, 2), 5, "Limit Status");
    
    p2buffer += sprintf (p2buffer, "CONF:EVDO:MEAS%ld:MEV:LIM:OBW:MULT ", instance);
    switch (limitStatus)
    {
        case RSCMWEVM_VAL_STATE_OFF:
            p2buffer += sprintf (p2buffer, "OFF");
            break;
        case RSCMWEVM_VAL_STATE_ON:
            p2buffer += sprintf (p2buffer, "ON");
            break;
        case RSCMWEVM_VAL_STATE_VAL:
            p2buffer += sprintf (p2buffer, "%.12lG", limitValue);
            break;
    }
    
    checkErr (viPrintf (instrSession, "%s\n", buffer));
        
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error; 
}

/*****************************************************************************
 * Function: Configure Multi Eval Limits OBW Limit Set
 * Purpose:  This function configures the OBW limits for Limit Set A or B.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsOBWLimitSet
             (ViSession instrSession, ViInt32 instance, ViInt32 limitSet,
              ViReal64 oneIsolatedCarrier, ViReal64 twoAdjacentCarriers,
              ViReal64 threeAdjacentCarriers)

{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";

    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, limitSet, 0, 1), 3, "Limit Set");
	viCheckParm (rscmwevm_invalidViReal64Range (instrSession, oneIsolatedCarrier, 0, 16.0), 4, "Isolated Carrier Limit");
    viCheckParm (rscmwevm_invalidViReal64Range (instrSession, twoAdjacentCarriers, 0, 16.0), 5, "Adjacent Carrier Limit (1)");
    viCheckParm (rscmwevm_invalidViReal64Range (instrSession, threeAdjacentCarriers, 0, 16.0), 6, "Adjacent Carrier Limit (2)");
  
	sprintf (buffer, "CONF:EVDO:MEAS%ld:MEV:LIM:OBW:SET%s %.12lG,%.12lG,%.12lG", instance, limitSetArr[limitSet],
		 									oneIsolatedCarrier, twoAdjacentCarriers, threeAdjacentCarriers);
    
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    
    checkErr( rscmwevm_CheckStatus (instrSession));
	
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error; 
}
 	
/*****************************************************************************
 * Function: Configure Multi Eval Limits OBW Limit Check
 * Purpose:  This function sets the enabled state of OBW limit checks.
		     Depending on the current band class, either Limit Set A or B applies.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsOBWLimitCheck
             (ViSession instrSession, ViInt32 instance,
              ViBoolean OBWLimitCheck)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr( Rs_LockSession (instrSession, 0));
    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

     viCheckParm(rscmwevm_SetAttributeViBoolean(instrSession, rep_cap,
             RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_OBW_LIMIT_CHECK, OBWLimitCheck), 3, "OBW Limit Check");
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
} 		
	 
/*****************************************************************************
 * Function: Query Multi Eval Limits OBW Used Limit Set
 * Purpose:  This function returns the currently used OBW limit set.
*****************************************************************************/
ViStatus _VI_FUNC rscmwevm_QueryMEvalLimitsOBWUsedLimitSet
             (ViSession instrSession, ViInt32 instance, ViInt32 *limitSet)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr( Rs_LockSession (instrSession, 0));
    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

     viCheckParm(rscmwevm_GetAttributeViInt32(instrSession, rep_cap,
             RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_OBW_USED_LIMIT_SET, limitSet), 3, "Limit Set");
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

		 
/*****************************************************************************
 * Function: Configure Multi Eval Limits Carrier Lower
 * Purpose:  Defines limits for the ACP of the selected carrier at the
 *           individual negative offset frequencies.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsCarrierLower (ViSession instrSession,
                                                             ViInt32 instance,
                                                             ViInt32 arraySize,
                                                             ViBoolean states[],
                                                             ViReal64 limits[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar      *pCmd = buffer;
    ViInt32     i;

    checkErr( Rs_LockSession (instrSession, 0));
    
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, arraySize, 10, 10), 3, "Array Size");
    if (states == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 4, "States");
    };  
    if (limits == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Limits");
    };  
    
    for ( i = 0; i < 10; i ++ )
    {
        ViReal64 limit = limits[i];
        
        if ( limit < -80.0 || 10.0 < limit )
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 5, "Limit Range");
    }
    

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    pCmd += sprintf (pCmd, "CONF:EVDO:MEAS%ld:MEV:LIM:ACP:LOW ", instance);


    for ( i = 0; i < arraySize; i++ )
    {
        if ( states[i] )
            pCmd += sprintf(pCmd, "%lf,", limits[i]);
        else
            pCmd += sprintf(pCmd, "%s,", switchArr[states[i]]);
    }
    
    *pCmd = '\0';
    *--pCmd = '\n';
    
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (rscmwevm_CheckStatus(instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;     
}
/*****************************************************************************
 * Function: Configure Multi Eval Limits Carrier Upper
 * Purpose:  Defines limits for the ACP of the selected carrier at the
 *           individual positive offset frequencies.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsCarrierUpper (ViSession instrSession,
                                                             ViInt32 instance,
                                                             ViInt32 arraySize,
                                                             ViBoolean states[],
                                                             ViReal64 limits[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar      *pCmd = buffer;
    ViInt32     i;

    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, arraySize, 10, 10), 3, "Array Size");
    if (states == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 4, "States");
    };  
    if (limits == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Limits");
    };  
    
    for ( i = 0; i < arraySize; i ++ )
    {
        ViReal64 limit = limits[i];
        
        if ( limit < -80.0 || 10.0 < limit )
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 5, "Limit Range");
    }
    

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    pCmd += sprintf (pCmd, "CONF:EVDO:MEAS%ld:MEV:LIM:ACP:UPP ", instance);

    for ( i = 0; i < 10; i++ )
    {
        if ( states[i] )
            pCmd += sprintf(pCmd, "%lf,", limits[i]);
        else
            pCmd += sprintf(pCmd, "%s,", switchArr[states[i]]);
    }
    
    *pCmd = '\0';
    *--pCmd = '\n';

    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (rscmwevm_CheckStatus(instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;     
}

/// HIFN  Defines limits for the ACP of the selected carrier at the individual 
/// HIFN  negative offset frequencies.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     none
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:EVDO:MEAS<i>:MEValuation:LIMit:ACP:LOWer:ABSolute <Limit9>, 
/// HIFN  ..., <Limit0>
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Count of items at States and Offsets arrays.
/// HIPAR states/Array of 10 states.
/// HIPAR limits/Array of 10 limits
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsCarrierLowerAbsolute (ViSession instrSession,
                                                                     ViInt32 instance,
                                                                     ViInt32 arraySize,
                                                                     ViBoolean states[],
                                                                     ViReal64 limits[])
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar      *pCmd = buffer;
    ViInt32     i;

    checkErr( Rs_LockSession (instrSession, 0));
    
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, arraySize, 10, 10), 3, "Array Size");
    if (states == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 4, "States");
    };  
    if (limits == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Limits");
    };  
    
    for ( i = 0; i < arraySize; i ++ )
    {
        ViReal64 limit = limits[i];
        
        if ( limit < -80.0 || 10.0 < limit )
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 5, "Limit Range");
    }
    

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    pCmd += sprintf (pCmd, "CONF:EVDO:MEAS%ld:MEV:LIM:ACP:LOW:ABS ", instance);


    for ( i = 0; i < arraySize; i++ )
    {
        if ( states[i] )
            pCmd += sprintf(pCmd, "%lf,", limits[i]);
        else
            pCmd += sprintf(pCmd, "%s,", switchArr[states[i]]);
    }
    
    *pCmd = '\0';
    *--pCmd = '\n';
    
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (rscmwevm_CheckStatus(instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}


/// HIFN  Defines limits for the ACP of the selected carrier at the individual 
/// HIFN  positive offset frequencies.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     none
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:EVDO:MEAS<i>:MEValuation:LIMit:ACP:UPPer:ABSolute <Limit0>, 
/// HIFN  ..., <Limit9> 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Count of items at States and Offsets arrays.
/// HIPAR states/Array of 10 states.
/// HIPAR limits/Array of 10 limits
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsCarrierUpperAbsolute (ViSession instrSession,
                                                                     ViInt32 instance,
                                                                     ViInt32 arraySize,
                                                                     ViBoolean states[],
                                                                     ViReal64 limits[])
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar      *pCmd = buffer;
    ViInt32     i;

    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, arraySize, 10, 10), 3, "Array Size");
    if (states == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 4, "States");
    };  
    if (limits == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Limits");
    };  
    
    for ( i = 0; i < arraySize; i ++ )
    {
        ViReal64 limit = limits[i];
        
        if ( limit < -80.0 || 10.0 < limit )
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 5, "Limit Range");
    }
    

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    pCmd += sprintf (pCmd, "CONF:EVDO:MEAS%ld:MEV:LIM:ACP:UPP:ABS ", instance);

    for ( i = 0; i < arraySize; i++ )
    {
        if ( states[i] )
            pCmd += sprintf(pCmd, "%lf,", limits[i]);
        else
            pCmd += sprintf(pCmd, "%s,", switchArr[states[i]]);
    }
    
    *pCmd = '\0';
    *--pCmd = '\n';

    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (rscmwevm_CheckStatus(instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}


/// HIFN  Defines limits for the ACP of the selected carrier at the individual 
/// HIFN  negative offset frequencies.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     none
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:EVDO:MEAS<i>:MEValuation:LIMit:ACP:EXTended:LOWer <Limit9>, 
/// HIFN  ..., <Limit0>
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Count of items at States and Offsets arrays.
/// HIPAR states/Array of 20 states.
/// HIPAR limits/Array of 20 limits
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsCarrierLowerExtended (ViSession instrSession,
                                                                     ViInt32 instance,
                                                                     ViInt32 arraySize,
                                                                     ViBoolean states[],
                                                                     ViReal64 limits[])
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar      *pCmd = buffer;
    ViInt32     i;

    checkErr( Rs_LockSession (instrSession, 0));
    
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, arraySize, 20, 20), 3, "Array Size");
    if (states == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 4, "States");
    };  
    if (limits == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Limits");
    };  
    
    for ( i = 0; i < arraySize; i ++ )
    {
        ViReal64 limit = limits[i];
        
        if ( limit < -80.0 || 10.0 < limit )
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 5, "Limit Range");
    }
    

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    pCmd += sprintf (pCmd, "CONF:EVDO:MEAS%ld:MEV:LIM:ACP:EXT:LOW ", instance);


    for ( i = 0; i < arraySize; i++ )
    {
        if ( states[i] )
            pCmd += sprintf(pCmd, "%lf,", limits[i]);
        else
            pCmd += sprintf(pCmd, "%s,", switchArr[states[i]]);
    }
    
    *pCmd = '\0';
    *--pCmd = '\n';
    
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (rscmwevm_CheckStatus(instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}


/// HIFN  Defines limits for the ACP of the selected carrier at the individual 
/// HIFN  positive offset frequencies.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     none
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:EVDO:MEAS<i>:MEValuation:LIMit:ACP:EXTended:UPPer <Limit0>, 
/// HIFN  ..., <Limit9> 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Count of items at States and Offsets arrays.
/// HIPAR states/Array of 20 states.
/// HIPAR limits/Array of 20 limits
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsCarrierUpperExtended (ViSession instrSession,
                                                                     ViInt32 instance,
                                                                     ViInt32 arraySize,
                                                                     ViBoolean states[],
                                                                     ViReal64 limits[])
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar      *pCmd = buffer;
    ViInt32     i;

    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, arraySize, 20, 20), 3, "Array Size");
    if (states == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 4, "States");
    };  
    if (limits == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Limits");
    };  
    
    for ( i = 0; i < arraySize; i ++ )
    {
        ViReal64 limit = limits[i];
        
        if ( limit < -80.0 || 10.0 < limit )
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 5, "Limit Range");
    }
    

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    pCmd += sprintf (pCmd, "CONF:EVDO:MEAS%ld:MEV:LIM:ACP:EXT:UPP ", instance);

    for ( i = 0; i < arraySize; i++ )
    {
        if ( states[i] )
            pCmd += sprintf(pCmd, "%lf,", limits[i]);
        else
            pCmd += sprintf(pCmd, "%s,", switchArr[states[i]]);
    }
    
    *pCmd = '\0';
    *--pCmd = '\n';

    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (rscmwevm_CheckStatus(instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}


/// HIFN  Defines limits for the ACP of the selected carrier at the individual 
/// HIFN  negative offset frequencies.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     none
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:EVDO:MEAS<i>:MEValuation:LIMit:ACP:EXTended:LOWer:ABSolute 
/// HIFN  <Limit9>, ..., <Limit0>
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Count of items at States and Offsets arrays.
/// HIPAR states/Array of 20 states.
/// HIPAR limits/Array of 20 limits
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsCarrierLowerAbsoluteExtended (ViSession instrSession,
                                                                             ViInt32 instance,
                                                                             ViInt32 arraySize,
                                                                             ViBoolean states[],
                                                                             ViReal64 limits[])
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar      *pCmd = buffer;
    ViInt32     i;

    checkErr( Rs_LockSession (instrSession, 0));
    
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, arraySize, 20, 20), 3, "Array Size");
    if (states == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 4, "States");
    };  
    if (limits == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Limits");
    };  
    
    for ( i = 0; i < arraySize; i ++ )
    {
        ViReal64 limit = limits[i];
        
        if ( limit < -80.0 || 10.0 < limit )
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 5, "Limit Range");
    }
    

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    pCmd += sprintf (pCmd, "CONF:EVDO:MEAS%ld:MEV:LIM:ACP:EXT:LOW:ABS ", instance);


    for ( i = 0; i < arraySize; i++ )
    {
        if ( states[i] )
            pCmd += sprintf(pCmd, "%lf,", limits[i]);
        else
            pCmd += sprintf(pCmd, "%s,", switchArr[states[i]]);
    }
    
    *pCmd = '\0';
    *--pCmd = '\n';
    
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (rscmwevm_CheckStatus(instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}


/// HIFN  Defines limits for the ACP of the selected carrier at the individual 
/// HIFN  positive offset frequencies.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     none
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:EVDO:MEAS<i>:MEValuation:LIMit:ACP:EXTended:UPPer:ABSolute 
/// HIFN  <Limit0>, ..., <Limit9> 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Count of items at States and Offsets arrays.
/// HIPAR states/Array of 20 states.
/// HIPAR limits/Array of 20 limits
ViStatus _VI_FUNC rscmwevm_ConfigureMEvalLimitsCarrierUpperAbsoluteExtended (ViSession instrSession,
                                                                             ViInt32 instance,
                                                                             ViInt32 arraySize,
                                                                             ViBoolean states[],
                                                                             ViReal64 limits[])
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    ViChar      buffer[RS_MAX_MESSAGE_BUF_SIZE]="";
    ViChar      *pCmd = buffer;
    ViInt32     i;

    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, arraySize, 20, 20), 3, "Array Size");
    if (states == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 4, "States");
    };  
    if (limits == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Limits");
    };  
    
    for ( i = 0; i < arraySize; i ++ )
    {
        ViReal64 limit = limits[i];
        
        if ( limit < -80.0 || 10.0 < limit )
            viCheckParm(RS_ERROR_INVALID_PARAMETER, 5, "Limit Range");
    }
    

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    pCmd += sprintf (pCmd, "CONF:EVDO:MEAS%ld:MEV:LIM:ACP:EXT:UPP:ABS ", instance);

    for ( i = 0; i < arraySize; i++ )
    {
        if ( states[i] )
            pCmd += sprintf(pCmd, "%lf,", limits[i]);
        else
            pCmd += sprintf(pCmd, "%s,", switchArr[states[i]]);
    }
    
    *pCmd = '\0';
    *--pCmd = '\n';

    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    checkErr (rscmwevm_CheckStatus(instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/*****************************************************************************
 * Function: Multi Eval Measurement Init
 * Purpose:  Starts or restarts the measurement; the R&S CMW 500 enters the 
 *           "RUN" state. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_MEvalMeasInit(ViSession   instrSession,
                                         ViInt32 instance)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

   viCheckParm(rscmwevm_SetAttributeViString(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_INIT, NULL), 3, "Multi Eval Measurement Init");

Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Multi Eval Measurement Abort
 * Purpose:  Causes a running measurement to stop immediately; the R&S CMW 500 
 *           enters the "OFF" state. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_MEvalMeasAbort(ViSession   instrSession,
                                        ViInt32 instance)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm(rscmwevm_SetAttributeViString(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_ABORT, NULL), 3, "Multi Eval Measurement Abort");

Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Multi Eval Measurement Stop
 * Purpose:  Causes a running measurement to stop after the current evaluation 
 *           period is terminated and valid results are available; the R&S CMW 
 *           500 enters the "RDY" state. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_MEvalMeasStop(ViSession   instrSession,
                                        ViInt32 instance)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm(rscmwevm_SetAttributeViString(instrSession, rep_cap,
        RSCMWEVM_ATTR_MULTI_EVAL_MEASUREMENT_STOP, NULL), 3, "Multi Eval Measurement Stop");
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Query Multi Eval Measurement Status
 * Purpose:  Queries the main measurement state and the measurement substates. 
 *           Both measurement substates are relevant for running measurements 
 *           only. Use FETCh...MEASurement<instance>:STATe? to query the main 
 *           measurement state only. Use INITiate..., STOP..., ABORt... to 
 *           change the measurement state. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasurementStatus(ViSession   instrSession,
                                                        ViInt32 instance,
                                                        ViInt32*    mainState,
                                                        ViInt32*    syncState,
                                                        ViInt32*    resourcesState)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
   
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:STAT:ALL?", instance);

    checkErr(rscmwevm_QueryMeasStatus (instrSession, command, mainState, syncState,
        resourcesState));

    checkErr( rscmwevm_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Read Multi Eval EVM Trace
 * Purpose:  Starts a new single shot measurement and returns values of the 
 *           EVM trace results of the multi evaluation measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ReadMEvalMeasEVMTrace (ViSession instrSession,
                                                 ViInt32 instance,
                                                 ViInt32 resultType,
                                                 ViInt32 arraySize,
                                                 ViInt32 *reliability,
                                                 ViReal64 results[],
                                                 ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "READ:EVDO:MEAS%ld:MEV:TRAC:EVM:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Fetch Multi Eval EVM Trace
 * Purpose:  Returns values of the EVM trace results of the multi evaluation 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasEVMTrace (ViSession instrSession,
                                                  ViInt32 instance,
                                                  ViInt32 resultType,
                                                  ViInt32 arraySize,
                                                  ViInt32 *reliability,
                                                  ViReal64 results[],
                                                  ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:TRAC:EVM:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Read Multi Eval Magnitude Error Trace
 * Purpose:  Starts a new single shot measurement and returns the values of
 *           the magnitude error trace results of the multi evaluation 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ReadMEvalMeasMagnErrTrace (ViSession instrSession,
                                                           ViInt32 instance,
                                                           ViInt32 resultType,
                                                           ViInt32 arraySize,
                                                           ViInt32 *reliability,
                                                           ViReal64 results[],
                                                           ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "READ:EVDO:MEAS%ld:MEV:TRAC:MERR:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));

Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/*****************************************************************************
 * Function: Fetch Multi Eval Magnitude Error Trace
 * Purpose:  Returns the values of the magnitude error trace results of the 
 *           multi evaluation measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasMagnErrTrace (ViSession instrSession,
                                                            ViInt32 instance,
                                                            ViInt32 resultType,
                                                            ViInt32 arraySize,
                                                            ViInt32 *reliability,
                                                            ViReal64 results[],
                                                            ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:TRAC:MERR:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Read Multi Eval Phase Error Trace
 * Purpose:  Starts a new single shot measurement and returns the values of 
 *           the phase error trace results of the multi evaluation measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ReadMEvalMeasPhaseErrTrace (ViSession instrSession,
                                                            ViInt32 instance,
                                                            ViInt32 resultType,
                                                            ViInt32 arraySize,
                                                            ViInt32 *reliability,
                                                            ViReal64 results[],
                                                            ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "READ:EVDO:MEAS%ld:MEV:TRAC:PERR:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Fetch Multi Eval Phase Error Trace
 * Purpose:  Returns the values of the phase error trace results of the multi
 *           evaluation measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasPhaseErrTrace (ViSession instrSession,
                                                             ViInt32 instance,
                                                             ViInt32 resultType,
                                                             ViInt32 arraySize,
                                                             ViInt32 *reliability,
                                                             ViReal64 results[],
                                                             ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:TRAC:PERR:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Read Multi Eval ACP Trace
 * Purpose:  Starts a new single shot measurement and returns the adjacent
 *           channel power measured at a series of frequencies. The
 *           frequencies are determined by the offset values defined via the
 *           command CONFigure:EVDO:MEAS<instance>:MEValuation:ACP:FOFFsets.
 *           All defined offset values are considered (irrespective of their
 *           activation status). The current, average and maximum traces can
 *           be retrieved.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ReadMEvalMeasACPTrace (ViSession instrSession,
                                                  ViInt32 instance,
                                                  ViInt32 resultType,
                                                  ViInt32 arraySize,
                                                  ViInt32 *reliability,
                                                  ViReal64 results[],
                                                  ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "READ:EVDO:MEAS%ld:MEV:TRAC:ACP:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Fetch Multi Eval ACP Trace
 * Purpose:  Returns the adjacent channel power measured at a series of
 *           frequencies. The frequencies are determined by the offset values
 *           defined via the command
 *           CONFigure:EVDO:MEAS<instance>:MEValuation:ACP:FOFFsets. All
 *           defined offset values are considered (irrespective of their
 *           activation status). The current, average and maximum traces can
 *           be retrieved.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasACPTrace (ViSession instrSession,
                                                   ViInt32 instance,
                                                   ViInt32 resultType,
                                                   ViInt32 arraySize,
                                                   ViInt32 *reliability,
                                                   ViReal64 results[],
                                                   ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:TRAC:ACP:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns the adjacent channel power measured at a series of 
/// HIFN  frequencies.
/// HIFN     
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate:EVDO:MEAS<instance>:MEValuation:TRACe:ACP:AVERage?
/// HIFN     CALCulate:EVDO:MEAS<instance>:MEValuation:TRACe:ACP:CURRent?
/// HIFN     CALCulate:EVDO:MEAS<instance>:MEValuation:TRACe:ACP:MAXimum?
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwc2m_init or 
/// HIPAR instrSession/rscmwc2m_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/return limit check results for adjacent channel power.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasACPTraceLimitCheckResults(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32	resultType,
	ViInt32	arraySize,
	ViInt32*	reliability,
	ViInt32	results[],
	ViInt32*	returnedValues
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:TRAC:ACP:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Starts a new single shot measurement and returns the adjacent channel 
/// HIFN  power measured at a series of frequencies. The frequencies are 
/// HIFN  determined by the offset values defined via the command 
/// HIFN  CONFigure:EVDO:MEAS<instance>:MEValuation:ACP:FOFFsets. All defined 
/// HIFN  offset values are considered (irrespective of their activation status). 
/// HIFN  The current, average and maximum traces can be retrieved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_ReadMEvalMeasACPTraceAbsolute (ViSession instrSession,
                                                          ViInt32 instance,
                                                          ViInt32 resultType,
                                                          ViInt32 arraySize,
                                                          ViInt32 *reliability,
                                                          ViReal64 results[],
                                                          ViInt32 *returnedValues)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "READ:EVDO:MEAS%ld:MEV:TRAC:ACP:%s:ABS?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns the adjacent channel power measured at a series of 
/// HIFN  frequencies. The frequencies are determined by the offset values 
/// HIFN  defined via the command 
/// HIFN  CONFigure:EVDO:MEAS<instance>:MEValuation:ACP:FOFFsets. All defined 
/// HIFN  offset values are considered (irrespective of their activation status). 
/// HIFN  The current, average and maximum traces can be retrieved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasACPTraceAbsolute (ViSession instrSession,
                                                           ViInt32 instance,
                                                           ViInt32 resultType,
                                                           ViInt32 arraySize,
                                                           ViInt32 *reliability,
                                                           ViReal64 results[],
                                                           ViInt32 *returnedValues)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:TRAC:ACP:%s:ABS?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns the adjacent channel power measured at a series of 
/// HIFN  frequencies. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/return limit check results for adjacent channel power.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasACPTraceLimitCheckResultsAbsolute (ViSession instrSession,
                                                                            ViInt32 instance,
                                                                            ViInt32 resultType,
                                                                            ViInt32 arraySize,
                                                                            ViInt32 *reliability,
                                                                            ViInt32 results[],
                                                                            ViInt32 *returnedValues)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:TRAC:ACP:%s:ABS?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Starts a new single shot measurement and returns the adjacent channel 
/// HIFN  power measured at a series of frequencies. The frequencies are 
/// HIFN  determined by the offset values defined via the command 
/// HIFN  CONFigure:EVDO:MEAS<instance>:MEValuation:ACP:FOFFsets. All defined 
/// HIFN  offset values are considered (irrespective of their activation status). 
/// HIFN  The current, average and maximum traces can be retrieved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_ReadMEvalMeasACPTraceExtended (ViSession instrSession,
                                                          ViInt32 instance,
                                                          ViInt32 resultType,
                                                          ViInt32 arraySize,
                                                          ViInt32 *reliability,
                                                          ViReal64 results[],
                                                          ViInt32 *returnedValues)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "READ:EVDO:MEAS%ld:MEV:TRAC:ACP:EXT:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns the adjacent channel power measured at a series of 
/// HIFN  frequencies. The frequencies are determined by the offset values 
/// HIFN  defined via the command 
/// HIFN  CONFigure:EVDO:MEAS<instance>:MEValuation:ACP:FOFFsets. All defined 
/// HIFN  offset values are considered (irrespective of their activation status). 
/// HIFN  The current, average and maximum traces can be retrieved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasACPTraceExtended (ViSession instrSession,
                                                           ViInt32 instance,
                                                           ViInt32 resultType,
                                                           ViInt32 arraySize,
                                                           ViInt32 *reliability,
                                                           ViReal64 results[],
                                                           ViInt32 *returnedValues)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:TRAC:ACP:EXT:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns the adjacent channel power measured at a series of 
/// HIFN  frequencies.    
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/return limit check results for adjacent channel power.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasACPTraceLimitCheckResultsExtended (ViSession instrSession,
                                                                            ViInt32 instance,
                                                                            ViInt32 resultType,
                                                                            ViInt32 arraySize,
                                                                            ViInt32 *reliability,
                                                                            ViInt32 results[],
                                                                            ViInt32 *returnedValues)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:TRAC:ACP:EXT:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Starts a new single shot measurement and returns the adjacent channel 
/// HIFN  power measured at a series of frequencies. The frequencies are 
/// HIFN  determined by the offset values defined via the command 
/// HIFN  CONFigure:EVDO:MEAS<instance>:MEValuation:ACP:FOFFsets. All defined 
/// HIFN  offset values are considered (irrespective of their activation status). 
/// HIFN  The current, average and maximum traces can be retrieved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_ReadMEvalMeasACPTraceAbsoluteExtended (ViSession instrSession,
                                                                  ViInt32 instance,
                                                                  ViInt32 resultType,
                                                                  ViInt32 arraySize,
                                                                  ViInt32 *reliability,
                                                                  ViReal64 results[],
                                                                  ViInt32 *returnedValues)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "READ:EVDO:MEAS%ld:MEV:TRAC:ACP:EXT:%s:ABS?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns the adjacent channel power measured at a series of 
/// HIFN  frequencies. The frequencies are determined by the offset values 
/// HIFN  defined via the command 
/// HIFN  CONFigure:EVDO:MEAS<instance>:MEValuation:ACP:FOFFsets. All defined 
/// HIFN  offset values are considered (irrespective of their activation status). 
/// HIFN  The current, average and maximum traces can be retrieved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasACPTraceAbsoluteExtended (ViSession instrSession,
                                                                   ViInt32 instance,
                                                                   ViInt32 resultType,
                                                                   ViInt32 arraySize,
                                                                   ViInt32 *reliability,
                                                                   ViReal64 results[],
                                                                   ViInt32 *returnedValues)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:TRAC:ACP:EXT:%s:ABS?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns the adjacent channel power measured at a series of 
/// HIFN  frequencies.  
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/return limit check results for adjacent channel power.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasACPTraceLimitCheckResultsAbsoluteExtended (ViSession instrSession,
                                                                                    ViInt32 instance,
                                                                                    ViInt32 resultType,
                                                                                    ViInt32 arraySize,
                                                                                    ViInt32 *reliability,
                                                                                    ViInt32 results[],
                                                                                    ViInt32 *returnedValues)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:TRAC:ACP:EXT:%s:ABS?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Read Multi Eval ACP Out of Tolerance
 * Purpose:  Starts a new single shot measurement and returns the "Out of
 *           Tolerance" result, i.e. the percentage of measurement intervals
 *           of the statistic count exceeding the specified limits.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ReadMEvalMeasACPOutofTolerance (ViSession instrSession,
                                                   ViInt32 instance,
                                                   ViInt32 resultType,
                                                   ViInt32 arraySize,
                                                   ViInt32 *reliability,
                                                   ViReal64 results[],
                                                   ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
     
    sprintf (command, "READ:EVDO:MEAS%ld:MEV:ACP:%s?", instance,  resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Fetch Multi Eval ACP Out of Tolerance
 * Purpose:  Returns the "Out of Tolerance" result, i.e. the percentage of
 *           measurement intervals of the statistic count exceeding the
 *           specified limits.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasACPOutofTolerance (ViSession instrSession,
                                                   ViInt32 instance,
                                                   ViInt32 resultType,
                                                   ViInt32 arraySize,
                                                   ViInt32 *reliability,
                                                   ViReal64 results[],
                                                   ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 

    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
 
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:ACP:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Starts a new single shot measurement and returns the results in the 
/// HIFN  I/Q constellation diagram. Every fourth value corresponds to a 
/// HIFN  constellation point. The other values are located on the path between 
/// HIFN  two constellation points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result arrays.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR i/Returns the I amplitudes of the measurement.
/// HIPAR q/Returns the Q amplitudes of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_ReadMEvalMeasIQTrace (ViSession instrSession,
                                                 ViInt32 instance,
                                                 ViInt32 arraySize,
                                                 ViInt32 *reliability,
                                                 ViReal64 i[], ViReal64 q[],
                                                 ViInt32 *returnedValues)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
	ViReal64    *results;
	ViInt32     j = 0;
	
	results = malloc (2*arraySize*sizeof(ViReal64));
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    if (i == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "I");   
    };
	if (q == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Q");   
    };
    
    sprintf (command, "READ:EVDO:MEAS%ld:MEV:TRAC:IQ:CURR?\n", instance);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize*2, reliability, results, returnedValues));
    
	for (j=0; j<*returnedValues; j++)
	{
		if (j%2)
			q[(ViInt32)j/2] = results[j];
		else
			i[(ViInt32)j/2] = results[j];
	}
	
    checkErr( rscmwevm_CheckStatus (instrSession));
Error:
    if (results) free (results);
	Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns the results in the I/Q constellation diagram. Every fourth 
/// HIFN  value corresponds to a constellation point. The other values are 
/// HIFN  located on the path between two constellation points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result arrays.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR i/Returns the I amplitudes of the measurement.
/// HIPAR q/Returns the Q amplitudes of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasIQTrace (ViSession instrSession,
                                                  ViInt32 instance,
                                                  ViInt32 arraySize,
                                                  ViInt32 *reliability,
                                                  ViReal64 i[], ViReal64 q[],
                                                  ViInt32 *returnedValues)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
	ViReal64    *results;
	ViInt32     j = 0;
	
	results = malloc (2*arraySize*sizeof(ViReal64));
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    if (i == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "I");   
    };
	if (q == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Q");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:TRAC:IQ:CURR?\n", instance);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize*2, reliability, results, returnedValues));
    
	for (j=0; j<*returnedValues; j++)
	{
		if (j%2)
			q[(ViInt32)j/2] = results[j];
		else
			i[(ViInt32)j/2] = results[j];
	}
	
    checkErr (rscmwevm_CheckStatus (instrSession));
	
Error:
    if (results) free (results);
	Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Read Multi Eval ACP Out of Tolerance
 * Purpose:  Starts a new single shot measurement and returns values of the
 *           code domain power (CDP) I-Signal and Q-Signal traces. For a
 *           physical layer subtype 2 measurement the entire traces are
 *           retrieved. For a physical layer subtype 0/1 measurement only the
 *           RRI part is retrieved. The results of the current, average,
 *           maximum and minimum traces can be retrieved.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ReadMEvalMeasCodeDomainRRITrace (ViSession instrSession,
                                                            ViInt32 instance,
                                                            ViInt32 resultType,
                                                            ViInt32 codeDomain,
                                                            ViInt32 type,
                                                            ViInt32 arraySize,
                                                            ViInt32 *reliability,
                                                            ViReal64 results[],
                                                            ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, codeDomain, 0, 1), 4, "Code Domain");
    switch (codeDomain)
    {
        case RSCMWEVM_VAL_DOMAIN_ERROR:
            viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
            break;
        default:
            viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 3), 3, "Result Type");
        break;
    }
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, type, 0, 1), 5, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "READ:EVDO:MEAS%ld:MEV:TRAC:%s:%s:RRI:%s?", instance, 
        resultCodeDomainArr[codeDomain], resultCodeDomainMonitorTypeArr[type], resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Fetch Multi Eval Code Domain RRI Trace
 * Purpose:  Returns values of the code domain power (CDP) I-Signal and
 *           Q-Signal traces. For a physical layer subtype 2 measurement the
 *           entire traces are retrieved. For a physical layer subtype 0/1
 *           measurement only the RRI part is retrieved. The results of the
 *           current, average, maximum and minimum traces can be retrieved.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasCodeDomainRRITrace (ViSession instrSession,
                                                             ViInt32 instance,
                                                             ViInt32 resultType,
                                                             ViInt32 codeDomain,
                                                             ViInt32 type,
                                                             ViInt32 arraySize,
                                                             ViInt32 *reliability,
                                                             ViReal64 results[],
                                                             ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 3), 3, "Result Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, codeDomain, 0, 1), 4, "Code Domain");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, type, 0, 1), 5, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:TRAC:%s:%s:RRI:%s?", instance, 
        resultCodeDomainArr[codeDomain], resultCodeDomainMonitorTypeArr[type], resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}
/// HIFN  Returns limits check for values of the code domain power (CDP) 
/// HIFN  I-Signal and Q-Signal traces. For a physical layer subtype 2 
/// HIFN  measurement the entire traces are retrieved. For a physical layer 
/// HIFN  subtype 0/1 measurement only the RRI part is retrieved. The results of 
/// HIFN  the current, average, maximum and minimum traces can be retrieved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR codeDomain/Denotes the code domain type.
/// HIPAR type/Denotes the result type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns the limits check results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasCodeDomainRRITraceLimitsCheckResults (ViSession instrSession,
                                                                      ViInt32 instance,
                                                                      ViInt32 resultType,
                                                                      ViInt32 codeDomain,
                                                                      ViInt32 type,
                                                                      ViInt32 arraySize,
                                                                      ViInt32 *reliability,
                                                                      ViInt32 results[],
                                                                      ViInt32 *returnedValues)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 3), 3, "Result Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, codeDomain, 0, 1), 4, "Code Domain");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, type, 0, 1), 5, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:TRAC:%s:%s:RRI:%s?", instance, 
        resultCodeDomainArr[codeDomain], resultCodeDomainMonitorTypeArr[type], resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Fetch Multi Eval Code Domain RRI Trace
 * Purpose:  Return limit check results for the code domain error (CDE) or
 *           code domain power (CDP) I-Signal and Q-Signal bar graphs.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasCodeDomainRRITraceCheckLimit
             (ViSession instrSession, ViInt32 instance, ViInt32 codeDomain,
              ViInt32 type, ViInt32 arraySize, ViInt32 *reliability,
              ViReal64  results[], ViInt32 *returnedValues)

{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, codeDomain, 0, 1), 3, "Code Domain");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, type, 0, 1), 4, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Results")
;   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:TRAC:%s:%s:RRI:LIM?", instance, 
        resultCodeDomainArr[codeDomain], resultCodeDomainMonitorTypeArr[type]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Fetch Multi Eval Code Domain RRI Trace Check States
 * Purpose:  Return limit check results for the code domain error (CDE) or
 *           code domain power (CDP) I-Signal and Q-Signal bar graphs.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasCodeDomainRRITraceCheckStates
             (ViSession instrSession, ViInt32 instance, ViInt32 codeDomain,
              ViInt32 resultType, ViInt32 arraySize, ViInt32 *reliability,
              ViInt32 results[], ViInt32 *returnedValues)
      

{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, codeDomain, 0, 1), 3, "Code Domain");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 1), 4, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Results")
;   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:TRAC:%s:%s:RRI:STAT?", instance, 
        resultCodeDomainArr[codeDomain], resultCodeDomainMonitorTypeArr[resultType]);
    
    checkErr (rscmwevm_GetMeasTraceStates (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
                          
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}     
      
/*****************************************************************************
 * Function: Read Multi Eval Code Domain Pilot Trace
 * Purpose:  Starts a new single shot measurement and returns values of the
 *           code domain power (CDP) I-Signal and Q-Signal traces. Only the
 *           pilot part of a physical layer subtype 0/1 measurement is
 *           retrieved. The results of the current, average, maximum and
 *           minimum traces can be retrieved.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ReadMEvalMeasCodeDomainPilotTrace (ViSession instrSession,
                                                              ViInt32 instance,
                                                              ViInt32 resultType,
                                                              ViInt32 codeDomain,
                                                              ViInt32 type,
                                                              ViInt32 arraySize,
                                                              ViInt32 *reliability,
                                                              ViReal64 results[],
                                                              ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 3), 3, "Result Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, codeDomain, 0, 1), 4, "Code Domain");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, type, 0, 1), 5, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "READ:EVDO:MEAS%ld:MEV:TRAC:%s:%s:PIL:%s?", instance, 
        resultCodeDomainArr[codeDomain], resultCodeDomainMonitorTypeArr[type], resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Fetch Multi Eval Code Domain Pilot Trace
 * Purpose:  Returns values of the code domain power (CDP) I-Signal and
 *           Q-Signal traces. Only the pilot part of a physical layer subtype
 *           0/1 measurement is retrieved. The results of the current,
 *           average, maximum and minimum traces can be retrieved.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasCodeDomainPilotTrace (ViSession instrSession,
                                                               ViInt32 instance,
                                                               ViInt32 resultType,
                                                               ViInt32 codeDomain,
                                                               ViInt32 type,
                                                               ViInt32 arraySize,
                                                               ViInt32 *reliability,
                                                               ViReal64 results[],
                                                               ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 3), 3, "Result Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, codeDomain, 0, 1), 4, "Code Domain");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, type, 0, 1), 5, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:TRAC:%s:%s:PIL:%s?", instance, 
        resultCodeDomainArr[codeDomain], resultCodeDomainMonitorTypeArr[type], resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}
  
/// HIFN  Returns the limits check for values of the code domain power (CDP) 
/// HIFN  I-Signal and Q-Signal traces. Only the pilot part of a physical layer 
/// HIFN  subtype 0/1 measurement is retrieved. The results of the current, 
/// HIFN  average, maximum and minimum traces can be retrieved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR codeDomain/Denotes the code domain type.
/// HIPAR type/Denotes the result type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns the limits check results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasCodeDomainPilotTraceLimitsCheckResults (ViSession instrSession,
                                                                        ViInt32 instance,
                                                                        ViInt32 resultType,
                                                                        ViInt32 codeDomain,
                                                                        ViInt32 type,
                                                                        ViInt32 arraySize,
                                                                        ViInt32 *reliability,
                                                                        ViInt32 results[],
                                                                        ViInt32 *returnedValues)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 3), 3, "Result Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, codeDomain, 0, 1), 4, "Code Domain");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, type, 0, 1), 5, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:TRAC:%s:%s:PIL:%s?", instance, 
        resultCodeDomainArr[codeDomain], resultCodeDomainMonitorTypeArr[type], resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Fetch Multi Eval Code Domain Pilot Trace
 * Purpose:  Return limit check results for the code domain error (CDE) or
 *           code domain power (CDP) I-Signal and Q-Signal bar graphs.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasCodeDomainPilotTraceCheckLimit
             (ViSession instrSession, ViInt32 instance, ViInt32 codeDomain,
              ViInt32 type, ViInt32 arraySize, ViInt32 *reliability,
              ViReal64  results[], ViInt32 *returnedValues)

{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, codeDomain, 0, 1), 3, "Code Domain");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, type, 0, 1), 4, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Results")
;   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:TRAC:%s:%s:PIL:LIM?", instance, 
        resultCodeDomainArr[codeDomain], resultCodeDomainMonitorTypeArr[type]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));

Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Fetch Multi Eval Code Domain Pilot Trace Check States
 * Purpose:  Return limit check results for the code domain error (CDE) or
 *           code domain power (CDP) I-Signal and Q-Signal bar graphs.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasCodeDomainPilotTraceCheckStates
             (ViSession instrSession, ViInt32 instance, ViInt32 codeDomain,
              ViInt32 resultType, ViInt32 arraySize, ViInt32 *reliability,
              ViInt32 results[], ViInt32 *returnedValues)
      

{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, codeDomain, 0, 1), 3, "Code Domain");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 1), 4, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Results")
;   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:TRAC:%s:%s:PIL:STAT?", instance, 
        resultCodeDomainArr[codeDomain], resultCodeDomainMonitorTypeArr[resultType]);
    
    checkErr (rscmwevm_GetMeasTraceStates (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
                          
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}   

    
/*****************************************************************************
 * Function: Query Multi Eval ACP Out of Tolerance Limits Check Result
 * Purpose:  Return limit check of the "Out of Tolerance" error codes.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_QueryMEvalACPOutofToleranceLimitsCheckResult (ViSession instrSession,
                                                            ViInt32 instance,
                                                            ViInt32 resultType,
                                                            ViInt32 arraySize,
                                                            ViInt32 *reliability,
                                                            ViInt32 results[],
                                                            ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:ACP:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Read Multi Eval Channel Power Trace
 * Purpose:  Returns the channel power of the reverse link physical channels
 *           of both the I and Q signal.
 *           The slots for the Pilot and the RRI channel are evaluated within
 *           the same measurement slot.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ReadMEvalMeasChannelPowerTrace (ViSession instrSession,
                                                 ViInt32 instance,
                                                  ViInt32 resultType,
                                                  ViInt32 arraySize,
                                                  ViInt32 *reliability,
                                                  ViReal64 results[],
                                                  ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 3), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    sprintf (command, "READ:EVDO:MEAS%ld:MEV:TRAC:CP:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}   

/*****************************************************************************
 * Function: Fetch Multi Eval Channel Power Trace
 * Purpose:  Returns the channel power of the reverse link physical channels
 *           of both the I and Q signal.
 *           The slots for the Pilot and the RRI channel are evaluated within
 *           the same measurement slot.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasChannelPowerTrace (ViSession instrSession,
                                                 ViInt32 instance,
                                                  ViInt32 resultType,
                                                  ViInt32 arraySize,
                                                  ViInt32 *reliability,
                                                  ViReal64 results[],
                                                  ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 3), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:TRAC:CP:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Query Multi Eval Channel Power Trace Limits Check Result
 * Purpose:  Return limit check of the "Out of Tolerance" error codes.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_QueryMEvalChannelPowerTraceLimitsCheckResult (ViSession instrSession,
                                                            ViInt32 instance,
                                                            ViInt32 resultType,
                                                            ViInt32 arraySize,
                                                            ViInt32 *reliability,
                                                            ViInt32 results[],
                                                            ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 3), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:TRAC:CP:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/*****************************************************************************
 * Function: Fetch Multi Eval Channel Power Trace State
 * Purpose:  Return the main measurement state for the channel power (CP)
 *           trace results of the multi evaluation measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_FetchMEvalChannelPowerTraceState
             (ViSession instrSession, ViInt32 instance, ViInt32 arraySize,
                ViInt32 *reliability, ViInt32 results[], ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Results")
;   
    };
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:TRAC:CP:STAT?", instance);
    
    checkErr (rscmwevm_GetMeasTraceStates (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
                          
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}   

/*****************************************************************************
 * Function: Read Multi Eval Channel Power
 * Purpose: Returns the scalar channel power for the Data channel.
 *          The result is extended to the W2^4 and W1^2 I/Q values of the
 *          Data channel. Only available if subtype 2 is selected.
 *          Otherwise NAV is returned.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ReadMEvalMeasChannelPower (ViSession instrSession,
                                                 ViInt32 instance,
                                                  ViInt32 resultType,
                                                  ViInt32 arraySize,
                                                  ViInt32 *reliability,
                                                  ViReal64 results[],
                                                  ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 3), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    sprintf (command, "READ:EVDO:MEAS%ld:MEV:CP:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}   

/*****************************************************************************
 * Function: Fetch Multi Eval Channel Power
 * Purpose:  Returns the scalar channel power for the Data channel.
 *           The result is extended to the W2^4 and W1^2 I/Q values of the
 *           Data channel. Only available if subtype 2 is selected.
 *           Otherwise NAV is returned.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasChannelPower (ViSession instrSession,
                                                 ViInt32 instance,
                                                  ViInt32 resultType,
                                                  ViInt32 arraySize,
                                                  ViInt32 *reliability,
                                                  ViReal64 results[],
                                                  ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 3), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:CP:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Query Multi Eval Channel Power Limits Check Result
 * Purpose:  Return limit check of the "Out of Tolerance" error codes.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_QueryMEvalChannelPowerLimitsCheckResult (ViSession instrSession,
                                                            ViInt32 instance,
                                                            ViInt32 resultType,
                                                            ViInt32 arraySize,
                                                            ViInt32 *reliability,
                                                            ViInt32 results[],
                                                            ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 3), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:CP:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}   


/*****************************************************************************
 * Function: Read Multi Eval Occupied Bandwidth Trace
 * Purpose:  For the carrier or carrier group returns the lower and upper
 *           edge of the Occupied Bandwidth (OBW).
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ReadMEvalMeasOccupiedBandwidthTrace (ViSession instrSession,
                                                 ViInt32 instance,
												  ViInt32 carrierNumber,
                                                  ViInt32 resultType,
                                                  ViInt32 arraySize,
                                                  ViInt32 *reliability,
                                                  ViReal64 results[],
                                                  ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, carrierNumber, 1, 4), 3, "Carrier Number");
        viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 0), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, carrierNumber, 1, 4), 3, "Carrier Number");
	
    sprintf (command, "READ:EVDO:MEAS%ld:MEV:TRAC:OBW%ld:%s?", instance, carrierNumber, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}   

/*****************************************************************************
 * Function: Fetch Multi Eval Occupied Bandwidth Trace
 * Purpose:  Returns the traces of the current occupied bandwidth (OBW).
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasOccupiedBandwidthTrace (ViSession instrSession,
                                                 ViInt32 instance,
												 ViInt32 carrierNumber,
                                                  ViInt32 resultType,
                                                  ViInt32 arraySize,
                                                  ViInt32 *reliability,
                                                  ViReal64 results[],
                                                  ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, carrierNumber, 1, 4), 3, "Carrier Number");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 0), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
	sprintf (command, "FETC:EVDO:MEAS%ld:MEV:TRAC:OBW%ld:%s?", instance, carrierNumber, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns the limit check of the Occupied Bandwidth (OBW).
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     
/// HIFN  CALCulate:EVDO:MEASurement<instance>:MEValuation:TRACe:OBW<number>:CURRent?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR carrierNumber/Selects the isolated carrier or group carrier to be analyzed.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns limits check results values of the EVM trace results of the 
/// HIPAR results/multi evaluation measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasOccupiedBandwidthTraceLimitsCheckResult(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32	carrierNumber,
	ViInt32	resultType,
	ViInt32	arraySize,
	ViInt32*	reliability,
	ViInt32	results[],
	ViInt32*	returnedValues
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, carrierNumber, 1, 4), 3, "Carrier Number");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 0), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
   	
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:TRAC:OBW%ld:%s?", instance, carrierNumber, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Read Multi Eval Occupied Bandwidth
 * Purpose:  Return the current, average and maximum occupied bandwidth value
 *           result and the "Out of Tolerance" result, the percentage of
 *           measurement intervals of the statistic counts exceeding the
 *           specified limits.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ReadMEvalMeasOccupiedBandwidth (ViSession instrSession,
                                                 ViInt32 instance,
												 ViInt32 carrierNumber,
                                                  ViInt32 resultType,
                                                  ViInt32 arraySize,
                                                  ViInt32 *reliability,
                                                  ViReal64 results[],
                                                  ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, carrierNumber, 1, 4), 3, "Carrier Number");
	
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    }
	
    sprintf (command, "READ:EVDO:MEAS%ld:MEV:OBW%ld:%s?", instance, carrierNumber, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}   

/*****************************************************************************
 * Function: Fetch Multi Eval Occupied Bandwidth
 * Purpose:  Return the current, average and maximum occupied bandwidth value
 *           result and the "Out of Tolerance" result, the percentage of
 *           measurement intervals of the statistic counts exceeding the
 *           specified limits.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasOccupiedBandwidth (ViSession instrSession,
                                                 ViInt32 instance,
												 ViInt32 carrierNumber,
                                                  ViInt32 resultType,
                                                  ViInt32 arraySize,
                                                  ViInt32 *reliability,
                                                  ViReal64 results[],
                                                  ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, carrierNumber, 1, 4), 3, "Carrier Number");
	
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
	
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:OBW%ld:%s?", instance, carrierNumber, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));

Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Query Multi Eval Occupied Bandwidth Limits Check Result
 * Purpose:  Returns the limit check of the "Out of Tolerance" error codes
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_QueryMEvalOccupiedBandwidthLimitsCheckResult (ViSession instrSession,
                                                            ViInt32 instance,
															ViInt32 carrierNumber,
                                                            ViInt32 resultType,
                                                            ViInt32 arraySize,
                                                            ViInt32 *reliability,
                                                            ViInt32 results[],
                                                            ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, carrierNumber, 1, 4), 3, "Carrier Number");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 2), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
   	
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:OBW%ld:%s?", instance, carrierNumber, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}   


/*****************************************************************************
 * Function: Read Multi Eval Modulation Results
 * Purpose:  Starts a new single shot measurement and returns the results of
 *           the multi evaluation measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ReadMEvalMeasModulationResults (ViSession instrSession,
                                                          ViInt32 instance,
                                                          ViInt32 resultType,
                                                          ViInt32 arraySize,
                                                          ViInt32 *reliability,
                                                          ViReal64 results[],
                                                          ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 3), 3, "Result Type");
    
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "READ:EVDO:MEAS%ld:MEV:MOD:%s?", 
        instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Fetch Multi Eval Modulation Results
 * Purpose:  Returns the results of the multi evaluation measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasModulationResults (ViSession instrSession,
                                                           ViInt32 instance,
                                                           ViInt32 resultType,
                                                           ViInt32 arraySize,
                                                           ViInt32 *reliability,
                                                           ViReal64 results[],
                                                           ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 3), 3, "Result Type");
    
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:MOD:%s?", 
        instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Read Multi Eval Modulation Standard Deviation
 * Purpose:  Starts a new single shot measurement and returns the standard 
 *           deviation single value results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ReadMEvalMeasModulationStandardDeviation (ViSession instrSession,
                                                                    ViInt32 instance,
                                                                    ViInt32 arraySize,
                                                                    ViInt32 *reliability,
                                                                    ViReal64 results[],
                                                                    ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Results")
;   
    };
    
    sprintf (command, "READ:EVDO:MEAS%ld:MEV:MOD:SDEV?", 
        instance);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Fetch Multi Eval Modulation Standard Deviation
 * Purpose:  Returns the standard deviation single value results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasModulationStandardDeviation (ViSession instrSession,
                                                                     ViInt32 instance,
                                                                     ViInt32 arraySize,
                                                                     ViInt32 *reliability,
                                                                     ViReal64 results[],
                                                                     ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Results")
;   
    };
    
    sprintf (command, "FETCH:EVDO:MEAS%ld:MEV:MOD:SDEV?", 
        instance);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}
/*****************************************************************************
 * Function: Query Multi Eval Modulation Results Limits Check Result
 * Purpose:  Returns limit check of error codes of single value results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasModulationResultsLimitsCheckResult (ViSession instrSession,
                                                                ViInt32 instance,
                                                                ViInt32 resultType,
                                                                ViInt32 arraySize,
                                                                ViInt32 *reliability,
                                                                ViInt32 results[],
                                                                ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Results")
;   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:MOD:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}
/*****************************************************************************
 * Function: Query Multi Eval Modulation Results Standard Deviation Limits Check Result
 * Purpose:  Returns limit check of single value standart deviation results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasModulationResultsStandardDeviationLimitsCheckResult (ViSession instrSession,
                                                                          ViInt32 instance,
                                                                          ViInt32 arraySize,
                                                                          ViInt32 *reliability,
                                                                          ViInt32 results[],
                                                                          ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Results")
;   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:MOD:SDEV?", instance);
    checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Read Multi Eval Spectrum Trace
 * Purpose:  Returns the power specturm traces across the frequency span (8 MHz).
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ReadMEvalMeasSpectrumTrace (ViSession instrSession,
                                                 ViInt32 instance,
                                                  ViInt32 resultType,
                                                  ViInt32 arraySize,
                                                  ViInt32 *reliability,
                                                  ViReal64 results[],
                                                  ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 0), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results"); 
    }
    sprintf (command, "READ:EVDO:MEAS%ld:MEV:TRAC:SPEC:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}   

/*****************************************************************************
 * Function: Fetch Multi Eval Spectrum Trace
 * Purpose:  Returns the power specturm traces across the frequency span (8 MHz).
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasSpectrumTrace (ViSession instrSession,
                                                 ViInt32 instance,
                                                  ViInt32 resultType,
                                                  ViInt32 arraySize,
                                                  ViInt32 *reliability,
                                                  ViReal64 results[],
                                                  ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, 0, 0), 3, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:TRAC:SPEC:%s?", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));

Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
} 


/// HIFN  Returns all ACP value results for the selected segment in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR ACPResults/Returns the adjacent channel power measured at a series of 
/// HIPAR ACPResults/frequencies. The power offsets <Power Offset +n> (<Power Offset -n>) 
/// HIPAR ACPResults/refer to the average power at the carrier frequency plus (minus) the 
/// HIPAR ACPResults/frequency offset value number n.
/// HIPAR ATPWide/Returns the access terminal power, measured with the wideband filter.
/// HIPAR ATPNarrow/Returns the access terminal power, measured with a filter bandwidth of 
/// HIPAR ATPNarrow/1.23 MHz
/// HIPAR outOfTolerance/Returns the out of tolerance result, i.e. percentage of measurement 
/// HIPAR outOfTolerance/intervals of the statistic count exceeding the specified.
/// HIPAR validSlotsCount/Returns the number of evaluated valid slots in this segment.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeACPResults (ViSession instrSession,
                                                             ViInt32 instance,
                                                             ViInt32 segment,
                                                             ViInt32 resultType,
                                                             ViInt32 arraySize,
                                                             ViInt32 *reliability,
                                                             ViInt32 *segmentReliability,
                                                             ViReal64 ACPResults[],
                                                             ViReal64 *ATPWide,
                                                             ViReal64 *ATPNarrow,
                                                             ViReal64 *outOfTolerance,
                                                             ViInt32 *validSlotsCount,
                                                             ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar   buffer [150] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    ViInt32  j = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 4, "Result Type");
    if (ACPResults == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
    checkErr (Rs_ClearBeforeRead(instrSession));   
	
	sprintf (buffer, "FETC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:ACP:%s;*OPC\n", instance, segment, resultTypeArr[resultType]);
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = rscmwevm_atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    *segmentReliability = rscmwevm_atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    for (j=0; j<21; j++)
    {
        if (j<arraySize)
        {
            ACPResults[j] = rscmwevm_atof (p2buf);
            p2buf = strtok (NULL, ",\n");
            if (p2buf == NULL) break;
        }
        else 
        {
            p2buf = strtok (NULL, ",\n");
            break;
        }
    }
    if (returnedValues) *returnedValues=j;
 
    *ATPWide = rscmwevm_atof (p2buf);
    
    p2buf = strtok (NULL, ",\n");
    *ATPNarrow = rscmwevm_atof (p2buf);
    
    p2buf = strtok (NULL, ",\n");
    *outOfTolerance = rscmwevm_atof (p2buf);
    
    p2buf = strtok (NULL, ",\n");
    *validSlotsCount = rscmwevm_atol (p2buf);
    
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns all ACP limits check results for the selected segment in list 
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR ACPResults/Returns the adjacent channel power limit state measured at a series of 
/// HIPAR ACPResults/frequencies. The power offsets <Power Offset +n> (<Power Offset -n>) 
/// HIPAR ACPResults/refer to the average power at the carrier frequency plus (minus) the 
/// HIPAR ACPResults/frequency offset value number n.
/// HIPAR ATPWide/Returns the access terminal power limit state, measured with the 
/// HIPAR ATPWide/wideband filter.
/// HIPAR ATPNarrow/Returns the access terminal power limit state, measured with a filter 
/// HIPAR ATPNarrow/bandwidth of 1.23 MHz
/// HIPAR outOfTolerance/Returns the out of tolerance result limit state, i.e. percentage of 
/// HIPAR outOfTolerance/measurement intervals of the statistic count exceeding the specified.
/// HIPAR validSlotsCount/Returns the limit state for number of evaluated valid slots in this 
/// HIPAR validSlotsCount/segment.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeACPLimitsCheckResults (ViSession instrSession,
                                                                        ViInt32 instance,
                                                                        ViInt32 segment,
                                                                        ViInt32 resultType,
                                                                        ViInt32 arraySize,
                                                                        ViInt32 *reliability,
                                                                        ViInt32 *segmentReliability,
                                                                        ViInt32 ACPResults[],
                                                                        ViInt32 *ATPWide,
                                                                        ViInt32 *ATPNarrow,
                                                                        ViInt32 *outOfTolerance,
                                                                        ViInt32 *validSlotsCount,
                                                                        ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer [150] = "";
    ViChar*     ext_buf=NULL;
    ViChar*     p2buf;
    ViUInt32    ret_cnt = 0;
    ViInt32     j = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 4, "Result Type");
    if (ACPResults == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
    checkErr (Rs_ClearBeforeRead(instrSession));   

    sprintf (buffer, "CALC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:ACP:%s?;*OPC\n", instance, segment, resultTypeArr[resultType]);
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    *segmentReliability = rscmwevm_atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    for (j=0; j<21; j++)
    {
        if (j<arraySize)
        {   
            ACPResults[j] = rscmwevm_strCode(p2buf, arrayCodes);
            p2buf = strtok (NULL, ",\n");
            if (p2buf == NULL) break;
        }
        else 
        {
            p2buf = strtok (NULL, ",\n");
            break;
        }
    }
    if (returnedValues) *returnedValues=j;
 
    *ATPWide = rscmwevm_strCode(p2buf, arrayCodes);
    
    p2buf = strtok (NULL, ",\n");
    *ATPNarrow = rscmwevm_strCode(p2buf, arrayCodes);
    
    p2buf = strtok (NULL, ",\n");
    *outOfTolerance = rscmwevm_strCode(p2buf, arrayCodes);
    
    p2buf = strtok (NULL, ",\n");
    *validSlotsCount = rscmwevm_strCode(p2buf, arrayCodes);
    
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}



/// HIFN  Returns all ACP standard deviation results for the selected segment in 
/// HIFN  list mode. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR ACPResults/Returns the adjacent channel power measured at a series of 
/// HIPAR ACPResults/frequencies. The power offsets <Power Offset +n> (<Power Offset -n>) 
/// HIPAR ACPResults/refer to the average power at the carrier frequency plus (minus) the 
/// HIPAR ACPResults/frequency offset value number n.
/// HIPAR ATPWide/Returns the access terminal power, measured with the wideband filter.
/// HIPAR ATPNarrow/Returns the access terminal power, measured with a filter bandwidth of 
/// HIPAR ATPNarrow/1.23 MHz
/// HIPAR outOfTolerance/Returns the out of tolerance result, i.e. percentage of measurement 
/// HIPAR outOfTolerance/intervals of the statistic count exceeding the specified.
/// HIPAR validSlotsCount/Returns the number of evaluated valid slots in this segment.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeACPStandardDeviation (ViSession instrSession,
                                                                       ViInt32 instance,
                                                                       ViInt32 segment,
                                                                       ViInt32 arraySize,
                                                                       ViInt32 *reliability,
                                                                       ViInt32 *segmentReliability,
                                                                       ViReal64 ACPResults[],
                                                                       ViReal64 *ATPWide,
                                                                       ViReal64 *ATPNarrow,
                                                                       ViReal64 *outOfTolerance,
                                                                       ViInt32 *validSlotsCount,
                                                                       ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer [150] = "";
    ViChar*     ext_buf=NULL;
    ViChar*     p2buf;
    ViUInt32    ret_cnt = 0;
    ViInt32     j = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
    if (ACPResults == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
   	checkErr (Rs_ClearBeforeRead(instrSession));   
	sprintf (buffer, "FETC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:ACP:SDEV?;*OPC\n", instance, segment);
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    *segmentReliability = rscmwevm_atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    for (j=0; j<21; j++)
    {
        if (j<arraySize)
        {
            ACPResults[j] = rscmwevm_atof (p2buf);
            p2buf = strtok (NULL, ",\n");
            if (p2buf == NULL) break;
        }
        else 
        {
            p2buf = strtok (NULL, ",\n");
            break;
        }
    }
    if (returnedValues) *returnedValues=j;
 
    *ATPWide = rscmwevm_atof (p2buf);
    
    p2buf = strtok (NULL, ",\n");
    *ATPNarrow = rscmwevm_atof (p2buf);
    
    p2buf = strtok (NULL, ",\n");
    *outOfTolerance = rscmwevm_atof (p2buf);
    
    p2buf = strtok (NULL, ",\n");
    *validSlotsCount = rscmwevm_atol (p2buf);
    
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}



/// HIFN  Returns all ACP standard deviation limits check results for the 
/// HIFN  selected segment in list mode.   
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR ACPResults/Returns the limit check for adjacent channel power measured at a 
/// HIPAR ACPResults/series of frequencies. The power offsets <Power Offset +n> (<Power 
/// HIPAR ACPResults/Offset -n>) refer to the average power at the carrier frequency plus 
/// HIPAR ACPResults/(minus) the frequency offset value number n.
/// HIPAR ATPWide/Returns the access terminal power, measured with the wideband filter.
/// HIPAR ATPNarrow/Returns the access terminal power, measured with a filter bandwidth of 
/// HIPAR ATPNarrow/1.23 MHz
/// HIPAR outOfTolerance/Returns the out of tolerance result, i.e. percentage of measurement 
/// HIPAR outOfTolerance/intervals of the statistic count exceeding the specified.
/// HIPAR validSlotsCount/Returns the number of evaluated valid slots in this segment.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeACPStandardDeviationLimitsCheck (ViSession instrSession,
                                                                                  ViInt32 instance,
                                                                                  ViInt32 segment,
                                                                                  ViInt32 arraySize,
                                                                                  ViInt32 *reliability,
                                                                                  ViInt32 *segmentReliability,
                                                                                  ViInt32 ACPResults[],
                                                                                  ViInt32 *ATPWide,
                                                                                  ViInt32 *ATPNarrow,
                                                                                  ViInt32 *outOfTolerance,
                                                                                  ViInt32 *validSlotsCount,
                                                                                  ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer [150] = "";
    ViChar*     ext_buf=NULL;
    ViChar*     p2buf;
    ViUInt32    ret_cnt = 0;
    ViInt32     j = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
    if (ACPResults == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
	checkErr (Rs_ClearBeforeRead(instrSession));  
    sprintf (buffer, "CALC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:ACP:SDEV?;*OPC\n", instance, segment);
    checkErr (viWrite (instrSession, (ViBuf) buffer, (ViUInt32) strlen (buffer), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    *segmentReliability = rscmwevm_atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    for (j=0; j<21; j++)
    {
        if (j<arraySize)
        {
            ACPResults[j] = rscmwevm_strCode(p2buf, arrayCodes);
            p2buf = strtok (NULL, ",\n");
            if (p2buf == NULL) break;
        }
        else
        {
            p2buf = strtok (NULL, ",\n");
            break;
        }
    }
    if (returnedValues) *returnedValues=j;
 
    *ATPWide = rscmwevm_strCode(p2buf, arrayCodes);
    
    p2buf = strtok (NULL, ",\n");
    *ATPNarrow = rscmwevm_strCode(p2buf, arrayCodes);
    
    p2buf = strtok (NULL, ",\n");
    *outOfTolerance = rscmwevm_strCode(p2buf, arrayCodes);
    
    p2buf = strtok (NULL, ",\n");
    *validSlotsCount = rscmwevm_strCode(p2buf, arrayCodes);
    
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns all ACP value results for the selected segment in list mode.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     
/// HIFN  FETCh:EVDO:MEAS<instance>:MEValuation:LIST:SEGMent<no>:ACP:EXTended:CURRent?
/// HIFN     
/// HIFN  FETCh:EVDO:MEAS<instance>:MEValuation:LIST:SEGMent<no>:ACP:EXTended:AVERage?
/// HIFN     
/// HIFN  FETCh:EVDO:MEAS<instance>:MEValuation:LIST:SEGMent<no>:ACP:EXTended:MINimum?
/// HIFN     
/// HIFN  FETCh:EVDO:MEAS<instance>:MEValuation:LIST:SEGMent<no>:ACP:EXTended:MAXimum?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR ACPResults/Returns the adjacent channel power measured at a series of 
/// HIPAR ACPResults/frequencies. The power offsets <Power Offset +n> (<Power Offset -n>) 
/// HIPAR ACPResults/refer to the average power at the carrier frequency plus (minus) the 
/// HIPAR ACPResults/frequency offset value number n.
/// HIPAR ATPWide/Returns the access terminal power, measured with the wideband filter.
/// HIPAR ATPNarrow/Returns the access terminal power, measured with a filter bandwidth of 
/// HIPAR ATPNarrow/1.23 MHz
/// HIPAR outOfTolerance/Returns the out of tolerance result, i.e. percentage of measurement 
/// HIPAR outOfTolerance/intervals of the statistic count exceeding the specified.
/// HIPAR validSlotsCount/Returns the number of evaluated valid slots in this segment.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeACPExtendedResults (ViSession instrSession,
                                                                     ViInt32 instance,
                                                                     ViInt32 segment,
                                                                     ViInt32 resultType,
                                                                     ViInt32 arraySize,
                                                                     ViInt32 *reliability,
                                                                     ViInt32 *segmentReliability,
                                                                     ViReal64 ACPResults[],
                                                                     ViReal64 *ATPWide,
                                                                     ViReal64 *ATPNarrow,
                                                                     ViReal64 *outOfTolerance,
                                                                     ViInt32 *validSlotsCount,
                                                                     ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    ViInt32  j = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 4, "Result Type");
    if (ACPResults == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
	checkErr (Rs_ClearBeforeRead(instrSession)); 
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:ACP:EXT:%s?;*OPC\n", instance, segment, resultTypeArr[resultType]);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = rscmwevm_atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    *segmentReliability = rscmwevm_atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    for (j=0; j<41; j++)
    {
        if (j<arraySize)
        {
            ACPResults[j] = rscmwevm_atof (p2buf);
            p2buf = strtok (NULL, ",\n");
            if (p2buf == NULL) break;
        }
        else 
        {
            p2buf = strtok (NULL, ",\n");
            break;
        }
    }
    if (returnedValues) *returnedValues=j;
 
    *ATPWide = rscmwevm_atof (p2buf);
    
    p2buf = strtok (NULL, ",\n");
    *ATPNarrow = rscmwevm_atof (p2buf);
    
    p2buf = strtok (NULL, ",\n");
    *outOfTolerance = rscmwevm_atof (p2buf);
    
    p2buf = strtok (NULL, ",\n");
    *validSlotsCount = rscmwevm_atol (p2buf);
    
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns all ACP limits check results for the selected segment in list 
/// HIFN  mode.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     
/// HIFN  CALCulate:EVDO:MEAS<instance>:MEValuation:LIST:SEGMent<no>:ACP:EXTended:CURRent?
/// HIFN     
/// HIFN  CALCulate:EVDO:MEAS<instance>:MEValuation:LIST:SEGMent<no>:ACP:EXTended:AVERage?
/// HIFN     
/// HIFN  CALCulate:EVDO:MEAS<instance>:MEValuation:LIST:SEGMent<no>:ACP:EXTended:MINimum?
/// HIFN     
/// HIFN  CALCulate:EVDO:MEAS<instance>:MEValuation:LIST:SEGMent<no>:ACP:EXTended:MAXimum?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR ACPResults/Returns the adjacent channel power limit state measured at a series of 
/// HIPAR ACPResults/frequencies. The power offsets <Power Offset +n> (<Power Offset -n>) 
/// HIPAR ACPResults/refer to the average power at the carrier frequency plus (minus) the 
/// HIPAR ACPResults/frequency offset value number n.
/// HIPAR ATPWide/Returns the access terminal power limit state, measured with the 
/// HIPAR ATPWide/wideband filter.
/// HIPAR ATPNarrow/Returns the access terminal power limit state, measured with a filter 
/// HIPAR ATPNarrow/bandwidth of 1.23 MHz
/// HIPAR outOfTolerance/Returns the out of tolerance result limit state, i.e. percentage of 
/// HIPAR outOfTolerance/measurement intervals of the statistic count exceeding the specified.
/// HIPAR validSlotsCount/Returns the limit state for number of evaluated valid slots in this 
/// HIPAR validSlotsCount/segment.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeACPLimitsCheckExtendedResults (ViSession instrSession,
                                                                                ViInt32 instance,
                                                                                ViInt32 segment,
                                                                                ViInt32 resultType,
                                                                                ViInt32 arraySize,
                                                                                ViInt32 *reliability,
                                                                                ViInt32 *segmentReliability,
                                                                                ViInt32 ACPResults[],
                                                                                ViInt32 *ATPWide,
                                                                                ViInt32 *ATPNarrow,
                                                                                ViInt32 *outOfTolerance,
                                                                                ViInt32 *validSlotsCount,
                                                                                ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
    ViChar*     ext_buf=NULL;
    ViChar*     p2buf;
    ViUInt32    ret_cnt = 0;
    ViInt32     j = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 4, "Result Type");
    if (ACPResults == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
    checkErr (Rs_ClearBeforeRead(instrSession));     
	sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:ACP:EXT:%s?;*OPC\n", instance, segment, resultTypeArr[resultType]);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    *segmentReliability = rscmwevm_atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    for (j=0; j<41; j++)
    {
        if (j<arraySize)
        {   
            ACPResults[j] = rscmwevm_strCode(p2buf, arrayCodes);
            p2buf = strtok (NULL, ",\n");
            if (p2buf == NULL) break;
        }
        else 
        {
            p2buf = strtok (NULL, ",\n");
            break;
        }
    }
    if (returnedValues) *returnedValues=j;
 
    *ATPWide = rscmwevm_strCode(p2buf, arrayCodes);
    
    p2buf = strtok (NULL, ",\n");
    *ATPNarrow = rscmwevm_strCode(p2buf, arrayCodes);
    
    p2buf = strtok (NULL, ",\n");
    *outOfTolerance = rscmwevm_strCode(p2buf, arrayCodes);
    
    p2buf = strtok (NULL, ",\n");
    *validSlotsCount = rscmwevm_strCode(p2buf, arrayCodes);
    
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns all ACP standard deviation results for the selected segment in 
/// HIFN  list mode.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     
/// HIFN  FETCh:EVDO:MEAS<instance>:MEValuation:LIST:SEGMent<no>:ACP:EXTended:SDEViation?
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR ACPResults/Returns the adjacent channel power measured at a series of 
/// HIPAR ACPResults/frequencies. The power offsets <Power Offset +n> (<Power Offset -n>) 
/// HIPAR ACPResults/refer to the average power at the carrier frequency plus (minus) the 
/// HIPAR ACPResults/frequency offset value number n.
/// HIPAR ATPWide/Returns the access terminal power, measured with the wideband filter.
/// HIPAR ATPNarrow/Returns the access terminal power, measured with a filter bandwidth of 
/// HIPAR ATPNarrow/1.23 MHz
/// HIPAR outOfTolerance/Returns the out of tolerance result, i.e. percentage of measurement 
/// HIPAR outOfTolerance/intervals of the statistic count exceeding the specified.
/// HIPAR validSlotsCount/Returns the number of evaluated valid slots in this segment.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeACPStandardDeviationExtended (ViSession instrSession,
                                                                               ViInt32 instance,
                                                                               ViInt32 segment,
                                                                               ViInt32 arraySize,
                                                                               ViInt32 *reliability,
                                                                               ViInt32 *segmentReliability,
                                                                               ViReal64 ACPResults[],
                                                                               ViReal64 *ATPWide,
                                                                               ViReal64 *ATPNarrow,
                                                                               ViReal64 *outOfTolerance,
                                                                               ViInt32 *validSlotsCount,
                                                                               ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
    ViChar*     ext_buf=NULL;
    ViChar*     p2buf;
    ViUInt32    ret_cnt = 0;
    ViInt32     j = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
    if (ACPResults == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
	checkErr (Rs_ClearBeforeRead(instrSession));     
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:ACP:EXT:SDEV?;*OPC\n", instance, segment);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    *segmentReliability = rscmwevm_atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    for (j=0; j<41; j++)
    {
        if (j<arraySize)
        {
            ACPResults[j] = rscmwevm_atof (p2buf);
            p2buf = strtok (NULL, ",\n");
            if (p2buf == NULL) break;
        }
        else 
        {
            p2buf = strtok (NULL, ",\n");
            break;
        }
    }
    if (returnedValues) *returnedValues=j;
 
    *ATPWide = rscmwevm_atof (p2buf);
    
    p2buf = strtok (NULL, ",\n");
    *ATPNarrow = rscmwevm_atof (p2buf);
    
    p2buf = strtok (NULL, ",\n");
    *outOfTolerance = rscmwevm_atof (p2buf);
    
    p2buf = strtok (NULL, ",\n");
    *validSlotsCount = rscmwevm_atol (p2buf);
    
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns all ACP standard deviation limits check results for the 
/// HIFN  selected segment in list mode.  
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR ACPResults/Returns the limit check for adjacent channel power measured at a 
/// HIPAR ACPResults/series of frequencies. The power offsets <Power Offset +n> (<Power 
/// HIPAR ACPResults/Offset -n>) refer to the average power at the carrier frequency plus 
/// HIPAR ACPResults/(minus) the frequency offset value number n.
/// HIPAR ATPWide/Returns the access terminal power, measured with the wideband filter.
/// HIPAR ATPNarrow/Returns the access terminal power, measured with a filter bandwidth of 
/// HIPAR ATPNarrow/1.23 MHz
/// HIPAR outOfTolerance/Returns the out of tolerance result, i.e. percentage of measurement 
/// HIPAR outOfTolerance/intervals of the statistic count exceeding the specified.
/// HIPAR validSlotsCount/Returns the number of evaluated valid slots in this segment.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeACPStandardDeviationLimitsCheckExtended (ViSession instrSession,
                                                                                          ViInt32 instance,
                                                                                          ViInt32 segment,
                                                                                          ViInt32 arraySize,
                                                                                          ViInt32 *reliability,
                                                                                          ViInt32 *segmentReliability,
                                                                                          ViInt32 ACPResults[],
                                                                                          ViInt32 *ATPWide,
                                                                                          ViInt32 *ATPNarrow,
                                                                                          ViInt32 *outOfTolerance,
                                                                                          ViInt32 *validSlotsCount,
                                                                                          ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
    ViChar*     ext_buf=NULL;
    ViChar*     p2buf;
    ViUInt32    ret_cnt = 0;
    ViInt32     j = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
    if (ACPResults == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
	checkErr (Rs_ClearBeforeRead(instrSession));     
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:ACP:EXT:SDEV?;*OPC\n", instance, segment);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    *segmentReliability = rscmwevm_atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    for (j=0; j<41; j++)
    {
        if (j<arraySize)
        {
            ACPResults[j] = rscmwevm_strCode(p2buf, arrayCodes);
            p2buf = strtok (NULL, ",\n");
            if (p2buf == NULL) break;
        }
        else
        {
            p2buf = strtok (NULL, ",\n");
            break;
        }
    }
    if (returnedValues) *returnedValues=j;
 
    *ATPWide = rscmwevm_strCode(p2buf, arrayCodes);
    
    p2buf = strtok (NULL, ",\n");
    *ATPNarrow = rscmwevm_strCode(p2buf, arrayCodes);
    
    p2buf = strtok (NULL, ",\n");
    *outOfTolerance = rscmwevm_strCode(p2buf, arrayCodes);
    
    p2buf = strtok (NULL, ",\n");
    *validSlotsCount = rscmwevm_strCode(p2buf, arrayCodes);
    
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns channel power value results for the selected segment in list 
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR results/Returns RMS channel power values for the physical channels. Depending 
/// HIPAR results/on the subtype 0/1 or 2 some channels exist or not. If not available, 
/// HIPAR results/no power measurement is possible therefore NAV is returned.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeChannelPowerResults (ViSession instrSession,
                                                                      ViInt32 instance,
                                                                      ViInt32 segment,
                                                                      ViInt32 resultType,
                                                                      ViInt32 arraySize,
                                                                      ViInt32 *reliability,
                                                                      ViInt32 *segmentReliability,
                                                                      ViReal64 results[],
                                                                      ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:CP:%s?\n", instance, segment, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasList (instrSession, command, 15000, arraySize, reliability, segmentReliability, results, returnedValues));

    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns channel power value limits check results for the selected 
/// HIFN  segment in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR results/Returns the limits state of RMS channel power for the physical 
/// HIPAR results/channels. Depending on the subtype 0/1 or 2 some channels exist or not. 
/// HIPAR results/If not available, no power measurement is possible therefore NAV is 
/// HIPAR results/returned.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeChannelPowerLimitsCheckResults (ViSession instrSession,
                                                                                 ViInt32 instance,
                                                                                 ViInt32 segment,
                                                                                 ViInt32 resultType,
                                                                                 ViInt32 arraySize,
                                                                                 ViInt32 *reliability,
                                                                                 ViInt32 *segmentReliability,
                                                                                 ViInt32 results[],
                                                                                 ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:CP:%s?\n", instance, segment, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTraceLimitsAll (instrSession, command, 15000, arraySize, reliability, segmentReliability, results, returnedValues));
Error:
    return error;
}



/// HIFN  Return the states of the channels for power measurement for the 
/// HIFN  selected segment in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR results/Return the states of the channels for power measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeChannelPowerResultsState (ViSession instrSession,
                                                                           ViInt32 instance,
                                                                           ViInt32 segment,
                                                                           ViInt32 resultType,
                                                                           ViInt32 arraySize,
                                                                           ViInt32 *reliability,
                                                                           ViInt32 *segmentReliability,
                                                                           ViInt32 results[],
                                                                           ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS; 
    ViChar      command[50] = "";   
	resultType;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:CP:STAT?\n", instance, segment);
    checkErr (rscmwevm_GetMeasListState (instrSession, command, 15000, arraySize, reliability, segmentReliability, results, returnedValues));

Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/// HIFN  In list mode, returns the Occupied Bandwidth (OBW) result for
/// HIFN  selected segment and the carrier or carrier group addressed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR occupiedBandwidth/Returns the occupied bandwidth value.
/// HIPAR lowerFrequency/Returns lower edge of the selected carrier's or carrier group's occupied bandwidth.
/// HIPAR upperFrequency/Returns the upper edge of the selected carrier's or carrier group's occupied bandwidth.
/// HIPAR carrierNumber/Selects the isolated carrier or group carrier to be analyzed.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeOBWResults (ViSession instrSession,
                                                             ViInt32 instance,
                                                             ViInt32 segment,
															 ViInt32 carrierNumber,
                                                             ViInt32 resultType,
                                                             ViInt32 *reliability,
                                                             ViInt32 *segmentReliability,
                                                             ViReal64 *occupiedBandwidth,
                                                             ViReal64 *lowerFrequency,
                                                             ViReal64 *upperFrequency)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, carrierNumber, 1, 4), 4, "Carrier Number");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MAX), 5, "Result Type");
    if (occupiedBandwidth == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    if (lowerFrequency == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 9, "Lower Frequency");   
    };
    if (upperFrequency == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 10, "Upper Frequency");   
    };

    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:OBW%ld:%s?\n", instance, segment, carrierNumber, resultTypeArr[resultType]);
    
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);
    
    p2buf = strtok (NULL, ",\n");  
    *segmentReliability = rscmwevm_atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    *occupiedBandwidth = rscmwevm_atof (p2buf);
    
    if (resultType == RSCMWEVM_VAL_RESULT_CURRENT)
    {
        p2buf = strtok (NULL, ",\n");
        *lowerFrequency = rscmwevm_atof (p2buf);
    
        p2buf = strtok (NULL, ",\n");
        *upperFrequency = rscmwevm_atof (p2buf);
    }
    
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns occupied Bandwidth (OBW) limits check results for selected 
/// HIFN  segment in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR occupiedBandwidth/Returns the occupied bandwidth limit state.
/// HIPAR lowerFrequency/Returns the lower edge of the selected carrier's or carrier group's occupied bandwidth limit state.
/// HIPAR upperFrequency/Returns the upper edge of the selected carrier's or carrier group's occupied bandwidth limit state.
/// HIPAR carrierNumber/Selects the isolated carrier or group carrier to be analyzed. 
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeOBWLimitsCheckResults (ViSession instrSession,
                                                                        ViInt32 instance,
                                                                        ViInt32 segment,
																		ViInt32 carrierNumber,
                                                                        ViInt32 resultType,
                                                                        ViInt32 *reliability,
                                                                        ViInt32 *segmentReliability,
                                                                        ViInt32 *occupiedBandwidth,
                                                                        ViInt32 *lowerFrequency,
                                                                        ViInt32 *upperFrequency)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, carrierNumber, 1, 4), 4, "Carrier Number");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MAX), 5, "Result Type");

    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:OBW%ld:%s?\n", instance, segment, carrierNumber, resultTypeArr[resultType]);

    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);
    
    p2buf = strtok (NULL, ",\n");
    *segmentReliability = rscmwevm_atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    *occupiedBandwidth = rscmwevm_strCode (p2buf, arrayCodes);
    
    if (resultType == RSCMWEVM_VAL_RESULT_CURRENT)
    {
        p2buf = strtok (NULL, ",\n");
        *lowerFrequency = rscmwevm_strCode (p2buf, arrayCodes);
    
        p2buf = strtok (NULL, ",\n");
        *upperFrequency = rscmwevm_strCode (p2buf, arrayCodes);
    }
    
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  In list mode, returns the Occupied Bandwidth (OBW) result for
/// HIFN  selected segment and the carrier or carrier group. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR occupiedBandwidth/Returns the occupied bandwidth value.
/// HIPAR carrierNumber/Selects the isolated carrier or group carrier to be analyzed.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeOBWStandardDeviation (ViSession instrSession,
                                                                       ViInt32 instance,
                                                                       ViInt32 segment,
																	   ViInt32 carrierNumber,																	   
                                                                       ViInt32 *reliability,
                                                                       ViInt32 *segmentReliability,
                                                                       ViReal64 *occupiedBandwidth)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
		viCheckParm (rscmwevm_invalidViInt32Range (instrSession, carrierNumber, 1, 4), 4, "Carrier Number");
    if (occupiedBandwidth == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };

	
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:OBW%ld:SDEV?\n", instance, segment, carrierNumber);
    
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);
     p2buf = strtok (NULL, ",\n");  
     
     *segmentReliability = rscmwevm_atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    *occupiedBandwidth = rscmwevm_atof (p2buf);

    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  In list mode, returns the Occupied Bandwidth (OBW) standard
/// HIFN  deviation limits check results for segment and the carrier or carrier group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR occupiedBandwidth/Returns the occupied bandwidth limit state.
/// HIPAR carrierNumber/Selects the isolated carrier or group carrier to be analyzed.  
ViStatus _VI_FUNC rscmwevm_QueryFetchMEvalMeasListModeOBWStandardDeviationLimitsCheck (ViSession instrSession,
                                                                                       ViInt32 instance,
                                                                                       ViInt32 segment,
																					   ViInt32 carrierNumber,																					   
                                                                                       ViInt32 *reliability,
                                                                                       ViInt32 *segmentReliability,
                                                                                       ViInt32 *occupiedBandwidth)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
   
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, carrierNumber, 1, 4), 4, "Carrier Number");
   
	sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:OBW%ld:SDEV?\n", instance, segment, carrierNumber);

    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);
    
    p2buf = strtok (NULL, ",\n");
    *segmentReliability = rscmwevm_atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    *occupiedBandwidth = rscmwevm_strCode (p2buf, arrayCodes);
    
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns modulation single value results for selected segment in list 
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR results/Returns the single slot modulation results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeModulationResults (ViSession instrSession,
                                                                    ViInt32 instance,
                                                                    ViInt32 segment,
                                                                    ViInt32 resultType,
                                                                    ViInt32 arraySize,
                                                                    ViInt32 *reliability,
                                                                    ViInt32 *segmentReliability,
                                                                    ViReal64 results[],
                                                                    ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 4, "Result Type");
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:MOD:%s?\n", instance, segment, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasList (instrSession, command, 15000, arraySize, reliability, segmentReliability, results, returnedValues));

Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns modulation limits check results for selected segment in list 
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR results/Returns the single slot modulation results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeModulationLimitsCheckResults (ViSession instrSession,
                                                                               ViInt32 instance,
                                                                               ViInt32 segment,
                                                                               ViInt32 resultType,
                                                                               ViInt32 arraySize,
                                                                               ViInt32 *reliability,
                                                                               ViInt32 *segmentReliability,
                                                                               ViInt32 results[],
                                                                               ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:MOD:%s?\n", instance, segment, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTraceLimitsAll (instrSession, command, 15000, arraySize, reliability, segmentReliability, results, returnedValues));

Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns modulation standard deviation single value results for 
/// HIFN  selected segment in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR results/Returns the single slot modulation results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeModulationStandardDeviation (ViSession instrSession,
                                                                              ViInt32 instance,
                                                                              ViInt32 segment,
                                                                              ViInt32 arraySize,
                                                                              ViInt32 *reliability,
                                                                              ViInt32 *segmentReliability,
                                                                              ViReal64 results[],
                                                                              ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
   
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:MOD:SDEV?\n", instance, segment);
    checkErr (rscmwevm_GetMeasList (instrSession, command, 15000, arraySize, reliability, segmentReliability, results, returnedValues));


Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns modulation standard deviation limits check results for 
/// HIFN  selected segment in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR results/Returns the single slot modulation results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeModulationStandardDeviationLimitsCheck (ViSession instrSession,
                                                                                         ViInt32 instance,
                                                                                         ViInt32 segment,
                                                                                         ViInt32 arraySize,
                                                                                         ViInt32 *reliability,
                                                                                         ViInt32 *segmentReliability,
                                                                                         ViInt32 results[],
                                                                                         ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:MOD:SDEV?\n", instance, segment);
    checkErr (rscmwevm_GetMeasTraceLimitsAll (instrSession, command, 15000, arraySize, reliability, segmentReliability, results, returnedValues));

Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}


/// HIFN  Returns the scalar channel power for the Data channel results for the 
/// HIFN  selected segment in list mode. The result is extended to the W2^4 and 
/// HIFN  W1^2 I/Q values of the Data channel. Only available if subtype 2 is 
/// HIFN  selected. Otherwise NAV is returned. in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR results/Returns the scalar channel power for the Data channel results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeDWCChannelPowerResults (ViSession instrSession,
                                                                         ViInt32 instance,
                                                                         ViInt32 segment,
                                                                         ViInt32 resultType,
                                                                         ViInt32 arraySize,
                                                                         ViInt32 *reliability,
                                                                         ViInt32 *segmentReliability,
                                                                         ViReal64 results[],
                                                                         ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:DWCP:%s?\n", instance, segment, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasList (instrSession, command, 15000, arraySize, reliability, segmentReliability, results, returnedValues));

    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns the limits state of scalar channel power for the Data channel 
/// HIFN  results for the selected segment in list mode. The result is extended 
/// HIFN  to the W2^4 and W1^2 I/Q values of the Data channel. Only available if 
/// HIFN  subtype 2 is selected. Otherwise NAV is returned. in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Selects the segment.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR results/Returns the scalar channel power for the Data channel limits check 
/// HIPAR results/results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeDWCChannelPowerLimitsCheckResults (ViSession instrSession,
                                                                                    ViInt32 instance,
                                                                                    ViInt32 segment,
                                                                                    ViInt32 resultType,
                                                                                    ViInt32 arraySize,
                                                                                    ViInt32 *reliability,
                                                                                    ViInt32 *segmentReliability,
                                                                                    ViInt32 results[],
                                                                                    ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, segment, 1, 200), 3, "Segment");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:SEGM%ld:DWCP:%s?\n", instance, segment, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasTraceLimitsAll (instrSession, command, 15000, arraySize, reliability, segmentReliability, results, returnedValues));
Error:
    return error;
}

/// HIFN  Returns the segment reliability for all measured list mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwc2m_init or 
/// HIPAR instrSession/rscmwc2m_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/Array of integer values of the reliability indicator for all measured 
/// HIPAR segmentReliability/list mode segments.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeAllReliabilityResults (ViSession instrSession,
                                                               ViInt32 instance,
                                                               ViInt32 numberOfActiveSegments,
                                                               ViInt32 *reliability,
                                                               ViInt32 segmentReliability[],
                                                               ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
    if (segmentReliability == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Segment Reliability");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:SREL?\n", instance);
	
    checkErr (rscmwevm_GetMeasReliabilitiesAll (instrSession, command, 15000, numberOfActiveSegments,  
													reliability, segmentReliability, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns limit check of the segment reliability results for all 
/// HIFN  segments in list mode.
/// HIFN     
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate:EVDO:MEAS<i>:MEValuation:LIST:SRELiability?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwc2m_init or 
/// HIPAR instrSession/rscmwc2m_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberofActiveSegments/Number of active segments.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/Returns limits check results values of the segment reliability results 
/// HIPAR segmentReliability/for all segments in list mode of the multi evaluation measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeAllReliabilityLimitsCheckResult(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32	numberofActiveSegments,
	ViInt32*	reliability,
	ViInt32	segmentReliability[],
	ViInt32*	returnedValues
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberofActiveSegments, 1, 200), 3, "Number Of Active Segments");
    if (segmentReliability == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Segment Reliability");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:SREL?\n", instance);
	
    checkErr (rscmwevm_GetMeasReliabilitiesAll (instrSession, command, 15000, numberofActiveSegments,  
													reliability, segmentReliability, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns all ACP value results for the all segments in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR ACPResults/Returns the adjacent channel power measured at a series of 
/// HIPAR ACPResults/frequencies. The power offsets <Power Offset +n> (<Power Offset -n>) 
/// HIPAR ACPResults/refer to the average power at the carrier frequency plus (minus) the 
/// HIPAR ACPResults/frequency offset value number n.
/// HIPAR ATPWide/Returns the access terminal power, measured with the wideband filter.
/// HIPAR ATPNarrow/Returns the access terminal power, measured with a filter bandwidth of 
/// HIPAR ATPNarrow/1.23 MHz
/// HIPAR outOfTolerance/Returns the out of tolerance result, i.e. percentage of measurement 
/// HIPAR outOfTolerance/intervals of the statistic count exceeding the specified.
/// HIPAR validSlotsCount/Returns the number of evaluated valid slots in this segment.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeAllACPResults (ViSession instrSession,
                                                                ViInt32 instance,
                                                                ViInt32 numberOfActiveSegments,
                                                                ViInt32 resultType,
                                                                ViInt32 arraySize,
                                                                ViInt32 *reliability,
                                                                ViInt32 *segmentReliability,
                                                                ViReal64 ACPResults[],
                                                                ViReal64 *ATPWide,
                                                                ViReal64 *ATPNarrow,
                                                                ViReal64 *outOfTolerance,
                                                                ViInt32 *validSlotsCount,
                                                                ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    ViInt32  i = 0,
             n = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 4, "Result Type");
    if (ACPResults == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
	checkErr (Rs_ClearBeforeRead(instrSession));     
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:ACP:%s?;*OPC\n", instance, resultTypeArr[resultType]);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    while (p2buf != NULL && i < numberOfActiveSegments)
    {
        segmentReliability[i] = rscmwevm_atol (p2buf);
        p2buf = strtok (NULL, ",\n");
            
        for (n = 0; n < 21; n++);
        {
            if (n<arraySize)
            {
                ACPResults[arraySize*i + n] = rscmwevm_atof (p2buf);
                p2buf = strtok (NULL, ",\n");
            }
            else 
            {
                p2buf = strtok (NULL, ",\n");
                break;
            }
        }
        ATPWide[i] = rscmwevm_atof (p2buf);
        p2buf = strtok (NULL, ",\n");
        
        ATPNarrow[i] = rscmwevm_atof (p2buf);
        p2buf = strtok (NULL, ",\n");
        
        outOfTolerance[i] = rscmwevm_atof (p2buf);
        p2buf = strtok (NULL, ",\n");
        
        validSlotsCount[i] = rscmwevm_atol (p2buf);
        p2buf = strtok (NULL, ",\n");
        
        i++;
    }
    if (returnedValues) *returnedValues=i;
    
    checkErr( rscmwevm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}



/// HIFN  Returns all ACP limits check results for the all segments in list 
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR ACPResults/Returns the adjacent channel power limit state measured at a series of 
/// HIPAR ACPResults/frequencies. The power offsets <Power Offset +n> (<Power Offset -n>) 
/// HIPAR ACPResults/refer to the average power at the carrier frequency plus (minus) the 
/// HIPAR ACPResults/frequency offset value number n.
/// HIPAR ATPWide/Returns the access terminal power limit state, measured with the 
/// HIPAR ATPWide/wideband filter.
/// HIPAR ATPNarrow/Returns the access terminal power limit state, measured with a filter 
/// HIPAR ATPNarrow/bandwidth of 1.23 MHz
/// HIPAR outOfTolerance/Returns the out of tolerance result limit state, i.e. percentage of 
/// HIPAR outOfTolerance/measurement intervals of the statistic count exceeding the specified.
/// HIPAR validSlotsCount/Returns the limit state for number of evaluated valid slots in this 
/// HIPAR validSlotsCount/segment.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeAllACPLimitsCheckResults (ViSession instrSession,
                                                                           ViInt32 instance,
                                                                           ViInt32 numberOfActiveSegments,
                                                                           ViInt32 resultType,
                                                                           ViInt32 arraySize,
                                                                           ViInt32 *reliability,
                                                                           ViInt32 *segmentReliability,
                                                                           ViInt32 ACPResults[],
                                                                           ViInt32 *ATPWide,
                                                                           ViInt32 *ATPNarrow,
                                                                           ViInt32 *outOfTolerance,
                                                                           ViInt32 *validSlotsCount,
                                                                           ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViString ResultStateArr[] = {"OK","ULEU","ULEL",NULL};
    ViChar      command[150] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    ViInt32  i = 0,
             n = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 4, "Result Type");
    if (ACPResults == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
	checkErr (Rs_ClearBeforeRead(instrSession));     
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:ACP:%s?;*OPC\n", instance, resultTypeArr[resultType]);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    while (p2buf != NULL && i < numberOfActiveSegments)
    {
        segmentReliability[i] = rscmwevm_atol (p2buf);
        p2buf = strtok (NULL, ",\n");
            
        for (n = 0; n < 21; n++);
        {
            if (n<arraySize)
            {
                ACPResults[arraySize*i + n] = rscmwevm_StringIndex (ResultStateArr,p2buf);
                p2buf = strtok (NULL, ",\n");
            }
            else 
            {
                p2buf = strtok (NULL, ",\n");
                break;
            }
        }
        ATPWide[i] = rscmwevm_StringIndex (ResultStateArr,p2buf);
        p2buf = strtok (NULL, ",\n");
        
        ATPNarrow[i] = rscmwevm_StringIndex (ResultStateArr,p2buf);
        p2buf = strtok (NULL, ",\n");
        
        outOfTolerance[i] = rscmwevm_StringIndex (ResultStateArr,p2buf);
        p2buf = strtok (NULL, ",\n");
        
        validSlotsCount[i] = rscmwevm_StringIndex (ResultStateArr,p2buf);
        p2buf = strtok (NULL, ",\n");
        
        i++;
    }
    if (returnedValues) *returnedValues=i;
    
    checkErr( rscmwevm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}



/// HIFN  Returns all ACP standard deviation results for the all segments in 
/// HIFN  list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR ACPResults/Returns the adjacent channel power measured at a series of 
/// HIPAR ACPResults/frequencies. The power offsets <Power Offset +n> (<Power Offset -n>) 
/// HIPAR ACPResults/refer to the average power at the carrier frequency plus (minus) the 
/// HIPAR ACPResults/frequency offset value number n.
/// HIPAR ATPWide/Returns the access terminal power, measured with the wideband filter.
/// HIPAR ATPNarrow/Returns the access terminal power, measured with a filter bandwidth of 
/// HIPAR ATPNarrow/1.23 MHz
/// HIPAR outOfTolerance/Returns the out of tolerance result, i.e. percentage of measurement 
/// HIPAR outOfTolerance/intervals of the statistic count exceeding the specified.
/// HIPAR validSlotsCount/Returns the number of evaluated valid slots in this segment.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeAllACPStandardDeviation (ViSession instrSession,
                                                                          ViInt32 instance,
                                                                          ViInt32 numberOfActiveSegments,
                                                                          ViInt32 arraySize,
                                                                          ViInt32 *reliability,
                                                                          ViInt32 *segmentReliability,
                                                                          ViReal64 ACPResults[],
                                                                          ViReal64 *ATPWide,
                                                                          ViReal64 *ATPNarrow,
                                                                          ViReal64 *outOfTolerance,
                                                                          ViInt32 *validSlotsCount,
                                                                          ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    ViInt32  i = 0,
             n = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
    if (ACPResults == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
    checkErr (Rs_ClearBeforeRead(instrSession));     
	sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:ACP:SDEV?;*OPC\n", instance);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    while (p2buf != NULL && i < numberOfActiveSegments)
    {
        segmentReliability[i] = rscmwevm_atol (p2buf);
        p2buf = strtok (NULL, ",\n");
            
        for (n = 0; n < 21; n++);
        {
            if (n<arraySize)
            {
                ACPResults[arraySize*i + n] = rscmwevm_atof (p2buf);
                p2buf = strtok (NULL, ",\n");
            }
            else 
            {
                p2buf = strtok (NULL, ",\n");
                break;
            }
        }
        ATPWide[i] = rscmwevm_atof (p2buf);
        p2buf = strtok (NULL, ",\n");
        
        ATPNarrow[i] = rscmwevm_atof (p2buf);
        p2buf = strtok (NULL, ",\n");
        
        outOfTolerance[i] = rscmwevm_atof (p2buf);
        p2buf = strtok (NULL, ",\n");
        
        validSlotsCount[i] = rscmwevm_atol (p2buf);
        p2buf = strtok (NULL, ",\n");
        
        i++;
    }
    if (returnedValues) *returnedValues=i;
    
    checkErr( rscmwevm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
} 


/// HIFN  Returns all ACP standard deviation limits check results for the all 
/// HIFN  segments in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR ACPResults/Returns the limit check for adjacent channel power measured at a 
/// HIPAR ACPResults/series of frequencies. The power offsets <Power Offset +n> (<Power 
/// HIPAR ACPResults/Offset -n>) refer to the average power at the carrier frequency plus 
/// HIPAR ACPResults/(minus) the frequency offset value number n.
/// HIPAR ATPWide/Returns the access terminal power, measured with the wideband filter.
/// HIPAR ATPNarrow/Returns the access terminal power, measured with a filter bandwidth of 
/// HIPAR ATPNarrow/1.23 MHz
/// HIPAR outOfTolerance/Returns the out of tolerance result, i.e. percentage of measurement 
/// HIPAR outOfTolerance/intervals of the statistic count exceeding the specified.
/// HIPAR validSlotsCount/Returns the number of evaluated valid slots in this segment.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeAllACPStandardDeviationLimitsCheck (ViSession instrSession,
                                                                                     ViInt32 instance,
                                                                                     ViInt32 numberOfActiveSegments,
                                                                                     ViInt32 arraySize,
                                                                                     ViInt32 *reliability,
                                                                                     ViInt32 *segmentReliability,
                                                                                     ViInt32 ACPResults[],
                                                                                     ViInt32 *ATPWide,
                                                                                     ViInt32 *ATPNarrow,
                                                                                     ViInt32 *outOfTolerance,
                                                                                     ViInt32 *validSlotsCount,
                                                                                     ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViString ResultStateArr[] = {"OK","ULEU","ULEL",NULL};
    ViChar      command[150] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    ViInt32  i = 0,
             n = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
    if (ACPResults == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
	checkErr (Rs_ClearBeforeRead(instrSession));     
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:ACP:SDEV?;*OPC\n", instance);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    while (p2buf != NULL && i < numberOfActiveSegments)
    {
        segmentReliability[i] = rscmwevm_atol (p2buf);
        p2buf = strtok (NULL, ",\n");
            
        for (n = 0; n < 21; n++);
        {
            if (n<arraySize)
            {
                ACPResults[arraySize*i + n] = rscmwevm_StringIndex (ResultStateArr,p2buf);
                p2buf = strtok (NULL, ",\n");
            }
            else 
            {
                p2buf = strtok (NULL, ",\n");
                break;
            }
        }
        ATPWide[i] = rscmwevm_StringIndex (ResultStateArr,p2buf);
        p2buf = strtok (NULL, ",\n");
        
        ATPNarrow[i] = rscmwevm_StringIndex (ResultStateArr,p2buf);
        p2buf = strtok (NULL, ",\n");
        
        outOfTolerance[i] = rscmwevm_StringIndex (ResultStateArr,p2buf);
        p2buf = strtok (NULL, ",\n");
        
        validSlotsCount[i] = rscmwevm_StringIndex (ResultStateArr,p2buf);
        p2buf = strtok (NULL, ",\n");
        
        i++;
    }
    if (returnedValues) *returnedValues=i;
    
    checkErr( rscmwevm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns all ACP value results for the all segments in list mode.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     FETCh:EVDO:MEAS<instance>:MEValuation:LIST:ACP:EXTended:AVERage?
/// HIFN     FETCh:EVDO:MEAS<instance>:MEValuation:LIST:ACP:EXTended:AVERage?
/// HIFN     FETCh:EVDO:MEAS<instance>:MEValuation:LIST:ACP:EXTended:MINimum?
/// HIFN     FETCh:EVDO:MEAS<instance>:MEValuation:LIST:ACP:EXTended:MAXimum?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR ACPResults/Returns the adjacent channel power measured at a series of 
/// HIPAR ACPResults/frequencies. The power offsets <Power Offset +n> (<Power Offset -n>) 
/// HIPAR ACPResults/refer to the average power at the carrier frequency plus (minus) the 
/// HIPAR ACPResults/frequency offset value number n.
/// HIPAR ATPWide/Returns the access terminal power, measured with the wideband filter.
/// HIPAR ATPNarrow/Returns the access terminal power, measured with a filter bandwidth of 
/// HIPAR ATPNarrow/1.23 MHz
/// HIPAR outOfTolerance/Returns the out of tolerance result, i.e. percentage of measurement 
/// HIPAR outOfTolerance/intervals of the statistic count exceeding the specified.
/// HIPAR validSlotsCount/Returns the number of evaluated valid slots in this segment.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeAllACPExtendedResults (ViSession instrSession,
                                                                        ViInt32 instance,
                                                                        ViInt32 numberOfActiveSegments,
                                                                        ViInt32 resultType,
                                                                        ViInt32 arraySize,
                                                                        ViInt32 *reliability,
                                                                        ViInt32 *segmentReliability,
                                                                        ViReal64 ACPResults[],
                                                                        ViReal64 *ATPWide,
                                                                        ViReal64 *ATPNarrow,
                                                                        ViReal64 *outOfTolerance,
                                                                        ViInt32 *validSlotsCount,
                                                                        ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    ViInt32  i = 0,
             n = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 4, "Result Type");
    if (ACPResults == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
	checkErr (Rs_ClearBeforeRead(instrSession));     
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:ACP:EXT:%s?;*OPC\n", instance, resultTypeArr[resultType]);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    while (p2buf != NULL && i < numberOfActiveSegments)
    {
        segmentReliability[i] = rscmwevm_atol (p2buf);
        p2buf = strtok (NULL, ",\n");
            
        for (n = 0; n < 21; n++);
        {
            if (n<arraySize)
            {
                ACPResults[arraySize*i + n] = rscmwevm_atof (p2buf);
                p2buf = strtok (NULL, ",\n");
            }
            else 
            {
                p2buf = strtok (NULL, ",\n");
                break;
            }
        }
        ATPWide[i] = rscmwevm_atof (p2buf);
        p2buf = strtok (NULL, ",\n");
        
        ATPNarrow[i] = rscmwevm_atof (p2buf);
        p2buf = strtok (NULL, ",\n");
        
        outOfTolerance[i] = rscmwevm_atof (p2buf);
        p2buf = strtok (NULL, ",\n");
        
        validSlotsCount[i] = rscmwevm_atol (p2buf);
        p2buf = strtok (NULL, ",\n");
        
        i++;
    }
    if (returnedValues) *returnedValues=i;
    
    checkErr( rscmwevm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns all ACP limits check results for the all segments in list 
/// HIFN  mode.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate:EVDO:MEAS<instance>:MEValuation:LIST:ACP:EXTended:CURRent?
/// HIFN     CALCulate:EVDO:MEAS<instance>:MEValuation:LIST:ACP:EXTended:AVERage?
/// HIFN     CALCulate:EVDO:MEAS<instance>:MEValuation:LIST:ACP:EXTended:MINimum?
/// HIFN     CALCulate:EVDO:MEAS<instance>:MEValuation:LIST:ACP:EXTended:MAXimum?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR ACPResults/Returns the adjacent channel power limit state measured at a series of 
/// HIPAR ACPResults/frequencies. The power offsets <Power Offset +n> (<Power Offset -n>) 
/// HIPAR ACPResults/refer to the average power at the carrier frequency plus (minus) the 
/// HIPAR ACPResults/frequency offset value number n.
/// HIPAR ATPWide/Returns the access terminal power limit state, measured with the 
/// HIPAR ATPWide/wideband filter.
/// HIPAR ATPNarrow/Returns the access terminal power limit state, measured with a filter 
/// HIPAR ATPNarrow/bandwidth of 1.23 MHz
/// HIPAR outOfTolerance/Returns the out of tolerance result limit state, i.e. percentage of 
/// HIPAR outOfTolerance/measurement intervals of the statistic count exceeding the specified.
/// HIPAR validSlotsCount/Returns the limit state for number of evaluated valid slots in this 
/// HIPAR validSlotsCount/segment.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeAllACPLimitsCheckExtendedResults (ViSession instrSession,
                                                                                   ViInt32 instance,
                                                                                   ViInt32 numberOfActiveSegments,
                                                                                   ViInt32 resultType,
                                                                                   ViInt32 arraySize,
                                                                                   ViInt32 *reliability,
                                                                                   ViInt32 *segmentReliability,
                                                                                   ViInt32 ACPResults[],
                                                                                   ViInt32 *ATPWide,
                                                                                   ViInt32 *ATPNarrow,
                                                                                   ViInt32 *outOfTolerance,
                                                                                   ViInt32 *validSlotsCount,
                                                                                   ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViString ResultStateArr[] = {"OK","ULEU","ULEL",NULL};
    ViChar      command[150] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    ViInt32  i = 0,
             n = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 4, "Result Type");
    if (ACPResults == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
	checkErr (Rs_ClearBeforeRead(instrSession));  
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:ACP:EXT:%s?;*OPC\n", instance, resultTypeArr[resultType]);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    while (p2buf != NULL && i < numberOfActiveSegments)
    {
        segmentReliability[i] = rscmwevm_atol (p2buf);
        p2buf = strtok (NULL, ",\n");
            
        for (n = 0; n < 21; n++);
        {
            if (n<arraySize)
            {
                ACPResults[arraySize*i + n] = rscmwevm_StringIndex (ResultStateArr,p2buf);
                p2buf = strtok (NULL, ",\n");
            }
            else 
            {
                p2buf = strtok (NULL, ",\n");
                break;
            }
        }
        ATPWide[i] = rscmwevm_StringIndex (ResultStateArr,p2buf);
        p2buf = strtok (NULL, ",\n");
        
        ATPNarrow[i] = rscmwevm_StringIndex (ResultStateArr,p2buf);
        p2buf = strtok (NULL, ",\n");
        
        outOfTolerance[i] = rscmwevm_StringIndex (ResultStateArr,p2buf);
        p2buf = strtok (NULL, ",\n");
        
        validSlotsCount[i] = rscmwevm_StringIndex (ResultStateArr,p2buf);
        p2buf = strtok (NULL, ",\n");
        
        i++;
    }
    if (returnedValues) *returnedValues=i;
    
    checkErr( rscmwevm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns all ACP standard deviation results for the all segments in 
/// HIFN  list mode.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     FETCh:EVDO:MEAS<instance>:MEValuation:LIST:ACP:EXTended:SDEViation?
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR ACPResults/Returns the adjacent channel power measured at a series of 
/// HIPAR ACPResults/frequencies. The power offsets <Power Offset +n> (<Power Offset -n>) 
/// HIPAR ACPResults/refer to the average power at the carrier frequency plus (minus) the 
/// HIPAR ACPResults/frequency offset value number n.
/// HIPAR ATPWide/Returns the access terminal power, measured with the wideband filter.
/// HIPAR ATPNarrow/Returns the access terminal power, measured with a filter bandwidth of 
/// HIPAR ATPNarrow/1.23 MHz
/// HIPAR outOfTolerance/Returns the out of tolerance result, i.e. percentage of measurement 
/// HIPAR outOfTolerance/intervals of the statistic count exceeding the specified.
/// HIPAR validSlotsCount/Returns the number of evaluated valid slots in this segment.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeAllACPStandardDeviationExtended (ViSession instrSession,
                                                                                  ViInt32 instance,
                                                                                  ViInt32 numberOfActiveSegments,
                                                                                  ViInt32 arraySize,
                                                                                  ViInt32 *reliability,
                                                                                  ViInt32 *segmentReliability,
                                                                                  ViReal64 ACPResults[],
                                                                                  ViReal64 *ATPWide,
                                                                                  ViReal64 *ATPNarrow,
                                                                                  ViReal64 *outOfTolerance,
                                                                                  ViInt32 *validSlotsCount,
                                                                                  ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    ViInt32  i = 0,
             n = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
    if (ACPResults == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
	checkErr (Rs_ClearBeforeRead(instrSession));  
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:ACP:EXT:SDEV?;*OPC\n", instance);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    while (p2buf != NULL && i < numberOfActiveSegments)
    {
        segmentReliability[i] = rscmwevm_atol (p2buf);
        p2buf = strtok (NULL, ",\n");
            
        for (n = 0; n < 21; n++);
        {
            if (n<arraySize)
            {
                ACPResults[arraySize*i + n] = rscmwevm_atof (p2buf);
                p2buf = strtok (NULL, ",\n");
            }
            else 
            {
                p2buf = strtok (NULL, ",\n");
                break;
            }
        }
        ATPWide[i] = rscmwevm_atof (p2buf);
        p2buf = strtok (NULL, ",\n");
        
        ATPNarrow[i] = rscmwevm_atof (p2buf);
        p2buf = strtok (NULL, ",\n");
        
        outOfTolerance[i] = rscmwevm_atof (p2buf);
        p2buf = strtok (NULL, ",\n");
        
        validSlotsCount[i] = rscmwevm_atol (p2buf);
        p2buf = strtok (NULL, ",\n");
        
        i++;
    }
    if (returnedValues) *returnedValues=i;
    
    checkErr( rscmwevm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns all ACP standard deviation limits check results for the all 
/// HIFN  segments in list mode.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     
/// HIFN  CALCulate:EVDO:MEAS<instance>:MEValuation:LIST:ACP:EXTended:SDEViation?
/// HIFN     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR ACPResults/Returns the limit check for adjacent channel power measured at a 
/// HIPAR ACPResults/series of frequencies. The power offsets <Power Offset +n> (<Power 
/// HIPAR ACPResults/Offset -n>) refer to the average power at the carrier frequency plus 
/// HIPAR ACPResults/(minus) the frequency offset value number n.
/// HIPAR ATPWide/Returns the access terminal power, measured with the wideband filter.
/// HIPAR ATPNarrow/Returns the access terminal power, measured with a filter bandwidth of 
/// HIPAR ATPNarrow/1.23 MHz
/// HIPAR outOfTolerance/Returns the out of tolerance result, i.e. percentage of measurement 
/// HIPAR outOfTolerance/intervals of the statistic count exceeding the specified.
/// HIPAR validSlotsCount/Returns the number of evaluated valid slots in this segment.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeAllACPStandardDeviationLimitsCheckExtended (ViSession instrSession,
                                                                                             ViInt32 instance,
                                                                                             ViInt32 numberOfActiveSegments,
                                                                                             ViInt32 arraySize,
                                                                                             ViInt32 *reliability,
                                                                                             ViInt32 *segmentReliability,
                                                                                             ViInt32 ACPResults[],
                                                                                             ViInt32 *ATPWide,
                                                                                             ViInt32 *ATPNarrow,
                                                                                             ViInt32 *outOfTolerance,
                                                                                             ViInt32 *validSlotsCount,
                                                                                             ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViString ResultStateArr[] = {"OK","ULEU","ULEL",NULL};
    ViChar      command[150] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    ViInt32  i = 0,
             n = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
    if (ACPResults == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
	checkErr (Rs_ClearBeforeRead(instrSession));  
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:ACP:EXT:SDEV?;*OPC\n", instance);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    while (p2buf != NULL && i < numberOfActiveSegments)
    {
        segmentReliability[i] = rscmwevm_atol (p2buf);
        p2buf = strtok (NULL, ",\n");
            
        for (n = 0; n < 21; n++);
        {
            if (n<arraySize)
            {
                ACPResults[arraySize*i + n] = rscmwevm_StringIndex (ResultStateArr,p2buf);
                p2buf = strtok (NULL, ",\n");
            }
            else 
            {
                p2buf = strtok (NULL, ",\n");
                break;
            }
        }
        ATPWide[i] = rscmwevm_StringIndex (ResultStateArr,p2buf);
        p2buf = strtok (NULL, ",\n");
        
        ATPNarrow[i] = rscmwevm_StringIndex (ResultStateArr,p2buf);
        p2buf = strtok (NULL, ",\n");
        
        outOfTolerance[i] = rscmwevm_StringIndex (ResultStateArr,p2buf);
        p2buf = strtok (NULL, ",\n");
        
        validSlotsCount[i] = rscmwevm_StringIndex (ResultStateArr,p2buf);
        p2buf = strtok (NULL, ",\n");
        
        i++;
    }
    if (returnedValues) *returnedValues=i;
    
    checkErr( rscmwevm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns channel power value results for the all segments in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR results/Returns RMS channel power values for the physical channels. Depending 
/// HIPAR results/on the subtype 0/1 or 2 some channels exist or not. If not available, 
/// HIPAR results/no power measurement is possible therefore NAV is returned.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeAllChannelPowerResults (ViSession instrSession,
                                                                         ViInt32 instance,
                                                                         ViInt32 numberOfActiveSegments,
                                                                         ViInt32 resultType,
                                                                         ViInt32 arraySize,
                                                                         ViInt32 *reliability,
                                                                         ViInt32 *segmentReliability,
                                                                         ViReal64 results[],
                                                                         ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:CP:%s?\n", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasListAll (instrSession, command, 15000, numberOfActiveSegments, arraySize, reliability, segmentReliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns channel power value limits check results for the all  segments 
/// HIFN  in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR results/Returns the limits state of RMS channel power for the physical 
/// HIPAR results/channels. Depending on the subtype 0/1 or 2 some channels exist or not. 
/// HIPAR results/If not available, no power measurement is possible therefore NAV is 
/// HIPAR results/returned.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeAllChannelPowerLimitsCheckResults (ViSession instrSession,
                                                                                    ViInt32 instance,
                                                                                    ViInt32 numberOfActiveSegments,
                                                                                    ViInt32 resultType,
                                                                                    ViInt32 arraySize,
                                                                                    ViInt32 *reliability,
                                                                                    ViInt32 *segmentReliability,
                                                                                    ViInt32 results[],
                                                                                    ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:CP:%s?\n", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_QueryMeasListAll (instrSession, command, 15000, numberOfActiveSegments, arraySize, reliability, segmentReliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}


/// HIFN  Return the states of the channels for power measurement for the all 
/// HIFN  segments in list mode.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     FETCh:EVDO:MEAS<instance>:MEValuation:LIST:CP:STATe?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR results/Return the states of the channels for power measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeAllChannelPowerResultsState (ViSession instrSession,
                                                                              ViInt32 instance,
                                                                              ViInt32 numberOfActiveSegments,
                                                                              ViInt32 resultType,
                                                                              ViInt32 arraySize,
                                                                              ViInt32 *reliability,
                                                                              ViInt32 *segmentReliability,
                                                                              ViInt32 results[],
                                                                              ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
	resultType;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:CP:STAT?\n", instance);
    checkErr (rscmwevm_GetMeasListStateAll (instrSession, command, 15000, numberOfActiveSegments, arraySize, reliability, segmentReliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}



/// HIFN  For the carrier or carrier group  returns Occupied Bandwidth
/// HIFN  (OBW) results for all segments in list mode. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.   
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR occupiedBandwidth/Returns the occupied bandwidth value.
/// HIPAR lowerEdge/Returns the lower edge of the selected carrier's or carrier group's occupied bandwidth.
/// HIPAR upperEdge/Returns the upper edge of the selected carrier's or carrier group's occupied bandwidth.
/// HIPAR carrierNumber/Selects the isolated carrier or group carrier to be analyzed.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeAllOBWResults (ViSession instrSession,
                                                                ViInt32 instance,
                                                                ViInt32 numberOfActiveSegments,
																ViInt32 carrierNumber,
                                                                ViInt32 resultType,
                                                                ViInt32 *reliability,
                                                                ViInt32 *segmentReliability,
                                                                ViReal64 *occupiedBandwidth,
                                                                ViReal64 *lowerEdge,
                                                                ViReal64 *upperEdge)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = ""; 
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    ViInt32  i = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, carrierNumber, 1, 4), 4, "Carrier Number");
	
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MAX), 5, "Result Type");
    if (occupiedBandwidth == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Occupied Bandwidth");   
    };
	

    checkErr (Rs_ClearBeforeRead(instrSession));  
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:OBW%ld:%s?;*OPC\n", instance, carrierNumber, resultTypeArr[resultType]);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    while (p2buf != NULL && i < numberOfActiveSegments)
    {
        segmentReliability[i] = rscmwevm_atol (p2buf);
        p2buf = strtok (NULL, ",\n");
            
        occupiedBandwidth[i] = rscmwevm_atof (p2buf);
        p2buf = strtok (NULL, ",\n");
        
        if (resultType ==  RSCMWEVM_VAL_RESULT_CURRENT)
        {
            lowerEdge[i] = rscmwevm_atof (p2buf);
            p2buf = strtok (NULL, ",\n");
        
            upperEdge[i] = rscmwevm_atof (p2buf);
            p2buf = strtok (NULL, ",\n");
        }
        i++;
    }
   
 
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  For the carrier or carrier group returns Occupied Bandwidth
/// HIFN  (OBW) limits check results for all segments in list mode. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.   
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR occupiedBandwidth/Returns the occupied bandwidth limit state.
/// HIPAR lowerEdge/Returns the lower edge of the selected carrier's or carrier group's of the occupied bandwidth limit state.
/// HIPAR upperEdge/Returns the upper edge of the selected carrier's or carrier group's of the occupied bandwidth limit state.
/// HIPAR carrierNumber/Selects the isolated carrier or group carrier to be analyzed. 
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeAllOBWLimitsCheckResults (ViSession instrSession,
                                                                           ViInt32 instance,
                                                                           ViInt32 numberOfActiveSegments,
																		   ViInt32 carrierNumber,
                                                                           ViInt32 resultType,
                                                                           ViInt32 *reliability,
                                                                           ViInt32 *segmentReliability,
                                                                           ViInt32 *occupiedBandwidth,
                                                                           ViInt32 *lowerEdge,
                                                                           ViInt32 *upperEdge)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = ""; 
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    ViInt32  i = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, carrierNumber, 1, 4), 4, "Carrier Number");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MAX), 5, "Result Type");
    if (occupiedBandwidth == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Occupied Bandwidth");   
    };

    checkErr (Rs_ClearBeforeRead(instrSession));  
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:OBW%ld:%s?;*OPC\n", instance, carrierNumber,resultTypeArr[resultType]);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    while (p2buf != NULL && i < numberOfActiveSegments)
    {
        segmentReliability[i] = rscmwevm_atol (p2buf);
        p2buf = strtok (NULL, ",\n");
            
        occupiedBandwidth[i] = rscmwevm_strCode(p2buf, arrayCodes);
        p2buf = strtok (NULL, ",\n");
        
        if (resultType ==  RSCMWEVM_VAL_RESULT_CURRENT)
        {
            lowerEdge[i] = rscmwevm_strCode(p2buf, arrayCodes);
            p2buf = strtok (NULL, ",\n");
        
            upperEdge[i] = rscmwevm_strCode(p2buf, arrayCodes);
            p2buf = strtok (NULL, ",\n");
        }
        i++;
    }
   
 
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  For the carrier or carrier group returns Occupied Bandwidth
/// HIFN  (OBW) standard deviation results for all segments in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.   
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR occupiedBandwidth/Returns the occupied bandwidth value.
/// HIPAR carrierNumber/Selects the isolated carrier or group carrier to be analyzed.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeAllOBWStandardDeviation (ViSession instrSession,
                                                                          ViInt32 instance,
                                                                          ViInt32 numberOfActiveSegments,
																		  ViInt32 carrierNumber,
                                                                          ViInt32 *reliability,
                                                                          ViInt32 *segmentReliability,
                                                                          ViReal64 *occupiedBandwidth)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = ""; 
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    ViInt32  i = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, carrierNumber, 1, 4), 4, "Carrier Number");
    if (occupiedBandwidth == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Occupied Bandwidth");   
    };

	checkErr (Rs_ClearBeforeRead(instrSession));  
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:OBW%ld:SDEV?;*OPC\n", instance, carrierNumber);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    while (p2buf != NULL && i < numberOfActiveSegments)
    {
        segmentReliability[i] = rscmwevm_atol (p2buf);
        p2buf = strtok (NULL, ",\n");
            
        occupiedBandwidth[i++] = rscmwevm_atof (p2buf);
        p2buf = strtok (NULL, ",\n");
    }
   
 
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  For the carrier or carrier group returns occupied Bandwidth
/// HIFN  (OBW) standard deviation limits check results for all segments in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR occupiedBandwidth/Returns the occupied bandwidth limit state.
/// HIPAR carrierNumber/Selects the isolated carrier or group carrier to be analyzed. 
ViStatus _VI_FUNC rscmwevm_QueryFetchMEvalMeasListModeAllOBWStandardDeviationLimitsCheck (ViSession instrSession,
                                                                                          ViInt32 instance,
                                                                                          ViInt32 numberOfActiveSegments,
																						  ViInt32 carrierNumber,																						  
                                                                                          ViInt32 *reliability,
                                                                                          ViInt32 *segmentReliability,
                                                                                          ViInt32 *occupiedBandwidth)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = ""; 
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    ViInt32  i = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, carrierNumber, 1, 4), 4, "Carrier Number");
    if (occupiedBandwidth == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Occupied Bandwidth");   
    };

    checkErr (Rs_ClearBeforeRead(instrSession));  
	sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:OBW%ld:SDEV?;*OPC\n", instance, carrierNumber);
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rscmwevm_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVM_ERROR_INCORRECT_RESPONSE;
    if (reliability != NULL)
    *reliability = atol (p2buf);

    p2buf = strtok (NULL, ",\n");
    
    while (p2buf != NULL && i < numberOfActiveSegments)
    {
        segmentReliability[i] = rscmwevm_atol (p2buf);
        p2buf = strtok (NULL, ",\n");
            
        occupiedBandwidth[i++] = rscmwevm_strCode(p2buf, arrayCodes);
        p2buf = strtok (NULL, ",\n");
    }
   
 
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns modulation single value results for all segments in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR results/Returns the single slot modulation results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeAllModulationResults (ViSession instrSession,
                                                                       ViInt32 instance,
                                                                       ViInt32 numberOfActiveSegments,
                                                                       ViInt32 resultType,
                                                                       ViInt32 arraySize,
                                                                       ViInt32 *reliability,
                                                                       ViInt32 *segmentReliability,
                                                                       ViReal64 results[],
                                                                       ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:MOD:%s?\n", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasListAll (instrSession, command, 15000, numberOfActiveSegments, arraySize, reliability, segmentReliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns modulation limits check results for all segments in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR results/Returns the single slot modulation results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeAllModulationLimitsCheckResults (ViSession instrSession,
                                                                                  ViInt32 instance,
                                                                                  ViInt32 numberOfActiveSegments,
                                                                                  ViInt32 resultType,
                                                                                  ViInt32 arraySize,
                                                                                  ViInt32 *reliability,
                                                                                  ViInt32 *segmentReliability,
                                                                                  ViInt32 results[],
                                                                                  ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:MOD:%s?\n", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_QueryMeasListAll (instrSession, command, 15000, numberOfActiveSegments, arraySize, reliability, segmentReliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}


/// HIFN  Returns modulation standard deviation single value results for all 
/// HIFN  segments in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR results/Returns the single slot modulation results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeAllModulationStandardDeviation (ViSession instrSession,
                                                                                 ViInt32 instance,
                                                                                 ViInt32 numberOfActiveSegments,
                                                                                 ViInt32 arraySize,
                                                                                 ViInt32 *reliability,
                                                                                 ViInt32 *segmentReliability,
                                                                                 ViReal64 results[],
                                                                                 ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:MOD:SDEV?\n", instance);
    checkErr (rscmwevm_GetMeasListAll (instrSession, command, 15000, numberOfActiveSegments, arraySize, reliability, segmentReliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}



/// HIFN  Returns modulation standard deviation limits check results for all 
/// HIFN  segments in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR results/Returns the single slot modulation results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeAllModulationStandardDeviationLimitsCheck (ViSession instrSession,
                                                                                            ViInt32 instance,
                                                                                            ViInt32 numberOfActiveSegments,
                                                                                            ViInt32 arraySize,
                                                                                            ViInt32 *reliability,
                                                                                            ViInt32 *segmentReliability,
                                                                                            ViInt32 results[],
                                                                                            ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:MOD:SDEV?\n", instance);
    checkErr (rscmwevm_QueryMeasListAll (instrSession, command, 15000, numberOfActiveSegments, arraySize, reliability, segmentReliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/// HIFN  Returns the scalar channel power for the Data channel results for the 
/// HIFN  all segments in list mode. The result is extended to the W2^4 and W1^2 
/// HIFN  I/Q values of the Data channel. Only available if subtype 2 is 
/// HIFN  selected. Otherwise NAV is returned. in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR results/Returns the scalar channel power for the Data channel results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeAllDWCChannelPowerResults (ViSession instrSession,
                                                                            ViInt32 instance,
                                                                            ViInt32 numberOfActiveSegments,
                                                                            ViInt32 resultType,
                                                                            ViInt32 arraySize,
                                                                            ViInt32 *reliability,
                                                                            ViInt32 *segmentReliability,
                                                                            ViReal64 results[],
                                                                            ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:DWCP:%s?\n", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_GetMeasListAll (instrSession, command, 15000, numberOfActiveSegments, arraySize, reliability, segmentReliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

 
/// HIFN  Returns the limits state of scalar channel power for the Data channel 
/// HIFN  results for the all segments in list mode. The result is extended to 
/// HIFN  the W2^4 and W1^2 I/Q values of the Data channel. Only available if 
/// HIFN  subtype 2 is selected. Otherwise NAV is returned. in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for ACP result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR segmentReliability/The segment reliability indicates whether one of the following
/// HIPAR results/Returns the scalar channel power for the Data channel limits check 
/// HIPAR results/results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeAllDWCChannelPowerLimitsCheckResults (ViSession instrSession,
                                                                                       ViInt32 instance,
                                                                                       ViInt32 numberOfActiveSegments,
                                                                                       ViInt32 resultType,
                                                                                       ViInt32 arraySize,
                                                                                       ViInt32 *reliability,
                                                                                       ViInt32 *segmentReliability,
                                                                                       ViInt32 results[],
                                                                                       ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:DWCP:%s?\n", instance, resultTypeArr[resultType]);
    checkErr (rscmwevm_QueryMeasListAll (instrSession, command, 15000, numberOfActiveSegments, arraySize, reliability, segmentReliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/// HIFN  Returns narrowband and wideband (statistical) power values in the ACP 
/// HIFN  measurement for all active list mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR type/Denotes the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of power values.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMultiEvalListModeAllACPSpecificResults (ViSession instrSession,
                                                               ViInt32 instance,
                                                               ViInt32 numberOfActiveSegments,
                                                               ViInt32 resultType,
                                                               ViInt32 type,
                                                               ViInt32 *reliability,
                                                               ViReal64 results[],
                                                               ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, type, RSCMWEVM_VAL_TYPE_MS_POWER_NARROW, RSCMWEVM_VAL_TYPE_MS_POWER_WIDE), 5, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:ACP:%s:%s?", instance, 
						resultACPBandArr[type], resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns narrowband and wideband (statistical) power limit check values 
/// HIFN  in the ACP measurement for all active list mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR type/Denotes the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of limit check results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMultiEvalListModeAllACPSpecificLimitsCheckResults (ViSession instrSession,
                                                                          ViInt32 instance,
                                                                          ViInt32 numberOfActiveSegments,
                                                                          ViInt32 resultType,
                                                                          ViInt32 type,
                                                                          ViInt32 *reliability,
                                                                          ViInt32 results[],
                                                                          ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, type, RSCMWEVM_VAL_TYPE_MS_POWER_NARROW, RSCMWEVM_VAL_TYPE_MS_POWER_WIDE), 5, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:ACP:%s:%s?", instance, 
						resultACPBandArr[type], resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession (instrSession, 0);

    return error;
}

/// HIFN  Returns adjacent channel power (statistical) values for the selected 
/// HIFN  off-center channels and all active list mode segments.   
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR direction/Denotes the direction.
/// HIPAR frequencyPoint/The relative channel offset.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns adjacent channel power (statistical) values for the selected 
/// HIPAR results/off-center channels and all active list mode segments.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMultiEvalListModeAllACPOffCenterResults (ViSession instrSession,
                                                                  ViInt32 instance,
                                                                  ViInt32 numberOfActiveSegments,
                                                                  ViInt32 resultType,
                                                                  ViInt32 direction,
                                                                  ViInt32 frequencyPoint,
                                                                  ViInt32 *reliability,
                                                                  ViReal64 results[],
                                                                  ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MAX), 4, "Result Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, direction, RSCMWEVM_VAL_DIRECTION_ACPP, RSCMWEVM_VAL_DIRECTION_ACPM), 5, "Direction");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:ACP:%s%ld:%s?", instance, 
						resultACPDirectionArr[direction], frequencyPoint, resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns adjacent channel power (statistical) limit check values for 
/// HIFN  the selected off-center channels and all active list mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR direction/Denotes the direction.
/// HIPAR frequencyPoint/The relative channel offset.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of limit check results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMultiEvalListModeAllACPOffCenterLimitsCheckResults (ViSession instrSession,
                                                                           ViInt32 instance,
                                                                           ViInt32 numberOfActiveSegments,
                                                                           ViInt32 resultType,
                                                                           ViInt32 direction,
                                                                           ViInt32 frequencyPoint,
                                                                           ViInt32 *reliability,
                                                                           ViInt32 results[],
                                                                           ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MAX), 4, "Result Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, direction, RSCMWEVM_VAL_DIRECTION_ACPP, RSCMWEVM_VAL_DIRECTION_ACPM), 5, "Direction");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:ACP:%s%ld:%s?", instance, 
						resultACPDirectionArr[direction], frequencyPoint, resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns adjacent channel power (statistical) values for the selected 
/// HIFN  off-center channels and all active list mode segments.  
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR direction/Denotes the direction.
/// HIPAR frequencyPoint/The relative channel offset.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns adjacent channel power (statistical) values for the selected 
/// HIPAR results/off-center channels and all active list mode segments.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMultiEvalListModeAllACPOffCenterExtendedResults (ViSession instrSession,
                                                                                 ViInt32 instance,
                                                                                 ViInt32 numberOfActiveSegments,
                                                                                 ViInt32 resultType,
                                                                                 ViInt32 direction,
                                                                                 ViInt32 frequencyPoint,
                                                                                 ViInt32 *reliability,
                                                                                 ViReal64 results[],
                                                                                 ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MAX), 4, "Result Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, direction, RSCMWEVM_VAL_DIRECTION_ACPP, RSCMWEVM_VAL_DIRECTION_ACPM), 5, "Direction");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:ACP:EXT:%s%ld:%s?", instance, 
						resultACPDirectionArr[direction], frequencyPoint, resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}


/// HIFN  Returns adjacent channel power (statistical) limit check values for 
/// HIFN  the selected off-center channels and all active list mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR direction/Denotes the direction.
/// HIPAR frequencyPoint/The relative channel offset.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of limit check results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMultiEvalListModeAllACPOffCenterLimitsCheckExtendedResults (ViSession instrSession,
                                                                                            ViInt32 instance,
                                                                                            ViInt32 numberOfActiveSegments,
                                                                                            ViInt32 resultType,
                                                                                            ViInt32 direction,
                                                                                            ViInt32 frequencyPoint,
                                                                                            ViInt32 *reliability,
                                                                                            ViInt32 results[],
                                                                                            ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MAX), 4, "Result Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, direction, RSCMWEVM_VAL_DIRECTION_ACPP, RSCMWEVM_VAL_DIRECTION_ACPM), 5, "Direction");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:ACP:EXT:%s%ld:%s?", instance, 
						resultACPDirectionArr[direction], frequencyPoint, resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns the out of tolerance percentages in the adjacent channel power 
/// HIFN  measurement for all active list mode segments.    
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/The percentage of measurement intervals of the statistic count
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMultiEvalListModeAllACPOutOfToleranceResults (ViSession instrSession,
                                                                     ViInt32 instance,
                                                                     ViInt32 numberOfActiveSegments,
                                                                     ViInt32 *reliability,
                                                                     ViReal64 results[],
                                                                     ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Results")
;   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:ACP:OTOL?", instance);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns the statistic count of the spectrum measurement for all active 
/// HIFN  list mode segments.    
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Number of evaluated valid slots.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMultiEvalListModeAllACPStatisticCountResults (ViSession instrSession,
                                                                     ViInt32 instance,
                                                                     ViInt32 numberOfActiveSegments,
                                                                     ViInt32 *reliability,
                                                                     ViReal64 results[],
                                                                     ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Results")
;   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:ACP:STC?", instance);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns the state of a particular reverse link channel  in the channel 
/// HIFN  power measurement for all active segments.      
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR channelType/Selects the reverse channel type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns the state of a particular reverse link channel.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeAllSpecificChannelPowerState (ViSession instrSession,
                                                                      ViInt32 instance,
                                                                      ViInt32 numberOfActiveSegments,
                                                                      ViInt32 channelType,
                                                                      ViInt32 *reliability,
                                                                      ViInt32 results[],
                                                                      ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, channelType, RSCMWEVM_VAL_RESULT_CP_CHANNEL_RRI, RSCMWEVM_VAL_RESULT_CP_CHANNEL_DATA), 4, "Channel Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:CP:%s:STAT?", instance, resultChannelTypeArr[channelType]);
    checkErr (rscmwevm_GetMeasTraceState (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns Channel Power (CP) results of specified channel for all 
/// HIFN  segments in list mode.     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR channelType/Selects the reverse channel type.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns the RMS Channel Power values for specified channel.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeAllSpecificChannelPowerResults (ViSession instrSession,
                                                                       ViInt32 instance,
                                                                       ViInt32 numberOfActiveSegments,
                                                                       ViInt32 channelType,
                                                                       ViInt32 resultType,
                                                                       ViInt32 *reliability,
                                                                       ViReal64 results[],
                                                                       ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, channelType, RSCMWEVM_VAL_RESULT_CP_CHANNEL_RRI, RSCMWEVM_VAL_RESULT_CP_CHANNEL_DATA), 4, "Channel Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 5, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:CP:%s:%s?", instance, 
						resultChannelTypeArr[channelType], resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns Channel Power (CP) limit check results of specified channel for all 
/// HIFN  segments in list mode.     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR channelType/Selects the reverse channel type.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns limits check results values of the Channel Power (CP) results 
/// HIPAR results/for all segments in list mode of the multi evaluation measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeAllSpecificChannelPowerLimitCheckResults (ViSession instrSession,
                                                                                 ViInt32 instance,
                                                                                 ViInt32 numberOfActiveSegments,
                                                                                 ViInt32 channelType,
                                                                                 ViInt32 resultType,
                                                                                 ViInt32 *reliability,
                                                                                 ViInt32 results[],
                                                                                 ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, channelType, RSCMWEVM_VAL_RESULT_CP_CHANNEL_RRI, RSCMWEVM_VAL_RESULT_CP_CHANNEL_DATA), 4, "Channel Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 5, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:CP:%s:%s?", instance, 
						resultChannelTypeArr[channelType], resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns lower or upper OBW frequencies for all active list mode 
/// HIFN  segments. The values are taken from the "Current" measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR frequency/Selects lower or upper frequency.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returned lower or upper OBW frequencies.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMultiEvalListModeAllOBWFrequenciesResults (ViSession instrSession,
                                                                  ViInt32 instance,
                                                                  ViInt32 numberOfActiveSegments,
                                                                  ViInt32 frequency,
                                                                  ViInt32 *reliability,
                                                                  ViReal64 results[],
                                                                  ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, frequency, RSCMWEVM_VAL_RESULT_LOW, RSCMWEVM_VAL_RESULT_UPP), 4, "Frequency");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:OBW:FREQ:%s?", instance, resultOBWFreqArr[frequency]);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns the occupied bandwidth (statistical) values for all active 
/// HIFN  list mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns the occupied bandwidth (statistical) values for all active 
/// HIPAR results/list mode segments.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMultiEvalListModeAllOBWSpecificResults (ViSession instrSession,
                                                               ViInt32 instance,
                                                               ViInt32 numberOfActiveSegments,
                                                               ViInt32 resultType,
                                                               ViInt32 *reliability,
                                                               ViReal64 results[],
                                                               ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:OBW:FREQ:%s?", instance, resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns the occupied bandwidth (statistical) limit check values for 
/// HIFN  all active list mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of limit check results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMultiEvalListModeAllOBWSpecificLimitsCheckResults (ViSession instrSession,
                                                                          ViInt32 instance,
                                                                          ViInt32 numberOfActiveSegments,
                                                                          ViInt32 resultType,
                                                                          ViInt32 *reliability,
                                                                          ViInt32 results[],
                                                                          ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:OBW:FREQ:%s?", instance, resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns error vector magnitude values for all measured list mode 
/// HIFN  segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR type/Denotes the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of error vector magnitude values.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMultiEvalListModeAllModulationEVMResults (ViSession instrSession,
                                                                 ViInt32 instance,
                                                                 ViInt32 numberOfActiveSegments,
                                                                 ViInt32 resultType,
                                                                 ViInt32 type,
                                                                 ViInt32 *reliability,
                                                                 ViReal64 results[],
                                                                 ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, type, RSCMWEVM_VAL_TYPE_RMS, RSCMWEVM_VAL_TYPE_PEAK), 5, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:MOD:EVM:%s:%s?", instance, 
									resultModulationTypeArr[type], resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns error vector magnitude limit check results for all measured 
/// HIFN  list mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR type/Denotes the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of limit check results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMultiEvalListModeAllModulationEVMLimitsCheckResults (ViSession instrSession,
                                                                            ViInt32 instance,
                                                                            ViInt32 numberOfActiveSegments,
                                                                            ViInt32 resultType,
                                                                            ViInt32 type,
                                                                            ViInt32 *reliability,
                                                                            ViInt32 results[],
                                                                            ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, type, RSCMWEVM_VAL_TYPE_RMS, RSCMWEVM_VAL_TYPE_PEAK), 5, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:MOD:EVM:%s:%s?", instance, 
									resultModulationTypeArr[type], resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns magnitude error values for all measured list mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR type/Denotes the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of magnitude error values.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMultiEvalListModeAllModulationMagnitudeErrorResults (ViSession instrSession,
                                                                            ViInt32 instance,
                                                                            ViInt32 numberOfActiveSegments,
                                                                            ViInt32 resultType,
                                                                            ViInt32 type,
                                                                            ViInt32 *reliability,
                                                                            ViReal64 results[],
                                                                            ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr (Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, type, RSCMWEVM_VAL_TYPE_RMS, RSCMWEVM_VAL_TYPE_PEAK), 5, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:MOD:MERR:%s:%s?", instance, 
									resultModulationTypeArr[type], resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns magnitude error limit check results for all measured list mode 
/// HIFN  segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR type/Denotes the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of limit check results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMultiEvalListModeAllModulationMagnitudeErrorLimitsCheckResults (ViSession instrSession,
                                                                                       ViInt32 instance,
                                                                                       ViInt32 numberOfActiveSegments,
                                                                                       ViInt32 resultType,
                                                                                       ViInt32 type,
                                                                                       ViInt32 *reliability,
                                                                                       ViInt32 results[],
                                                                                       ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, type, RSCMWEVM_VAL_TYPE_RMS, RSCMWEVM_VAL_TYPE_PEAK), 5, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:MOD:MERR:%s:%s?", instance, 
									resultModulationTypeArr[type], resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns phase error values for all measured list mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR type/Denotes the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of phase error values.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMultiEvalListModeAllModulationPhaseErrorResults (ViSession instrSession,
                                                                        ViInt32 instance,
                                                                        ViInt32 numberOfActiveSegments,
                                                                        ViInt32 resultType,
                                                                        ViInt32 type,
                                                                        ViInt32 *reliability,
                                                                        ViReal64 results[],
                                                                        ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, type, RSCMWEVM_VAL_TYPE_RMS, RSCMWEVM_VAL_TYPE_PEAK), 5, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:MOD:PERR:%s:%s?", instance, 
									resultModulationTypeArr[type], resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns phase error limit check results for all measured list mode 
/// HIFN  segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR type/Denotes the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of limit check results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMultiEvalListModeAllModulationPhaseErrorLimitsCheckResults (ViSession instrSession,
                                                                                   ViInt32 instance,
                                                                                   ViInt32 numberOfActiveSegments,
                                                                                   ViInt32 resultType,
                                                                                   ViInt32 type,
                                                                                   ViInt32 *reliability,
                                                                                   ViInt32 results[],
                                                                                   ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, type, RSCMWEVM_VAL_TYPE_RMS, RSCMWEVM_VAL_TYPE_PEAK), 5, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:MOD:PERR:%s:%s?", instance, 
									resultModulationTypeArr[type], resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}
/// HIFN  Returns IQ origin offset for all measured list mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of I/Q origin offset values.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMultiEvalListModeAllModulationIQOriginOffsetResults (ViSession instrSession,
                                                                            ViInt32 instance,
                                                                            ViInt32 numberOfActiveSegments,
                                                                            ViInt32 resultType,
                                                                            ViInt32 *reliability,
                                                                            ViReal64 results[],
                                                                            ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:MOD:IQOF:%s?", instance, resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns IQ origin limit check results for all measured list mode 
/// HIFN  segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of limit check results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMultiEvalListModeAllModulationIQOriginOffsetLimitsCheckResults (ViSession instrSession,
                                                                                      ViInt32 instance,
                                                                                      ViInt32 numberOfActiveSegments,
                                                                                      ViInt32 resultType,
                                                                                      ViInt32 *reliability,
                                                                                      ViInt32 results[],
                                                                                      ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:MOD:IQOF:%s?", instance, resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns IQ Imbalance for all measured list mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of IQ Imbalance values.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMultiEvalListModeAllModulationIQImbalanceResults (ViSession instrSession,
                                                                         ViInt32 instance,
                                                                         ViInt32 numberOfActiveSegments,
                                                                         ViInt32 resultType,
                                                                         ViInt32 *reliability,
                                                                         ViReal64 results[],
                                                                         ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:MOD:IQIM:%s?", instance, resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns IQ imbalance limit check results for all measured list mode 
/// HIFN  segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of limit check results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMultiEvalListModeAllModulationIQImbalanceLimitsCheckResults (ViSession instrSession,
                                                                                    ViInt32 instance,
                                                                                    ViInt32 numberOfActiveSegments,
                                                                                    ViInt32 resultType,
                                                                                    ViInt32 *reliability,
                                                                                    ViInt32 results[],
                                                                                    ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:MOD:IQIM:%s?", instance, resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns  Carrier Frequency Error for all measured list mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of Carrier Frequency Error values.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMultiEvalListModeAllModulationCFEResults (ViSession instrSession,
                                                                 ViInt32 instance,
                                                                 ViInt32 numberOfActiveSegments,
                                                                 ViInt32 resultType,
                                                                 ViInt32 *reliability,
                                                                 ViReal64 results[],
                                                                 ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:MOD:FERR:%s?", instance, resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns carrier frequency error limit check results for all measured 
/// HIFN  list mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of limit check results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMultiEvalListModeAllModulationCFELimitsCheckResults (ViSession instrSession,
                                                                            ViInt32 instance,
                                                                            ViInt32 numberOfActiveSegments,
                                                                            ViInt32 resultType,
                                                                            ViInt32 *reliability,
                                                                            ViInt32 results[],
                                                                            ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:MOD:FERR:%s?", instance, resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns  transmit time error for all measured list mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of transmit time error values.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMultiEvalListModeAllModulationTTEResults (ViSession instrSession,
                                                                 ViInt32 instance,
                                                                 ViInt32 numberOfActiveSegments,
                                                                 ViInt32 resultType,
                                                                 ViInt32 *reliability,
                                                                 ViReal64 results[],
                                                                 ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:MOD:TERR:%s?", instance, resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns transmit time error limit check results for all measured list 
/// HIFN  mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of limit check results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMultiEvalListModeAllModulationTTELimitsCheckResults (ViSession instrSession,
                                                                            ViInt32 instance,
                                                                            ViInt32 numberOfActiveSegments,
                                                                            ViInt32 resultType,
                                                                            ViInt32 *reliability,
                                                                            ViInt32 results[],
                                                                            ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:MOD:TERR:%s?", instance, resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns MS power values for all measured list mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR type/Denotes the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of MS power values.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMultiEvalListModeAllModulationMSPowerResults (ViSession instrSession,
                                                                     ViInt32 instance,
                                                                     ViInt32 numberOfActiveSegments,
                                                                     ViInt32 resultType,
                                                                     ViInt32 type,
                                                                     ViInt32 *reliability,
                                                                     ViReal64 results[],
                                                                     ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, type, RSCMWEVM_VAL_TYPE_MS_POWER_NARROW, RSCMWEVM_VAL_TYPE_MS_POWER_WIDE), 5, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:MOD:%s:%s?", instance, 
						resultBandArr[type], resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns MS Power limit check results for all measured list mode 
/// HIFN  segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR type/Denotes the result type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of limit check results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMultiEvalListModeAllModulationMSPowerLimitsCheckResults (ViSession instrSession,
                                                                                ViInt32 instance,
                                                                                ViInt32 numberOfActiveSegments,
                                                                                ViInt32 resultType,
                                                                                ViInt32 type,
                                                                                ViInt32 *reliability,
                                                                                ViInt32 results[],
                                                                                ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_SDEV), 4, "Result Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, type, RSCMWEVM_VAL_TYPE_MS_POWER_NARROW, RSCMWEVM_VAL_TYPE_MS_POWER_WIDE), 5, "Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:MOD:%s:%s?", instance, 
						resultBandArr[type], resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns  waveform quality for all measured list mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of waveform quality values.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMultiEvalListModeAllModulationWaveformQualityResults (ViSession instrSession,
                                                                             ViInt32 instance,
                                                                             ViInt32 numberOfActiveSegments,
                                                                             ViInt32 resultType,
                                                                             ViInt32 *reliability,
                                                                             ViReal64 results[],
                                                                             ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MS_POW_MIN), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:MOD:WQU:%s?", instance, resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns waveform quality limit check results for all measured list 
/// HIFN  mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Array of limit check results.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMultiEvalListModeAllModulationWaveformQualityLimitsCheckResults (ViSession instrSession,
                                                                                        ViInt32 instance,
                                                                                        ViInt32 numberOfActiveSegments,
                                                                                        ViInt32 resultType,
                                                                                        ViInt32 *reliability,
                                                                                        ViInt32 results[],
                                                                                        ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MS_POW_MIN), 4, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:MOD:WQU:%s?", instance, resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns the out of tolerance percentages in the modulation measurement 
/// HIFN  for all active list mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/The percentage of measurement intervals of the statistic count
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMultiEvalListModeAllModulationOutOfToleranceResults (ViSession instrSession,
                                                                            ViInt32 instance,
                                                                            ViInt32 numberOfActiveSegments,
                                                                            ViInt32 *reliability,
                                                                            ViReal64 results[],
                                                                            ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Results")
;   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:MOD:OTOL?", instance);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}
/// HIFN  Returns the statistic count in the modulation measurement for all 
/// HIFN  active list mode segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Number of evaluated valid slots.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMultiEvalListModeAllModulationStatisticCountResults (ViSession instrSession,
                                                                            ViInt32 instance,
                                                                            ViInt32 numberOfActiveSegments,
                                                                            ViInt32 *reliability,
                                                                            ViReal64 results[],
                                                                            ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Results")
;   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:MOD:STC?", instance);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns the scalar channel power for the Data channel results for all 
/// HIFN  active list mode segments.     
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR dataType/Selects the data type. W1^2 means Walsh code One Two and W2^4 means 
/// HIPAR dataType/Walsh code Two Four.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns the scalar channel power for the Data channel results for all 
/// HIPAR results/active list mode segments.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchMEvalMeasListModeAllDWCPowerSpecificResults (ViSession instrSession,
                                                                    ViInt32 instance,
                                                                    ViInt32 numberOfActiveSegments,
                                                                    ViInt32 dataType,
                                                                    ViInt32 resultType,
                                                                    ViInt32 *reliability,
                                                                    ViReal64 results[],
                                                                    ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, dataType, RSCMWEVM_VAL_RESULT_DWC_WOTI, RSCMWEVM_VAL_RESULT_DWC_WTFQ), 4, "Data Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 5, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:MEV:LIST:DWCP:%s:%s?", instance, 
						resultDWCTypeArr[dataType], resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns the scalar channel power for the Data channel limits check 
/// HIFN  results for all active list mode segments.       
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfActiveSegments/Number of active segments.
/// HIPAR dataType/Selects the data type. W1^2 means Walsh code One Two and W2^4 means 
/// HIPAR dataType/Walsh code Two Four.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns the scalar channel power for the Data channel limits check 
/// HIPAR results/results for all active list mode segments.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryMEvalMeasListModeAllDWCPowerSpecificLimitsCheckResults (ViSession instrSession,
                                                                               ViInt32 instance,
                                                                               ViInt32 numberOfActiveSegments,
                                                                               ViInt32 dataType,
                                                                               ViInt32 resultType,
                                                                               ViInt32 *reliability,
                                                                               ViInt32 results[],
                                                                               ViInt32 *returnedValues)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";   
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, numberOfActiveSegments, 1, 200), 3, "Number Of Active Segments");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, dataType, RSCMWEVM_VAL_RESULT_DWC_WOTI, RSCMWEVM_VAL_RESULT_DWC_WTFQ), 4, "Data Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, resultType, RSCMWEVM_VAL_RESULT_CURRENT, RSCMWEVM_VAL_RESULT_MIN), 5, "Result Type");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:MEV:LIST:DWCP:%s:%s?", instance, 
						resultDWCTypeArr[dataType], resultTypeArr[resultType]);
    
	checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, numberOfActiveSegments, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Configures the OLTR measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR timeout/Sets a timeout for the measurement.
/// HIPAR numberOfSequences/Sets the number of measurement sequences within a single OLTR 
/// HIPAR numberOfSequences/measurement. Each sequence consists of a power UP or power DOWN step, 
/// HIPAR numberOfSequences/followed by a power step in the opposite direction.
/// HIPAR direction/Defines the direction of the first power step within an OLTR 
/// HIPAR direction/measurement. For each subsequent power step the direction is toggled.
/// HIPAR powerStep/Defines the size of the power steps, i.e. the increases and decreases 
/// HIPAR powerStep/in the total BSS power during the OLTR measurement.
/// HIPAR powerInterval/The duration of the reference power interval as number of power
/// HIPAR guardInterval/The duration of the guard interval as number of power control.
ViStatus _VI_FUNC rscmwevm_ConfigureOLTRMeasurement (ViSession instrSession,
                                            ViInt32 instance, ViReal64 timeout,
                                            ViInt32 numberOfSequences,
                                            ViInt32 direction,
                                            ViReal64 powerStep,
                                            ViInt32 powerInterval,
                                            ViInt32 guardInterval,
											ViBoolean measureOnException)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[50] = "";
    
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm(rscmwevm_SetAttributeViReal64 (instrSession, rep_cap,
        RSCMWEVM_ATTR_OLTR_MEASUREMENT_TIMEOUT, timeout), 3, "Timeout");
    
    viCheckParm(rscmwevm_SetAttributeViInt32 (instrSession, rep_cap,
        RSCMWEVM_ATTR_OLTR_MEASUREMENT_NUMBER_OF_SEQUENCES, numberOfSequences), 4, "Number Of Sequences");
	
	viCheckParm(rscmwevm_SetAttributeViInt32 (instrSession, rep_cap,
        RSCMWEVM_ATTR_OLTR_MEASUREMENT_POWER_STEP_DIRECTION, direction), 5, "Direction");
    
    viCheckParm(rscmwevm_SetAttributeViReal64 (instrSession, rep_cap,
        RSCMWEVM_ATTR_OLTR_MEASUREMENT_POWER_STEP, powerStep), 6, "Power Step");
	
	viCheckParm(rscmwevm_SetAttributeViInt32 (instrSession, rep_cap,
        RSCMWEVM_ATTR_OLTR_MEASUREMENT_REF_POWER_INTERVAL, powerInterval), 7, "Power Interval");
    
    viCheckParm(rscmwevm_SetAttributeViInt32 (instrSession, rep_cap,
        RSCMWEVM_ATTR_OLTR_MEASUREMENT_GUARD_INTERVAL, guardInterval), 8, "Guard Interval");
	
	viCheckParm(rscmwevm_SetAttributeViBoolean (instrSession, rep_cap,
        RSCMWEVM_ATTR_OLTR_MEASUREMENT_RESULTS_REJECTION, measureOnException), 8, "Measure On Exception");
    
    checkErr( rscmwevm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/// HIFN  Queries the duration of the reference power interval time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR powerIntervalTime/Gets the duration of the reference power time,
ViStatus _VI_FUNC rscmwevm_QueryOLTRMeasurementReferencePowerIntervalTime (ViSession instrSession,
                                                            ViInt32 instance,
                                                            ViReal64 *powerIntervalTime)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[50] = "";
    
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm(rscmwevm_GetAttributeViReal64 (instrSession, rep_cap,
        RSCMWEVM_ATTR_OLTR_MEASUREMENT_REF_POWER_INTERVAL_TIME, powerIntervalTime), 3, "Power Interval Timet");
    
    checkErr( rscmwevm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/// HIFN  Gets the duration of the guard intervals.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSCMWEVM_ATTR_OLTR_MEASUREMENT_GUARD_INTERVAL_TIME
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:EVDO:MEAS<instance>:OLTR:GINTerval:TIME?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR guardIntervalTime/Gets the duration of the guard intervals, i.e. the intervals 
/// HIPAR guardIntervalTime/succeeding the OLTR evaluation intervals and preceding the reference 
/// HIPAR guardIntervalTime/power intervals.
ViStatus _VI_FUNC rscmwevm_QueryOLTRMeasurementGuardIntervalTime(
	ViSession	instrSession,
	ViInt32	instance,
	ViReal64*	guardIntervalTime
)
{
    ViStatus	error = VI_SUCCESS;
    ViChar	rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));
    
    viCheckParm (rscmwevm_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVM_ATTR_OLTR_MEASUREMENT_GUARD_INTERVAL_TIME,
                                                guardIntervalTime), 3, "Guard Interval Time");
    
Error:
    Rs_UnlockSession(instrSession, 0);    
    return error;
}

/// HIFN  Returns the repetition mode of the measurement. The repetition mode 
/// HIFN  specifies whether the measurement is stopped after a single-shot or 
/// HIFN  repeated continuously.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR repetition/Returns the repetition mode of the measurement. The repetition mode 
/// HIPAR repetition/specifies whether the measurement is stopped after a single-shot or 
/// HIPAR repetition/repeated continuously.
ViStatus _VI_FUNC rscmwevm_QueryOLTRMeasurementRepetition (ViSession instrSession,
                                                  ViInt32 instance,
                                                  ViInt32 *repetition)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[50] = "";
    
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm(rscmwevm_GetAttributeViInt32 (instrSession, rep_cap,
        RSCMWEVM_ATTR_OLTR_MEASUREMENT_REPETITION, repetition), 3, "Repetition");
    
    checkErr( rscmwevm_CheckStatus (instrSession));
Error:
    Rs_UnlockSession(instrSession, 0);
    
    return error;
}

/// HIFN  Sets initial lower limit for open loop power control step response 
/// HIFN  (3GPP2 C.S0033).
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSCMWEVM_ATTR_OLTR_MEASUREMENT_LIMITS_ILOWER
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:EVDO:MEASurement<instance>:OLTR:LIMit:ILOWer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwc2m_init or 
/// HIPAR instrSession/rscmwc2m_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR initialLowerLimit/Sets the lower limit for the limit violation calculation.
ViStatus _VI_FUNC rscmwevm_ConfigureOLTRInitialLowerLimit(
	ViSession	instrSession,
	ViInt32	instance,
	ViReal64	initialLowerLimit
)
{
    ViStatus	error = VI_SUCCESS;
    ViChar	rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, 0));

    checkErr(rscmwevm_instanceRepCap (instrSession, instance, rep_cap));
    
    viCheckParm (rscmwevm_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVM_ATTR_OLTR_MEASUREMENT_LIMITS_ILOWER,
                                                initialLowerLimit), 3, "Initial Lower Limit");
    
Error:
    Rs_UnlockSession(instrSession, 0);    
    return error;
}

/// HIFN  Starts or restarts the measurement; the R&S CMW 500 enters the "RUN" 
/// HIFN  state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
ViStatus _VI_FUNC rscmwevm_OLTRMeasurementInit (ViSession instrSession,
                                       ViInt32 instance)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr (rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

   viCheckParm (rscmwevm_SetAttributeViString(instrSession, rep_cap,
        RSCMWEVM_ATTR_OLTR_MEASUREMENT_INIT, NULL), 3, "OLTR Measurement Init");

Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Causes a running measurement to stop immediately; the R&S CMW 500 
/// HIFN  enters the "OFF" state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
ViStatus _VI_FUNC rscmwevm_OLTRMeasurementAbort (ViSession instrSession,
                                        ViInt32 instance)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr (rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

   viCheckParm (rscmwevm_SetAttributeViString(instrSession, rep_cap,
        RSCMWEVM_ATTR_OLTR_MEASUREMENT_ABORT, NULL), 3, "OLTR Measurement Abort");

Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Causes a running measurement to stop after the current evaluation 
/// HIFN  period is terminated and valid results are available; the R&S CMW 500 
/// HIFN  enters the "RDY" state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
ViStatus _VI_FUNC rscmwevm_OLTRMeasurementStop (ViSession instrSession,
                                       ViInt32 instance)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr (rscmwevm_instanceRepCap (instrSession, instance, rep_cap));

   viCheckParm (rscmwevm_SetAttributeViString(instrSession, rep_cap,
        RSCMWEVM_ATTR_OLTR_MEASUREMENT_STOP, NULL), 3, "OLTR Measurement Stop");

Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Queries the main measurement state and the measurement substates. Both 
/// HIFN  measurement substates are relevant for running measurements only. Use 
/// HIFN  FETCh...MEASurement<instance>:STATe? to query the main measurement 
/// HIFN  state only. Use INITiate..., STOP..., ABORt... to change the 
/// HIFN  measurement state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR mainState/Returns main state.
/// HIPAR syncState/Returns synchronization state.
/// HIPAR resourcesState/Returns resources state.
ViStatus _VI_FUNC rscmwevm_QueryOLTRMeasurementStatus (ViSession instrSession,
                                              ViInt32 instance,
                                              ViInt32 *mainState,
                                              ViInt32 *syncState,
                                              ViInt32 *resourcesState)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
   
    sprintf (command, "FETC:EVDO:MEAS%ld:OLTR:STAT:ALL?", instance);

    checkErr(rscmwevm_QueryMeasStatus (instrSession, command, mainState, syncState,
        resourcesState));

    checkErr (rscmwevm_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Starts a measurement and returns limit violation results "State
/// HIFN Down Power"/"State Up Power" in 20 equidistant parts of the 100
/// HIFN ms interval following the power down/up step.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR sequence/Sets the number of the sequence.
/// HIPAR direction/Defines the step direction.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement result; see Reliability Indicator.
/// HIPAR results/Returns limit violation results "State Down Power"/"State Up Power".
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_ReadOLTRTraceState (ViSession instrSession,
                                      ViInt32 instance, ViInt32 sequence,
                                      ViInt32 direction, ViInt32 arraySize,
                                      ViInt32 *reliability, ViInt32 results[],
                                      ViInt32 *returnedValues)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, sequence, 1, 5), 3, "Sequence");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, direction, RSCMWEVM_VAL_OLTR_DOWN, RSCMWEVM_VAL_OLTR_UP), 4, "Direction");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "READ:EVDO:MEAS%ld:OLTR:SEQ%ld:TRACe:%s:STAT?", instance, sequence, DirectionArr[direction]);
    checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns limit violation results "State Down Power"/"State Up Power" in 20 equidistant
/// HIFN  parts of the 100 ms interval following the power down/up step.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR sequence/Sets the number of the sequence.
/// HIPAR direction/Defines the step direction.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement result; see Reliability Indicator.
/// HIPAR results/Returns limit violation results "State Down Power"/"State Up Power".
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchOLTRTraceState (ViSession instrSession,
                                       ViInt32 instance, ViInt32 sequence,
                                       ViInt32 direction, ViInt32 arraySize,
                                       ViInt32 *reliability, ViInt32 results[],
                                       ViInt32 *returnedValues)

{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, sequence, 1, 5), 3, "Sequence");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, direction, RSCMWEVM_VAL_OLTR_DOWN, RSCMWEVM_VAL_OLTR_UP), 4, "Direction");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:OLTR:SEQ%ld:TRACe:%s:STAT?", instance, sequence, DirectionArr[direction]);
    checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error; 
}

/// HIFN  Returns the limits check for values of the limit violation results 
/// HIFN  "State Down Power"/"State Up Power" in 20 equidistant parts of the 100 
/// HIFN  ms interval following the power down/up step.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     
/// HIFN  CALCulate:EVDO:MEAS<instance>:OLTR:SEQuence<Sequence>:TRACe:DOWN:STATe?
/// HIFN     CALCulate:EVDO:MEAS<instance>:OLTR:SEQuence<Sequence>:TRACe:UP:STATe?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwc2m_init or 
/// HIPAR instrSession/rscmwc2m_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR sequence/The number of the sequence whose trace shall be queried; must
/// HIPAR direction/Defines the step direction.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns the limits check results of the limit violation results "State 
/// HIPAR results/Down Power"/"State Up Power" in 20 equidistant parts of the 100 ms 
/// HIPAR results/interval following the power down/up step.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_QueryOLTRTraceStateLimitsCheckResults(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32	sequence,
	ViInt32	direction,
	ViInt32	arraySize,
	ViInt32*	reliability,
	ViInt32	results[],
	ViInt32*	returnedValues
)
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, sequence, 1, 5), 3, "Sequence");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, direction, RSCMWEVM_VAL_OLTR_DOWN, RSCMWEVM_VAL_OLTR_UP), 4, "Direction");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 7, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:MEAS%ld:OLTR:SEQ%ld:TRACe:%s:STAT?", instance, sequence, DirectionArr[direction]);
    checkErr (rscmwevm_GetMeasTraceLimits (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, 0);

    return error; 
}

/// HIFN  Starts a measurement and returns the values of the OLTR and reference 
/// HIFN  power traces. For each sequence, UP/DOWN commands return the results of 
/// HIFN  the 100 ms interval following the power up/down step.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR traceType/Denotes the result trace type.
/// HIPAR sequence/The number of the sequence whose trace shall be queried; must
/// HIPAR direction/Defines the step direction.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns the resulting power values of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_ReadOLTRTrace (ViSession instrSession, ViInt32 instance,
                                 ViInt32 traceType, ViInt32 sequence,
                                 ViInt32 direction, ViInt32 arraySize,
                                 ViInt32 *reliability, ViReal64 results[],
                                 ViInt32 *returnedValues)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, traceType, RSCMWEVM_VAL_OLTR_TRACE_TYPE_OLTR, RSCMWEVM_VAL_OLTR_TRACE_TYPE_RPOW), 3, "Trace Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, sequence, 1, 5), 4, "Sequence");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, direction, RSCMWEVM_VAL_OLTR_DOWN, RSCMWEVM_VAL_OLTR_UP), 5, "Direction");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
    sprintf (command, "READ:EVDO:MEAS%ld:%s:SEQ%ld:TRAC:%s?", instance, resultOLTRTraceArr[traceType], sequence, resultOLTRDirectionArr[direction]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
    checkErr( rscmwevm_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/// HIFN  Returns the values of the OLTR and reference power traces. For each 
/// HIFN  sequence, UP/DOWN commands return the results of the 100 ms interval 
/// HIFN  following the power up/down step.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR traceType/Denotes the result trace type.
/// HIPAR sequence/The number of the sequence whose trace shall be queried; must
/// HIPAR direction/Defines the step direction.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR results/Returns the resulting power values of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwevm_FetchOLTRTrace (ViSession instrSession, ViInt32 instance,
                                  ViInt32 traceType, ViInt32 sequence,
                                  ViInt32 direction, ViInt32 arraySize,
                                  ViInt32 *reliability, ViReal64 results[],
                                  ViInt32 *returnedValues)
{
	ViStatus    error   = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, traceType, RSCMWEVM_VAL_OLTR_TRACE_TYPE_OLTR, RSCMWEVM_VAL_OLTR_TRACE_TYPE_RPOW), 3, "Trace Type");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, sequence, 1, 5), 4, "Sequence");
	viCheckParm (rscmwevm_invalidViInt32Range (instrSession, direction, RSCMWEVM_VAL_OLTR_DOWN, RSCMWEVM_VAL_OLTR_UP), 5, "Direction");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 8, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:MEAS%ld:%s:SEQ%ld:TRAC:%s?", instance, resultOLTRTraceArr[traceType], sequence, resultOLTRDirectionArr[direction]);
    checkErr (rscmwevm_GetMeasTrace (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
    checkErr( rscmwevm_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: rscmwevm_SetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to set
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/
/* ViInt32 */
ViStatus _VI_FUNC rscmwevm_SetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViReal64 */
ViStatus _VI_FUNC rscmwevm_SetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 value)
{
     return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViString */
ViStatus _VI_FUNC rscmwevm_SetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViString value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rscmwevm_SetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViSession */
ViStatus _VI_FUNC rscmwevm_SetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/*****************************************************************************
 * Function: rscmwevm_GetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to get
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rscmwevm_GetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt32), (ViInt32 *)value);
}

/* ViReal64 */
ViStatus _VI_FUNC rscmwevm_GetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViReal64), (ViReal64 *)value);
}

/* ViString */
ViStatus _VI_FUNC rscmwevm_GetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViInt32 bufSize,
                                                 ViChar value[])
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            bufSize, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rscmwevm_GetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViBoolean), (ViBoolean *)value);
}

/* ViSession */
ViStatus _VI_FUNC rscmwevm_GetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViSession), (ViSession *)value);
}


/*****************************************************************************
 * Function: Rs_GetAttributeRepCapNameId
 * Purpose:  This function returns the repeated capability id(s) that
 *           belongs to the attribute you specify. More then one id is listed
 *           in comma separated string.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_GetAttributeRepeatedCapabilityIds (ViSession instrSession,
                                                            ViUInt32 attributeID,
                                                            ViInt32 bufferSize,
                                                            ViChar _VI_FAR repeatedCapabilityIds[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapNameId[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapNameId (instrSession, attributeID, repCapNameId));

    strncpy (repeatedCapabilityIds, repCapNameId, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIds[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/*****************************************************************************
 * Function: Get Attribute Repeated Capability Id Name(s)
 * Purpose:  This function returns the repeated capability name(s) that
 *           belongs to the attribute and single repeated capability id you
 *           may specify. More then one name is listed in comma separated
 *           string.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_GetAttributeRepeatedCapabilityIdNames (ViSession instrSession,
                                                                ViUInt32 attributeID,
                                                                ViString repeatedCapabilityId,
                                                                ViInt32 bufferSize,
                                                                ViChar _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapName[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapName (instrSession,
                                           attributeID,
                                           repeatedCapabilityId,
                                           repCapName));

    strncpy (repeatedCapabilityIdNames, repCapName, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIdNames[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/*****************************************************************************
 * Function: rscmwevm_CheckAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to check
 *           attribute values directly.  These functions check the value you
 *           specify even if you set the rscmwevm_ATTR_RANGE_CHECK
 *           attribute to VI_FALSE.  There are typesafe versions for ViInt32,
 *           ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rscmwevm_CheckAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViInt32 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViReal64 */
ViStatus _VI_FUNC rscmwevm_CheckAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViReal64 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViString */
ViStatus _VI_FUNC rscmwevm_CheckAttributeViString (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViString value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rscmwevm_CheckAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViBoolean value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViSession */
ViStatus _VI_FUNC rscmwevm_CheckAttributeViSession (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViSession value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/*****************************************************************************
 * Function: Reset
 * Purpose:  This function resets the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_reset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, 0));

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr( viPrintf (instrSession, "*RST\n"));
        }

    checkErr( rscmwevm_DefaultInstrSetup (instrSession));
    checkErr( rscmwevm_CheckStatus (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, 0);

    return error;
}

/*****************************************************************************
 * Function: Self Test
 * Purpose:  This function executes the instrument self-test and returns the
 *           result.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_self_test (ViSession instrSession, ViInt16 *testResult,
                                      ViChar testMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, 0));

    if (testResult == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result");
    if (testMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr( viPrintf (instrSession, "*TST?;*OPC\n"));
        viCheckErr( rscmwevm_WaitForOPC (instrSession, 600000));  
        viCheckErr( viScanf (instrSession, "%hd", testResult));
        if (*testResult == 0)
            strcpy (testMessage, "Self-Test Passed.");
        else
            strcpy (testMessage, "Self-Test Failed.");
        }
    else
        {
        /* Simulate Self Test */
        *testResult = 0;
        strcpy (testMessage, "No error.");
        }

    checkErr( rscmwevm_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, 0);
    return error;
}

/*****************************************************************************
 * Function: Self Test Results
 * Purpose:  This function executes the instrument self-test and returns the
 *           result.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_SelfTestResults (ViSession instrSession,
                                            ViInt32 bufferSize,
                                            ViChar results[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      command[] = "DIAG:SERV:STES:RES?\n";
    
    checkErr( Rs_LockSession (instrSession, 0));

    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (viRead (instrSession, (ViBuf) results, (ViUInt32) bufferSize, NULL));

    checkErr( rscmwevm_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, 0);
    return error;

}

/// HIFN This function switches ON/OFF various error checkings performed inside of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches option checking On or Off.
/// HIPAR rangeChecking/This control enables or disables range checking.
/// HIPAR statusChecking/This control enables or disables instrument state checking
ViStatus _VI_FUNC rscmwevm_ConfigureErrorChecking (ViSession instrSession,
                                                  ViBoolean optionChecking,
                                                  ViBoolean rangeChecking,
                                                  ViBoolean statusChecking)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     p2value         = 0;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);

    sessionProperties -> optionChecking = optionChecking;
    p2value = (ViInt32) statusChecking;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, &p2value));
    p2value = (ViInt32) rangeChecking;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_RANGE_CHECK, 0, &p2value));
	
Error:
    return error;
}

/// HIFN  This function sets the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR opcTimeout/This control sets the timeout value.
ViStatus _VI_FUNC rscmwevm_SetOPCTimeout (ViSession instrSession,
                                         ViUInt32 opcTimeout)
{
    ViStatus    error = VI_SUCCESS; 
    
    checkErr( Rs_SetAttribute (instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, &opcTimeout));
    
Error:    
    return error;
}

/// HIFN  This function returns the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/This control returns the timeout value.
ViStatus _VI_FUNC rscmwevm_GetOPCTimeout (ViSession instrSession,
                                         ViUInt32 *timeout)
{
    ViStatus    error = VI_SUCCESS; 
    
    checkErr( Rs_GetAttribute (instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, (unsigned int) sizeof(timeout), timeout));
    
Error:    
    return error;
}

/// HIFN This function specifies the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIFN Note that the actual timeout used may be higher than the one
/// HIFN requested.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR VISATimeout/Specifies the minimum timeout value to use (in milliseconds)
/// HIPAR VISATimeout/when accessing the device associated with the given session.
/// HIPAR VISATimeout/Note that the actual timeout used may be higher than the one
/// HIPAR VISATimeout/requested.
ViStatus _VI_FUNC rscmwevm_SetVISATimeout (ViSession instrSession,
                                        ViUInt32 VISATimeout)
{
	return viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, VISATimeout);
}

/// HIFN This function queries the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR VISATimeout/Queries the minimum timeout value to use (in milliseconds)
/// HIPAR VISATimeout/when accessing the device associated with the given session.
ViStatus _VI_FUNC rscmwevm_GetVISATimeout (ViSession instrSession,
                                        ViUInt32 *VISATimeout)
{
	return viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, VISATimeout);
}

/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwevm_ClearStatus(
	ViSession	instrSession
)
{
    ViStatus  error = VI_SUCCESS;

	checkErr (Rs_LockSession (instrSession, 0));

	checkErr (viWrite (instrSession, (ViBuf) "*CLS\n", 5, NULL));
	checkErr (viClear(instrSession));
	checkErr (rscmwevm_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, 0);    
    return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of IDQueryResponse buffer
/// HIPAR idQueryResponse/Returns the ID Query response string. The array should consist of at 
/// HIPAR idQueryResponse/least 256 elements.
ViStatus _VI_FUNC rscmwevm_IDQueryResponse (ViSession instrSession,
                                         ViUInt32 bufferSize,
                                         ViChar IDQueryResponse[])
{
	ViStatus	error = VI_SUCCESS;

	checkErr (Rs_LockSession (instrSession, 0));
	
	checkErr (viWrite (instrSession, (ViBuf) "*IDN?\n", 6, NULL));
	checkErr (viRead (instrSession, (ViBuf) IDQueryResponse, bufferSize, NULL));
    checkErr (rscmwevm_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, 0);    
	return error;
}

/// HIFN  Stops further command processing until all commands sent before *WAI 
/// HIFN  have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwevm_ProcessAllPreviousCommands(
	ViSession	instrSession
)
{
    ViStatus  error = VI_SUCCESS;

	checkErr (Rs_LockSession (instrSession, 0));

    viCheckParm(rscmwevm_SetAttributeViString(instrSession, "",
        RSCMWEVM_ATTR_PROCESS_ALL_PREVIOUS_COMMANDS, NULL), 2, "Process All Previous Commands");
Error:
    Rs_UnlockSession(instrSession, 0);    
    return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevm_init or 
/// HIPAR instrSession/rscmwevm_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR opc/Queries the OPC.
ViStatus _VI_FUNC rscmwevm_QueryOPC(
	ViSession	instrSession,
	ViInt32*	opc
)
{
    ViStatus  error = VI_SUCCESS;
	ViChar      cmd[100] = "*OPC?\n";

	checkErr (Rs_LockSession (instrSession, 0));

	viCheckErr (viWrite (instrSession, (ViBuf) cmd, (ViUInt32) strlen (cmd), NULL));
	viCheckErr (viRead (instrSession, (ViBuf) cmd, 100UL, NULL));
	sscanf (cmd, "%ld", opc);
Error:
    Rs_UnlockSession(instrSession, 0);    
    return error;
}

/*****************************************************************************
 * Function: Instrument Status Checking                                                 
 * Purpose:  This function switches ON/OFF instrument status checking of the 
 *           instrument 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_setCheckStatus (ViSession instrSession, ViBoolean statusChecking)
{
	ViStatus    error = VI_SUCCESS;
    ViInt32     p2value         = 0;
    
    p2value = (ViInt32) statusChecking;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, &p2value));

Error:
    return error;
}
/*****************************************************************************
 * Function: Option Checking                                                 
 * Purpose:  This function switches ON/OFF option checking of the instrument 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_setCheckOption (ViSession instrSession,
                                  ViBoolean optionCheckingState)
{
	ViStatus    error = VI_SUCCESS;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);

    sessionProperties -> optionChecking = optionCheckingState;

    return error;
}

/*****************************************************************************
 * Function: Revision Query
 * Purpose:  This function returns the driver and instrument revisions.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_revision_query (ViSession instrSession, ViChar driverRev[],
                                           ViChar instrRev[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, 0));

    if (driverRev == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Rs_GetAttribute (instrSession, NULL, RSCMWEVM_ATTR_SPECIFIC_DRIVER_REVISION,
                                        0, 256, driverRev));
    checkErr( Rs_GetAttribute (instrSession, "", RSCMWEVM_ATTR_INSTRUMENT_FIRMWARE_REVISION,
                                        0, 256, instrRev));
    checkErr( rscmwevm_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, 0);
    return error;
}

/*****************************************************************************
 * Function:    System Version
 * Purpose:     This function queries the number of the SCPI version, which 
 *              is relevant for the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_SystemVersion (ViSession instrSession,
                                          ViInt32 length, 
                                          ViChar systemVersion[])

{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, 0)); 
    
    viCheckParm( rscmwevm_GetAttributeViString (instrSession, "", 
                                                RSCMWEVM_ATTR_SYSTEM_VERSION, length,
                                                systemVersion),
                                                2, "System Version");
    checkErr( rscmwevm_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, 0); 
    return error;
}

/*****************************************************************************
 * Function: Error Query
 * Purpose:  This function queries the instrument error queue and returns
 *           the result.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_error_query (ViSession instrSession, ViInt32 *errorCode,
                                        ViChar errorMessage[])
{
    ViStatus    error = VI_SUCCESS;
	ViChar      buffer[RSCMWEVM_IO_BUFFER_SIZE] = "";
    ViInt32     oldCode = 0;
    ViChar      oldMessage[256] = "";
    ViChar      *p2buf = NULL;
    ViInt32     i = 0;
	ViUInt32    retCnt = 0;

	p2buf = (char *)malloc (1);
    *p2buf='\0';
        
    do
	{
		checkErr (viWrite (instrSession, (ViBuf) ":SYST:ERR?\n", 11, NULL));
		checkErr (viRead (instrSession, (ViBuf)buffer, RSCMWEVM_IO_BUFFER_SIZE, &retCnt));

	    if  (sscanf (buffer, "%ld,%*[^0-9a-zA-Z]%256[^\"]", &oldCode, oldMessage) != 2)
        {
            return RS_ERROR_UNEXPECTED_RESPONSE;
        }
        else
        {
            if (oldCode != 0)
            {
                *errorCode = oldCode;
                p2buf = (char *)realloc (p2buf, strlen(p2buf) + strlen (oldMessage) + 2);
                strcat (p2buf, oldMessage);
                strcat (p2buf, "\n");
                i++;
            }
            else if (i == 0)
            {
                *errorCode = oldCode;
                strncpy (errorMessage, oldMessage, 1024);
            }
        }

    } while (oldCode != 0);
    
    if (i > 0)
    {
        p2buf[strlen (p2buf) - 1] = '\0'; 
        strncpy (errorMessage, p2buf, 1024);
    }
   
Error:
	if (p2buf) free (p2buf);

    return error;    
}

/*****************************************************************************
 * Function: Error List
 * Purpose:  This function reads all system messages and returns a list of 
 *           comma separated strings.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ErrorList (ViSession instrSession,
                                      ViInt32 bufferSize, 
                                      ViChar errors[])
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, 0));

    if (bufferSize < 0)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Buffer Size");
    if (errors == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Errors");

    viCheckErr (rscmwevm_GetAttributeViString( instrSession, "", RSCMWEVM_ATTR_SYSTEM_ALL_ERRORS, 
        bufferSize, errors));

Error:
    (void) Rs_UnlockSession(instrSession, 0);
    return error;
}

/*****************************************************************************
 * Function: Error Message
 * Purpose:  This function translates the error codes returned by this
 *           instrument driver into user-readable strings.
 *
 *           Note:  The caller can pass NULL for the vi parameter.  This
 *           is useful if one of the init functions fail.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_error_message (ViSession instrSession, ViStatus errorCode,
                                          ViChar errorMessage[])
{
    ViStatus    error   = VI_SUCCESS;

    static      RsStringValueTable errorTable =
        {
        /*=CHANGE:================================================================*
            Insert instrument driver specific error codes here.  Example:

            {rscmwevm_ERROR_TOO_MANY_SAMPLES,   "Sample Count cannot exceed 512."},
         *=============================================================END=CHANGE=*/
            RSCMWEVM_ERROR_CODES_AND_MSGS,
            {0, NULL}
        };

    (void) Rs_LockSession(instrSession, 0);

        /* all VISA and RS error codes are handled as well as codes in the table */
    if (errorMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Rs_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, errorTable));

Error:
    (void) Rs_UnlockSession(instrSession, 0);
    return error;
}

/*****************************************************************************
 * Function: Reliability Indicator
 * Purpose:  This function converts a reliability code returned by an
 *           instrument driver function into a user-readable string.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_ReliabilityIndicator (ViSession instrSession,
                                                 ViInt32 reliabilityCode,
                                                 ViChar reliabilityMessage[])
{
	ViStatus	error = VI_SUCCESS;
	ViInt16     cnt     = 0;
	ViBoolean   found = VI_FALSE;
	
	static      RsStringValueTable reliabilityTable =
	{
		{0,       "OK"},
		{1,       "Measurement Timeout"},
		{2,       "Capture Buffer Overflow"},
		{3,       "Overdriven"},
		{4,       "Underdriven"},
		{6,       "Trigger Timeout"},
		{7,       "Acquisition Error"},
		{8,       "Sync Error"},
		{9,       "Uncal"},
		{15,      "Reference Frequency Error"},
		{16,      "RF Not Available"},
		{17,      "RF Level not Settled"},
		{18,      "RF Frequency not Settled"},
		{19,      "Call not Established"},
		{20,      "Call Type not Usable"},
		{21,      "Call Lost"},
		{23,      "Missing Option"},
		{24,      "Invalid RF Setting"},
		{26,      "Resource Conflict"},
		{27,      "No Sensor Connected"},
		{30,      "File not Found"},
		{40,      "ARB File CRC Error"},
		{42,      "ARB Header Tag Invalid"},
		{43,      "ARB Segment Overflow"},
		{44,      "ARB File not Found"},
		{45,      "ARB Memory Overflow"},
		{50,      "Startup Error"},
		{51,      "No Reply"},
		{52,      "Connection Error"},
		{53,      "Configuration Error"},
		{54,      "Filesystem Error"},
		{60,      "Invalid RF-Connector Setting"},
		{101,     "Firmware Error"},
		{102,     "Unidentified Error"},
		{103,     "Parameter Error"},
		{104,     "Not Functional"},
		{0, NULL}
	};
	
	(void) Rs_LockSession(instrSession, 0);
	
    for (cnt = 0; reliabilityTable[cnt].string; cnt++)
    {
	    if (reliabilityTable[cnt].value == reliabilityCode)
	    {
	        sprintf (reliabilityMessage, "%s", reliabilityTable[cnt].string);
			if (reliabilityCode)
				error = RSCMWEVM_ERROR_RELIABILITY_INDICATOR;
			found = VI_TRUE;
	    }
	}
	if (!found)
	{
		sprintf (reliabilityMessage, "Unrecognized reliability indicator! Report problem to R&S support!");

		error = VI_WARN_UNKNOWN_STATUS;
	}
	
	(void) Rs_UnlockSession (instrSession, 0);
    return error;
}

/*****************************************************************************
 * Function: Preset
 * Purpose:  This function initiates an instrument reset.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_Preset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckErr (rscmwevm_SetAttributeViString (instrSession, "", 
                RSCMWEVM_ATTR_SYSTEM_PRESET, NULL));
    
    checkErr( rscmwevm_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, 0);
    return error;
}

/// HIFN  This function sets fast sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsvna_init or 
/// HIPAR instrSession/rsvna_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR fastSweepMode/Fast sweep mode
ViStatus _VI_FUNC rscmwevm_SetFastSweepMode (ViSession instrSession,
                                             ViInt32 fastSweepMode)
{
	ViStatus	error = VI_SUCCESS;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);
	viCheckParm(rscmwevm_invalidViInt32Range (instrSession, fastSweepMode, RSCMWEVM_VAL_FAST_SWEEP_NORMAL, RSCMWEVM_VAL_FAST_SWEEP_FAST), 2, "Fast Sweep Mode");
	
	sessionProperties->fastSweepInstrument = fastSweepMode;
Error:
	return error;
}

/*****************************************************************************
 * Function: Get Error and Clear Error Functions
 * Purpose:  These functions enable the instrument driver user to
 *           get or clear the error information the driver associates with the
 *           RS session.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_GetError (ViSession instrSession,
                                     ViStatus *errorCode,
                                     ViInt32 bufferSize,
                                     ViChar description[])
{
    ViStatus    error       = VI_SUCCESS;
    ViStatus    primary     = VI_SUCCESS,
                secondary   = VI_SUCCESS;
    ViChar      elaboration[RS_MAX_MESSAGE_BUF_SIZE]    = "";
    ViChar      errorMessage[RS_MAX_MESSAGE_BUF_SIZE]  = "";
    ViChar     *appendPoint = errorMessage;
    ViInt32     actualSize  = 0;
    ViBoolean   locked      = VI_FALSE;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);

    /* Lock Session */
    checkErr( Rs_LockSession(instrSession, &locked));

    /* Test for nulls and acquire error data */
    if (bufferSize != 0)
    {
        if (errorCode == NULL)
        {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error");
        }
        if (sessionProperties == NULL)
        {
            primary = VI_ERROR_RSRC_NFOUND;
            secondary = 0;
        }
        else
        {
            checkErr( Rs_GetErrorInfo (instrSession, &primary, &secondary, elaboration));
        }
    }

    else
        {
            if (sessionProperties == NULL)
            {
                (void) Rs_UnlockSession (instrSession, &locked);
                return 1024;
            }
            else
            {
                checkErr( Rs_GetAttribute (instrSession, NULL, RS_ATTR_ERROR_ELABORATION, 0, 256, elaboration));
                checkErr( Rs_GetAttribute (instrSession, NULL, RS_ATTR_SECONDARY_ERROR, 0, (ViInt32) sizeof (secondary), &secondary));
                checkErr( Rs_GetAttribute (instrSession, NULL, RS_ATTR_PRIMARY_ERROR, 0, (ViInt32) sizeof (primary), &primary));
            }
        }

    /* Format data */
    if (primary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rscmwevm_error_message (instrSession, primary, msg));
        appendPoint += sprintf(appendPoint, "Primary Error: (Hex 0x%08X) %s\n", (unsigned int) primary, msg);
        }

    if (secondary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rscmwevm_error_message (instrSession, secondary, msg));
        appendPoint += sprintf(appendPoint, "Secondary Error: (Hex 0x%08X) %s\n", (unsigned int) secondary, msg);
        }

    if (strlen (elaboration) > 0)
        {
        sprintf(appendPoint, "Elaboration: %s", elaboration);
        }

    actualSize = (ViInt32) (strlen(errorMessage) + 1);

    /* Prepare return values */
    if (bufferSize == 0)
        {
        error = (ViStatus) actualSize;
        }
    else
        {
        if (bufferSize > 0)
            {
            if (actualSize > bufferSize)
                {
                error = actualSize;
                actualSize = bufferSize;
                }
            }
        if (description == NULL)
            {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Description");
            }
        memcpy(description, errorMessage, (size_t)(actualSize-1));
        description[actualSize-1] = '\0';
        }

    if (errorCode)
        {
        *errorCode = primary;
        }

Error:
    (void) Rs_UnlockSession (instrSession, &locked);
    return error;
}

ViStatus _VI_FUNC rscmwevm_ClearError (ViSession instrSession)
{
    return Rs_ClearErrorInfo (instrSession);
}

/// HIFN This function is used to read data from the instrument and write 
/// HIFN it to a user specified file on the host computer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from instrument
/// HIPAR source/to control computer.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from instrument to control computer applies.
ViStatus _VI_FUNC rscmwevm_ReadToFileFromInstrument(ViSession instrSession,
	ViString source,
	ViString destination)
{
	ViStatus    error = VI_SUCCESS;
	ViUInt32    ret_count = 0;
	ViUInt64    length = 0;
	ViChar      buffer[RSCMWEVM_IO_BUFFER_SIZE] = "";
	ViUInt16 intfc = VI_INTF_GPIB;
	ViChar      *pbuffer = NULL;
	int i;

	checkErr(Rs_LockSession(instrSession, NULL));

	viCheckParm(!source, 2, "Source");
	viCheckParm(!destination, 3, "Destination");

	checkErr(viGetAttribute(instrSession, VI_ATTR_INTF_TYPE, &intfc));
	switch (intfc)
	{
	case VI_INTF_ASRL: /* Serial intfc. */
		viCheckErr(viSetAttribute(instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
		break;
	}
	viCheckErr(viSetAttribute(instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));

	viCheckErr(viPrintf(instrSession, ":MMEMory:DATA? '%s'\n", source));
	checkErr(viRead(instrSession, (ViBuf)buffer, 1, NULL));


	if (buffer[0] == '#')
	{   /* Binary data */
		/* Disable termination character */
		checkErr(viSetAttribute(instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));

		/* Parse header */
		checkErr(viRead(instrSession, (ViBuf)buffer, 1, NULL));

		buffer[1] = '\0';
		if (buffer[0] == '(')
		{
			for (i = 0; i < 20; i++)
			{
				checkErr(viRead(instrSession, (ViBuf)buffer + i, 1, NULL));

				if (buffer[i] == ')')
				{
					buffer[i] = '\0';
					sscanf(buffer, "%lld", &length);
					break;
				}
			}
			if (i == 20)
			{
				//couldn't find the ')' character
				checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
			}
		}
		else if (buffer[0] == '0')
		{
			length = 0;
		}
		else
		{
			sscanf(buffer, "%lld", &length);
			if (length > 9 || length < 1)
			{
				//expected number 1 to 9
				checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
			}

			checkErr(viRead(instrSession, (ViBuf)buffer, (ViUInt32)length, NULL));
			buffer[length] = '\0';
			sscanf(buffer, "%lld", &length);
		}

		if (length > 0)
		{
			viCheckErr(Rs_ReadToFile2(instrSession, destination, length, RS_VAL_TRUNCATE));
			/* Flush garbage */
			viScanf(instrSession, "%*[^\r\n]");
			if (error > 0) error = VI_SUCCESS;
		}
		else
		{
			/* Unlimited size of data */
			checkErr(Rs_ReadDataUnknownLength(instrSession, &pbuffer, &ret_count));
		}
	}
	else
	{
		checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
	}

	viSetAttribute(instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
	checkErr(rscmwevm_CheckStatus(instrSession));

Error:
	viSetAttribute(instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
	if (pbuffer) free(pbuffer);
	switch (intfc)
	{
	case VI_INTF_ASRL:
		viSetAttribute(instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
		break;
	}
	(void)Rs_UnlockSession(instrSession, NULL);

	return error;
}

/// HIFN This function is used to read data from the host computer and 
/// HIFN write it to a user specified file in the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from control
/// HIPAR source/computer to instrument.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from control computer to instrument applies.
ViStatus _VI_FUNC rscmwevm_WriteFromFileToInstrument(ViSession instrSession,
	ViString source,
	ViString destination)
{
	ViStatus    error = VI_SUCCESS;
	ViUInt32    digits = 0;
	FILE        *sourceFile = NULL;
	ViUInt64    fileSize = 0;
#if defined (_CVI_)
	ViUInt32    fileSize32 = 0;
#endif

	checkErr(Rs_LockSession(instrSession, NULL));
	viCheckParm(!source, 2, "Source");
	viCheckParm(!destination, 3, "Destination");

	/* Open source file from host controller */
	if ((sourceFile = fopen(source, "rb")) == NULL)
		viCheckErr(RS_ERROR_CANNOT_OPEN_FILE);

	/* Get size of file */
	(void)fseek(sourceFile, 0, SEEK_END);
#if defined (_CVI_)
	GetFileSize(source, &fileSize32);
	fileSize = (ViUInt64)fileSize32;
#elif defined(_MSC_VER)
	fileSize = (ViUInt64)_ftelli64(sourceFile);
#else
	fileSize = (ViUInt64)ftell(sourceFile);
#endif
	(void)fseek(sourceFile, 0, SEEK_SET);

	/* Create command header */
	digits = (ViUInt32)((int)floor(log10((double)fileSize))) + 1;
	(void)viSetAttribute(instrSession, VI_ATTR_SEND_END_EN, VI_FALSE);
	if (digits < 10)
	{
		viCheckErr(viPrintf(instrSession, ":MMEM:DATA '%s',#%ld%lld", destination, digits, fileSize));
	}
	else
	{
		viCheckErr(viPrintf(instrSession, ":MMEM:DATA '%s',#(%lld)", destination, fileSize));
	}

	viCheckErr(Rs_WriteFromFile2(instrSession, source, fileSize, 0));

	checkErr(rscmwevm_CheckStatus(instrSession));

Error:
	if (sourceFile) (void)fclose(sourceFile);

	(void)Rs_UnlockSession(instrSession, NULL);

	return error;
}

/*****************************************************************************
 * Function: WriteInstrData and ReadInstrData Functions
 * Purpose:  These functions enable the instrument driver user to
 *           write and read commands directly to and from the instrument.
 *
 *           Note:  These functions bypass the RS attribute state caching.
 *                  WriteInstrData invalidates the cached values for all
 *                  attributes.
 *****************************************************************************/

ViStatus _VI_FUNC rscmwevm_WriteInstrData (ViSession instrSession, ViString writeBuffer)
{
    return Rs_WriteInstrData (instrSession, writeBuffer);
}

ViStatus _VI_FUNC rscmwevm_ReadInstrData (ViSession instrSession, ViInt32 numBytes,
                                          ViChar rdBuf[], ViUInt32 *bytesRead)
{
    return Rs_ReadInstrData (instrSession, numBytes, rdBuf, bytesRead);
}

/*****************************************************************************
 * Function: Query ViBoolean
 * Purpose:  This function queries the ViBoolean value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_QueryViBoolean (ViSession instrSession,
                                           ViString command, 
                                           ViBoolean *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, 0)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==NULL)
        viCheckParm (RS_ERROR_NULL_POINTER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%d", command, value));
    checkErr( rscmwevm_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, 0);
    return error;
}

/*****************************************************************************
 * Function: Query ViInt32
 * Purpose:  This function queries the ViInt32 value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_QueryViInt32 (ViSession instrSession,
                                         ViString command, 
                                         ViInt32 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, 0)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==NULL)
        viCheckParm (RS_ERROR_NULL_POINTER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%ld", command, value));
    checkErr( rscmwevm_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, 0);
    return error;
}

/*****************************************************************************
 * Function: Query ViReal64
 * Purpose:  This function queries the ViReal64 value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_QueryViReal64 (ViSession instrSession,
                                          ViString command, 
                                          ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, 0)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==NULL)
        viCheckParm (RS_ERROR_NULL_POINTER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%le", command, value));
    checkErr( rscmwevm_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, 0);
    return error;
}


/*****************************************************************************
 * Function: Query ViString
 * Purpose:  This function queries the ViString value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_QueryViString (ViSession instrSession,
                                          ViString command, 
                                          ViInt32 bufferSize,
                                          ViChar _VI_FAR value[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = NULL;
    ViUInt32        bytesRead;
    
    checkErr( Rs_LockSession (instrSession, 0)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==NULL)
        viCheckParm (RS_ERROR_NULL_POINTER, 3, "Value");
    viCheckErr (viPrintf (instrSession, "%s\n", command));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &bytesRead));  
    if (bytesRead>=(ViUInt32)bufferSize)
    {
        strncpy(value, pbuffer, (size_t) (bufferSize-1));
        value[bufferSize-1]='\0';
        error = (ViStatus) (bytesRead+1);
    } 
    else
    {
        strncpy(value, pbuffer, (size_t) bytesRead); 
        value[bytesRead]='\0';   
    }
    checkErr( rscmwevm_CheckStatus (instrSession)); 
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, 0);
    return error;
}

/*****************************************************************************
 * Function: rscmwevm_close
 * Purpose:  This function closes the instrument.
 *
 *           Note:  This function must unlock the session before calling
 *           Rs_Dispose.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_close (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, 0));
    checkErr( rscmwevm_RsClose (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, 0);
    (void) Rs_Dispose (instrSession);

    return error;
}


/*****************************************************************************
 * Function: rscmwevm_RsClose
 * Purpose:  This function performs all of the drivers clean-up operations
 *           except for closing the RS session.  This function is called by
 *           rscmwevm_close or by an RS class driver.
 *
 *           Note:  This function must close the I/O session and set
 *           RS_ATTR_IO_SESSION to 0.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevm_RsClose (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    /* Do not lock here. The caller manages the lock. */

    if(instrSession != 0)
        {
        ViSession   rmSession = 0;

        checkErr( Rs_GetAttribute (instrSession, NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViUInt32) sizeof (rmSession), &rmSession));
        
        viPrintf (instrSession, "&GTL");
        
        (void) viClose (instrSession);
        (void) viClose (rmSession);
        }
Error:    
    return error;
}

/*****************************************************************************
 *------------------------ Global Session Callbacks -------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: Rs_CheckStatusCallback
 * Purpose:  This function queries the instrument to determine if it has
 *           encountered an error. If the instrument has encountered an
 *           error, this function returns the RS_ERROR_INSTRUMENT_SPECIFIC
 *           error code. This function is called by the
 *           Rs_CheckStatus utility function. The
 *           Rs_SetAttribute and Rs_GetAttribute functions invoke this
 *           function when the optionFlags parameter includes the
 *           RS_VAL_DIRECT_USER_CALL flag.
 *
 *           The user can disable calls to this function by setting the
 *           RSCMWEVM_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE.  The driver can
 *           disable the check status callback for a particular attribute by
 *           setting the RS_VAL_DONT_CHECK_STATUS flag.
 *
 *****************************************************************************/
ViStatus rscmwevm_CheckStatusCallback (ViSession instrSession)
{
    ViUInt32    ret_cnt;
    ViStatus    error   = VI_SUCCESS;
    ViInt16     esr     = 0;
    ViChar      buffer[1024];
    ViInt32     code = 0;
    
    viCheckErr (viWrite (instrSession, (ViBuf) "*STB?\n", 6, NULL));
    viCheckErr (viRead (instrSession, (ViPBuf) buffer, 4, &ret_cnt));
    buffer[ret_cnt] = '\0';
    esr = (ViInt16) atol (buffer);

    if ((esr & IEEE_488_2_ERROR_MASK) != 0)
    {
        rscmwevm_error_query (instrSession, &code, buffer);
        
        checkErr (Rs_SetErrorInfo (instrSession, VI_TRUE, RS_ERROR_INSTR_SPECIFIC, code, buffer));
        
        viCheckErr (RS_ERROR_INSTR_SPECIFIC);
    }

Error:

    return error;
}

/*****************************************************************************
 * Function: Rs_WaitForOPCCallback
 * Purpose:  This function waits until the instrument has finished processing
 *           all pending operations. This function is called by the
 *           Rs_WaitForOPC utility function. The RS engine invokes
 *           this function in the following two cases:
 *           - Before invoking the read callback for attributes for which the
 *             RS_VAL_WAIT_FOR_OPC_BEFORE_READS flag is set.
 *           - After invoking the write callback for attributes for which the
 *             RS_VAL_WAIT_FOR_OPC_AFTER_WRITES flag is set.
 *
 *****************************************************************************/
ViStatus rscmwevm_WaitForOPCCallback (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    ViUInt16    response    = 0;
    ViUInt16    intfc   = VI_INTF_GPIB;
    ViInt32     opc_timeout = 0;
    clock_t     start_time = clock();
    ViReal64    opc_timeout_ms = 0.0;
    double      elapsed_time_ms;
    double      ms_divider = (double)CLOCKS_PER_SEC / 1000.0;

    checkErr( Rs_GetAttribute (instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0,
        (ViInt32) sizeof (opc_timeout), &opc_timeout));

    opc_timeout_ms = (ViReal64) opc_timeout;

    viCheckErr( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));

    switch (intfc)
    {
        case VI_INTF_ASRL: /* Serial intfc */
            
            /* Not supported */

        break;
        default:

            do
            {
                error = viReadSTB (instrSession, &response);
                if ((response & IEEE_488_2_SRE_MASK) != 0)
                    break;

                elapsed_time_ms = (double)(clock() - start_time)/ms_divider;
                if (elapsed_time_ms > opc_timeout_ms)
                    return VI_ERROR_TMO;
                
                if (elapsed_time_ms < 10.0)
                {/* Do it as fast as possible, therefore no Sleep */
                }
                else if (elapsed_time_ms < 100.0)
                    Sleep (1);
                else if (elapsed_time_ms < 1000.0)
                    Sleep (5);
                else if (elapsed_time_ms < 5000.0)
                    Sleep (10);
                else if (elapsed_time_ms < 10000.0)
                    Sleep (50);
                else if (elapsed_time_ms < 20000.0)
                    Sleep (100);
                else if (elapsed_time_ms < 30000.0)
                    Sleep (500);
                else
                    Sleep (1000);
                
            } while ((response & IEEE_488_2_SRE_MASK) == 0);

            if ((response & IEEE_488_2_ERROR_MASK) != 0)
            {
                error =  RS_ERROR_INSTRUMENT_STATUS;
                
            }
        }
Error:
    return error;
}

