/*****************************************************************************
 *  Rohde&Schwarz CMW Radiocommunication Analyzer 1xEV-DO Signaling driver
 *  VXIpnp Instrument Driver
 *  Original Release: February 2010
 *  By: Martin Krcmar
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *     see ChangeLog
 *****************************************************************************/
#include <math.h>
#include <float.h>
#include "rscmwevs.h"
#include "rscmwevs_utility.h"

#include "time.h"
#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
    #define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
    #include <sys/timeb.h>
    #include <sys/types.h>
    #include <unistd.h>
    #define Sleep sleep
#else                       /* ANSI compatible */
        #include <windows.h>
#endif

#if defined (_CVI_)
	#include <utility.h>
#endif
/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
static ViString packetSizeArr[] = {"S128","S256","S512","S1K","S2K","S3K",
                                   "S4K","S5K","TOT", NULL};
static ViString rfInputConnectorArr[] = {"RF1C","RF2C","RF3C","RF4C","RFAC","RFBC","RF1O","RF3O","RFAO",
                                         "R11C","R12C","R13C","R14C",
                                         "R21C","R22C","R23C","R24C",
                                         "R31C","R32C","R33C","R34C",
                                         "R41C","R42C","R43C","R44C",
                                         "R11O","R13O","R21O","R23O",
                                         "R31O","R33O","R41O","R43O", NULL};
static ViString rfInputConverterArr[] = {"RX1","RX2","RX3","RX4","RX11","RX12","RX13","RX14","RX21",
                                         "RX22","RX23","RX24","RX31","RX32","RX33","RX34","RX41","RX42",
                                         "RX43","RX44",NULL};
static ViString rfOutputConnectorArr[] = {"RF1C","RF2C","RF3C","RF4C","RFAC","RFBC","RF1O","RF3O","RFAO",
                                          "R11C","R12C","R13C","R14C",
                                          "R21C","R22C","R23C","R24C",
                                          "R31C","R32C","R33C","R34C",
                                          "R41C","R42C","R43C","R44C",
                                          "R11O","R13O","R21O","R23O",
                                          "R31O","R33O","R41O","R43O", NULL};
static ViString rfOutputConverterArr[] = {"TX1","TX2","TX3","TX4", "TX11","TX12","TX13","TX14","TX21",
                                          "TX22","TX23","TX24","TX31","TX32","TX33","TX34","TX41","TX42",
                                          "TX43","TX44", NULL};
static ViString iqOutputConnectorArr[] = {"IQ2O","IQ4O","IQ6O","IQ8O","I12O","I14O","I16O","I18O","I22O",
                                          "I24O","I26O","I28O","I32O","I34O","I36O","I38O","I42O","I44O",
                                          "I46O","I48O",NULL}; 
static ViString ScenarioArr[] = {"SCEL","HMOD","SCF", "HML",NULL};
static ViString PowerModeArr[] = {"Absolute","Relative",NULL};
static ViString PERLinkTypeArr[] = {"FLPer","RLPer",NULL};
static ViString MeasurementTypeArr[] = {"MeasPER","MeasTHR",NULL};
static ViString MeasurementArr[] = {"PER","THR",NULL};
static ViString DataRateArr[] = {"R0K0","R19K2","R38K4","R76K8","R115k2","R153k6","R230k4","R307k2","R460k8","R614k4","R921k6","R1228k8","R1843k2","TOTAL",NULL};
static ViString RLPTableArr[] = {"RES","RACK","NAK","SUMM",NULL};
static ViString IPStatTableArr[] = {"PPPT","DRAT",NULL};
static ViString SignArr[]={"NEG","POS",NULL};
static ViString switchArr[] = {"OFF","ON",NULL};
static ViString TechnologyArr[] = {"TechEVDO","TechCDMA","TechLTE",NULL};
static ViString TechnologyStringArr[] = {"EVDO","CDMA","LTE",NULL};
static ViString EVDObandClassArr[] = {"USC","KCEL","NAPC","TACS","JTAC","KPCS","N45T","IM2K","NA7C","B18M","NA9C","NA8S","PA4M","PA8M","IEXT","USPC","AWS","U25B","U25F","PS7C","LO7C","LBAN","SBAN",NULL};
static ViString LTEbandClassArr[] = {"","OB1","OB2","OB3","OB4","OB5","OB6","OB7","OB8","OB9","OB10","OB11","OB12","OB13","OB14","OB15","OB16","OB17","OB18","OB19","OB20","OB21","OB22","OB23","OB24","OB25","OB26","OB27","OB28","OB29","OB30","OB31","OB32","OB33","OB34","OB35","OB36","OB37","OB38","OB39","OB40","OB41","OB42","OB43","OB44","UDEF",NULL};
static ViString IPVersionArr[] = {"IP4","IP6",NULL};
static ViString faderArr[] = {"EXT","INT", NULL};
static ViString categoryArr[] = {"INFO","WARN","ERR","CONT", NULL};

/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/
ViStatus rscmwevs_CheckStatusCallback (ViSession io);
ViStatus rscmwevs_WaitForOPCCallback (ViSession io);
ViStatus _VI_FUNC  rscmwevs_RsClose (ViSession vi);

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: rscmwevs_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rscmwevs_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_init (ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == NULL)
        {
        (void) Rs_SetErrorInfo (0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    checkErr( rscmwevs_InitWithOptions (resourceName, IDQuery, resetDevice,
                                      "QueryInstrStatus=1", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rscmwevs_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession
)
{
    ViStatus                error = VI_SUCCESS;
    ViChar                  newResourceName[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar                  newOptionString[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViBoolean               isLogicalName = 0;
    ViSession               instrSession = 0, rmSession = 0;
    RsAttrPropertiesPtr     *attrList=NULL;
    RsRepCapPtr             repCapTable = NULL;

    if (newInstrSession == NULL)
        {
        (void) Rs_SetErrorInfo (0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                         VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
        checkErr( RS_ERROR_INVALID_PARAMETER);
        }

    /* Prepare resource manager session handle */
    checkErr (viOpenDefaultRM (&rmSession));

    /* Establish communication session with the instrument */

    viCheckErr( viOpen (rmSession, resourceName, 0, 0, &instrSession));
	
	viCheckErr (viClear (instrSession));
    
    /* Configure VISA Formatted I/O */
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 5000 ));
    viCheckErr( viSetBuf (instrSession, (ViUInt16) (VI_READ_BUF | VI_WRITE_BUF), 4000));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));

    checkErr( Rs_GetInfoFromResourceName (resourceName,
                                          (ViString)optionString,
                                          newResourceName,
                                          newOptionString,
                                          &isLogicalName));

    ATTRLIST(attrList,rscmwevs); 
    checkErr( Rs_SpecificDriverNew (instrSession, "rscmwevs", newOptionString, attrList));
   
    /* Build repeated capability table */
    REPCAPTABLE(repCapTable,rscmwevs);
    checkErr( Rs_BuildRepCapTable (instrSession, repCapTable));

    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_VISA_RM_SESSION, 0, &rmSession));

    /* With remote control via Ethernet, remote control is not automatically
       set by means of a command. The instrument must be explicitly set to
       the REMOTE state, e.g. by sending the intf command &GTR
    */
    checkErr (viWrite (instrSession, (ViBuf) "&GTR\n", 5, NULL));
    
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice == VI_TRUE)
    {
        checkErr( rscmwevs_reset (instrSession));
    }
    else  /*- Send Default Instrument Setup ---------------------------------*/
        checkErr( rscmwevs_DefaultInstrSetup (instrSession));

    /*- Identification Query ------------------------------------------------*/
    if (IDQuery == VI_TRUE)
        {
        ViChar rdBuffer[RSCMWEVS_IO_BUFFER_SIZE] = "";

        checkErr( Rs_GetAttribute (instrSession, NULL, (ViAttr) RSCMWEVS_ATTR_ID_QUERY_RESPONSE,
                                   0, RSCMWEVS_IO_BUFFER_SIZE, rdBuffer));

        if (strncmp (rdBuffer, RSCMWEVS_VALID_ID_RESPONSE_STRING, strlen(RSCMWEVS_VALID_ID_RESPONSE_STRING)) != 0)
            {
            viCheckErr( VI_ERROR_FAIL_ID_QUERY);
            }
        }

    if (isLogicalName == VI_FALSE)
        {
        ViInt32 oldFlag = 0;

        checkErr (Rs_GetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, &oldFlag));
        checkErr (Rs_SetAttributeFlags (instrSession, RS_ATTR_IO_RESOURCE_DESCRIPTOR, oldFlag & 0xfffb | 0x0010));
        checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_IO_RESOURCE_DESCRIPTOR, 0, newResourceName));
        }
    

    if (isLogicalName == VI_TRUE)
        checkErr( Rs_ApplyDefaultSetup (instrSession));

    /* Instrument session accessible by the end user is native VISA session,
       or when simulated virtual instrument (vi) session. */
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:

    if (error < VI_SUCCESS)
    {
        if (instrSession != 0)
        {
            (void) viClose (instrSession);
            (void) Rs_Dispose (instrSession);
        }
        
        if (rmSession != 0)
            (void) viClose (rmSession);
    }
    else
    {
        *newInstrSession = instrSession;
    }

    return error;
}

/// HIFN Specifies the repetition mode of the pattern execution.
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
/// HIPAR instance/Document parameter here.
/// HIPAR repetition/Document parameter here.
ViStatus _VI_FUNC rscmwevs_ConfigureReversePowerControlRepetition (ViSession instrSession,
                                                                   ViInt32 instance,
                                                                   ViInt32 repetition)
{   
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm(rscmwevs_SetAttributeViInt32(instrSession, rep_cap,
        RSCMWEVS_ATTR_REVERSE_POWER_CONTROL_REPETITION, repetition), 3, "Repetition");

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN Configures power control segment properties.
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
/// HIPAR instance/Document parameter here.
/// HIPAR segment/Document parameter here.
/// HIPAR powerControlBits/Document parameter here.
/// HIPAR segmentLength/Document parameter here.
ViStatus _VI_FUNC rscmwevs_ConfigureReversePowerControlSegment (ViSession instrSession,
                                                                ViInt32 instance,
                                                                ViInt32 segment,
                                                                ViInt32 powerControlBits,
                                                                ViInt32 segmentLength)
{
	ViStatus	error = VI_SUCCESS;
    ViChar      rep_cap[10] = "";   
    checkErr (Rs_LockSession (instrSession, NULL));

	sprintf (rep_cap, "i%ld,seg%ld", instance, segment);
	
	viCheckParm(rscmwevs_SetAttributeViInt32(instrSession, rep_cap, RSCMWEVS_ATTR_POWER_CONTROL_SEGMENT_BITS, powerControlBits), 4, "Segment Bits");
	viCheckParm(rscmwevs_SetAttributeViInt32(instrSession, rep_cap, RSCMWEVS_ATTR_POWER_CONTROL_SEGMENT_LENGTH, segmentLength), 5, "Segment Length");
	  
Error:
	Rs_UnlockSession (instrSession, NULL);       
	return error;
}

/// HIFN Starts and in continuous mode also stops the execution of the user specific pattern.
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
/// HIPAR instance/Document parameter here.
/// HIPAR runSequenceState/Document parameter here.
ViStatus _VI_FUNC rscmwevs_StartReversePowerControl (ViSession instrSession,
                                                     ViInt32 instance,
                                                     ViBoolean runSequenceState)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_REVERSE_POWER_CONTROL_RUN,
                                                 runSequenceState), 3, "Run Sequence State");  
	
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Configures the time settings. 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR timeSource/Sets the time source for the derivation of the CMDA system time.
/// HIPAR syncCode/This setting will be applied if "Time Source" has been set to "Sync 
/// HIPAR syncCode/Code". The "Sync Code" is an internally calculated value that allows 
/// HIPAR syncCode/for the same time settings of two synchronized SUUs. Two SUUs in one 
/// HIPAR syncCode/CMW are automatically synchronized if the software option KS888 is 
/// HIPAR syncCode/present. In order to synchronize two SUUs in two CMWs, the multibox 
/// HIPAR syncCode/scenario needs to be used.
/// HIPAR applyTimeAt/Defines when the configured time source shall be applied to the SUU 
/// HIPAR applyTimeAt/hosting the signaling application.
ViStatus _VI_FUNC rscmwevs_ConfigureTimeSettings (ViSession instrSession,
                                                  ViInt32 instance,
                                                  ViInt32 timeSource,
                                                  ViString syncCode,
                                                  ViInt32 applyTimeAt)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_TIME_SOURCE,
                                                timeSource), 3, "timeSource");

	if (timeSource == RSCMWEVS_VAL_TIME_SOURCE_SYNC)
	{
		viCheckParm (rscmwevs_SetAttributeViString (instrSession, rep_cap,
	                                                RSCMWEVS_ATTR_TIME_SYNC_CODE,
	                                                syncCode), 3, "Sync Code");
	}
	
	viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_APPLY_TIME_AT,
                                                applyTimeAt), 3, "Apply Time At");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Adjusts track of leap second correction to Coordinated Universal Time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR leapSeconds/Sets the time source for the derivation of the CMDA system time.
/// HIPAR leapSeconds/Sets track of leap second correction to Coordinated Universal Time.
ViStatus _VI_FUNC rscmwevs_ConfigureLeapSecondCorrection (ViSession instrSession,
                                                          ViInt32 instance,
                                                          ViInt32 leapSeconds)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_CONFIGURE_LEAP_SECOND_CORRECTION,
                                                leapSeconds), 3, "Leap Seconds");

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Defines the time offset from Coordinated Universal Time (UTC) according a local time zone.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR sign/Sets the sign.
/// HIPAR hour/Sets the hour differences from the Coordinated Universal Time UTC.
/// HIPAR minute/Sets the minute differences from the Coordinated Universal Time UTC.
ViStatus _VI_FUNC rscmwevs_ConfigureLocalTimeOffset (ViSession instrSession,
                                                     ViInt32 instance,
                                                     ViInt32 sign, ViInt32 hour,
                                                     ViInt32 minute)
{
   
	ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, sign, RSCMWEVS_VAL_NEG, RSCMWEVS_VAL_POS), 3, "Sign");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, hour, 0, 17), 4, "hour");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, minute, 0, 59), 5, "Minute");
    
    sprintf (command, "CONF:EVDO:SIGN%ld:SYST:LTOF %s,%ld,%ld", instance, SignArr[sign], hour, minute);
    
    viCheckErr (viPrintf (instrSession, command));
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Queries time offset from UTC in hexadecimal format according a local time zone.
/// HIFN  rscmwevs_QueryLocalTimeOffset
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR localTimeOffset/Displays time offset from UTC in hexadecimal format according a local time zone.
ViStatus _VI_FUNC rscmwevs_QueryLocalTimeOffset (ViSession instrSession,
                                                 ViInt32 instance,
                                                 ViChar localTimeOffset[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViString (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_QUERY_LOCAL_TIME_OFFSET,
                                               RSCMWEVS_IO_BUFFER_SIZE, localTimeOffset), 3, "Local Time Offset");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Configures external DAU.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSCMWEVS_ATTR_USE_EXTERNAL_DAU
/// HIFN     RSCMWEVS_ATTR_EXTERNAL_DAU_NETWORK_SEGMENT
/// HIFN     RSCMWEVS_ATTR_EXTERNAL_DAU_SUBNET_NODE_ID
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:EVDO:SIGN<instance>:CONNection:EDAU:ENABle
/// HIFN     CONFigure:EVDO:SIGN<instance>:CONNection:EDAU:NSEGment
/// HIFN     CONFigure:EVDO:SIGN<instance>:CONNection:EDAU:NID
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR useExternalDAU/Enables use of an external DAU. The two instruments must be connected 
/// HIPAR useExternalDAU/via LAN. For details, refer to the DAU documentation.
/// HIPAR networkSegment/Specifies the network segment of the instrument where the external DAU 
/// HIPAR networkSegment/is installed.
/// HIPAR subnetNodeID/Specifies the subnet node ID of the instrument where the external DAU 
/// HIPAR subnetNodeID/is installed.
ViStatus _VI_FUNC rscmwevs_ConfigureExternalDAU(
	ViSession	instrSession,
	ViInt32	instance,
	ViBoolean	useExternalDAU,
	ViInt32	networkSegment,
	ViInt32	subnetNodeID
)
{
    ViStatus  error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViBoolean (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_USE_EXTERNAL_DAU,
                                               useExternalDAU), 3, "Use External DAU");
 
    viCheckParm ( rscmwevs_SetAttributeViInt32(instrSession, rep_cap,
                                                RSCMWEVS_ATTR_EXTERNAL_DAU_NETWORK_SEGMENT,
                                                networkSegment), 4, "Network Segment");
 
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_EXTERNAL_DAU_SUBNET_NODE_ID,
                                                subnetNodeID), 5, "Subnet Node ID");	
 
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}


/// HIFN  Time setting for CDMA system time source DATE (see 
/// HIFN  rscmwevs_ConfigureTimeSettings). 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR hour/Sets the hour.
/// HIPAR minute/Sets the minute.
/// HIPAR second/Sets the second.
ViStatus _VI_FUNC rscmwevs_ConfigureTime (ViSession instrSession,
                                          ViInt32 instance, ViInt32 hour,
                                          ViInt32 minute, ViInt32 second)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, hour, 0, 23), 3, "hour");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, minute, 0, 59), 4, "Minute");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, second, 0, 59), 5, "Second");
   
    
    sprintf (command, "CONF:EVDO:SIGN%ld:SYST:TIME %ld,%ld,%ld", instance, hour, minute, second);
    
    viCheckErr (viPrintf (instrSession, command));
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Date setting for CDMA system time source DATE (see 
/// HIFN  rscmwevs_ConfigureTimeSettings). 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR day/Sets the day.
/// HIPAR month/Sets the month.
/// HIPAR year/Sets the year.
ViStatus _VI_FUNC rscmwevs_ConfigureDate (ViSession instrSession,
                                          ViInt32 instance, ViInt32 day,
                                          ViInt32 month, ViInt32 year)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, day, 1, 31), 3, "day");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, month, 1, 12), 4, "month");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, year, 2012, 9999), 5, "year");
    
    sprintf (command, "CONF:EVDO:SIGN%ld:SYST:DATE %ld,%ld,%ld", instance, day, month, year);
    
    viCheckErr (viPrintf (instrSession, command));
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure RF Output Signal Attenuation
 * Purpose:  Defines an external attenuation (or gain, if the value is 
 *           negative), to be applied to the RF output connector.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRFOutputSignalAttenuation(
    ViSession   instrSession,
    ViInt32 instance,
    ViReal64    externalAttenuation
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_OUTPUT_EXTERNAL_ATTENUATION,
                                                externalAttenuation), 3, "External Attenuation");

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure RF Input Signal Attenuation
 * Purpose:  Defines an external attenuation (or gain, if the value is 
 *           negative), to be applied to the RF input connector.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRFInputSignalAttenuation(
    ViSession   instrSession,
    ViInt32 instance,
    ViReal64    externalAttenuation
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_INPUT_EXTERNAL_ATTENUATION,
                                                externalAttenuation), 3, "External Attenuation");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  This function returns the active scenario.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR scenario/Returns the active scenario.
/// HIPAR fader/Only returned for fading scenario (SCF), indicates whether internal or 
/// HIPAR fader/external fading is active.
ViStatus _VI_FUNC rscmwevs_QueryRFSignalActiveScenarioFader(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32*	scenario,
	ViInt32*	fader
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[1024];
    ViChar      *p2buf;

    
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckErr (viQueryf (instrSession, "ROUT:EVDO:SIGN%ld:SCEN?\n","%[^\n]", 
                instance, command));
    
    p2buf = strtok (command, ",");
    if (p2buf == NULL)
        return RS_ERROR_UNEXPECTED_RESPONSE;
    *scenario = rscmwevs_StringIndex (ScenarioArr,p2buf);
    if (*scenario < 0)
        return RS_ERROR_UNEXPECTED_RESPONSE;
	
	if (*scenario == RSCMWEVS_VAL_SCENARIO_SCF)
	{
	    p2buf = strtok (NULL, ",");
	    if (p2buf == NULL)
	        return RS_ERROR_UNEXPECTED_RESPONSE;
	    *fader = rscmwevs_StringIndex (faderArr,p2buf);
	    if (*fader < 0)
	        return RS_ERROR_UNEXPECTED_RESPONSE; 
	}

    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;    
}

/// HIFN  This function selects scenario in which an AT registers in CDMA2000 
/// HIFN  1xRTT and opens a session in CDMA2000 1xEV-DO at the same time.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     ROUTe:EVDO:SIGN<instance>:SCENario:HMODe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR rfInputConnector/Defines the RF input connector.
/// HIPAR rfInputConverter/Defines the RF input converter.
/// HIPAR rfOutputConnector/Selects the RF output connector.
/// HIPAR rfOutputConverter/Defines the RF output converter.
ViStatus _VI_FUNC rscmwevs_ConfigureHybridMode(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32	rfInputConnector,
	ViInt32	rfInputConverter,
	ViInt32	rfOutputConnector,
	ViInt32	rfOutputConverter
)
{
    ViStatus	error = VI_SUCCESS;
    ViChar	command[100] = "";

    checkErr (Rs_LockSession (instrSession, NULL));


    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, rfInputConnector, RSCMWEVS_VAL_CONN_RF1C, RSCMWEVS_VAL_CONN_R44C), 3, "RF Input Connector");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, rfInputConverter, RSCMWEVS_VAL_CONV_RX1, RSCMWEVS_VAL_CONV_RX44), 4, "RF Input Converter");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, rfOutputConnector, RSCMWEVS_VAL_CONN_RF1C, RSCMWEVS_VAL_CONN_R43O), 5, "RF Output Connector");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, rfOutputConverter, RSCMWEVS_VAL_CONV_TX1, RSCMWEVS_VAL_CONV_TX44), 6, "RF Output Converter");
    

    sprintf (command, "ROUT:EVDO:SIGN%ld:SCEN:HMOD %s,%s,%s,%s", instance, rfInputConnectorArr[rfInputConnector], rfInputConverterArr[rfInputConverter], rfOutputConnectorArr[rfOutputConnector], rfOutputConverterArr[rfOutputConverter]);  

    viCheckErr (viPrintf (instrSession, command));
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  This function activates the "Hybrid Mode Lite" scenario and selects 
/// HIFN  the signal path.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     ROUTe:EVDO:SIGN<instance>:SCENario:HMLite
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR rfInputConnector/Defines the RF input connector.
/// HIPAR rfInputConverter/Defines the RF input converter.
/// HIPAR rfOutputConnector/Selects the RF output connector.
/// HIPAR rfOutputConverter/Defines the RF output converter.
ViStatus _VI_FUNC rscmwevs_ConfigureHybridModeLite(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32	rfInputConnector,
	ViInt32	rfInputConverter,
	ViInt32	rfOutputConnector,
	ViInt32	rfOutputConverter
)
{
    ViStatus	error = VI_SUCCESS;
    ViChar	command[100] = "";

    checkErr (Rs_LockSession (instrSession, NULL));


    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, rfInputConnector, RSCMWEVS_VAL_CONN_RF1C, RSCMWEVS_VAL_CONN_R44C), 3, "RF Input Connector");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, rfInputConverter, RSCMWEVS_VAL_CONV_RX1, RSCMWEVS_VAL_CONV_RX44), 4, "RF Input Converter");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, rfOutputConnector, RSCMWEVS_VAL_CONN_RF1C, RSCMWEVS_VAL_CONN_R43O), 5, "RF Output Connector");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, rfOutputConverter, RSCMWEVS_VAL_CONV_TX1, RSCMWEVS_VAL_CONV_TX44), 6, "RF Output Converter");
    

    sprintf (command, "ROUT:EVDO:SIGN%ld:SCEN:HML %s,%s,%s,%s", instance, rfInputConnectorArr[rfInputConnector], rfInputConverterArr[rfInputConverter], rfOutputConnectorArr[rfOutputConnector], rfOutputConverterArr[rfOutputConverter]);  

    viCheckErr (viPrintf (instrSession, command));
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Activates the scenario "Standard Cell Fading: Internal" and selects 
/// HIFN  the RF input (RX) and output (TX) path, i.e. RF connectors and RX/TX 
/// HIFN  modules and the I/Q board to be used for fading.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR RFInputConnector/Defines the RF input connector.
/// HIPAR RFInputConverter/Defines the RF input converter.
/// HIPAR RFOutputConnector/Selects the RF output connector.
/// HIPAR RFOutputConverter/Defines the RF output converter.
ViStatus _VI_FUNC rscmwevs_ConfigureSignalRoutingStandardCellFading (ViSession instrSession,
                                                                            ViInt32 instance,
                                                                            ViInt32 RFInputConnector,
                                                                            ViInt32 RFInputConverter,
                                                                            ViInt32 RFOutputConnector,
                                                                            ViInt32 RFOutputConverter)
{
	ViStatus	error = VI_SUCCESS;
	ViChar buffer[1024] = ""; 
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, RFInputConnector, RSCMWEVS_VAL_CONN_RF1C, RSCMWEVS_VAL_CONN_R44C), 3, "RF Input Connector");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, RFInputConverter, RSCMWEVS_VAL_CONV_RX1, RSCMWEVS_VAL_CONV_RX44), 4, "RF Input Converter");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, RFOutputConnector, RSCMWEVS_VAL_CONN_RF1C, RSCMWEVS_VAL_CONN_R43O), 5, "RF Output Connector");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, RFOutputConverter, RSCMWEVS_VAL_CONV_TX1, RSCMWEVS_VAL_CONV_TX44), 6, "RF Output Converter");

    sprintf (buffer, "ROUT:EVDO:SIGN%ld:SCEN:SCF:INT %s,%s,%s,%s", instance, 
		rfInputConnectorArr[RFInputConnector], rfInputConverterArr[RFInputConverter],
		rfOutputConnectorArr[RFOutputConnector], rfOutputConverterArr[RFOutputConverter]);  

    checkErr (viPrintf (instrSession, "%s\n", buffer));
        
    checkErr (rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}


/// HIFN  Activates the "Standard Cell Fading: External" scenario and selects 
/// HIFN  the RF connectors, RX/TX modules and digital I/Q output connector to be 
/// HIFN  used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR RFInputConnector/Defines the RF input connector.
/// HIPAR RFInputConverter/Defines the RF input converter.
/// HIPAR RFOutputConnector/Selects the RF output connector.
/// HIPAR RFOutputConverter/Defines the RF output converter.
/// HIPAR IQOutputConnector/Selects the IQ output connector.
ViStatus _VI_FUNC rscmwevs_ConfigureSignalRoutingStandardCellFadingExternal (ViSession instrSession,
                                                                             ViInt32 instance,
                                                                             ViInt32 RFInputConnector,
                                                                             ViInt32 RFInputConverter,
                                                                             ViInt32 RFOutputConnector,
                                                                             ViInt32 RFOutputConverter,
                                                                             ViInt32 IQOutputConnector)
{
	ViStatus	error = VI_SUCCESS;
	ViChar buffer[1024] = ""; 
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, RFInputConnector, RSCMWEVS_VAL_CONN_RF1C, RSCMWEVS_VAL_CONN_R44C), 3, "RF Input Connector");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, RFInputConverter, RSCMWEVS_VAL_CONV_RX1, RSCMWEVS_VAL_CONV_RX44), 4, "RF Input Converter");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, RFOutputConnector, RSCMWEVS_VAL_CONN_RF1C, RSCMWEVS_VAL_CONN_R43O), 5, "RF Output Connector");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, RFOutputConverter, RSCMWEVS_VAL_CONV_TX1, RSCMWEVS_VAL_CONV_TX44), 6, "RF Output Converter");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, IQOutputConnector, RSCMWEVS_VAL_CONN_IQ2O, RSCMWEVS_VAL_CONN_I48O), 6, "RF Output Converter");

    sprintf (buffer, "ROUT:EVDO:SIGN%ld:SCEN:SCF:EXT %s,%s,%s,%s,%s", instance, 
		rfInputConnectorArr[RFInputConnector], rfInputConverterArr[RFInputConverter],
		rfOutputConnectorArr[RFOutputConnector], rfOutputConverterArr[RFOutputConverter],
												iqOutputConnectorArr[IQOutputConnector]);  

    checkErr (viPrintf (instrSession, "%s\n", buffer));
        
    checkErr (rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}


/// HIFN  Specifies properties of the baseband signal at the I/Q input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR path/Selects the path to be configured.
/// HIPAR PEP/Set the peak envelope power of the incoming baseband signal.
/// HIPAR level/Average level of the incoming baseband signal (without noise).
ViStatus _VI_FUNC rscmwevs_ConfigureIQInput (ViSession instrSession,
                                             ViInt32 instance, ViInt32 path,
                                             ViReal64 PEP, ViReal64 level)
{
	ViStatus	error = VI_SUCCESS;
	ViChar buffer[1024] = ""; 
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, path, 1, 2), 3, "Path");
    viCheckParm (rscmwevs_invalidViReal64Range (instrSession, PEP, -60.0, 0.0), 4, "PEP");

    sprintf (buffer, "CONF:EVDO:SIGN%ld:IQIN:PATH%ld %le,%le", instance, path, PEP, level);;  

    checkErr (viPrintf (instrSession, "%s\n", buffer));
        
    checkErr (rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}




/// HIFN  Queries properties of the baseband signal at the I/Q output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR path/Selects the path to be configured.
/// HIPAR sampleRate/Returns the sample rate.
/// HIPAR PEP/Returns the peak envelope power of the incoming baseband signal.
/// HIPAR crestFactor/Crest factor of the baseband signal
ViStatus _VI_FUNC rscmwevs_QueryIQOutput (ViSession instrSession,
                                             ViInt32 instance, ViInt32 path,
                                             ViInt32 *sampleRate, ViReal64 *PEP, ViReal64 *crestFactor)
{
	ViStatus	error = VI_SUCCESS;
	ViChar command[1024] = "";
	ViChar buffer[1024] = ""; 
	ViChar *p2buf;
	ViString sampleRateArr[] = {"M100"};
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, path, 1, 2), 3, "Path");
	
	sprintf (command, "SENS:EVDO:SIGN%ld:IQO:PATH%ld?", instance, path);
	
	viCheckErr (rscmwevs_QueryViString (instrSession, command, 1024, buffer));
    
    p2buf = strtok (buffer, ",");
    
	if (p2buf == NULL)
        return RS_ERROR_UNEXPECTED_RESPONSE;
    
	*sampleRate = rscmwevs_StringIndex (sampleRateArr, p2buf);
	
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        return RS_ERROR_UNEXPECTED_RESPONSE;

	*PEP = rscmwevs_atof (p2buf);
	
	p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        return RS_ERROR_UNEXPECTED_RESPONSE;
    
	*crestFactor = rscmwevs_atof (p2buf);
	
    checkErr (rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Enables IP-based data tests involving the Data Application Unit (DAU). 
/// HIFN  Can only be set in signal OFF state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR enableDataEndToEnd/Enables IP-based data tests involving the Data Application Unit (DAU). 
/// HIPAR enableDataEndToEnd/Can only be set in signal OFF state.
ViStatus _VI_FUNC rscmwevs_ConfigureDataEndToEndState (ViSession instrSession,
                                                       ViInt32 instance,
                                                       ViBoolean enableDataEndToEnd)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViBoolean (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_DATA_END_TO_END_STATE,
                                               enableDataEndToEnd), 3, "Enable Data End To End");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure RF Signal Frequency
 * Purpose:  Selects the CDMA2000/1xEV-DO band class and RF channel for the 
 *           signaling tests. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRFSignalFrequency(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 bandClass,
    ViInt32 channel
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_OPERATING_BAND_CLASS,
                                               bandClass), 3, "Band Class");
    
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_RF_CHANNEL,
                                               channel), 4, "Channel");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query RF Signal Link Frequency
 * Purpose:  Queries the forward / reverse link frequency, depending on the 
 *           selected band class and channel 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryRFSignalLinkFrequency(
    ViSession   instrSession,
    ViInt32 instance,
    ViReal64*   forwardLinkFrequency,
    ViReal64*   reverseLinkFrequency
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_QUERY_FORWARD_LINK_FREQUENCY,
                                                forwardLinkFrequency), 3, "Forward Link Frequency");
    
    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_QUERY_REVERSE_LINK_FREQUENCY,
                                                reverseLinkFrequency), 4, "Reverse Link Frequency");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure RF Signal Frequency Offset
 * Purpose:  Configures the nominal forward link frequency of the selected 
 *           band class and RF channel by a frequency offset. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRFSignalFrequencyOffset(
    ViSession   instrSession,
    ViInt32 instance,
    ViReal64    frequencyOffset
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_FREQUENCY_OFFSET,
                                                frequencyOffset), 3, "Frequency Offset");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure RF Signal RF Power
 * Purpose:  Defines the absolute power of the generated forward 1xEV-DO 
 *           signal, excluding a possible AWGN contribution. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRFSignalRFPower(
    ViSession   instrSession,
    ViInt32 instance,
    ViReal64    _1xevDOPower
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_SIGNAL_RF_POWER,
                                                _1xevDOPower), 3, "1xEV-DO Power");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query RF Signal Total Output Power
 * Purpose:  Queries the output power at the selected RF output connector: 
 *           the sum of the "1xEV-DO Power and AWGN.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryRFSignalTotalOutputPower(
    ViSession   instrSession,
    ViInt32 instance,
    ViReal64*   outputPower
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_QUERY_RF_SIGNAL_OUTPUT_POWER,
                                                outputPower), 3, "Output Power");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure RF Signal Expected Power
 * Purpose:  This function selects the algorithm which the tester uses to 
 *           configure its input path.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRFSignalExpectedPower(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 expectedPowerMode,
    ViReal64    manualExpectedPower
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_EXPECTED_POWER_MODE,
                                               expectedPowerMode), 3, "Expected Power Mode");
    
    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_MANUAL_EXPECTED_POWER,
                                                manualExpectedPower), 4, "Manual Expected Power");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query RF Signal Expected Power
 * Purpose:  Queries the calculated value of the expected input power from 
 *           the AT.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryRFSignalExpectedPower(
    ViSession   instrSession,
    ViInt32 instance,
    ViReal64*   expectedNominalPower
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_QUERY_RF_SIGNAL_EXPECTED_POWER,
                                                expectedNominalPower), 3, "Expected Nominal Power");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure RF Signal AWGN
 * Purpose:  Configures the general AWGN generator settings.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRFSignalAWGN(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 awgnMode,
    ViReal64    awgnLevel
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_AWGN_GENERATOR_MODE,
                                               awgnMode), 3, "AWGN Mode");
    
    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_AWGN_GENERATOR_LEVEL,
                                                awgnLevel), 4, "AWGN Level");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Forward Traffic Channel
 * Purpose:  Defines the preamble MAC index of the MAC layer packets 
 *           transmitted on the forward traffic channel. 
 *           
 *           Note(s):
 *           
 *           (1) Preamble MAC index of the MAC layer packets transmitted on 
 *           the forward traffic channel. The MAC index specifies the address 
 *           of the single user MAC layer packets and the packet size of 
 *           multi-user MAC layer packets.
 *           
 *           (2) The R&S CMW transmits single user packets. With MAC layer 
 *           protocol revision 0, MAC indices between 5 and 63 are supported. 
 *           Revision A provides the additional values 72 to 127. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureForwardTrafficChannel(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 MACIndexRev0,
    ViInt32 MACIndexRevA,
    ViInt32 MACIndexRevB
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, MACIndexRev0, 5, 63), 3, "MAC Index Rev 0");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, MACIndexRevA, 5, 127), 4, "MAC Index Rev A");
    
    sprintf (command, "CONF:EVDO:SIGN%ld:MAC:IND %ld,%ld,%ld\n", instance, MACIndexRev0, MACIndexRevA, MACIndexRevB);
    
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr( rscmwevs_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Reverse Power Control
 * Purpose:  Configures the closed loop power control settings. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureReversePowerControl(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 powerControlBits,
    ViReal64    stepSize
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_REVERSE_POWER_CONTROL_BITS,
                                               powerControlBits), 3, "Power Control Bits");
    
    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_REVERSE_POWER_CONTROL_STEP_SIZE,
                                                stepSize), 4, "Step Size");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Routing
 * Purpose:  This function selects the input and output connectors for the 
 *           generated RF signal.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRouting(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 rfInputConnector,
    ViInt32 rfInputConverter,
    ViInt32 rfOutputConnector,
    ViInt32 rfOutputConverter
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, rfInputConnector, RSCMWEVS_VAL_CONN_RF1C, RSCMWEVS_VAL_CONN_R44C), 3, "RF Input Connector");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, rfInputConverter, RSCMWEVS_VAL_CONV_RX1, RSCMWEVS_VAL_CONV_RX44), 4, "RF Input Converter");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, rfOutputConnector, RSCMWEVS_VAL_CONN_RF1C, RSCMWEVS_VAL_CONN_R43O), 5, "RF Output Connector");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, rfOutputConverter, RSCMWEVS_VAL_CONV_TX1, RSCMWEVS_VAL_CONV_TX44), 6, "RF Output Converter");
    
    sprintf (command, "ROUT:EVDO:SIGN%ld:SCEN:SCEL %s,%s,%s,%s", instance, rfInputConnectorArr[rfInputConnector],
        rfInputConverterArr[rfInputConverter], rfOutputConnectorArr[rfOutputConnector],
        rfOutputConverterArr[rfOutputConverter]);
    
    viCheckErr (viPrintf (instrSession, command));
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query Signal Routing
 * Purpose:  This function returns the configured routing settings.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QuerySignalRouting (ViSession instrSession,
                                               ViInt32 instance,
                                               ViInt32 *scenario,
                                               ViChar controller[],
                                               ViInt32 *RXConnector,
                                               ViInt32 *RXConverter,
                                               ViInt32 *TXConnector,
                                               ViInt32 *TXConverter,
                                               ViInt32 *IQOutputConnector)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[1024];
    ViChar      *p2buf;

    
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckErr (viQueryf (instrSession, "ROUT:EVDO:SIGN%ld?\n"," %[^\r\n]", 
                instance, command));
    
    p2buf = strtok (command, ",");
    if (p2buf == NULL)
        return RS_ERROR_UNEXPECTED_RESPONSE;
    *scenario = rscmwevs_StringIndex (ScenarioArr,p2buf);
    if (*scenario < 0)
        return RS_ERROR_UNEXPECTED_RESPONSE;
     
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        return RS_ERROR_UNEXPECTED_RESPONSE;
    strcpy (controller, p2buf);                                        
    if (*controller < 0)
        return RS_ERROR_UNEXPECTED_RESPONSE;

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        return RS_ERROR_UNEXPECTED_RESPONSE;
    *RXConnector = rscmwevs_StringIndex (rfInputConnectorArr,p2buf);
    if (*RXConnector < 0)
        return RS_ERROR_UNEXPECTED_RESPONSE;
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        return RS_ERROR_UNEXPECTED_RESPONSE;
    *RXConverter = rscmwevs_StringIndex (rfInputConverterArr,p2buf);
    if (*RXConverter < 0)
        return RS_ERROR_UNEXPECTED_RESPONSE;
    
    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        return RS_ERROR_UNEXPECTED_RESPONSE;
    *TXConnector = rscmwevs_StringIndex (rfOutputConnectorArr,p2buf);
    if (*TXConnector < 0)
        return RS_ERROR_UNEXPECTED_RESPONSE;

    p2buf = strtok (NULL, ",");
    if (p2buf == NULL)
        return RS_ERROR_UNEXPECTED_RESPONSE;
    *TXConverter = rscmwevs_StringIndex (rfOutputConverterArr,p2buf);
    if (*TXConverter < 0)
        return RS_ERROR_UNEXPECTED_RESPONSE;
	
	p2buf = strtok (NULL, ",\n");
    if (p2buf != NULL)
	{
    	*IQOutputConnector = rscmwevs_StringIndex (iqOutputConnectorArr,p2buf);
    	if (*IQOutputConnector < 0)
        	return RS_ERROR_UNEXPECTED_RESPONSE;
	}

    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query RF Signal Active Scenario
 * Purpose:  Queries the active scenario.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryRFSignalActiveScenario (ViSession instrSession,
                                                        ViInt32 instance,
                                                        ViInt32* scenario)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_ACTIVE_SCENARIO,
                                               scenario), 3, "Scenario");
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Forward Traffic Channel Index
 * Purpose:  Configures the data rate index for FTAP packets. Data rate 
 *           index "0" stops the flow of FTAP packets to the AT.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureForwardTrafficChannelIndex(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 index
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_LAYER_FORWARD_TRAFFIC_CHANNEL_INDEX,
                                               index), 3, "Index");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query Forward Traffic Channel
 * Purpose:  Queries the Forward Traffic Channel data rate and slot count.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryForwardTrafficChannel(
    ViSession   instrSession,
    ViInt32 instance,
    ViReal64*   rate,
    ViInt32*    slots
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_QUERY_LAYER_FORWARD_TRAFFIC_CHANNEL_DATA_RATE,
                                                rate), 3, "Rate");
    viCheckParm (rscmwevs_GetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_QUERY_LAYER_FORWARD_TRAFFIC_CHANNEL_SLOTS,
                                               slots), 4, "Slots");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query Forward Traffic Channel Loopback
 * Purpose:  Queries whether the AT under test can transmit FTAP loopback 
 *           packets in order to provide Packet Error Rate (PER) information.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryForwardTrafficChannelLoopback(
    ViSession   instrSession,
    ViInt32 instance,
    ViBoolean*  loopbackEnable
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViBoolean (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_QUERY_LAYER_FORWARD_TRAFFIC_CHANNEL_LOOPBACK_ENABLE,
                                                 loopbackEnable), 3, "Loopback Enable");

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configures ACK Traffic Channel
 * Purpose:  Configures the ACK channel in the reverse signal that the AT 
 *           uses to acknowledge/not acknowledge test packets received on the 
 *           forward traffic channel.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfiguresACKTrafficChannel(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 ackChFixedMode
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_LAYER_ACK_TRAFFIC_CHANNEL,
                                               ackChFixedMode), 3, "Ack Ch Fixed Mode");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Reverse Traffic Channel Index
 * Purpose:  Selects the minimum and maximum data rate index for RTAP 
 *           packets.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureReverseTrafficChannelIndex(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 minIndex,
    ViInt32 maxIndex
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_LAYER_ACK_TRAFFIC_CHANNEL_MAX_INDEX,
                                               maxIndex), 4, "Max Index");
    
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_LAYER_ACK_TRAFFIC_CHANNEL_MIN_INDEX,
                                               minIndex), 3, "Min Index");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query Reverse Traffic Channel Rate
 * Purpose:  Queries the data rate for the selected maximum and minimum data 
 *           rate index.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryReverseTrafficChannelRate(
    ViSession   instrSession,
    ViInt32 instance,
    ViReal64*   minDataRate,
    ViReal64*   maxDataRate
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_QUERY_LAYER_ACK_TRAFFIC_CHANNEL_DATA_RATE_MIN,
                                                minDataRate), 3, "Min Data Rate");
    
    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_QUERY_LAYER_ACK_TRAFFIC_CHANNEL_DATA_RATE_MAX,
                                                maxDataRate), 4, "Max Data Rate");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Forward Traffic Channel A Packet Type
 * Purpose:  Configures the packet type for FETAP packets
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureForwardTrafficChannelAPacketType(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 packetType
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_LAYER_FORWARD_TRAFFIC_CHANNEL_A_PACKET_TYPE,
                                               packetType), 3, "Packet Type");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query Forward Traffic Channel A
 * Purpose:  Queries the Forward Traffic Channel A index, packet size, data 
 *           rate and slot count.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryForwardTrafficChannelA(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32*    index,
    ViInt32*    packetSize,
    ViReal64*   dataRate,
    ViInt32*    slots
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_QUERY_LAYER_FORWARD_TRAFFIC_CHANNEL_A_INDEX,
                                               index), 3, "Index");
    
    viCheckParm (rscmwevs_GetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_QUERY_LAYER_FORWARD_TRAFFIC_CHANNEL_A_PACKET_SIZE,
                                               packetSize), 4, "Packet Size");
    
    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_QUERY_LAYER_FORWARD_TRAFFIC_CHANNEL_A_DATA_RATE,
                                                dataRate), 5, "Data Rate");
    
    viCheckParm (rscmwevs_GetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_QUERY_LAYER_FORWARD_TRAFFIC_CHANNEL_A_SLOTS,
                                               slots), 6, "Slots");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query Forward Traffic Channel A Loopback
 * Purpose:  Queries whether the AT under test can transmit FETAP loopback 
 *           packets in order to provide Packet Error Rate (PER) information.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryForwardTrafficChannelALoopback(
    ViSession   instrSession,
    ViInt32 instance,
    ViBoolean*  loopbackEnable
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViBoolean (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_QUERY_LAYER_FORWARD_TRAFFIC_CHANNEL_A_LOOPBACK_ENABLE,
                                                 loopbackEnable), 3, "Loopback Enable");

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configures ACK Traffic Channel A
 * Purpose:  Configures the ACK channel in the reverse signal that the AT 
 *           uses to acknowledge/not acknowledge test packets received on the 
 *           forward traffic channel. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfiguresACKTrafficChannelA(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 ackChFixedMode
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_LAYER_ACK_TRAFFIC_CHANNEL_A,
                                               ackChFixedMode), 3, "Ack Ch Fixed Mode");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query ACK Traffic Channel A
 * Purpose:  Queries the status of the "ACK Channel Modulation Type Fixed" 
 *           mode where the AT can select a specific modulation type for the 
 *           reverse ACK channel. At present, this mode is disabled.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryACKTrafficChannelA(
    ViSession   instrSession,
    ViInt32 instance,
    ViBoolean*  ackChFixedModulation
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViBoolean (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_QUERY_LAYER_ACK_TRAFFIC_CHANNEL_A,
                                                 ackChFixedModulation), 3, "ACK Ch Fixed Modulation");

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Reverse Traffic Channel A Index
 * Purpose:  Selects the minimum and maximum data rate index for RETAP 
 *           packets.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureReverseTrafficChannelAIndex(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 minIndex,
    ViInt32 maxIndex
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_LAYER_ACK_CHANNEL_A_MAX_INDEX,
                                               maxIndex), 4, "Max Index");
    
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_LAYER_ACK_CHANNEL_A_MIN_INDEX,
                                               minIndex), 3, "Min Index");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query Reverse Traffic Channel A Rate
 * Purpose:  Queries the data rate for the selected maximum and minimum data 
 *           rate index.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryReverseTrafficChannelARate(
    ViSession   instrSession,
    ViInt32 instance,
    ViReal64*   minDataRate,
    ViReal64*   maxDataRate
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_QUERY_LAYER_ACK_CHANNEL_A_DATA_RATE_MIN,
                                                minDataRate), 3, "Min Data Rate");
    
    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_QUERY_LAYER_ACK_CHANNEL_A_DATA_RATE_MAX,
                                                maxDataRate), 4, "Max Data Rate");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  The termination target for RETAP test packets.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR terminationTarget/The termination target for RETAP test packets.
ViStatus _VI_FUNC rscmwevs_ConfigureReverseTrafficTerminationTarget (ViSession instrSession,
                                                                     ViInt32 instance,
                                                                     ViInt32 terminationTarget)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32(instrSession, rep_cap,
                                                RSCMWEVS_ATTR_LAYER_ACK_CHANNEL_A_TERMINATION_TARGET,
                                                terminationTarget), 3, "Termination Target");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Selects the packet application the R&S CMW500 initially proposes to 
/// HIFN  the AT druing session negotiation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR packetApplication/Selects the packet application the R&S CMW500 initially proposes to 
/// HIPAR packetApplication/the AT druing session negotiation.
ViStatus _VI_FUNC rscmwevs_ConfigurePacketPreferredApplication (ViSession instrSession,
                                                                ViInt32 instance,
                                                                ViInt32 packetApplication)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32(instrSession, rep_cap,
                                                RSCMWEVS_ATTR_LAYER_ACK_CHANNEL_A_PREFERED_APPLICATION,
                                                packetApplication), 3, "Packet Application");  
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Forward Traffic Channel A MAC
 * Purpose:  Configures the MAC layer for subtype 2 signals. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureForwardTrafficChannelAMAC(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 dscValue,
    ViReal64    dscChannelGainBase
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_LAYER_FORWARD_TRAFFIC_CHANNEL_A_DSC_VALUE,
                                               dscValue), 3, "DSC Value");
    
    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_LAYER_FORWARD_TRAFFIC_CHANNEL_A_DSC_GAIN_BASE,
                                                dscChannelGainBase), 4, "DSC Channel Gain Base");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure RF Carrier Channel
 * Purpose:  Defines the channel settings for a carrier in the sector
 *           implemented by the Signaling Application. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRFCarrierChannel (ViSession instrSession,
                                                      ViInt32 instance,
                                                      ViInt32 carrier,
                                                      ViInt32 carrierChannel,
                                                      ViInt32 carrierChannelOffset)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_RF_CARRIER_SETTING,
                                               carrier), 3, "Carrier");
    
    if (carrier == 0)
    {
        viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                                   RSCMWEVS_ATTR_RF_CARRIER_CHANNEL,
                                                   carrierChannel), 4, "Carrier Channel");
    }
    else
    {
        viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                                   RSCMWEVS_ATTR_RF_CARRIER_CHANNEL_OFFSET,
                                                   carrierChannelOffset), 5, "Carrier Channel Offset");
    }
    
    
Error:
    Rs_UnlockSession(instrSession, NULL);
    return error;
}

/*****************************************************************************
 * Function: Query RF Carrier Link Frequency
 * Purpose:  Returns the (forward and reverse) link frequency for a carrier
 *           in the cell implemented by the Signaling Application.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryRFCarrierLinkFrequency (ViSession instrSession,
                                                        ViInt32 instance,
                                                        ViReal64 *forwardLinkFrequency,
                                                        ViReal64 *reverseLinkFrequency)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_RF_CARRIER_FORWARD_LINK_FREQUENCY,
                                               forwardLinkFrequency), 3, "Forward Link Frequency");
 
    
    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_RF_CARRIER_REVERSE_LINK_FREQUENCY,
                                               reverseLinkFrequency), 4, "Reverse Link Frequency");
    
Error:
    Rs_UnlockSession(instrSession, NULL); 
    return error;
}

/*****************************************************************************
 * Function: Configure RF Carrier Power Level
 * Purpose:  Defines the absolute and relative EV-DO power level for a
 *           carrier.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRFCarrierPowerLevel (ViSession instrSession,
                                                         ViInt32 instance,
                                                         ViInt32 powerMode,
                                                         ViReal64 level)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSCMWEVS_IO_BUFFER_SIZE] = "";
    
    checkErr (Rs_LockSession (instrSession, NULL));
    
    sprintf(buffer, "i%ld,%s", instance, PowerModeArr[powerMode]);

    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, buffer,
                                               RSCMWEVS_ATTR_RF_CARRIER_POWER_LEVEL,
                                               level), 3, "Level");
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure RF Carrier Pilot States
 * Purpose:  Configures the pilot.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRFCarrierPilotState (ViSession instrSession,
                                                         ViInt32 instance,
                                                         ViInt32 pilot,
                                                         ViBoolean active_onAN,
                                                         ViBoolean assigned_toAT)
{
   ViStatus    error = VI_SUCCESS;
   ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_RF_CARRIER_PILOT_SETTING,
                                               pilot), 3, "Pilot");
 
    viCheckParm (rscmwevs_SetAttributeViBoolean (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_RF_CARRIER_ACTIVE_ON_AN,
                                                active_onAN), 4, "Active on AN");
    
    if (active_onAN == VI_TRUE)
    {
        viCheckParm (rscmwevs_SetAttributeViBoolean (instrSession, rep_cap,
                                                    RSCMWEVS_ATTR_RF_CARRIER_ASSIGNED_TO_AT,
                                                    assigned_toAT), 5, "Assigned to AT");
    }
    
Error:
    Rs_UnlockSession(instrSession, NULL);
    return error;      
}

/*****************************************************************************
 * Function: Query RF Carrier Pilot AT State
 * Purpose:  Queries if a pilot is currently acquired by the AT, i.e. if the
 *           corresponding carrier is currently carrying traffic.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryRFCarrierPilotATState (ViSession instrSession,
                                                       ViInt32 instance,
                                                       ViBoolean *acquired_byAT)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViBoolean (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_RF_CARRIER_PILOT_AT_STATE,
                                                 acquired_byAT), 3, "Acquired by AT");
    
Error:
    Rs_UnlockSession(instrSession, NULL);
    return error;       
}

/*****************************************************************************
 * Function: Configure Session Layer
 * Purpose:  Configures the session layer.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureSessionLayer(
    ViSession   instrSession,
    ViInt32 instance,
    ViReal64    inactiveSessionTimeout,
    ViBoolean   subnetIncluded
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_SESSION_LAYER_INACTIVE_SESSION_TIMEOUT,
                                                inactiveSessionTimeout), 3, "Inactive Session Timeout");
    
    viCheckParm (rscmwevs_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_SESSION_LAYER_SUBNET_INCLUDED,
                                                 subnetIncluded), 4, "Subnet Included");

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Connection Layer
 * Purpose:  Configures the connection layer for subtype 0/1 signals.
 *           
 *           Note(s):
 *           
 *           (1)  The settings for subtype 0/1 signals and for subtype 2 
 *           signals are identical and overwrite each other. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureConnectionLayer(
    ViSession   instrSession,
    ViInt32 instance,
    ViReal64    pilotDropThreshold,
    ViInt32 reverseLinkFrameOffset,
    ViBoolean   redirectOverheadMessages
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_CONNECTION_LAYER_PILOT_DROP_THRESHOLD,
                                                pilotDropThreshold), 3, "Pilot Drop Threshold");
    
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_CONNECTION_LAYER_REVERSE_LINK_FRAME_OFFSET,
                                               reverseLinkFrameOffset), 4, "Reverse Link Frame Offset");
    
    viCheckParm (rscmwevs_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_CONNECTION_LAYER_REDIRECT_OVERHEAD_MESSAGES,
                                                 redirectOverheadMessages), 5, "Redirect Overhead Messages");

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Sets the T2P values in the session negotiation with respect to a test 
/// HIFN  purpose.
/// HIFN     
/// HIFN     Attribute:
/// HIFN     RSCMWEVS_ATTR_MAC_LAYER_T2P_VALUE_OPTIMIZATION
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:EVDO:SIGN<instance>:LAYer:MAC:TTOPt
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR t2pValueOptimization/Sets the T2P values in the session negotiation with respect to a test 
/// HIPAR t2pValueOptimization/purpose.
ViStatus _VI_FUNC rscmwevs_ConfigureMACLayerT2PValueOptimization(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32	t2pValueOptimization
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_MAC_LAYER_T2P_VALUE_OPTIMIZATION,
                                               t2pValueOptimization), 3, "T2P Value Optimization");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}


/*****************************************************************************
 * Function: Configure MAC Layer Control Protocol
 * Purpose:  This function configures the MAC layer.
 *           
 *           Note(s):
 *           
 *           (1) All settings are valid for subtype 0/1 and for subtype 2 
 *           signals.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureMACLayerControlProtocol(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 dataRate,
    ViInt32 maxProbeSequences,
    ViInt32 interProbeBackoff,
    ViInt32 interProbeSequenceBackoff
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_MAC_LAYER_PROTOCOL_DATA_RATE,
                                               dataRate), 3, "Data Rate");
    
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_MAC_LAYER_PROTOCOL_MAX_PROBE_SEQUENCES,
                                               maxProbeSequences), 4, "Max Probe Sequences");
    
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_MAC_LAYER_PROTOCOL_INTER_PROBE_BACKOFF,
                                               interProbeBackoff), 5, "Inter Probe Backoff");
    
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_MAC_LAYER_PROTOCOL_INTER_PROBE_SEQUENCE_BACKOFF,
                                               interProbeSequenceBackoff), 6, "Inter Probe Sequence Backoff");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query MAC Layer Acces Session Seed
 * Purpose:  This function queries the session seed parameter which is 
 *           negotiated between the R&S CMW and the AT. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryMACLayerAccesSessionSeed(
    ViSession   instrSession,
    ViInt32 instance,
    ViReal64*   sessionSeed
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_QUERY_MAC_LAYER_ACCES_SESSION_SEED,
                                                sessionSeed), 3, "Session Seed");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure MAC Layer
 * Purpose:  This function configures the MAC layer.
 *           
 *           Note(s):
 *           
 *           (1) All settings are valid for subtype 0/1 and for subtype 2 
 *           signals.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureMACLayer(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 subtype,
    ViInt32 drcCover,
    ViInt32 drcLength,
    ViReal64    drcChannelGain,
    ViReal64    ackChannelGain
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
    ViAttr      attr[4][2] = {{RSCMWEVS_ATTR_MAC_LAYER_PROTOCOL_DRC_COVER, RSCMWEVS_ATTR_MAC_LAYER_SUBTYPE2_PROTOCOL_DRC_COVER},
                              {RSCMWEVS_ATTR_MAC_LAYER_PROTOCOL_DRC_LENGTH,RSCMWEVS_ATTR_MAC_LAYER_SUBTYPE2_PROTOCOL_DRC_LENGTH},
                              {RSCMWEVS_ATTR_MAC_LAYER_PROTOCOL_DRC_CHANNEL_GAIN,RSCMWEVS_ATTR_MAC_LAYER_SUBTYPE2_PROTOCOL_DRC_CHANNEL_GAIN},
                              {RSCMWEVS_ATTR_MAC_LAYER_PROTOCOL_ACK_CHANNEL_GAIN,RSCMWEVS_ATTR_MAC_LAYER_SUBTYPE2_PROTOCOL_ACK_CHANNEL_GAIN}};

    checkErr (Rs_LockSession (instrSession, NULL));
	viCheckErr (rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, subtype, RSCMWEVS_VAL_LAYER_MAC_SUBTYPE01, RSCMWEVS_VAL_LAYER_MAC_SUBTYPE2), 3, "Subtype");

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               attr[0][subtype],
                                               drcCover), 4, "DRC Cover");
    
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               attr[1][subtype],
                                               drcLength), 5, "DRC Length");
    
    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                attr[2][subtype],
                                                drcChannelGain), 6, "DRC Channel Gain");
    
    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                attr[3][subtype],
                                                ackChannelGain), 7, "ACK Channel Gain");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Reverse Traffic Channel MAC Data Rate
 * Purpose:  Defines the ratio of the Reverse Traffic Channel power at 
 *           different data rates to the Reverse Pilot Channel power.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureReverseTrafficChannelMACDataRate(
    ViSession   instrSession,
    ViInt32 instance,
    ViReal64    data96_kbps,
    ViReal64    data192_kbps,
    ViReal64    data384_kbps,
    ViReal64    data768_kbps,
    ViReal64    data1536_kbps
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViReal64Range (instrSession, data96_kbps, -2.0, 1.75), 3, "Data 9.6 kbps");
    viCheckParm (rscmwevs_invalidViReal64Range (instrSession, data192_kbps, -2.0, 1.75), 4, "Data 19.2 kbps");
    viCheckParm (rscmwevs_invalidViReal64Range (instrSession, data384_kbps, -2.0, 1.75), 5, "Data 38.4 kbps");
    viCheckParm (rscmwevs_invalidViReal64Range (instrSession, data768_kbps, -2.0, 1.75), 6, "Data 76.8 kbps");
    viCheckParm (rscmwevs_invalidViReal64Range (instrSession, data1536_kbps, -2.0, 1.75), 7, "Data 153.6 kbps");
    
    sprintf (command, "CONF:EVDO:SIGN%ld:LAY:MAC:DRTP:DRAT %lf,%lf,%lf,%lf,%lf\n", 
                                                                       instance, 
                                                                       data96_kbps, 
                                                                       data192_kbps,
                                                                       data384_kbps,
                                                                       data768_kbps,
                                                                       data1536_kbps);
    
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr( rscmwevs_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Reverse Traffic Channel MAC Increase Transition
 * Purpose:  Defines the probability of the access terminal to increase its 
 *           transmission rate to the next higher data rate.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureReverseTrafficChannelMACIncreaseTransition(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 data96_kbps,
    ViInt32 data192_kbps,
    ViInt32 data384_kbps,
    ViInt32 data768_kbps
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, data96_kbps, 0, 255), 3, "Data 9.6 kbps");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, data192_kbps, 0, 255), 4, "Data 19.2 kbps");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, data384_kbps, 0, 255), 5, "Data 38.4 kbps");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, data768_kbps, 0, 255), 6, "Data 76.8 kbps");
    
    sprintf (command, "CONF:EVDO:SIGN%ld:LAY:MAC:DRTP:ITR #H%X,#H%X,#H%X,#H%X\n", 
                                                                       instance, 
                                                                       data96_kbps, 
                                                                       data192_kbps,
                                                                       data384_kbps,
                                                                       data768_kbps);
    
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr( rscmwevs_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Reverse Traffic Channel MAC Decrease Transition
 * Purpose:  Defines the probability of the access terminal to decrease its 
 *           transmission rate to the next higher data rate. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureReverseTrafficChannelMACDecreaseTransition(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 data192_kbps,
    ViInt32 data384_kbps,
    ViInt32 data768_kbps,
    ViInt32 data1536_kbps
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, data192_kbps, 0, 255), 3, "Data 19.2 kbps");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, data384_kbps, 0, 255), 4, "Data 38.4 kbps");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, data768_kbps, 0, 255), 5, "Data 76.8 kbps");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, data1536_kbps, 0, 255), 6, "Data 153.6 kbps");
    
    sprintf (command, "CONF:EVDO:SIGN%ld:LAY:MAC:DRTP:DTR #H%X,#H%X,#H%X,#H%X\n", 
                                                                       instance, 
                                                                       data192_kbps,
                                                                       data384_kbps,
                                                                       data768_kbps,
                                                                       data1536_kbps);
    
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr( rscmwevs_CheckStatus (instrSession));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Forward Traffic Channel B Packet Type
 * Purpose:  Selects the type of FMCTAP test packets that shall be sent on a
 *           carrier.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureForwardTrafficChannelBPacketType (ViSession instrSession,
                                                                      ViInt32 instance,
                                                                      ViInt32 packetType)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_LAYER_FORWARD_TRAFFIC_CHANNEL_B_PACKET_TYPE,
                                               packetType), 3, "Packet Type");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query Forward Traffic Channel B
 * Purpose:  Queries the Forward Traffic Channel B index, packet size, data
 *           rate and slot count.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryForwardTrafficChannelB (ViSession instrSession,
                                                        ViInt32 instance,
                                                        ViInt32 *index,
                                                        ViInt32 *packetSize,
                                                        ViReal64 *dataRate,
                                                        ViInt32 *slots)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_QUERY_LAYER_FORWARD_TRAFFIC_CHANNEL_B_INDEX,
                                               index), 3, "Index");
    
    viCheckParm (rscmwevs_GetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_QUERY_LAYER_FORWARD_TRAFFIC_CHANNEL_B_PACKET_SIZE,
                                               packetSize), 4, "Packet Size");
    
    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_QUERY_LAYER_FORWARD_TRAFFIC_CHANNEL_B_DATA_RATE,
                                               dataRate), 5, "Data Rate");
   
    viCheckParm (rscmwevs_GetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_QUERY_LAYER_FORWARD_TRAFFIC_CHANNEL_B_SLOTS,
                                               slots), 6, "Slots");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query Forward Traffic Channel B Loopback
 * Purpose:  Queries whether the AT under test shall transmit FMCTAP loopback
 *           packets in order to provide Packet Error Rate (PER) information.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryForwardTrafficChannelBLoopback (ViSession instrSession,
                                                                ViInt32 instance,
                                                                ViBoolean *loopbackEnable)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViBoolean (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_QUERY_LAYER_FORWARD_TRAFFIC_CHANNEL_B_LOOPBACK_ENABLE,
                                                 loopbackEnable), 3, "Loopback Enable");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure ACK Traffic Channel B
 * Purpose:  Configures the ACK channel bit fixed mode for a carrier.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureACKTrafficChannelB (ViSession instrSession,
                                                        ViInt32 instance,
                                                        ViInt32 fixedMode)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_LAYER_ACK_TRAFFIC_CHANNEL_B,
                                               fixedMode), 3, "Fixed Mode");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query ACK Traffic Channel B
 * Purpose:  Queries if the the ACK Channel Modulation Type Fixed mode (see
 *           3GPP2 C.S0029) is enabled for a carrier. Currently this mode is
             always switched off.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryACKTrafficChannelB (ViSession instrSession,
                                                    ViInt32 instance,
                                                    ViBoolean *fixedModulationType)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViBoolean (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_QUERY_LAYER_ACK_TRAFFIC_CHANNEL_B,
                                                 fixedModulationType), 3, "Fixed Modulation Type");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error; 
}

/*****************************************************************************
 * Function: Configure Reverse Traffic Channel B Index
 * Purpose:  Selects the minimum and maximum packet size index for RMCTAP
 *           test packets on this carrier.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureReverseTrafficChannelBIndex (ViSession instrSession,
                                                                 ViInt32 instance,
                                                                 ViInt32 minIndex,
                                                                 ViInt32 maxIndex)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_LAYER_ACK_CHANNEL_B_MIN_INDEX,
                                               minIndex), 3, "Min Index");
    
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_LAYER_ACK_CHANNEL_B_MAX_INDEX,
                                               maxIndex), 4, "Max Index");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;   
}

/*****************************************************************************
 * Function: Query Reverse Traffic Channel B Packet Size
 * Purpose:  Queries the minimum and the maximum RMCTAP test packet size on a
             carrier.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryReverseTrafficChannelBPacketSize (ViSession instrSession,
                                                                  ViInt32 instance,
                                                                  ViReal64 *minPacketSize,
                                                                  ViReal64 *maxPacketSize)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_LAYER_RMCTAP_MIN_PACKET_SIZE,
                                                minPacketSize), 3, "Min Packet Size");
    
    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_LAYER_RMCTAP_MAX_PACKET_SIZE,
                                                maxPacketSize), 4, "Max Packet Size");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error; 
}

/*****************************************************************************
 * Function: Configure Network System Parameters
 * Purpose:  This function provides basic network settings.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureNetworkSystemParameters(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 release,
    ViInt32 systemID
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_SYSTEM_ID,
                                               systemID), 3, "System ID");
    
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_RELEASE,
                                               release), 4, "Release");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Network Access Properties
 * Purpose:  This function defines general settings for the simulated access 
 *           network.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureNetworkAccessProperties(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 pnOffset,
    ViReal64    callLossDetectTime,
    ViReal64    atForwardPacketActivity,
	ViBoolean interRATEnabled
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_ACCESS_PROPERTIES_PN_OFFSET,
                                               pnOffset), 3, "PN Offset");
    
    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_NETWORK_ACCESS_PROPERTIES_CALL_LOSS_DETECT_TIME,
                                                callLossDetectTime), 4, "Call Loss Detect Time");
    
    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_NETWORK_ACCESS_PROPERTIES_AT_FORWARD_PACKET_ACTIVITY,
                                                atForwardPacketActivity), 5, "AT Forward Packet Activity");
	viCheckParm (rscmwevs_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_NETWORK_ACCESS_PROPERTIES_IRAT_ENABLED,
                                                 interRATEnabled), 6, "Inter RAT Enabled");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Network Sector
 * Purpose:  This function defines information of the R&S CMW's sector.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureNetworkSector(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 colorCode,
    ViInt32 subnetMask,
    ViInt32 countryCode
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_SECTOR_COLOR_CODE,
                                               colorCode), 3, "Color Code");
    
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_SECTOR_SUBNET_MASK,
                                               subnetMask), 4, "Subnet Mask");
    
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_SECTOR_COUNTRY_CODE,
                                               countryCode), 5, "Country Code");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Network ANSI 41
 * Purpose:  This function configures the ANSI-41 format of the sector 
 *           settings.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureNetworkANSI41(
    ViSession   instrSession,
    ViInt32 instance,
    ViString    sectorID,
    ViInt32 numberOfParityBits
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_SECTOR_FORMAT,
                                               RSCMWEVS_VAL_NETWORK_SECTOR_FORMAT_A41N), 1, "Dummy");
    
    viCheckParm (rscmwevs_SetAttributeViString (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_NETWORK_ANSI_41_SECTOR_ID,
                                                sectorID), 3, "Sector ID");
    
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_ANSI_41_NUMBER_OF_PARITY_BITS,
                                               numberOfParityBits), 4, "Number Of Parity Bits");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query Network ANSI 41 Overall ID
 * Purpose:  This function queries the 128-bit overall ID.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryNetworkANSI41OverallID(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 bufferSize,
    ViChar  overallID[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViString (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_QUERY_NETWORK_ANSI_41_OVERALL_ID,
                                                bufferSize, overallID), 4, "Overall ID");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Network Manual
 * Purpose:  This function configures Overall ID for "Manual" format.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureNetworkManual(
    ViSession   instrSession,
    ViInt32     instance,
    ViString    overallID
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_SECTOR_FORMAT,
                                               RSCMWEVS_VAL_NETWORK_SECTOR_FORMAT_MAN), 1, "Dummy");
    
    viCheckParm (rscmwevs_SetAttributeViString (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_NETWORK_MANUAL_OVERALL_ID,
                                                overallID), 3, "Overall ID");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Network Access Probes State
 * Purpose:  This function configures whether the tester acknowledges or 
 *           ignores access probes from the AT.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureNetworkAccessProbesState(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 accessProbes
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_ACCESS_PROBE_STATE,
                                               accessProbes), 3, "Access Probes");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Network Access Probes Power
 * Purpose:  This function configures the access probes power settings.
  *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureNetworkAccessProbesPower(
    ViSession   instrSession,
    ViInt32 instance,
    ViReal64    initialAdjust,
    ViReal64    openLoopAdjust,
    ViReal64    probeIncrement,
    ViInt32 probesperSequence
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_NETWORK_ACCESS_PROBES_INITIAL_ADJUST,
                                                initialAdjust), 3, "Initial Adjust");
    
    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_NETWORK_ACCESS_PROBES_OPEN_LOOP_ADJUST,
                                                openLoopAdjust), 4, "Open Loop Adjust");
    
    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_NETWORK_ACCESS_PROBES_INCREMENT,
                                                probeIncrement), 5, "Probe Increment");
    
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_ACCESS_PROBES_PER_SEQUENCE,
                                               probesperSequence), 6, "Probes per Sequence");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Network Access Probes Lengths
 * Purpose:  This function configures the access probes length settings.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureNetworkAccessProbesLengths(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 preambleLength,
    ViInt32 accessCycleDuration,
    ViInt32 preambleLengthSlots
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_ACCESS_PROBES_PREAMBLE_LENGTH,
                                               preambleLength), 3, "Preamble Length");
    
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_ACCESS_PROBES_CYCLE_DURATION,
                                               accessCycleDuration), 4, "Access Cycle Duration");
    
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_ACCESS_PROBES_PREAMBLE_LENGTH_SLOTS,
                                               preambleLengthSlots), 5, "Preamble Length Slots");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Network Access Probes Sector Access Max Rate
 * Purpose:  This function configures the maximum rate at which the AT may 
 *           transmit on the access channel.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureNetworkAccessProbesSectorAccessMaxRate(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 sectorAccessMaxRate
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_ACCESS_PROBES_SECTOR_ACCESS_MAXIMUM_RATE,
                                               sectorAccessMaxRate), 3, "Sector Access Max Rate");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  This function allows to specify the parameters required for EAP-AKA 
/// HIFN  access authentication that is required for EMPA.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR secretKey/The shared authentication key (32 hexadecimal digits).
/// HIPAR operatorVariantKey/The value OPc derived from the "Secret Key" and the operator dependent 
/// HIPAR operatorVariantKey/value OP (32 hexadecimal digits).
/// HIPAR authenticationManagementField/The AMF input parameter (4 hexadecimal digits) of the AKA' 
/// HIPAR authenticationManagementField/authentication algorithm
/// HIPAR sequenceNumber/The sequence ssent to the AT in the EAP-Request/AKA'-Challenge 
/// HIPAR sequenceNumber/message.
ViStatus _VI_FUNC rscmwevs_ConfigureNetworkSecuritySettings (ViSession instrSession,
                                                             ViInt32 instance,
                                                             ViString secretKey,
                                                             ViString operatorVariantKey,
                                                             ViString authenticationManagementField,
                                                             ViString sequenceNumber)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViString (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_SECURITY_SECRET_KEY,
                                               secretKey), 3, "Secret Key");
	
	viCheckParm (rscmwevs_SetAttributeViString (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_SECURITY_OPERATOR_VARIANT_KEY,
                                               operatorVariantKey), 4, "Operator Variant Key");
	
	viCheckParm (rscmwevs_SetAttributeViString (instrSession, rep_cap,
                                   				RSCMWEVS_ATTR_NETWORK_SECURITY_AUTHENTICATION_FIELD,
                                   authenticationManagementField), 5, "Authentication Management Field");
	
	viCheckParm (rscmwevs_SetAttributeViString (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_SECURITY_SEQUENCE_NUMBER,
                                               sequenceNumber), 6, "Sequence Number");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  This function defines neighbor cell information to be broadcast to the 
/// HIFN  AT. LTE is the only supported neighbor cell technology in the current 
/// HIFN  release. 
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSCMWEVS_ATTR_NETWORK_NEIGHBOR_CELL_RX_LEVEL_MIN_LTE
/// HIFN     RSCMWEVS_ATTR_NETWORK_NEIGHBOR_CELL_TRESH_SERVING
/// HIFN     RSCMWEVS_ATTR_NETWORK_NEIGHBOR_CELL_MAX_RESELECTION_TIMER
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:EVDO:SIGN<instance>:NCELl:RLMeutra
/// HIFN     CONFigure:EVDO:SIGN<instance>:NCELl:THRServing
/// HIFN     CONFigure:EVDO:SIGN<instance>:NCELl:MRTimer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR rxLevelMinLTE/Configures the low reselection threshold value for LTE neighbor cells. 
/// HIPAR rxLevelMinLTE/
/// HIPAR treshServing/Specifies the limit for pilot power level below which the AT triggers 
/// HIPAR treshServing/cell reselection to a neighbor cell in the candidate set.
/// HIPAR maxReselectionTimer/Maximum time for the access terminal to execute the cell reselection.
ViStatus _VI_FUNC rscmwevs_ConfigureNeighborCell(
	ViSession	instrSession,
	ViInt32	instance,
	ViReal64	rxLevelMinLTE,
	ViReal64	treshServing,
	ViReal64	maxReselectionTimer
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_NEIGHBOR_CELL_RX_LEVEL_MIN_LTE,
                                               rxLevelMinLTE), 3, "RX Level Min LTE");
	
	viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_NEIGHBOR_CELL_TRESH_SERVING,
                                               treshServing), 4, "Tresh Serving");
	
	viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                   				RSCMWEVS_ATTR_NETWORK_NEIGHBOR_CELL_MAX_RESELECTION_TIMER,
                                   				maxReselectionTimer), 5, "Max Reselection Timer");
	
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;    
}

/// HIFN  Sets the low reselection threshold for all enabled neighbour cells.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR set_forAll/Enables and disables the set for all relevant technologies.
/// HIPAR values/Sets the low threshold value.
ViStatus _VI_FUNC rscmwevs_ConfigureAllNeighborCellLowThreshold (ViSession instrSession,
                                                                 ViInt32 instance,
                                                                 ViBoolean set_forAll,
                                                                 ViInt32 values)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, values, 0, 31), 4, "Values");
    
    sprintf (command, "CONF:EVDO:SIGN%ld:NCEL:ALL:THR:LOW %s, %ld", instance, switchArr[set_forAll], values);
    
    viCheckErr (viPrintf (instrSession, command));
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Sets the low threshold for the 1xEV-DO cell reselection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR technology/Selects the technology.
/// HIPAR values/Sets the the value for low threshold for reselection.
ViStatus _VI_FUNC rscmwevs_ConfigureNeighborCellLowThreshold (ViSession instrSession,
                                                              ViInt32 instance,
                                                              ViInt32 technology,
                                                              ViInt32 values)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[105] = "";
    ViChar      rep_cap_1[100] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));
	sprintf (rep_cap_1, "%s", TechnologyArr[technology]);
    
    strcat (rep_cap, ",");
    strcat (rep_cap, rep_cap_1);
	
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_NEIGHBOR_CELL_LOW_THRESHOLD,
                                               values), 4, "Values");
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error; 
}

/// HIFN  Enables and configures up to 16 1xEVDO or CDMA2000 neighbor cells.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR technology/Selects the technology.
/// HIPAR cellNumber/Sets te number of cells.
/// HIPAR enable/Enables signaling of neighbor cells.
/// HIPAR bandClass/Selects the band class.
/// HIPAR channel/Sets the channel.
/// HIPAR cellID/Sets cell ID of neighbor cell.
ViStatus _VI_FUNC rscmwevs_Configure1xEVDOandCDMANeighborCells (ViSession instrSession,
                                                                ViInt32 instance,
																ViInt32 technology,
																ViInt32 cellNumber,
                                                                ViBoolean enable,
                                                                ViInt32 bandClass,
                                                                ViInt32 channel,
                                                                ViInt32 cellID)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, technology, RSCMWEVS_VAL_EVDO, RSCMWEVS_VAL_CDMA), 3, "Technology");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, cellNumber, 1, 16), 4, "Cell Number");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, bandClass, RSCMWEVS_VAL_BAND_USC, RSCMWEVS_VAL_BAND_SBAN), 6, "Band Class");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, channel, 1, 1323), 7, "Channel");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, cellID, 0, 511), 8, "Cell ID");
	
    sprintf (command, "CONF:EVDO:SIGN%ld:NCEL:%s:CELL%ld %s,%s,%ld,%ld", instance, TechnologyStringArr[technology] ,cellNumber, switchArr[enable], EVDObandClassArr[bandClass], channel, cellID);
    
    viCheckErr (viPrintf (instrSession, command));
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Enables and configures up to 16 LTE neighbor cells.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR cellNumber/Sets te number of cells.
/// HIPAR enable/Enables signaling of neighbor cells.
/// HIPAR bandClass/Selects the band class.
/// HIPAR channel/Sets the channel.
ViStatus _VI_FUNC rscmwevs_ConfigureLTENeighborCells (ViSession instrSession,
                                                      ViInt32 instance,
                                                      ViInt32 cellNumber,
                                                      ViBoolean enable,
                                                      ViInt32 bandClass,
                                                      ViInt32 channel)
{

	ViStatus    error = VI_SUCCESS;
    ViChar      command[150] = "";
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, cellNumber, 1, 16), 3, "Cell Number");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, bandClass, RSCMWEVS_VAL_BAND_OB1, RSCMWEVS_VAL_BAND_UDEF), 5, "Band Class");
	
    sprintf (command, "CONF:EVDO:SIGN%ld:NCEL:LTE:CELL%ld %s,%s,%ld", instance ,cellNumber, switchArr[enable], LTEbandClassArr[bandClass], channel);
    
    viCheckErr (viPrintf (instrSession, command));
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Specifies the minimum required quality threshold of the reselection 
/// HIFN  target cell.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR cellNumber/Sets te number of cells.
/// HIPAR treshx/Specifies the minimum required quality threshold of the reselection 
/// HIPAR treshx/target cell.
ViStatus _VI_FUNC rscmwevs_ConfigureLTENeighborCellsTreshX(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32	cellNumber,
	ViInt32	treshx
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[105] = "";
    ViChar      rep_cap_1[100] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));
	viCheckParm (rscmwevs_invalidViInt32Range (instrSession, cellNumber, 1, 16), 3, "Cell Number"); 
	
	sprintf (rep_cap_1, "cell%ld", cellNumber);
    
    strcat (rep_cap, ",");
    strcat (rep_cap, rep_cap_1);
	
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_NETWORK_NEIGHBOR_CELL_LTE_TRESHX,
                                               treshx), 4, "TreshX");
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;     
}

/// HIFN  This function configures the state and sets up the fading simulator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR enableFadingSimulator/Enables/disables the fading simulator.
/// HIPAR profile/Selects a propagation condition profile.
/// HIPAR startSeed/Sets the start seed for the pseudo-random fading algorithm.
/// HIPAR insertionLossMode/Sets the insertion loss mode.
/// HIPAR insertionLoss/Sets the insertion loss for the fading simulator.
ViStatus _VI_FUNC rscmwevs_ConfigureFadingSimulator(
	ViSession	instrSession,
	ViInt32	instance,
	ViBoolean	enableFadingSimulator,
	ViInt32	profile,
	ViInt32	startSeed,
	ViInt32	insertionLossMode,
	ViReal64	insertionLoss
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViBoolean (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_FADING_SIMULATOR_ENABLED,
                                               enableFadingSimulator), 3, "Enable Fading Simulator");
	
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_FADING_SIMULATOR_PROFILE,
                                               profile), 4, "Profile");
	
	viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_FADING_SIMULATOR_START_SEED,
                                               startSeed), 5, "Start Seed");
	
	viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_FADING_SIMULATOR_INSERTION_LOSS_MODE,
                                               insertionLossMode), 6, "Insertion Loss Mode");
	
	if (insertionLossMode == RSCMWEVS_VAL_FADING_INSERTION_LOSS_MODE_USER)
	{
		viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_FADING_SIMULATOR_INSERTION_LOSS,
                                               insertionLoss), 7, "Insertion Loss");
	}
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;    
}


/// HIFN  This function configures the restart mode of the fading simulator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR restartMode/Sets the restart mode of the fading simulator.
ViStatus _VI_FUNC rscmwevs_ConfigureFadingSimulatorRestartMode(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32	restartMode
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr (rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_FADING_SIMULATOR_RESTART_MODE,
                                               restartMode), 3, "Restart Mode");
Error:    
    Rs_UnlockSession(instrSession, NULL);    
    return error;    
}


/// HIFN  This function restarts the fading process in Manual mode (see 
/// HIFN  rscmwevs_ConfigureFadingSimulatorRestartMode).
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSCMWEVS_ATTR_FADING_SIMULATOR_RESTART
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:CDMA:SIGN<instance>:FADing:FSIMulator:RESTart
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
ViStatus _VI_FUNC rscmwevs_RestartFadingSimulator(
	ViSession	instrSession,
	ViInt32	instance
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckErr (rscmwevs_SetAttributeViString (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_FADING_SIMULATOR_RESTART,
                                               NULL));
Error:    
    Rs_UnlockSession(instrSession, NULL);    
    return error;       
}


/// HIFN  This function returns the percentage of clipped samples for one output 
/// HIFN  path.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSCMWEVS_ATTR_FADING_SIMULATOR_CLIPPED_SAMPLES
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:CDMA:SIGN<instance>:FADing:FSIMulator:ILOSs:CSAMples?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR clippedSamples/Returns the percentage of clipped samples for one output path.
ViStatus _VI_FUNC rscmwevs_QueryFadingSimulatorClippedSamples(
	ViSession	instrSession,
	ViInt32	instance,
	ViReal64*	clippedSamples
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr (rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_FADING_SIMULATOR_CLIPPED_SAMPLES,
                                               clippedSamples), 3, "Clipped Samples");
Error:    
    Rs_UnlockSession(instrSession, NULL);    
    return error;       
}


/// HIFN  This function returns noise power values, calculated from the downlink 
/// HIFN  power, the cell bandwidth and the fading module AWGN settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR noisePower_SystemBW/Queries the calculated noise power on the DL channel, i.e. within the 
/// HIPAR noisePower_SystemBW/cell bandwidth.
/// HIPAR noisePower_TotalBW/Queries the total noise power.
/// HIPAR signalNoisePower/Queries the calculated total power (signal + noise) on the DL channel, 
/// HIPAR signalNoisePower/i.e. within the cell bandwidth.
ViStatus _VI_FUNC rscmwevs_QueryInternalFadingNoisePowerValues(
	ViSession	instrSession,
	ViInt32	instance,
	ViReal64*	noisePower_SystemBW,
	ViReal64*	noisePower_TotalBW,
	ViReal64*	signalNoisePower
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr (rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_FADING_SIMULATOR_POWER_NOISE,
                                               noisePower_SystemBW), 3, "Noise Power System BW");
	
	viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_FADING_SIMULATOR_POWER_NOISE_TOTAL,
                                               noisePower_TotalBW), 4, "Noise Power Total BW");
	
	viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_FADING_SIMULATOR_POWER_NOISE_SUM,
                                               signalNoisePower), 5, "Signal Noise Power");
	
Error:    
    Rs_UnlockSession(instrSession, NULL);    
    return error;      
}


/// HIFN  This function configures the state and configure the AWGN insertion on 
/// HIFN  the fading module.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR enableFadingModuleAWGN/Enables or disables AWGN insertion via the fading module.
/// HIPAR minNoiseSystemBWRatio/Specifies the AWGN minimal noise to system bandwidth ratio for
/// HIPAR signalToNoiseRatio/Specifies the signal to noise ratio for the AWGN inserted on the 
/// HIPAR signalToNoiseRatio/internal fading module.
ViStatus _VI_FUNC rscmwevs_ConfigureFadingModuleAWGN(
	ViSession	instrSession,
	ViInt32	instance,
	ViBoolean	enableFadingModuleAWGN,
	ViReal64	minNoiseSystemBWRatio,
	ViReal64	signalToNoiseRatio
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViBoolean (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_FADING_AWGN_ENABLED,
                                               enableFadingModuleAWGN), 3, "Enable Fading Module AWGN");
	
    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_FADING_AWGN_BANDWIDTH_RATIO,
                                               minNoiseSystemBWRatio), 4, "min Noise System BW Ratio");
	
	viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_FADING_AWGN_SIGNAL_TO_NOISE_RATIO,
                                               signalToNoiseRatio), 5, "Signal To Noise Ratio");
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;     
}


/// HIFN  Queries the bandwidth of the AWGN inserted via the internal fading 
/// HIFN  module.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSCMWEVS_ATTR_FADING_AWGN_NOISE_BANDWIDTH
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:CDMA:SIGN<instance>:FADing:AWGN:BWIDth:NOISe?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR noiseBandwidth/Queries the bandwidth of the AWGN inserted via the internal fading 
/// HIPAR noiseBandwidth/module.
ViStatus _VI_FUNC rscmwevs_QueryFadingModuleAWGNNoiseBandwidth(
	ViSession	instrSession,
	ViInt32	instance,
	ViReal64*	noiseBandwidth
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr (rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_FADING_AWGN_NOISE_BANDWIDTH,
                                               noiseBandwidth), 3, "Noise Bandwidth");
Error:    
    Rs_UnlockSession(instrSession, NULL);    
    return error;        
}

/// HIFN  Configures the message monitoring.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR signalingToLogging/If enabled, messages exchanged between the 1xEV-DO signaling 
/// HIPAR signalingToLogging/application and the DUT are reported to the selected logging PC.
/// HIPAR IPv4Address/Selects the IP address (i.e. the logging PC) to which the signaling 
/// HIPAR IPv4Address/messages shall be reported. The available IPv4 addresses are configured 
/// HIPAR IPv4Address/in the "Logging" part of the global
ViStatus _VI_FUNC rscmwevs_ConfigureMessageMonitoring (ViSession instrSession,
                                                       ViInt32 instance,
                                                       ViBoolean signalingToLogging,
                                                       ViInt32 IPv4Address)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViBoolean (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_SIGNALING_TO_LOGGING_STATE,
                                               signalingToLogging), 3, "Signaling To Logging");
	
	viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_SIGNALING_MESSAGES_IP_ADDRESS,
                                               IPv4Address), 4, "IPv4 Address");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Generator State
 * Purpose:  Turns the 1xEV-DO Signaling generator (forward 1xEV-DO signal) 
 *           off or on.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureGeneratorState(
    ViSession   instrSession,
    ViInt32 instance,
    ViBoolean   signalState
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_GENERATOR_STATE,
                                                 signalState), 3, "Signal State");

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query Generator State
 * Purpose:  Returns detailed information about the "1xEV-DO Signaling" 
 *           generator state.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryGeneratorState(
    ViSession   instrSession,
    ViInt32 instance,
    ViBoolean*  mainState,
    ViInt32*    syncState
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command [100] = "";
    ViChar      response [150] = "";
    ViChar*     p2buf;
    ViString    mainStateArr[] = {"OFF","ON",NULL};
    ViString    syncStateArr[] = {"PEND","ADJ",NULL};
    
    checkErr (Rs_LockSession (instrSession, NULL));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    
    sprintf (command, "SOUR:EVDO:SIGN%ld:STAT:ALL?\n", instance);
    
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (viRead (instrSession, (ViBuf) response, 150, NULL));
    
    p2buf = strtok (response, ",");
    if (p2buf == NULL)
        return RSCMWEVS_ERROR_INCORRECT_STATE;
    *mainState = (ViBoolean) rscmwevs_StringIndex (mainStateArr,p2buf);
    if (*mainState < 0)
        return RSCMWEVS_ERROR_INCORRECT_STATE;

    p2buf = strtok (NULL, "\n");
    if (p2buf == NULL)
        return RSCMWEVS_ERROR_INCORRECT_STATE;
    *syncState = rscmwevs_StringIndex (syncStateArr,p2buf);
    if (*syncState < 0)
        return RSCMWEVS_ERROR_INCORRECT_STATE;

    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Connection State
 * Purpose:  Controls the setup and release of an 1xEV-DO connection. The 
 *           function initiates a transition between different connection 
 *           states.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureConnectionState(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 action
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_CONNECTION_STATE,
                                               action), 3, "Action");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query Connection State
 * Purpose:  Returns the connection state of an 1xEV-DO connection.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryConnectionState(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32*    connectionState
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_QUERY_CONNECTION_STATE,
                                               connectionState), 3, "Connection State");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Returns the state of the packet data (PPP) connection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR connectionState/Returns the state of the packet data (PPP) connection.
ViStatus _VI_FUNC rscmwevs_QueryPacketDataState (ViSession instrSession,
                                                 ViInt32 instance,
                                                 ViInt32 *connectionState)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_QUERY_PACKET_DATA_STATE,
                                               connectionState), 3, "Connection State");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Test Application
 * Purpose:  This function configures the Test Application.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureTestApplication(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 testApplicationStream,
    ViInt32 testApplicationMode
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_TEST_APPLICATION_STREAM,
                                               testApplicationStream), 3, "Test Application Stream");
    
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_TEST_APPLICATION_MODE,
                                               testApplicationMode), 4, "Test Application Mode");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  This function configures the preferred standard to be signaled during 
/// HIFN  the connection setup.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSCMWEVS_ATTR_TEST_APPLICATION_PREFERRED_PACKET_MODE
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:EVDO:SIGN<instance>:LAYer:APPLication:PACKet:MODE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR testApplicationMode/Sets the preferred standard to be signaled during the connection 
/// HIPAR testApplicationMode/setup.
ViStatus _VI_FUNC rscmwevs_ConfigureTestApplicationPreferredPacketMode(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32	testApplicationMode
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_TEST_APPLICATION_PREFERRED_PACKET_MODE,
                                               testApplicationMode), 3, "Test Application Mode");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;    
}

/// HIFN  Queries the stream and the state of the Default Signaling Application. 
/// HIFN  The response is fixed: The Default Signaling Application is always 
/// HIFN  enabled and assigned to stream 0.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwwmax_init or 
/// HIPAR instrSession/rscmwwmax_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR result/Returns the stream and the state of the Default Signaling Application.
ViStatus _VI_FUNC rscmwevs_QueryTestApplicationInformation (ViSession instrSession,
                                                            ViInt32 instance,
                                                            ViInt32 arraySize,
                                                            ViChar result[])
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViString (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_TEST_APPLICATION_INFO,
                                                arraySize, result), 3, "Result");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Indicates whether an inter RAT handover is supported (as agreed during 
/// HIFN  session negotiation). 
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR interRAT/Indicates whether an inter RAT handover is supported (as agreed during 
/// HIPAR interRAT/session negotiation).
ViStatus _VI_FUNC rscmwevs_QueryInterRAT (ViSession instrSession,
                                          ViInt32 instance, ViBoolean *interRAT)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViBoolean (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_QUERY_INTER_RAT,
                                               interRAT), 3, "Inter RAT");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Queries the the quality of the RX signal from the connected AT. 
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSCMWEVS_ATTR_QUERY_RX_POWER
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     SENSe:EVDO:SIGN<instance>:TEST:RX:POWer:STATe?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR rxPower/Queries the the quality of the RX signal from the connected AT.
ViStatus _VI_FUNC rscmwevs_QueryRXPower(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32*	rxPower
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViInt32 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_QUERY_RX_POWER,
                                                rxPower), 3, "RX Power");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;    
}

/// HIFN  Returns the active test or packet applications along with the streams 
/// HIFN  they are using.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR application/Comma-separated string of tuples s:<Application Name>.
ViStatus _VI_FUNC rscmwevs_QueryActiveApplication (ViSession instrSession,
                                            ViInt32 instance,
											ViInt32 arraySize,
                                            ViChar application[])
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViString (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_QUERY_ACTIVE_APPLICATION,
                                                arraySize, application), 3, "Application");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Queries the negotiated packet standard of the current connection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR negotiatedMode/Queries the negotiated packet standard of the current connection.
ViStatus _VI_FUNC rscmwevs_QueryNegotiatedMode(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32*	negotiatedMode
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViInt32 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_QUERY_NEGOTIATED_MODE,
                                                negotiatedMode), 3, "Negotiated Mode");
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;     
}

/*****************************************************************************
 * Function: Query Physical Layer Subtype
 * Purpose:  Queries the physical layer subtype in use.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryPhysicalLayerSubtype(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32*    physicalLayerSubtype
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_QUERY_PHYSICAL_LAYER_SUBTYPE,
                                               physicalLayerSubtype), 3, "Physical Layer Subtype");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query UATI
 * Purpose:  Queries the Unicast Access Terminal Identifier (UATI) of the 
 *           AT. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryUATI(
    ViSession   instrSession,
    ViInt32 instance,
    ViChar  uati[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViString (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_QUERY_UATI,
                                                RSCMWEVS_IO_BUFFER_SIZE, uati), 3, "UATI");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Sets the hard-coded Electronic Serial Number of the connected AT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR esn/Sets the hard-coded Electronic Serial Number of the connected AT.
ViStatus _VI_FUNC rscmwevs_ConfigureTestESN(
	ViSession	instrSession,
	ViInt32	instance,
	ViString	esn
)
{
 	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViString (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_TEST_ESN,
                                                esn), 3, "ESN");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;   
}

/*****************************************************************************
 * Function: Query ESN
 * Purpose:  Queries the Electronic Serial Number of the connected AT.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryESN(
    ViSession   instrSession,
    ViInt32 instance,
    ViChar  esn[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViString (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_QUERY_ESN,
                                                RSCMWEVS_IO_BUFFER_SIZE, esn), 3, "ESN");

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Sets the mobile equipment identifier of the connected AT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR meid/Sets the mobile equipment identifier of the connected AT.
ViStatus _VI_FUNC rscmwevs_ConfigureTestMEID(
	ViSession	instrSession,
	ViInt32	instance,
	ViString	meid
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViString (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_TEST_MEID,
                                                meid), 3, "MEID");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;      
}

/*****************************************************************************
 * Function: Query MEID
 * Purpose:  Queries the Mobile Equipment Identifier of the connected AT. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryMEID(
    ViSession   instrSession,
    ViInt32 instance,
    ViChar  meid[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViString (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_QUERY_MEID,
                                                RSCMWEVS_IO_BUFFER_SIZE, meid), 3, "MEID");

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}
/// HIFN  Queries whether the AT supports either evolved HRPD or the High Rate 
/// HIFN  Packet Data.
/// HIFN     
/// HIFN     Attribute(s):
/// HIFN     RSCMWEVS_ATTR_QUERY_EHRPD
/// HIFN     
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:EVDO:SIGN<i>:CSTatus:EHRPd?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR ehrpd/Queries whether the AT supports either evolved HRPD or the High Rate 
/// HIPAR ehrpd/Packet Data.
ViStatus _VI_FUNC rscmwevs_QueryeHRPD(
	ViSession	instrSession,
	ViInt32	instance,
	ViBoolean*	ehrpd
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViBoolean (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_QUERY_EHRPD,
                                                 ehrpd), 3, "eHRPD");

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query Control Channel Cycle
 * Purpose:  Queries the control channel cycle in which the AT transitions 
 *           out of the dormant state in order to monitor the Control 
 *           Channel.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryControlChannelCycle(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32*    preferredCtrlChannelCycle
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_QUERY_PREFERRED_CONTROL_CHANNEL_CYCLE,
                                               preferredCtrlChannelCycle), 3, "Preferred Ctrl Channel Cycle");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query Control Channel Status
 * Purpose:  Queries the state of the "Preferred Control Channel Enable" 
 *           flag. The flag indicates whether or not the AT selects the 
 *           Preferred Control Channel Cycle.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryControlChannelStatus(
    ViSession   instrSession,
    ViInt32 instance,
    ViBoolean*  preferredCtrlChannelStatus
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViBoolean (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_QUERY_PREFERRED_CONTROL_CHANNEL_STATUS,
                                                 preferredCtrlChannelStatus), 3, "Preferred Ctrl Channel Status");

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query Connection Status
 * Purpose:  Queries the current state of a forward and reverse link carrier.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryCarrierStatus (ViSession instrSession,
                                                  ViInt32 instance,
                                                  ViInt32 *activeFwdLinkCarriers,
                                                  ViInt32 *activeRevLinkCarriers)
{   
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViInt32 (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_QUERY_ACTIVE_FORWARD_LINK_CARRIERS,
                                                 activeFwdLinkCarriers), 3, "Active Fwd Link Carriers");
    
    viCheckParm (rscmwevs_GetAttributeViInt32 (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_QUERY_ACTIVE_REVERSE_LINK_CARRIERS,
                                                 activeRevLinkCarriers), 4, "Active Rev Link Carriers");

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query Long Code Mask
 * Purpose:  Queries the reverse traffic channel in phase and quadrature
 *           phase long code mask associated with the access terminal's
 *           session.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryLongCodeMask (ViSession instrSession,
                                              ViInt32 instance,
                                              ViChar longCodeMaskI[],
                                              ViChar longCodeMaskQ[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViString (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_QUERY_LONG_CODE_MASK_I, 512,
                                                 longCodeMaskI), 3, "Long Code Mask I");
    
    viCheckParm (rscmwevs_GetAttributeViString (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_QUERY_LONG_CODE_MASK_Q, 512,
                                                 longCodeMaskQ), 4, "Long Code Mask Q");

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error; 
}

/// HIFN  Queries the maximum reverse link bandwith reported by the AT.
/// HIFN     
/// HIFN     Attribute:
/// HIFN     RSCMWEVS_ATTR_QUERY_MAX_REVERSE_BANDWIDTH
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR maxReverseBandwidth/Queries the maximum reverse link bandwith reported by the AT.
ViStatus _VI_FUNC rscmwevs_QueryMaxReverseBandwidth(
	ViSession	instrSession,
	ViInt32	instance,
	ViReal64*	maxReverseBandwidth
)
{   
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_QUERY_MAX_REVERSE_BANDWIDTH,
                                                 maxReverseBandwidth), 3, "Max Reverse Bandwidth");

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}
	
/*****************************************************************************
 * Function: Query Data Application Unit IP Address
 * Purpose:  Returns the IPv4 address (<n> = 4) or the IPv6 prefix (<n> = 6)
 *           assigned to the AT by the DAU.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryDataApplicationUnitIPAddress (ViSession instrSession,
                                                              ViInt32 instance,
                                                              ViInt32 IPVersion,
                                                              ViInt32 bufferSize,
                                                              ViChar  IPAddress[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[105] = "";
    ViChar      rep_cap_1[100] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));
	sprintf (rep_cap_1, "%s", IPVersionArr[IPVersion]);
    
    strcat (rep_cap, ",");
    strcat (rep_cap, rep_cap_1);

    viCheckParm (rscmwevs_GetAttributeViString (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_QUERY_DATA_APPLICATION_UNIT_IP_ADDRESS,
                                                bufferSize, IPAddress), 5, "IP Address");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Handoff Target
 * Purpose:  This function configures a handoff to another 1xEV-DO band 
 *           class.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureHandoffTarget(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 destinationBandClass,
    ViInt32 destinationChannel
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_HANDOFF_BAND_CLASS,
                                               destinationBandClass), 3, "Destination Band Class");
    
    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_HANDOFF_CHANNEL,
                                               destinationChannel), 4, "Destination Channel");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure Handoff Carrier Target
 * Purpose:  Defines the channel and RF channel offset for a carrier in the 
 *           handoff destination cell.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureHandoffCarrierTarget (ViSession instrSession,
                                                           ViInt32 instance,
                                                           ViInt32 carrier,
                                                           ViInt32 carrierChannel,
                                                           ViReal64 RFChannelOffset)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_RF_CARRIER_SETTING,
                                               carrier), 3, "Carrier");
    
    if (carrier == 0)
    {
        viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_HANDOFF_CARRIER_CHANNEL_TARGET,
                                               carrierChannel), 4, "Carrier Channel");
    }
    else
    {
        viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_HANDOFF_CHANNEL_OFFSET,
                                               RFChannelOffset), 5, "RF Channel Offset");
    }
    
Error:
    Rs_UnlockSession(instrSession, NULL);
    return error;
}

/*****************************************************************************
 * Function: Query Handoff Carrier Link Frequency
 * Purpose:  RReturns the forward and reverse link frequency for a carrier in
 *           the handoff destination cell.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryHandoffCarrierLinkFrequency (ViSession instrSession,
                                                             ViInt32 instance,
                                                             ViReal64 *forwardLinkFrequency,
                                                             ViReal64 *reverseLinkFrequency)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_HANDOFF_FORWARD_LINK_FREQUENCY,
                                                forwardLinkFrequency), 3, "Forward Link Frequency");
    
    viCheckParm (rscmwevs_GetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_HANDOFF_REVERSE_LINK_FREQUENCY,
                                                reverseLinkFrequency), 4, "Reverse Link Frequency");
    
Error:
    Rs_UnlockSession(instrSession, NULL);
    return error; 
}

/*****************************************************************************
 * Function: Configure Handoff Carrier Pilot States
 * Purpose:  Defines the states of a pilot in the handoff destination cell.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureHandoffCarrierPilotStates (ViSession instrSession,
                                                               ViInt32 instance,
                                                               ViInt32 pilot,
                                                               ViBoolean active_onAN,
                                                               ViBoolean assigned_toAT)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_RF_CARRIER_PILOT_SETTING,
                                               pilot), 3, "Pilot");
 
    viCheckParm (rscmwevs_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_HANDOFF_PILOT_ACTIVE_ON_AN,
                                                 active_onAN), 4, "Active on AN");

    if (active_onAN == VI_TRUE)
    {
        viCheckParm (rscmwevs_SetAttributeViBoolean (instrSession, rep_cap,
                                                     RSCMWEVS_ATTR_HANDOFF_PILOT_ASSIGNED_TO_AT,
                                                     assigned_toAT), 5, "Assigned to AT");
    }
    
Error:
    Rs_UnlockSession(instrSession, NULL);
    return error;  
}

/*****************************************************************************
 * Function: Initiate Handoff
 * Purpose:  This function initiates a handoff to a band class.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_InitiateHandoff(
    ViSession   instrSession,
    ViInt32 instance
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckErr (rscmwevs_SetAttributeViString (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_INITIATE_HANDOFF, NULL));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query Event Log Last Entry
 * Purpose:  Queries the latest entry of the event log.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryEventLogLastEntry (
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32	timestampBufferSize,
	ViInt32	eventBufferSize,
	ViChar	timeStamp[],
	ViInt32*	category,
	ViChar	event[]
)
{
	ViStatus	error = VI_SUCCESS;
	ViChar      command[150] = ""; 
    ViChar*  p2buf;
    ViChar      response [RSCMWEVS_IO_BUFFER_SIZE] = "";
    
    checkErr(Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
   
    sprintf (command, "SENS:EVDO:SIGN%ld:ELOG:LAST?", instance);
	
    checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    checkErr (viRead (instrSession, (ViBuf) response, RSCMWEVS_IO_BUFFER_SIZE, NULL));
	
	p2buf = strtok (response, ",");	
    if (p2buf == NULL)
        return RSCMWEVS_ERROR_INCORRECT_STATE;
	strncpy (timeStamp, p2buf, timestampBufferSize);
	
    p2buf = strtok (NULL, ",\n");  
    if (p2buf == NULL)
        return RSCMWEVS_ERROR_INCORRECT_STATE;
    *category = rscmwevs_StringIndex (categoryArr, p2buf);
    if (*category < 0)
        return RSCMWEVS_ERROR_INCORRECT_STATE;

	p2buf = strtok (NULL, ",\n"); 
    if (p2buf == NULL)
        return RSCMWEVS_ERROR_INCORRECT_STATE;
    strncpy (event, p2buf, eventBufferSize);   
	
	p2buf = strtok (NULL, ",\n"); 
	
	checkErr( rscmwevs_CheckStatus (instrSession));
Error:
	Rs_UnlockSession(instrSession, NULL);  
	
	return error;	
}

/*****************************************************************************
 * Function: Query Event Log All Entries
 * Purpose:  Queries all entries of the event log.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryEventLogAllEntries (ViSession instrSession,
                                                   ViInt32 instance,
                                                   ViInt32 arraySize,
                                                   ViChar entries[])
{
    ViStatus	error = VI_SUCCESS;
	ViChar      command[50] = "";
		
    checkErr(Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
   
    sprintf (command, "SENS:EVDO:SIGN%ld:ELOG:ALL?", instance);
    
	checkErr (rscmwevs_GetCatalog(instrSession, command, 15000, arraySize, entries));
	
Error:
	Rs_UnlockSession(instrSession, NULL);  
	
	return error;
}

/*****************************************************************************
 * Function: Configure RX Quality Selected Carrier For Results
 * Purpose:  Selects the carrier whose results shall be retrieved.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRXQualitySelectedCarrierForResults (ViSession instrSession,
                                                                        ViInt32 instance,
                                                                        ViInt32 selectedCarrier)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                              RSCMWEVS_ATTR_RX_QUALITY_SELECT_CARRIER_FOR_RESULTS, 
                                              selectedCarrier), 3,"Selected Carrier");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;  
}

/*****************************************************************************
 * Function: Configure RX Quality Measurement Results
 * Purpose:  This function configures which receiver quality measurements 
 *           will be displayed in the "1xEV-DO RX Quality" main dialog.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRXQualityMeasurementResults(
    ViSession   instrSession,
    ViInt32 instance,
    ViBoolean   forwardLinkPER,
    ViBoolean   forwardLinkPerformance,
    ViBoolean   reverseLinkPER,
    ViBoolean   reverseLinkPerformance
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_RX_QUALITY_ASSIGN_VIEWS_FORWARD_LINK_PER,
                                                 forwardLinkPER), 3, "Forward Link PER");

    viCheckParm (rscmwevs_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_RX_QUALITY_ASSIGN_VIEWS_FORWARD_LINK_PERFORMANCE,
                                                 forwardLinkPerformance), 4, "Forward Link Performance");
    
    viCheckParm (rscmwevs_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_RX_QUALITY_ASSIGN_VIEWS_REVERSE_LINK_PER,
                                                 reverseLinkPER), 5, "Reverse Link PER");

    viCheckParm (rscmwevs_SetAttributeViBoolean (instrSession, rep_cap,
                                                 RSCMWEVS_ATTR_RX_QUALITY_ASSIGN_VIEWS_REVERSE_LINK_PERFORMANCE,
                                                 reverseLinkPerformance), 6, "Reverse Link Performance");

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}


/// HIFN Defines a timeout for the measurement. The timer is started when the measurement is initiated via a READ or INIT command. It is not started if the measurement is initiated manually (ON/OFF key or RESTART/STOP key).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init 
/// HIPAR instrSession/or rscmwevs_InitWithOptions function.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR timeout/Defines a timeout for the measurement.
ViStatus _VI_FUNC rscmwevs_ConfigureRXQualityMeasurementTimeout (ViSession instrSession,
                                                                 ViInt32 instance,
                                                                 ViInt32 measurement,						
                                                                 ViReal64 timeout)
{
  	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";
	
	viCheckParm (rscmwevs_invalidViInt32Range (instrSession, measurement, RSCMWEVS_VAL_MEASUREMENT_PER, RSCMWEVS_VAL_MEASUREMENT_THR), 3, "Measurement");

    checkErr (Rs_LockSession (instrSession, NULL));

	checkErr(rscmwevs_instanceRepCap(instrSession, instance, rep_cap));

	switch(measurement)
    {
        case RSCMWEVS_VAL_MEASUREMENT_PER:
            
        viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                   RSCMWEVS_ATTR_RX_QUALITY_PER_TIMEOUT, 
                                                   timeout), 4,"Timeout");
        break;
    
    
        case RSCMWEVS_VAL_MEASUREMENT_THR:
        
        viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                   RSCMWEVS_ATTR_RX_QUALITY_THROUGHPUT_TIMEOUT, 
                                                   timeout), 4,"Timeout");
        break;
        default:
            viCheckParm (RS_ERROR_INVALID_PARAMETER, 3, "Measurement");
    }
    
  Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;

}

/*****************************************************************************
 * Function: Configure RX Quality Measurement Results
 * Purpose:  Speifies the repetition mode of the PER (Packet Error Rate) and
 *           Throughput measurement. The repetition mode specifies whether
 *           the measurement is stopped after a single-shot or repeated
 *           continuously.also this function defines the time interval for
 *           evaluation and displaying the new set of measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRXQualityMeasurementRepetition (ViSession instrSession,
                                                                    ViInt32 instance,
                                                                    ViInt32 measurement,
                                                                    ViInt32 repetition,
                                                                    ViReal64 updatePeriod)
{
   ViStatus    error = VI_SUCCESS;
   ViChar      rep_cap[5] = "";
   
        viCheckParm (rscmwevs_invalidViInt32Range (instrSession, measurement, RSCMWEVS_VAL_MEASUREMENT_PER, RSCMWEVS_VAL_MEASUREMENT_THR), 3, "Measurement");
   
   checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));
   
   switch(measurement)
    {
        case RSCMWEVS_VAL_MEASUREMENT_PER:
            
        viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                                   RSCMWEVS_ATTR_RX_QUALITY_PER_REPETITION, 
                                                   repetition), 4,"Repetition");
        break;
    
    
        case RSCMWEVS_VAL_MEASUREMENT_THR:
        
        viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                                   RSCMWEVS_ATTR_RX_QUALITY_THROUGHPUT_REPETITION, 
                                                   repetition), 4,"Repetition");
        break;
        
    }
    
    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_RX_QUALITY_UPDATE_PERIOD, 
                                                updatePeriod), 4,"Update Period");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;   
    
}

/*****************************************************************************
 * Function: Configure RX Quality PER Measurement Stop Condition
 * Purpose:  Qualifies whether the measurement is stopped after a failed
 *           limit check or continued. SLFail means that the measurement is
 *           stopped and reaches the RDY state as soon as one of the results
 *           exceeds the limits.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRXQualityPERMeasurementStopCondition (ViSession instrSession,
                                                                          ViInt32 instance,
                                                                          ViInt32 PERLinkType,
                                                                          ViInt32 stopCondition)
{
   ViStatus    error = VI_SUCCESS;
   ViChar      buffer[RSCMWEVS_IO_BUFFER_SIZE] = "";  
   
   checkErr (Rs_LockSession (instrSession, NULL));
   
   sprintf(buffer, "i%ld,%s", instance, PERLinkTypeArr[PERLinkType]);

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, buffer,
                                               RSCMWEVS_ATTR_RX_QUALITY_LINK_STOP_CONDITION,
                                               stopCondition), 4, "Stop Condition");
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure RX Quality Forward PER Measurement Sent Packets
 * Purpose:  Defines the length of a single shot forward link PER
 *           measurement, i.e. the maximum number of packets sent.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRXQualityForwardPERMeasurementSentPackets (ViSession instrSession,
                                                                               ViInt32 instance,
                                                                               ViInt32 maxTestPacketsSent)
{
   ViStatus    error = VI_SUCCESS;
   ViChar      rep_cap[5] = ""; 
   
   checkErr (Rs_LockSession (instrSession, NULL));
   
   checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap)); 
   
   viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                                   RSCMWEVS_ATTR_RX_QUALITY_FORWARD_MAX_TEST_PACKETS_SENT, 
                                                   maxTestPacketsSent), 3,"Max Test Packets Sent");
   
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure RX Quality Reverse PER Measurement Sent Packets
 * Purpose:  Defines the length of a single shot reverse link PER
 *           measurement, i.e. the maximum number of packets sent.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRXQualityReversePERMeasurementSentPackets (ViSession instrSession,
                                                                               ViInt32 instance,
                                                                               ViInt32 maxPacketsSent)
{
   ViStatus    error = VI_SUCCESS;
   ViChar      rep_cap[5] = "";
   
   checkErr (Rs_LockSession (instrSession, NULL));
   
   checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap)); 
   
   viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                                   RSCMWEVS_ATTR_RX_QUALITY_REVERSE_MAX_PACKETS_SENT, 
                                                   maxPacketsSent), 3,"Max Packets Sent");
   
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error; 
}

/*****************************************************************************
 * Function: Configure RX Quality PER Measurement Sent Packets
 * Purpose:  Defines the maximum duration of the "Forward Link Performance"
 *           and "Reverse Link Performance" measurement as a number of 26.66
             ms CDMA frames.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRXQualityThroughputMeasurementFrames (ViSession instrSession,
                                                                          ViInt32 instance,
                                                                          ViInt32 linkType,
                                                                          ViInt32 maxFrames)
{
   ViStatus    error = VI_SUCCESS;
   ViChar      rep_cap[5] = ""; 
   
   
	viCheckParm (rscmwevs_invalidViInt32Range (instrSession, linkType, RSCMWEVS_VAL_LINK_FORWARD, RSCMWEVS_VAL_LINK_REVERSE), 3, "Link Type");
    
   checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));
   
   switch(linkType)
    {
        case RSCMWEVS_VAL_LINK_FORWARD:
            
        viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                                   RSCMWEVS_ATTR_RX_QUALITY_FORWARD_LINK_PERFORMANCE_MAX_FRAMES, 
                                                   maxFrames), 4,"Max Frames");
        break;
    
    
        case RSCMWEVS_VAL_LINK_REVERSE:
        
        viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                                   RSCMWEVS_ATTR_RX_QUALITY_REVERSE_LINK_PERFORMANCE_MAX_FRAMES, 
                                                   maxFrames), 4,"Max Frames");
        break;
        
    }
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure RX Quality Measurement Limits
 * Purpose:  Configures BER measurement parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRXQualityMeasurementLimits(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 PERLinkType,
    ViReal64    maxPERLimit,
    ViReal64    minConfidenceLevel
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSCMWEVS_IO_BUFFER_SIZE] = "";  
   
   checkErr (Rs_LockSession (instrSession, NULL));
   
   sprintf(buffer, "i%ld,%s", instance, PERLinkTypeArr[PERLinkType]);

    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, buffer,
                                                RSCMWEVS_ATTR_RX_QUALITY_MAX_PER_LIMIT,
                                                maxPERLimit), 4, "Max PER Limit");
    
    viCheckParm (rscmwevs_SetAttributeViReal64 (instrSession, buffer,
                                                RSCMWEVS_ATTR_RX_QUALITY_MIN_CONFIDENCE_LEVEL,
                                                minConfidenceLevel), 5, "Min Confidence Level");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Configure RX Quality Measurement Limits
 * Purpose:  Defines whether the limits specified for the forward and reverse
 *           link PER (Packet Error Rate) measurements shall be evaluated per
 *           carrier or over all carriers. This only affects multi-carrier
 *           evaluations.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ConfigureRXQualityCarrierEvaluation (ViSession instrSession,
                                                                ViInt32 instance,
                                                                ViInt32 limitEvaluation)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                              RSCMWEVS_ATTR_RX_QUALITY_EVALUATION_DECISION, 
                                              limitEvaluation), 3,"Limit Evaluation");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;    
}

/*****************************************************************************
 * Function: RX Quality Measurement Init
 * Purpose:  Starts or restarts the measurement; the R&S CMW 500 enters the 
 *           "RUN" state. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_RXQualityMeasurementInit(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 measurement
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSCMWEVS_IO_BUFFER_SIZE] = "";  
   
    checkErr (Rs_LockSession (instrSession, NULL));
   
    sprintf(buffer, "i%ld,%s", instance, MeasurementTypeArr[measurement]);

    viCheckErr (rscmwevs_SetAttributeViString (instrSession, buffer,
                                               RSCMWEVS_ATTR_RX_QUALITY_INIT, NULL));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: RX Quality Measurement Abort
 * Purpose:  Causes a running measurement to stop immediately; the R&S CMW 
 *           500 enters the "OFF" state. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_RXQualityMeasurementAbort(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 measurement
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSCMWEVS_IO_BUFFER_SIZE] = "";  
   
    checkErr (Rs_LockSession (instrSession, NULL));
   
    sprintf(buffer, "i%ld,%s", instance, MeasurementTypeArr[measurement]);

    viCheckErr (rscmwevs_SetAttributeViString (instrSession, buffer,
                                               RSCMWEVS_ATTR_RX_QUALITY_ABORT, NULL));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: RX Quality Measurement Stop
 * Purpose:  Causes a running measurement to stop after the current 
 *           evaluation period is terminated and valid results are available; 
 *           the R&S CMW 500 enters the "RDY" state. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_RXQualityMeasurementStop(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 measurement
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer[RSCMWEVS_IO_BUFFER_SIZE] = "";  
   
    checkErr (Rs_LockSession (instrSession, NULL));
   
    sprintf(buffer, "i%ld,%s", instance, MeasurementTypeArr[measurement]);

    viCheckErr (rscmwevs_SetAttributeViString (instrSession, buffer,
                                               RSCMWEVS_ATTR_RX_QUALITY_STOP, NULL));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: RX Quality Measurement Clear
 * Purpose:  Clears the statistics for all receiver quality measurements and 
 *           re-starts the measurements. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_RXQualityMeasurementClear(
    ViSession   instrSession,
    ViInt32 instance
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckErr (rscmwevs_SetAttributeViString (instrSession, rep_cap,
                                               RSCMWEVS_ATTR_RX_QUALITY_CLEAR, NULL));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Selects the view to be shown when the display is switched on during 
/// HIFN  remote control.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR display/Selects the display.
ViStatus _VI_FUNC rscmwevs_ConfigureRXQualityRemoteDisplay(
	ViSession	instrSession,
	ViInt32	instance,
	ViInt32	display
)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViInt32 (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_RX_QUALITY_REMOTE_DISPLAY,
                                                display), 3, "Display");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;     
}

/*****************************************************************************
 * Function: Query RX Quality Measurement Status
 * Purpose:  Queries the main measurement state and the measurement 
 *           substates. Both measurement substates are relevant for running 
 *           measurements only. Use FETCh...SIGN<instance>:STATe? to query 
 *           the main measurement state only. Use INITiate..., STOP..., 
 *           ABORt... to change the measurement state. 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryRXQualityMeasurementStatus(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 measurement,
    ViInt32*    mainState,
    ViInt32*    syncState,
    ViInt32*    resourceState
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
   
    sprintf (command, "FETC:EVDO:SIGN%ld:%s:STAT:ALL?", instance, MeasurementArr[measurement]);

    checkErr(rscmwevs_QueryMeasStatus (instrSession, command, mainState, syncState,
        resourceState));

    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Read RX Quality Forward Link PER Results
 * Purpose:  Starts a new single shot measurement and returns the results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ReadRXQualityForwardLinkPERResults(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 arraySize,
    ViInt32*    reliability,
    ViReal64    results[],
    ViInt32*    returnedValues
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Results");   
    };
    
    sprintf (command, "READ:EVDO:SIGN%ld:RXQ:FLP?", instance);
    checkErr (rscmwevs_GetMeasResults (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Fetch RX Quality Forward Link PER Results
 * Purpose:  Returns the measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_FetchRXQualityForwardLinkPERResults(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 arraySize,
    ViInt32*    reliability,
    ViReal64    results[],
    ViInt32*    returnedValues
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Results");   
    };
    
    sprintf (command, "FETC:EVDO:SIGN%ld:RXQ:FLP?", instance);
    checkErr (rscmwevs_GetMeasResults (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query RX Quality Forward Link PER Limit Check Results
 * Purpose:  Returns limit check results.
 *           
 *           Attribute(s):
 *           no attribute
 *           
 *           Remote-control command(s):
 *           CALCulate:EVDO:SIGN<instance>:RXQuality:FLPer? 
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryRXQualityForwardLinkPERLimitCheckResults(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 arraySize,
    ViInt32*    reliability,
    ViInt32 results[],
    ViInt32*    returnedValues
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Results");   
    };
    
    sprintf (command, "CALC:EVDO:SIGN%ld:RXQ:FLP?\n", instance);
    checkErr (rscmwevs_GetLimitCheckResults (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Fetch RX Quality Forward Link PER Measurement Status
 * Purpose:  Returns a string containing status information about the 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_FetchRXQualityForwardLinkPERMeasurementStatus(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 arraySize,
    ViChar  results[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViString (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_FETCH_RX_QUALITY_FORWARD_LINK_PER_MEASUREMENT_STATUS,
                                                arraySize, results), 4, "Results");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Returns status information about the carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwwmax_init or 
/// HIPAR instrSession/rscmwwmax_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR status/Returns status information about the carrier.
ViStatus _VI_FUNC rscmwevs_FetchRXQualityForwardLinkPERMeasurementCarrierStatus (ViSession instrSession,
                                                                                 ViInt32 instance,
                                                                                 ViInt32 *reliability,
                                                                                 ViInt32 *status)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    
    sprintf (command, "FETC:EVDO:SIGN%ld:RXQ:FLP:CST?\n", instance);
    checkErr (rscmwevs_GetCarrierStatus (instrSession, command, 15000, reliability, status));
    
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Read RX Quality Forward Link Performance Results
 * Purpose:  Starts a new single shot measurement and returns the results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ReadRXQualityForwardLinkPerformanceResults(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 packetSize,
    ViInt32 arraySize,
    ViInt32*    reliability,
    ViReal64    results[],
    ViInt32*    returnedValues
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, packetSize, RSCMWEVS_VAL_PACKET_SIZE_S128, RSCMWEVS_VAL_PACKET_SIZE_ALL), 3, "Packet Size");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Results");   
    };
    
    if (packetSize == RSCMWEVS_VAL_PACKET_SIZE_ALL)
        sprintf (command, "READ:EVDO:SIGN%ld:RXQ:FLPF?", instance);
    else
        sprintf (command, "READ:EVDO:SIGN%ld:RXQ:FLPF? %s", instance, packetSizeArr[packetSize]);
    
    checkErr (rscmwevs_GetMeasResults (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Fetch RX Quality Forward Link Performance Results
 * Purpose:  Returns the measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_FetchRXQualityForwardLinkPerformanceResults(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 packetSize,
    ViInt32 arraySize,
    ViInt32*    reliability,
    ViReal64    results[],
    ViInt32*    returnedValues
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, packetSize, RSCMWEVS_VAL_PACKET_SIZE_S128, RSCMWEVS_VAL_PACKET_SIZE_ALL), 3, "Packet Size");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Results");   
    };
    
    if (packetSize == RSCMWEVS_VAL_PACKET_SIZE_ALL)
        sprintf (command, "FETC:EVDO:SIGN%ld:RXQ:FLPF?", instance);
    else
        sprintf (command, "FETC:EVDO:SIGN%ld:RXQ:FLPF? %s", instance, packetSizeArr[packetSize]);
    
    checkErr (rscmwevs_GetMeasResults (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query RX Quality Forward Link Performance Limit Check Results
 * Purpose:  Returns limit check results.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryRXQualityForwardLinkPerformanceLimitCheckResults(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 packetSize,
    ViInt32 arraySize,
    ViInt32*    reliability,
    ViInt32 results[],
    ViInt32*    returnedValues
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, packetSize, RSCMWEVS_VAL_PACKET_SIZE_S128, RSCMWEVS_VAL_PACKET_SIZE_ALL), 3, "Packet Size");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 5, "Results");   
    };
    
    if (packetSize == RSCMWEVS_VAL_PACKET_SIZE_ALL)
        sprintf (command, "CALC:EVDO:SIGN%ld:RXQ:FLPF?\n", instance);
    else
        sprintf (command, "CALC:EVDO:SIGN%ld:RXQ:FLPF? %s\n", instance, packetSizeArr[packetSize]);
    
    checkErr (rscmwevs_GetLimitCheckResults (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Fetch RX Quality Forward Link Performance Measurement Status
 * Purpose:  Returns a string containing status information about the 
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_FetchRXQualityForwardLinkPerformanceMeasurementStatus(
    ViSession   instrSession,
    ViInt32 instance,
    ViInt32 arraySize,
    ViChar  results[]
)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViString (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_FETCH_RX_QUALITY_FORWARD_LINK_PERFORMANCE_MEASUREMENT_STATUS,
                                                arraySize, results), 4, "Results");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Returns status information about the carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwwmax_init or 
/// HIPAR instrSession/rscmwwmax_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR status/Returns status information about the carrier.
ViStatus _VI_FUNC rscmwevs_FetchRXQualityForwardLinkPerformanceMeasurementCarrierStatus (ViSession instrSession,
                                                                                         ViInt32 instance,
                                                                                         ViInt32 *reliability,
                                                                                         ViInt32 *status)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    
    sprintf (command, "FETC:EVDO:SIGN%ld:RXQ:FLPF:CST?\n", instance);
    checkErr (rscmwevs_GetCarrierStatus (instrSession, command, 15000, reliability, status));
    
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Read RX Quality Reverse Link PER Results
 * Purpose:  Reads the "Reverse Link PER" measurement results for the
 *           specified Data Rate (and the selected carrier in multi-carrier
 *           scenarios)
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ReadRXQualityReverseLinkPERResults (ViSession instrSession,
                                                               ViInt32 instance,
                                                               ViInt32 dataRate,
                                                               ViInt32 arraySize,
                                                               ViInt32 *reliability,
                                                               ViReal64 results[],
                                                               ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "READ:EVDO:SIGN%ld:RXQ:RLP? %s", instance, DataRateArr[dataRate]);
    checkErr (rscmwevs_GetMeasResults (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Fetch RX Quality Reverse Link PER Results
 * Purpose:  Returns the "Reverse Link PER" measurement results for the
 *           specified Data Rate (and the selected carrier in multi-carrier
 *           scenarios)
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_FetchRXQualityReverseLinkPERResults (ViSession instrSession,
                                                                ViInt32 instance,
                                                                ViInt32 dataRate,
                                                                ViInt32 arraySize,
                                                                ViInt32 *reliability,
                                                                ViReal64 results[],
                                                                ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "FETC:EVDO:SIGN%ld:RXQ:RLP? %s", instance, DataRateArr[dataRate]);
    checkErr (rscmwevs_GetMeasResults (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error; 
}

/*****************************************************************************
 * Function: Fetch RX Quality Reverse Link PER Results
 * Purpose:  Returns the "Reverse Link PER" measurement results for the
 *           specified Data Rate (and the selected carrier in multi-carrier
 *           scenarios)
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryRXQualityReverseLinkPERLimitCheckResults (ViSession instrSession,
                                                                          ViInt32 instance,
                                                                          ViInt32 dataRate,
                                                                          ViInt32 arraySize,
                                                                          ViInt32 *reliability,
                                                                          ViInt32 results[],
                                                                          ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    if (results == NULL)
    {
        viCheckParm (RS_ERROR_NULL_POINTER, 6, "Results");
    };
    
    sprintf (command, "CALC:EVDO:SIGN%ld:RXQ:RLP? %s", instance, DataRateArr[dataRate]);
    checkErr (rscmwevs_GetLimitCheckResults (instrSession, command, 15000, arraySize, reliability, results, returnedValues));
    
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Fetch RX Quality Reverse Link PER Measurement Status
 * Purpose:  Returns a string containing status information about the
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_FetchRXQualityReverseLinkPERMeasurementStatus (ViSession instrSession,
                                                                          ViInt32 instance,
                                                                          ViInt32 arraySize,
                                                                          ViChar results[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViString (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_FETCH_RX_QUALITY_REVERSE_LINK_PER_MEASUREMENT_STATUS,
                                                arraySize, results), 4, "Results");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error; 
}

/// HIFN  Returns status information about the carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwwmax_init or 
/// HIPAR instrSession/rscmwwmax_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR status/Returns status information about the carrier.
ViStatus _VI_FUNC rscmwevs_FetchRXQualityReverseLinkPERMeasurementCarrierStatus (ViSession instrSession,
                                                                                 ViInt32 instance,
                                                                                 ViInt32 *reliability,
                                                                                 ViInt32 *status)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    
    sprintf (command, "FETC:EVDO:SIGN%ld:RXQ:RLP:CST?\n", instance);
    checkErr (rscmwevs_GetCarrierStatus (instrSession, command, 15000, reliability, status));
    
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Read RX Quality Reverse Link Performance Results
 * Purpose:  Reads the results of the "Reverse Link Performance" throughput
 *           measurement for the specified data rate (and the selected
 *           carrier in multi-carrier scenarios),
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ReadRXQualityReverseLinkPerformanceResults (ViSession instrSession,
                                                                       ViInt32 instance,
                                                                       ViInt32 dataRate,
                                                                       ViInt32 arraySize,
                                                                       ViInt32 *reliability,
                                                                       ViInt32 *packetSize,
                                                                       ViReal64 results[],
                                                                       ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, RSCMWEVS_VAL_DATA_RATE_R0K0, RSCMWEVS_VAL_DATA_RATE_TOTAL), 3, "Data Rate");
    
    sprintf (command, "READ:EVDO:SIGN%ld:RXQ:RLPF? %s\n", instance, DataRateArr[dataRate]);
    
    checkErr (rscmwevs_GetPerformanceResults (instrSession, command, 15000, arraySize, reliability, packetSize, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Fetch RX Quality Reverse Link Performance Results
 * Purpose:  Returns the results of the "Reverse Link Performance" throughput
 *           measurement for the specified data rate (and the selected
 *           carrier in multi-carrier scenarios),
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_FetchRXQualityReverseLinkPerformanceResults (ViSession instrSession,
                                                                        ViInt32 instance,
                                                                        ViInt32 dataRate,
                                                                        ViInt32 arraySize,
                                                                        ViInt32 *reliability,
                                                                        ViInt32 *packetSize,
                                                                        ViReal64 results[],
                                                                        ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, RSCMWEVS_VAL_DATA_RATE_R0K0, RSCMWEVS_VAL_DATA_RATE_TOTAL), 3, "Data Rate");
    
    sprintf (command, "FETC:EVDO:SIGN%ld:RXQ:RLPF? %s\n", instance, DataRateArr[dataRate]);
    
    checkErr (rscmwevs_GetPerformanceResults (instrSession, command, 15000, arraySize, reliability, packetSize, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Query RX Quality Reverse Link Performance Limit Check Results
 * Purpose:  Returns limit check results reverse Link Performance.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryRXQualityReverseLinkPerformanceLimitCheckResults (ViSession instrSession,
                                                                                  ViInt32 instance,
                                                                                  ViInt32 dataRate,
                                                                                  ViInt32 arraySize,
                                                                                  ViInt32 *reliability,
                                                                                  ViInt32 *packetSize,
                                                                                  ViInt32 results[],
                                                                                  ViInt32 *returnedValues)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, RSCMWEVS_VAL_DATA_RATE_R0K0, RSCMWEVS_VAL_DATA_RATE_TOTAL), 3, "Data Rate");
    
    sprintf (command, "CALC:EVDO:SIGN%ld:RXQ:RLPF? %s\n", instance, DataRateArr[dataRate]);
    
    checkErr (rscmwevs_GetLimitCheckPerformanceResults (instrSession, command, 15000, arraySize, reliability, packetSize, results, returnedValues));
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: Fetch RX Quality Reverse Link Performance Measurement Status
 * Purpose:  Returns a string containing status information about the
 *           measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_FetchRXQualityReverseLinkPerformanceMeasurementStatus (ViSession instrSession,
                                                                                  ViInt32 instance,
                                                                                  ViInt32 arraySize,
                                                                                  ViChar results[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_GetAttributeViString (instrSession, rep_cap,
                                                RSCMWEVS_ATTR_FETCH_RX_QUALITY_REVERSE_LINK_PERFORMANCE_MEASUREMENT_STATUS,
                                                arraySize, results), 4, "Results");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error; 
}

/// HIFN  Returns status information about the carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwwmax_init or 
/// HIPAR instrSession/rscmwwmax_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement 
/// HIPAR reliability/result; see Reliability Indicator.
/// HIPAR status/Returns status information about the carrier.
ViStatus _VI_FUNC rscmwevs_FetchRXQualityReverseLinkPerformanceMeasurementCarrierStatus (ViSession instrSession,
                                                                                         ViInt32 instance,
                                                                                         ViInt32 *reliability,
                                                                                         ViInt32 *status)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = ""; 
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    
    sprintf (command, "FETC:EVDO:SIGN%ld:RXQ:RLPF:CST?\n", instance);
    checkErr (rscmwevs_GetCarrierStatus (instrSession, command, 15000, reliability, status));
    
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Enables or disables view. For a disabled view results are neither displayed nor calculated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwwmax_init or 
/// HIPAR instrSession/rscmwwmax_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR RLP_andIPStatistics/Enables or disables view.
ViStatus _VI_FUNC rscmwevs_ConfigureRXQualityDataRLPTableResultsState (ViSession instrSession,
                                                                       ViInt32 instance,
                                                                       ViBoolean RLP_andIPStatistics)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      rep_cap[5] = "";

    checkErr (Rs_LockSession (instrSession, NULL));

    checkErr(rscmwevs_instanceRepCap (instrSession, instance, rep_cap));

    viCheckParm (rscmwevs_SetAttributeViBoolean (instrSession, rep_cap,
                                              RSCMWEVS_ATTR_RX_QUALITY_DATA_RLP_TABLE_RESULTS_STATE, 
                                              RLP_andIPStatistics), 3,"RLP and IP Statistics");
    
Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;    
}

/// HIFN  This function returns the number of packets from the measured RLP 
/// HIFN  messages.      
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwwmax_init or 
/// HIPAR instrSession/rscmwwmax_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Selcts the result type.
/// HIPAR RX/Number of packets received in the last update period.
/// HIPAR RXTotal/Total number of packets received since the beginning of the PPP 
/// HIPAR RXTotal/connection
/// HIPAR TX/Number of packets transmitted in the last update period.
/// HIPAR TXTotal/Total number of packets transmitted.
ViStatus _VI_FUNC rscmwevs_QueryRXQualityDataRLPTableResults (ViSession instrSession,
                                                             ViInt32 instance,
                                                             ViInt32 resultType,
                                                             ViInt32 *RX,
                                                             ViInt32 *RXTotal,
                                                             ViInt32 *TX,
                                                             ViInt32 *TXTotal)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";
	ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, resultType, RSCMWEVS_VAL_DATA_RLP_RESULT_RESET, RSCMWEVS_VAL_DATA_RLP_RESULT_SUMMARY), 3, "Result Type");

    
    sprintf (command, "SENS:EVDO:SIGN%ld:RXQ:IPST:%s?\n", instance, RLPTableArr[resultType]);
    
	checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rscmwevs_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVS_ERROR_INCORRECT_RESPONSE; 
	*RX = rscmwevs_atol (p2buf);
	
	p2buf = strtok (NULL, ",\n");
	if (p2buf == NULL)
        return RSCMWEVS_ERROR_INCORRECT_RESPONSE;
	*RXTotal = rscmwevs_atol (p2buf);
	
	p2buf = strtok (NULL, ",\n"); 
	if (p2buf == NULL)
        return RSCMWEVS_ERROR_INCORRECT_RESPONSE;
	*TX = rscmwevs_atol (p2buf);
	
	p2buf = strtok (NULL, ",\n");
	if (p2buf == NULL)
        return RSCMWEVS_ERROR_INCORRECT_RESPONSE;
	*TXTotal = rscmwevs_atol (p2buf);   
    
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  This function returns the number of packets from the IP statistics 
/// HIFN  table.      
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwwmax_init or 
/// HIPAR instrSession/rscmwwmax_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Selcts the result type.
/// HIPAR RX/Number of packets received in the last update period.
/// HIPAR TX/Number of packets transmitted in the last update period.
ViStatus _VI_FUNC rscmwevs_QueryRXQualityDataIPStatisticTableResults (ViSession instrSession,
                                                                     ViInt32 instance,
                                                                     ViInt32 resultType,
                                                                     ViInt32 *RX,
                                                                     ViInt32 *TX)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";
	ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, resultType, RSCMWEVS_VAL_DATA_IP_TABLE_PPPT, RSCMWEVS_VAL_DATA_IP_TABLE_DRAT), 3, "Result Type");
    
    sprintf (command, "SENS:EVDO:SIGN%ld:RXQ:IPST:%s?\n", instance, IPStatTableArr[resultType]);
    
	checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rscmwevs_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVS_ERROR_INCORRECT_RESPONSE; 
	*RX = rscmwevs_atol (p2buf);
	
	p2buf = strtok (NULL, ",\n"); 
	if (p2buf == NULL)
        return RSCMWEVS_ERROR_INCORRECT_RESPONSE;
	*TX = rscmwevs_atol (p2buf);
	
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/// HIFN  Returns status information about the IP Statistic Table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR status/Returns status information about the IP Statistic Table.
ViStatus _VI_FUNC rscmwevs_QueryRXQualityDataIPStatisticStatus (ViSession instrSession,
                                                                ViInt32 instance,
                                                                ViInt32 *status)
{
	ViStatus    error = VI_SUCCESS;
    ViChar      command[50] = "";
	ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViUInt32 ret_cnt = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckParm (rscmwevs_invalidViInt32Range (instrSession, instance, 1, 16), 2, "Instance");
    
    sprintf (command, "SENS:EVDO:SIGN%ld:RXQ:IPST:STAT?\n", instance);
    
	checkErr (viWrite (instrSession, (ViBuf) command, (ViUInt32) strlen (command), NULL));
    
    checkErr (rscmwevs_CheckBeforeRead (instrSession, 15000));
    checkErr (Rs_ReadDataUnknownLength(instrSession, &ext_buf, &ret_cnt));

    p2buf = strtok (ext_buf, ",");
    if (p2buf == NULL)
        return RSCMWEVS_ERROR_INCORRECT_RESPONSE; 
	*status = rscmwevs_atol (p2buf); 
	
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    Rs_UnlockSession(instrSession, NULL);    
    return error;
}

/*****************************************************************************
 * Function: rscmwevs_SetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to set
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/
/* ViInt32 */
ViStatus _VI_FUNC rscmwevs_SetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViReal64 */
ViStatus _VI_FUNC rscmwevs_SetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 value)
{
     return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViString */
ViStatus _VI_FUNC rscmwevs_SetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViString value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rscmwevs_SetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViSession */
ViStatus _VI_FUNC rscmwevs_SetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession value)
{
    return Rs_SetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/*****************************************************************************
 * Function: rscmwevs_GetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to get
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rscmwevs_GetAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                ViUInt32 attributeId, ViInt32 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViInt32), (ViInt32 *)value);
}

/* ViReal64 */
ViStatus _VI_FUNC rscmwevs_GetAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViReal64 *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViReal64), (ViReal64 *)value);
}

/* ViString */
ViStatus _VI_FUNC rscmwevs_GetAttributeViString (ViSession instrSession, ViString channelName,
                                                 ViUInt32 attributeId, ViInt32 bufSize,
                                                 ViChar value[])
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            bufSize, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rscmwevs_GetAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViBoolean *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViBoolean), (ViBoolean *)value);
}

/* ViSession */
ViStatus _VI_FUNC rscmwevs_GetAttributeViSession (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViSession *value)
{
    return Rs_GetAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL,
                            (ViInt32) sizeof (ViSession), (ViSession *)value);
}


/*****************************************************************************
 * Function: Rs_GetAttributeRepCapNameId
 * Purpose:  This function returns the repeated capability id(s) that
 *           belongs to the attribute you specify. More then one id is listed
 *           in comma separated string.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_GetAttributeRepeatedCapabilityIds (ViSession instrSession,
                                                            ViUInt32 attributeID,
                                                            ViInt32 bufferSize,
                                                            ViChar _VI_FAR repeatedCapabilityIds[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapNameId[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 3, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapNameId (instrSession, attributeID, repCapNameId));

    strncpy (repeatedCapabilityIds, repCapNameId, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIds[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE : bufferSize] = '\0';

Error:
    return error;
}

/*****************************************************************************
 * Function: Get Attribute Repeated Capability Id Name(s)
 * Purpose:  This function returns the repeated capability name(s) that
 *           belongs to the attribute and single repeated capability id you
 *           may specify. More then one name is listed in comma separated
 *           string.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_GetAttributeRepeatedCapabilityIdNames (ViSession instrSession,
                                                                ViUInt32 attributeID,
                                                                ViString repeatedCapabilityId,
                                                                ViInt32 bufferSize,
                                                                ViChar _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      repCapName[RS_MAX_MESSAGE_BUF_SIZE] = "";

    if (bufferSize <= 0)
        viCheckParm(RS_ERROR_INVALID_PARAMETER, 4, "Buffer Size");

    viCheckErr (Rs_GetAttributeRepCapName (instrSession,
                                           attributeID,
                                           repeatedCapabilityId,
                                           repCapName));

    strncpy (repeatedCapabilityIdNames, repCapName, RS_MAX_MESSAGE_BUF_SIZE);
    repeatedCapabilityIdNames[(bufferSize > RS_MAX_MESSAGE_BUF_SIZE) ?
        RS_MAX_MESSAGE_BUF_SIZE - 1 : bufferSize - 1] = '\0';

Error:
    return error;
}

/*****************************************************************************
 * Function: rscmwevs_CheckAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to check
 *           attribute values directly.  These functions check the value you
 *           specify even if you set the rscmwevs_ATTR_RANGE_CHECK
 *           attribute to VI_FALSE.  There are typesafe versions for ViInt32,
 *           ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rscmwevs_CheckAttributeViInt32 (ViSession instrSession, ViString channelName,
                                                  ViUInt32 attributeId, ViInt32 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViReal64 */
ViStatus _VI_FUNC rscmwevs_CheckAttributeViReal64 (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViReal64 value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViString */
ViStatus _VI_FUNC rscmwevs_CheckAttributeViString (ViSession instrSession, ViString channelName,
                                                   ViUInt32 attributeId, ViString value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);
}

/* ViBoolean */
ViStatus _VI_FUNC rscmwevs_CheckAttributeViBoolean (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViBoolean value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/* ViSession */
ViStatus _VI_FUNC rscmwevs_CheckAttributeViSession (ViSession instrSession, ViString channelName,
                                                    ViUInt32 attributeId, ViSession value)
{
    return Rs_CheckAttribute (instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, &value);
}

/*****************************************************************************
 * Function: Reset
 * Purpose:  This function resets the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_reset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, 0));

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr( viPrintf (instrSession, "*RST\n"));
        }

    checkErr( rscmwevs_DefaultInstrSetup (instrSession));
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, NULL);

    return error;
}

/*****************************************************************************
 * Function: Self Test
 * Purpose:  This function executes the instrument self-test and returns the
 *           result.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_self_test (ViSession instrSession, ViInt16 *testResult,
                                      ViChar testMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, 0));

    if (testResult == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Test Result");
    if (testMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Test Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        viCheckErr( viPrintf (instrSession, "*TST?;*OPC\n"));
        viCheckErr( rscmwevs_WaitForOPC (instrSession, 600000));  
        viCheckErr( viScanf (instrSession, "%hd", testResult));
        if (*testResult == 0)
            strcpy (testMessage, "Self-Test Passed.");
        else
            strcpy (testMessage, "Self-Test Failed.");
        }
    else
        {
        /* Simulate Self Test */
        *testResult = 0;
        strcpy (testMessage, "No error.");
        }

    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/*****************************************************************************
 * Function: Revision Query
 * Purpose:  This function returns the driver and instrument revisions.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_revision_query (ViSession instrSession, ViChar driverRev[],
                                           ViChar instrRev[])
{
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, 0));

    if (driverRev == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Driver Revision");
    if (instrRev == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Instrument Revision");

    checkErr( Rs_GetAttribute (instrSession, NULL, RSCMWEVS_ATTR_SPECIFIC_DRIVER_REVISION,
                                        0, 256, driverRev));
    checkErr( Rs_GetAttribute (instrSession, "", RSCMWEVS_ATTR_INSTRUMENT_FIRMWARE_REVISION,
                                        0, 256, instrRev));
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/*****************************************************************************
 * Function:    System Version
 * Purpose:     This function queries the number of the SCPI version, which 
 *              is relevant for the instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_SystemVersion (ViSession instrSession,
                                          ViInt32 length, 
                                          ViChar systemVersion[])

{
    ViStatus    error = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, 0)); 
    
    viCheckParm( rscmwevs_GetAttributeViString (instrSession, "", 
                                                RSCMWEVS_ATTR_SYSTEM_VERSION, length,
                                                systemVersion),
                                                2, "System Version");
    checkErr( rscmwevs_CheckStatus (instrSession));
    
Error:
    (void) Rs_UnlockSession(instrSession, NULL); 
    return error;
}

/*****************************************************************************
 * Function: Error Query
 * Purpose:  This function queries the instrument error queue and returns
 *           the result.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_error_query (ViSession instrSession, ViInt32 *errorCode,
                                        ViChar errorMessage[])
{
    ViStatus    error   = VI_SUCCESS;
    ViChar      buffer[RSCMWEVS_IO_BUFFER_SIZE] = "";
    ViInt32     oldCode = 0;
    ViChar      oldMessage[256] = "";
    ViChar      *p2buf = NULL;
    ViInt32     i = 0;
    
    checkErr( Rs_LockSession (instrSession, 0));

    if (errorCode == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error Code");
    if (errorMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    if (Rs_Simulating(instrSession) == 0)                /* call only when locked */
        {
        p2buf = (char *)malloc (1);

        *p2buf='\0';
            
        do 
        {
        viCheckErr (rscmwevs_GetAttributeViString( instrSession, "", RSCMWEVS_ATTR_SYSTEM_ERROR, RSCMWEVS_IO_BUFFER_SIZE, buffer));
            if  (sscanf (buffer, "%ld,%*[^0-9a-zA-Z]%256[^\"]", &oldCode, oldMessage) != 2)
            {
            viCheckErr(RS_ERROR_UNEXPECTED_RESPONSE);
            }
            else
            {
                if (oldCode != 0)
                {
                    *errorCode = oldCode;
                    
                    p2buf = (char *)realloc (p2buf, strlen(p2buf) + strlen (oldMessage) + 2);
        
                    strcat (p2buf, oldMessage);

                    strcat (p2buf, "\n");
                    
                    i++;
                }
                else if (i == 0)
                {
                    *errorCode = oldCode;
    
                    strncpy (errorMessage, oldMessage, 1024);
                }
            }

        } while (oldCode != 0);
        
        if (i > 0)
        {
            p2buf[strlen (p2buf) - 1] = '\0'; 
            
            strncpy (errorMessage, p2buf, 1024);
        }
        }
    
    else
        {
            /* Simulate Error Query */
        *errorCode = 0;
        strcpy (errorMessage, "No error.");
        }

Error:
    if (p2buf) free (p2buf);
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/*****************************************************************************
 * Function: Error List
 * Purpose:  This function reads all system messages and returns a list of 
 *           comma separated strings.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ErrorList (ViSession instrSession,
                                      ViInt32 bufferSize, 
                                      ViChar errors[])
{
    ViStatus    error   = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, 0));

    if (bufferSize < 0)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Buffer Size");
    if (errors == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Errors");

    viCheckErr (rscmwevs_GetAttributeViString( instrSession, "", RSCMWEVS_ATTR_SYSTEM_ALL_ERRORS, 
        bufferSize, errors));

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/*****************************************************************************
 * Function: Error Message
 * Purpose:  This function translates the error codes returned by this
 *           instrument driver into user-readable strings.
 *
 *           Note:  The caller can pass NULL for the vi parameter.  This
 *           is useful if one of the init functions fail.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_error_message (ViSession instrSession, ViStatus errorCode,
                                          ViChar errorMessage[])
{
    ViStatus    error   = VI_SUCCESS;

    static      RsStringValueTable errorTable =
        {
            RSCMWEVS_ERROR_CODES_AND_MSGS,
            {0, NULL}
        };

    (void) Rs_LockSession(instrSession, NULL);

        /* all VISA and RS error codes are handled as well as codes in the table */
    if (errorMessage == NULL)
        viCheckParm( RS_ERROR_INVALID_PARAMETER, 3, "Null address for Error Message");

    checkErr( Rs_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, errorTable));

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/*****************************************************************************
 * Function: Reliability Indicator
 * Purpose:  This function converts a reliability code returned by an
 *           instrument driver function into a user-readable string.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_ReliabilityIndicator (ViSession instrSession,
                                                 ViInt32 reliabilityCode,
                                                 ViChar reliabilityMessage[])
{
	ViStatus	error = VI_SUCCESS;
	ViInt16     cnt     = 0;
	ViBoolean   found = VI_FALSE;
	
	static      RsStringValueTable reliabilityTable =
	{
		{0,       "OK"},
		{1,       "Measurement Timeout"},
		{2,       "Capture Buffer Overflow"},
		{3,       "Overdriven"},
		{4,       "Underdriven"},
		{6,       "Trigger Timeout"},
		{7,       "Acquisition Error"},
		{8,       "Sync Error"},
		{9,       "Uncal"},
		{15,      "Reference Frequency Error"},
		{16,      "RF Not Available"},
		{17,      "RF Level not Settled"},
		{18,      "RF Frequency not Settled"},
		{19,      "Call not Established"},
		{20,      "Call Type not Usable"},
		{21,      "Call Lost"},
		{23,      "Missing Option"},
		{24,      "Invalid RF Setting"},
		{26,      "Resource Conflict"},
		{27,      "No Sensor Connected"},
		{28,      "Unexpected Parameter Change"},
		{30,      "File not Found"},
		{31,      "No DTM reply"},
		{32,      "ACL Disconnected"},
		{40,      "ARB File CRC Error"},
		{42,      "ARB Header Tag Invalid"},
		{43,      "ARB Segment Overflow"},
		{44,      "ARB File not Found"},
		{45,      "ARB Memory Overflow"},
		{46,      "ARB Sample Rate out of Range"},
		{47,      "ARB Cycles out of Range"},
		{50,      "Startup Error"},
		{51,      "No Reply"},
		{52,      "Connection Error"},
		{53,      "Configuration Error"},
		{54,      "Filesystem Error"},
		{60,      "Invalid RF-Connector Setting"},
		{93,      "OCXO Oven Temperature too low"},
		{101,     "Firmware Error"},
		{102,     "Unidentified Error"},
		{103,     "Parameter Error"},
		{104,     "Not Functional"},
		{0, NULL}
	};
	
	(void) Rs_LockSession(instrSession, NULL);
	
    for (cnt = 0; reliabilityTable[cnt].string; cnt++)
    {
	    if (reliabilityTable[cnt].value == reliabilityCode)
	    {
	        sprintf (reliabilityMessage, "%s", reliabilityTable[cnt].string);
			if (reliabilityCode)
				error = RSCMWEVS_ERROR_RELIABILITY_INDICATOR;
			found = VI_TRUE;
	    }
	}
	if (!found)
	{
		sprintf (reliabilityMessage, "Unrecognized reliability indicator! Report problem to R&S support!");

		error = VI_WARN_UNKNOWN_STATUS;
	}
	
	(void) Rs_UnlockSession (instrSession, NULL);
    return error;
}

/*****************************************************************************
 * Function: Preset
 * Purpose:  This function initiates an instrument reset.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_Preset (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;
   
    checkErr( Rs_LockSession (instrSession, 0));

    viCheckErr (rscmwevs_SetAttributeViString (instrSession, "", 
                RSCMWEVS_ATTR_SYSTEM_PRESET, NULL));
    
    checkErr( rscmwevs_CheckStatus (instrSession));

Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwevs_ClearStatus (ViSession instrSession)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, NULL));
	
	checkErr (rscmwevs_WriteInstrData (instrSession, "*CLS\n"));
	
	checkErr( rscmwevs_CheckStatus (instrSession));

Error:
	(void) Rs_UnlockSession (instrSession, NULL);
    return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Defines the size of ID Response Query.
/// HIPAR IDQueryResponse/Returns the ID Query response string.
/// HIPAR returnedValues/Returns length of string readed from instrument.
ViStatus _VI_FUNC rscmwevs_IDQueryResponse (ViSession instrSession,
                                         ViInt32 bufferSize,
                                         ViChar IDQueryResponse[],
                                         ViInt32 *returnedValues)
{
	ViStatus	error = VI_SUCCESS;

	checkErr (Rs_LockSession (instrSession, NULL));

	checkErr (rscmwevs_QueryViString (instrSession, "*IDN?", bufferSize, IDQueryResponse));
	
	if (returnedValues)
	{
		if (error == VI_SUCCESS)
			*returnedValues= (ViInt32) strlen(IDQueryResponse);
		else	
		{
			*returnedValues=error;
			error = VI_SUCCESS;
		}
	}	
	
Error:
	(void) Rs_UnlockSession (instrSession, NULL);
    return error;
}

/// HIFN  Stops further command processing until all commands sent before
/// HIFN  *WAI have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwevs_ProcessAllPreviousCommands (ViSession instrSession)
{
	ViStatus	error = VI_SUCCESS;

	checkErr (Rs_LockSession (instrSession, NULL));
	
	checkErr (rscmwevs_WriteInstrData (instrSession, "*WAI\n"));
	
Error:
	(void) Rs_UnlockSession (instrSession, NULL);
    return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwevs_init or 
/// HIPAR instrSession/rscmwevs_InitWithOptions function.  The handle identifies a particular 
/// HIPAR instrSession/instrument session.
/// HIPAR OPC/Queries the OPC.
ViStatus _VI_FUNC rscmwevs_QueryOPC (ViSession instrSession, ViInt32 *OPC)
{
	ViStatus	error = VI_SUCCESS;
	
	checkErr (Rs_LockSession (instrSession, NULL));
	
	checkErr (rscmwevs_QueryViInt32 (instrSession, "*OPC?", OPC));

Error:
	(void) Rs_UnlockSession (instrSession, NULL);
    return error;
}

/*===========================================================================*/
/* Function: Set OPC Timeout                                                 */
/* Purpose:  Sets new OPC timeout                                            */
/*===========================================================================*/
ViStatus _VI_FUNC rscmwevs_SetOPCTimeout(ViSession   instrSession, ViUInt32 timeout)
{
    ViStatus    error = VI_SUCCESS; 
    
    checkErr( Rs_SetAttribute (instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, &timeout));  
    
Error:    
    return error;
}

/*===========================================================================*/
/* Function: Get OPC Timeout                                                 */
/* Purpose:  Gets new OPC timeout                                            */
/*===========================================================================*/
ViStatus _VI_FUNC rscmwevs_GetOPCTimeout(ViSession   instrSession, ViUInt32 *timeout)
{
    ViStatus    error = VI_SUCCESS; 
    
    checkErr( Rs_GetAttribute (instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, sizeof(timeout), timeout));  
    
Error:    
    return error;
}

/// HIFN This function specifies the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIFN Note that the actual timeout used may be higher than the one
/// HIFN requested.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR VISATimeout/Specifies the minimum timeout value to use (in milliseconds)
/// HIPAR VISATimeout/when accessing the device associated with the given session.
/// HIPAR VISATimeout/Note that the actual timeout used may be higher than the one
/// HIPAR VISATimeout/requested.
ViStatus _VI_FUNC rscmwevs_SetVISATimeout (ViSession instrSession,
                                        ViUInt32 VISATimeout)
{
	return viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, VISATimeout);
}

/// HIFN This function queries the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR VISATimeout/Queries the minimum timeout value to use (in milliseconds)
/// HIPAR VISATimeout/when accessing the device associated with the given session.
ViStatus _VI_FUNC rscmwevs_GetVISATimeout (ViSession instrSession,
                                        ViUInt32 *VISATimeout)
{
	return viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, VISATimeout);
}

/// HIFN This function switches ON/OFF various error checkings performed inside of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches option checking On or Off.
/// HIPAR rangeChecking/This control enables or disables range checking.
/// HIPAR statusChecking/This control enables or disables instrument state checking
ViStatus _VI_FUNC rscmwevs_ConfigureErrorChecking (ViSession instrSession,
                                                  ViBoolean optionChecking,
                                                  ViBoolean rangeChecking,
                                                  ViBoolean statusChecking)
{
    ViStatus    error = VI_SUCCESS;
    ViInt32     p2value         = 0;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);

    sessionProperties -> optionChecking = optionChecking;
    p2value = (ViInt32) statusChecking;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, &p2value));
    p2value = (ViInt32) rangeChecking;
    checkErr (Rs_SetAttribute (instrSession, "", RS_ATTR_RANGE_CHECK, 0, &p2value));
	
Error:
    return error;
}

/*****************************************************************************
 * Function: Get Error and Clear Error Functions
 * Purpose:  These functions enable the instrument driver user to
 *           get or clear the error information the driver associates with the
 *           RS session.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_GetError (ViSession instrSession,
                                     ViStatus *errorCode,
                                     ViInt32 bufferSize,
                                     ViChar description[])
{
    ViStatus    error       = VI_SUCCESS;
    ViStatus    primary     = VI_SUCCESS,
                secondary   = VI_SUCCESS;
    ViChar      elaboration[RS_MAX_MESSAGE_BUF_SIZE]    = "";
    ViChar      errorMessage[RS_MAX_MESSAGE_BUF_SIZE]  = "";
    ViChar     *appendPoint = errorMessage;
    ViInt32     actualSize  = 0;
    ViBoolean   locked      = VI_FALSE;
    RsSessionPropertiesPtr  sessionProperties = Rs_ViSession (instrSession);

    /* Lock Session */
    checkErr( Rs_LockSession(instrSession, &locked));

    /* Test for nulls and acquire error data */
    if (bufferSize != 0)
    {
        if (errorCode == NULL)
        {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 2, "Null address for Error");
        }
        if (sessionProperties == NULL)
        {
            primary = VI_ERROR_RSRC_NFOUND;
            secondary = 0;
        }
        else
        {
            checkErr( Rs_GetErrorInfo (instrSession, &primary, &secondary, elaboration));
        }
    }

    else
        {
            if (sessionProperties == NULL)
            {
                (void) Rs_UnlockSession (instrSession, &locked);
                return 1024;
            }
            else
            {
                checkErr( Rs_GetAttribute (instrSession, NULL, RS_ATTR_ERROR_ELABORATION, 0, 256, elaboration));
                checkErr( Rs_GetAttribute (instrSession, NULL, RS_ATTR_SECONDARY_ERROR, 0, (ViInt32) sizeof (secondary), &secondary));
                checkErr( Rs_GetAttribute (instrSession, NULL, RS_ATTR_PRIMARY_ERROR, 0, (ViInt32) sizeof (primary), &primary));
            }
        }

    /* Format data */
    if (primary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rscmwevs_error_message (instrSession, primary, msg));
        appendPoint += sprintf(appendPoint, "Primary Error: (Hex 0x%08X) %s\n", (unsigned int) primary, msg);
        }

    if (secondary != VI_SUCCESS)
        {
        ViChar msg[256] = "";
        checkErr( rscmwevs_error_message (instrSession, secondary, msg));
        appendPoint += sprintf(appendPoint, "Secondary Error: (Hex 0x%08X) %s\n", (unsigned int) secondary, msg);
        }

    if (strlen (elaboration) > 0)
        {
        sprintf(appendPoint, "Elaboration: %s", elaboration);
        }

    actualSize = (ViInt32) (strlen(errorMessage) + 1);

    /* Prepare return values */
    if (bufferSize == 0)
        {
        error = (ViStatus) actualSize;
        }
    else
        {
        if (bufferSize > 0)
            {
            if (actualSize > bufferSize)
                {
                error = actualSize;
                actualSize = bufferSize;
                }
            }
        if (description == NULL)
            {
            viCheckParm( RS_ERROR_INVALID_PARAMETER, 4, "Null address for Description");
            }
        memcpy(description, errorMessage, (size_t)(actualSize-1));
        description[actualSize-1] = '\0';
        }

    if (errorCode)
        {
        *errorCode = primary;
        }

Error:
    /* Unlock Session */
    (void) Rs_UnlockSession (instrSession, &locked);
    return error;
}

ViStatus _VI_FUNC rscmwevs_ClearError (ViSession instrSession)
{
    return Rs_ClearErrorInfo (instrSession);
}

/// HIFN This function is used to read data from the host computer and 
/// HIFN write it to a user specified file in the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from control
/// HIPAR source/computer to instrument.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from control computer to instrument applies.
ViStatus _VI_FUNC rscmwevs_WriteFromFileToInstrument(ViSession instrSession,
	ViString source,
	ViString destination)
{
	ViStatus    error = VI_SUCCESS;
    ViUInt32    digits = 0;
    FILE        *sourceFile = NULL;
    ViUInt64    fileSize = 0;
#if defined (_CVI_)
	ViUInt32      fileSize32 = 0;
#endif
	ViChar 		command[1024];
	
    checkErr( Rs_LockSession (instrSession, NULL));
    viCheckParm (!source, 2, "Source");
    viCheckParm (!destination, 3, "Destination");  
       
    /* Open source file from host controller */
    if ((sourceFile = fopen (source, "rb")) == NULL)
        viCheckErr(RS_ERROR_CANNOT_OPEN_FILE);

    /* Get size of file */
    (void) fseek (sourceFile, 0, SEEK_END);
#if defined (_CVI_)
    GetFileSize (source, &fileSize32);
	fileSize = (ViUInt64) fileSize32;
#elif defined(_MSC_VER)
    fileSize = (ViUInt64) _ftelli64 (sourceFile);
#else
    fileSize = (ViUInt64) ftell (sourceFile);
#endif
    (void) fseek (sourceFile, 0, SEEK_SET);
	
	(void) viSetAttribute (instrSession, VI_ATTR_SEND_END_EN, VI_FALSE); 

    /* Create command header */
    digits = (ViUInt32) ((int)floor(log10((double)fileSize))) + 1;
    if (digits < 10)
	{
		sprintf (command, ":MMEM:DATA '%s',#%ld%lld", destination, digits, fileSize);
	}
	else
	{
		sprintf (command, ":MMEM:DATA '%s',#(%lld)", destination, fileSize);
	}
	viCheckErr (viWrite(instrSession, (ViBuf) command, (ViUInt16)strlen(command), NULL));
	
    viCheckErr (Rs_WriteFromFile2(instrSession, source, fileSize, 0));
        
    checkErr( rscmwevs_CheckStatus (instrSession));
 
 Error:
    if (sourceFile) (void) fclose (sourceFile);
    
    (void) Rs_UnlockSession(instrSession, NULL);
    
    return error;
}

/// HIFN This function is used to read data from the instrument and write 
/// HIFN it to a user specified file on the host computer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize 
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from instrument
/// HIPAR source/to control computer.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from instrument to control computer applies.
ViStatus _VI_FUNC rscmwevs_ReadToFileFromInstrument(ViSession instrSession,
	ViString source,
	ViString destination)
{
	ViStatus    error = VI_SUCCESS;
    //ViInt32     count = 0;
    ViUInt32    ret_count = 0;
    ViUInt64    length = 0;
    ViChar      buffer[RSCMWEVS_IO_BUFFER_SIZE] = "";
    ViUInt16 intfc = VI_INTF_GPIB;     
	ViChar      *pbuffer = NULL;
	int i;
    ViChar 		command[1024];
	
    checkErr( Rs_LockSession (instrSession, NULL));
    
	viCheckParm (!source, 2, "Source");
	viCheckParm (!destination, 3, "Destination");    
    
	checkErr (viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));
    switch (intfc)
    {
        case VI_INTF_ASRL: /* Serial intfc. */
            viCheckErr( viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
        break;
    }
    viCheckErr( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));
    
	sprintf (command, ":MMEMory:DATA? '%s'\n", source);    
	viCheckErr (viWrite(instrSession, (ViBuf) command, (ViUInt16)strlen(command), NULL));
    
    checkErr (viRead (instrSession, (ViBuf) buffer, 1, NULL)); 

	
	if (buffer[0] == '#')
	{   /* Binary data */
		/* Disable termination character */
		checkErr(viSetAttribute(instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));

		/* Parse header */
		checkErr(viRead(instrSession, (ViBuf)buffer, 1, NULL));

		buffer[1] = '\0';
		if (buffer[0] == '(')
		{
			for (i = 0; i < 20; i++)
			{
				checkErr(viRead(instrSession, (ViBuf)buffer + i, 1, NULL));

				if (buffer[i] == ')')
				{
					buffer[i] = '\0';
					sscanf(buffer, "%lld", &length);
					break;
				}
			}
			if (i == 20)
			{
				//couldn't find the ')' character
				checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
			}
		}
		else if (buffer[0] == '0')
		{
			length = 0;
		}
		else
		{
			sscanf(buffer, "%llu", &length);
			if (length > 9 || length < 1)
			{
				//expected number 1 to 9
				checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
			}

			checkErr(viRead(instrSession, (ViBuf)buffer, (ViUInt32) length, NULL));
			buffer[length] = '\0';
			sscanf(buffer, "%llu", &length);
		}

		if (length > 0)
		{
			viCheckErr(Rs_ReadToFile2(instrSession, destination, length, RS_VAL_TRUNCATE));
			/* Flush garbage */
			viScanf(instrSession, "%*[^\r\n]");
			if (error > 0) error = VI_SUCCESS;
		}
		else
		{
			/* Unlimited size of data */
			checkErr(Rs_ReadDataUnknownLength(instrSession, &pbuffer, &ret_count));
		}
    }
    else
    {
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE);         
    }
	
    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);        
    checkErr( rscmwevs_CheckStatus (instrSession));
  
Error:
    viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
    if (pbuffer) free(pbuffer);
    switch (intfc)
    {
        case VI_INTF_ASRL:
            viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
        break;    
    }  
    (void) Rs_UnlockSession(instrSession, NULL);
    
    return error;
}

/*****************************************************************************
 * Function: WriteInstrData and ReadInstrData Functions
 * Purpose:  These functions enable the instrument driver user to
 *           write and read commands directly to and from the instrument.
 *
 *           Note:  These functions bypass the RS attribute state caching.
 *                  WriteInstrData invalidates the cached values for all
 *                  attributes.
 *****************************************************************************/

ViStatus _VI_FUNC rscmwevs_WriteInstrData (ViSession instrSession, ViString writeBuffer)
{
    return Rs_WriteInstrData (instrSession, writeBuffer);
}

ViStatus _VI_FUNC rscmwevs_ReadInstrData (ViSession instrSession, ViInt32 numBytes,
                                          ViChar rdBuf[], ViUInt32 *bytesRead)
{
    return Rs_ReadInstrData (instrSession, numBytes, rdBuf, bytesRead);
}

/*****************************************************************************
 * Function: Query ViBoolean
 * Purpose:  This function queries the ViBoolean value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryViBoolean (ViSession instrSession,
                                           ViString command, 
                                           ViBoolean *value)
{
    ViStatus    error = VI_SUCCESS;
    ViChar      buffer [10] = "";
    
    checkErr( Rs_LockSession (instrSession, 0)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==NULL)
        viCheckParm (RS_ERROR_NULL_POINTER, 3, "Value");
    
    viCheckErr (viQueryf (instrSession, "%s\n", "%[^\r\n]", command, buffer));
    if (strstr (buffer, "OFF") != NULL)
        *value = VI_FALSE;
    else
        *value = VI_TRUE;
            
    checkErr( rscmwevs_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/*****************************************************************************
 * Function: Query ViInt32
 * Purpose:  This function queries the ViInt32 value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryViInt32 (ViSession instrSession,
                                         ViString command, 
                                         ViInt32 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, 0)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==NULL)
        viCheckParm (RS_ERROR_NULL_POINTER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%ld", command, value));
    checkErr( rscmwevs_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/*****************************************************************************
 * Function: Query ViReal64
 * Purpose:  This function queries the ViReal64 value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryViReal64 (ViSession instrSession,
                                          ViString command, 
                                          ViReal64 *value)
{
    ViStatus    error = VI_SUCCESS;
    
    checkErr( Rs_LockSession (instrSession, 0)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==NULL)
        viCheckParm (RS_ERROR_NULL_POINTER, 3, "Value");
    viCheckErr (viQueryf (instrSession, "%s\n", "%le", command, value));
    checkErr( rscmwevs_CheckStatus (instrSession)); 
Error:
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}


/*****************************************************************************
 * Function: Query ViString
 * Purpose:  This function queries the ViString value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_QueryViString (ViSession instrSession,
                                          ViString command, 
                                          ViInt32 bufferSize,
                                          ViChar _VI_FAR value[])
{
    ViStatus    error = VI_SUCCESS;
    ViChar      *pbuffer = NULL;
    ViUInt32        bytesRead;
    
    checkErr( Rs_LockSession (instrSession, 0)); 
    if (strlen(command)==0)
        viCheckParm (RS_ERROR_INVALID_PARAMETER, 2, "Command");
    if (value==NULL)
        viCheckParm (RS_ERROR_NULL_POINTER, 3, "Value");
    viCheckErr (viPrintf (instrSession, "%s\n", command));
    viCheckErr(Rs_ReadDataUnknownLength (instrSession, &pbuffer, &bytesRead));  
    if (bytesRead>=(ViUInt32)bufferSize)
    {
        strncpy(value, pbuffer, (size_t) (bufferSize-1));
        value[bufferSize-1]='\0';
        error = (ViStatus) (bytesRead+1);
    } 
    else
    {
        strncpy(value, pbuffer, (size_t) bytesRead); 
        value[bytesRead]='\0';   
    }
    checkErr( rscmwevs_CheckStatus (instrSession)); 
Error:
    if (pbuffer) free (pbuffer);
    (void) Rs_UnlockSession(instrSession, NULL);
    return error;
}

/*****************************************************************************
 * Function: rscmwevs_close
 * Purpose:  This function closes the instrument.
 *
 *           Note:  This function must unlock the session before calling
 *           Rs_Dispose.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_close (ViSession instrSession)
{
    ViUInt16    intfc;
    ViStatus    error   = VI_SUCCESS;

    checkErr( Rs_LockSession (instrSession, 0));
    checkErr( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));
    if (intfc == VI_INTF_GPIB)
	     viGpibControlREN (instrSession, VI_GPIB_REN_DEASSERT);
    checkErr( rscmwevs_RsClose (instrSession));

Error:

    (void) Rs_UnlockSession (instrSession, NULL);
    (void) Rs_Dispose (instrSession);

    return error;
}

/*****************************************************************************
 * Function: rscmwevs_RsClose
 * Purpose:  This function performs all of the drivers clean-up operations
 *           except for closing the RS session.  This function is called by
 *           rscmwevs_close or by an RS class driver.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwevs_RsClose (ViSession instrSession)
{
    ViStatus    error   = VI_SUCCESS;

    /* Do not lock here. The caller manages the lock. */

    if(instrSession != 0)
        {
        ViSession   rmSession = 0;

        checkErr( Rs_GetAttribute (instrSession, NULL, RS_ATTR_VISA_RM_SESSION, 0, (ViUInt32) sizeof (rmSession), &rmSession));
        (void) viClose (instrSession);
        (void) viClose (rmSession);
        }
Error:    
    return error;
}

/*****************************************************************************
 *------------------------ Global Session Callbacks -------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: Rs_CheckStatusCallback
 * Purpose:  This function queries the instrument to determine if it has
 *           encountered an error. If the instrument has encountered an
 *           error, this function returns the RS_ERROR_INSTRUMENT_SPECIFIC
 *           error code. This function is called by the
 *           Rs_CheckStatus utility function. The
 *           Rs_SetAttribute and Rs_GetAttribute functions invoke this
 *           function when the optionFlags parameter includes the
 *           RS_VAL_DIRECT_USER_CALL flag.
 *
 *           The user can disable calls to this function by setting the
 *           RSCMWEVS_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE.  The driver can
 *           disable the check status callback for a particular attribute by
 *           setting the RS_VAL_DONT_CHECK_STATUS flag.
 *
 *****************************************************************************/
ViStatus rscmwevs_CheckStatusCallback (ViSession instrSession)
{
    ViUInt32    ret_cnt;
    ViStatus    error   = VI_SUCCESS;
    ViInt16     esr     = 0;
    ViChar      buffer[1024];
    ViInt32     code = 0;
    
    //viCheckErr (viQueryf (instrSession, "*ESR?\n", "%hd", &esr));
    viCheckErr (viWrite (instrSession, (ViBuf) "*STB?\n", 6, NULL));
    viCheckErr (viRead (instrSession, (ViPBuf) buffer, 4, &ret_cnt));
    buffer[ret_cnt] = '\0';
    esr = (ViInt16) atol (buffer);

    if ((esr & IEEE_488_2_ERROR_MASK) != 0)
    {
        rscmwevs_error_query (instrSession, &code, buffer);
        
        checkErr (Rs_SetErrorInfo (instrSession, VI_TRUE, RS_ERROR_INSTR_SPECIFIC, code, buffer));
        
        viCheckErr (RS_ERROR_INSTR_SPECIFIC);
    }

Error:

    return error;
}

/*****************************************************************************
 * Function: Rs_WaitForOPCCallback
 * Purpose:  This function waits until the instrument has finished processing
 *           all pending operations. This function is called by the
 *           Rs_WaitForOPC utility function. The RS engine invokes
 *           this function in the following two cases:
 *           - Before invoking the read callback for attributes for which the
 *             RS_VAL_WAIT_FOR_OPC_BEFORE_READS flag is set.
 *           - After invoking the write callback for attributes for which the
 *             RS_VAL_WAIT_FOR_OPC_AFTER_WRITES flag is set.
 *
 *****************************************************************************/
ViStatus rscmwevs_WaitForOPCCallback (ViSession instrSession)
{
    ViStatus    error = VI_SUCCESS;
    ViUInt16    response    = 0;
    ViUInt16    intfc   = VI_INTF_GPIB;
    ViInt32     opc_timeout = 0;
    clock_t     start_time = clock();
    ViReal64    opc_timeout_ms = 0.0;
    double      elapsed_time_ms;
    double      ms_divider = (double)CLOCKS_PER_SEC / 1000.0;

    checkErr( Rs_GetAttribute (instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0,
        (ViInt32) sizeof (opc_timeout), &opc_timeout));

    opc_timeout_ms = (ViReal64) opc_timeout;

    viCheckErr( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &intfc));

    switch (intfc)
    {
        case VI_INTF_ASRL: /* Serial intfc */
            
            /* Not supported */

        break;
        default:

            do
            {
                error = viReadSTB (instrSession, &response);
                if ((response & IEEE_488_2_SRE_MASK) != 0)
                    break;

                elapsed_time_ms = (double)(clock() - start_time)/ms_divider;
                if (elapsed_time_ms > opc_timeout_ms)
                    return VI_ERROR_TMO;
                
                if (elapsed_time_ms < 10.0)
                {/* Do it as fast as possible, therefore no Sleep */
                }
                else if (elapsed_time_ms < 100.0)
                    Sleep (1);
                else if (elapsed_time_ms < 1000.0)
                    Sleep (5);
                else if (elapsed_time_ms < 5000.0)
                    Sleep (10);
                else if (elapsed_time_ms < 10000.0)
                    Sleep (50);
                else if (elapsed_time_ms < 20000.0)
                    Sleep (100);
                else if (elapsed_time_ms < 30000.0)
                    Sleep (500);
                else
                    Sleep (1000);
                
            } while ((response & IEEE_488_2_SRE_MASK) == 0);

            if ((response & IEEE_488_2_ERROR_MASK) != 0)
            {
                error =  RS_ERROR_INSTRUMENT_STATUS;
                
            }
        }
Error:
    return error;
}
